	SUBROUTINE BINVALFITS(LU,MAP,BIN,XST,XEND,XINC,READ,KEY)
C
C       This subroutine reads a FITS map header and returns the
C  bin type (the physical unit assigned to a pixel), the starting
C  physical value, the ending physical value, and the incremental
C  physical value.  Alternately, these values may be written to a
C  FITS header.  When writing, the comment fields are determined
C  by the value of BIN.  This subroutine is based on BINVAL for SAD
C  files which ran under OpenVMS.
C			Donald H. Gudehus, June 21, 2005
C
C	LU	(C)	The logical unit for the FITS file
C	MAP	(C)	The map number of the FITS file
C	BIN	(C/R)	The bin type from the header
C	XST	(C/R)	The physical value at the center of the
C			first pixel in the map (X=1.5).  Note that
C			the left edge of the first pixel is 1.0 in
C			the MIIPS convention.
C	XEND	(R)	The physical value at the center of the last
C			pixel in the map (X=NAXIS1+0.5 or NAXIS2+0.5)
C	XINC	(C/R)	The incremental physical value
C	READ	(C)	A logical variable which if .TRUE. allows
C			the header to be read and if .FALSE. allows
C			the header to be written to.
C	KEY	(C)	An argument which if equal to 1 returns the
C			values for the horizontal direction, and if
C			equsl to 2, returns the values for the 
C			vertical direction.
C
	CHARACTER BIN*(*),CHFITSMHV*50,COMMENT*25,COMMENT2*25
	LOGICAL READ
	LOGICAL VERTICAL
C
	CHARACTER*68 FITSBIN
C
	IF (KEY.EQ.1) THEN
	  VERTICAL=.FALSE.
	ELSE IF (KEY.EQ.2) THEN
	  VERTICAL=.TRUE.
	END IF
C
	IF (READ) THEN
	  BIN=CHFITSMHV(LU,MAP,'CTYPE1','PIXEL')
	  IF (VERTICAL) BIN=CHFITSMHV(LU,MAP,'CTYPE2','PIXEL')
	  IF (BIN(1:1).EQ.' ') THEN
	    BIN=BIN(2:LEN(BIN)) !  Strips leading blanks
	  END IF
	  IBLANK=INDEX(BIN,' ')
C  Don't remove trailing blanks, because some keywords may consist of blanks
C	  IF (IBLANK.GT.0) THEN
C	    BIN=BIN(1:IBLANK-1) !  Removes trailing blanks
C	  END IF
	  IF (VERTICAL) THEN
	    REFPIX=R4FITSMHV(LU,MAP,'CRPIX2',1.5) !  The reference location 
	    REFVAL=R4FITSMHV(LU,MAP,'CRVAL2',1.5) !  The reference physical value
	    XINC=R4FITSMHV(LU,MAP,'CDELT2',1.0)
	    NI=I4FITSMHV(LU,MAP,'NAXIS2',0)
	  ELSE
	    REFPIX=R4FITSMHV(LU,MAP,'CRPIX1',1.5)
	    REFVAL=R4FITSMHV(LU,MAP,'CRVAL1',1.5)
	    XINC=R4FITSMHV(LU,MAP,'CDELT1',1.0)
	    NI=I4FITSMHV(LU,MAP,'NAXIS1',0)
	  END IF
	  XST=REFVAL-(REFPIX-1.5)*XINC !  Physical value at center of 
				       !  first pixel
	  XEND=XST+DFLOAT(NI-1)*XINC !  Physical value at center of last pixel
C
	ELSE  ! Writes the values
	  IF (BIN.EQ.'PIXEL') THEN !  SAD and FITS
	    COMMENT='pixels'
	    COMMENT2='pixels'
	  ELSE IF (BIN.EQ.'LAMBDA') THEN !  SAD and FITS values
				     !  Not recommended since LAMBDA is a
				     !  symbol rather than a physical variable
	    COMMENT='meters'
	    COMMENT2='meters/pixel'
	  ELSE IF (BIN.EQ.'WAVEL(M)') THEN !  SAD value only
	    COMMENT='meters'
	    COMMENT2='meters/pixel'
	  ELSE IF (BIN.EQ.'LOGLAMBDA') THEN !  SAD and FITS values
				     !  Not recommended since LAMBDA is a
				     !  symbol rather than a physical variable
	    COMMENT='log(meters)'
	    COMMENT2='log(meters)/pixel'
	  ELSE IF (BIN.EQ.'WAVELENGTH') THEN !  SAD value only
	    COMMENT='Angstroms'
	    COMMENT2='Angstroms/pixel'
	  ELSE IF (BIN.EQ.'WAVELENGTH(A)') THEN !  SAD value only
	    COMMENT='Angstroms'
	    COMMENT2='Angstroms/pixel'
	  ELSE IF (BIN.EQ.'WAVEL(A)') THEN !  SAD value only
	    COMMENT='Angstroms'
	    COMMENT2='Angstroms/pixel'
	  ELSE IF (BIN.EQ.'LOG(WAVEL(A))') THEN !  SAD value only
	    COMMENT='log(Angstroms)'
	    COMMENT2='log(Angstroms)/pixel'
	  ELSE IF (BIN.EQ.'FREQUENCY') THEN !  SAD value
	    COMMENT='hertz'
	    COMMENT2='hertz/pixel'
	  ELSE IF (BIN.EQ.'FREQ') THEN !  SAD and FITS
	    COMMENT='hertz'
	    COMMENT2='hertz/pixel'
	  ELSE IF (BIN.EQ.'LOG(FREQ)') THEN !  SAD value
	    COMMENT='log(hertz)'
	    COMMENT2='log(hertz)/pixel'
	  ELSE IF (BIN.EQ.'SPATIAL FREQUENCY') THEN !  SAD value
	    COMMENT='pixel pairs/pixel'
	    COMMENT2='pixel pairs/pixel/pixel'
	  ELSE IF (BIN.EQ.'LOG(SPATIAL FREQ)') THEN !  SAD value
	    COMMENT='log(pixel pairs)/pixel'
	    COMMENT2='log(pixel pairs/pixel)/pixel'
	  ELSE IF (BIN.EQ.'ANGLE') THEN !  SAD
	    COMMENT='arc sec'
	    COMMENT2='arc sec/pixel '
	  ELSE !  Recover gracefully if an unexpected bin type is 
	       !  received
	    COMMENT=' '
	    COMMENT2=' '
	  END IF
	  FITSBIN=''''//BIN//''''
	  IF (VERTICAL) THEN
	    CALL WRFITSMHST(LU,MAP,'CTYPE2',BIN,'C',COMMENT,.FALSE.)
	    CALL WRFITSMHV(LU,MAP,'CRPIX2',1.5D0,'R4','pixels',.FALSE.)
	    CALL WRFITSMHV(LU,MAP,'CRVAL2',DBLE(XST),'R4',COMMENT,.FALSE.)
	    CALL WRFITSMHV(LU,MAP,'CDELT2',DBLE(XINC),'R4',COMMENT2,.FALSE.)
	  ELSE
	    CALL WRFITSMHST(LU,MAP,'CTYPE1',FITSBIN,'C',COMMENT,.FALSE.)
	    CALL WRFITSMHV(LU,MAP,'CRPIX1',1.5D0,'R4','pixels',.FALSE.)
	    CALL WRFITSMHV(LU,MAP,'CRVAL1',DBLE(XST),'R4',COMMENT,.FALSE.)
	    CALL WRFITSMHV(LU,MAP,'CDELT1',DBLE(XINC),'R4',COMMENT2,.FALSE.)
	  END IF
	END IF
C
	RETURN
	END
