      SUBROUTINE CROP_FITS_SIMPLE
C
C       This subroutine will perform a simple crop on a FITS file, and
C  save to the same map, the end of the same file, the end of a different
C  file, or a new file.  One or more maps can be operated on, and all
C  maps are cropped to the same size.  All planes within each map are
C  operated on as well.
C			Donald H. Gudehus, Jan. 8, 2008
C
	INCLUDE 'miipspar.for'
	INCLUDE 'fitsfacts.for'
C
	CHARACTER*3 FILEFMT
	CHARACTER*50 CHFITSMHV
	CHARACTER*80 FILENAME_IN
	INTEGER STATUS
	INTEGER MAPS_IN(MAPMAX),NCOLS_OUT_ARRAY(MAPMAX),
	1NROWS_OUT_ARRAY(MAPMAX)
	INTEGER LOWER_LEFT(2),UPPER_RIGHT(2)
	REAL*4 DATAXY(MAXSIZE,MAXSIZE)
	REAL*8 DATADPXY(MAXSIZE,MAXSIZE)
C
	COMMON /MAPN/ IMNUM
	COMMON /PLANEN/ IPNUM
	CHARACTER*80 DNAME
	COMMON /DFILE/ DNAME
C
	LU_IN=11
	FILENAME_IN=' '
C
C  If we were to only operate on one map, the code could be as
C  follows:
C  Open the input file and select the map
C  Because we will process all the planes in a map, don't request a 
C  prompt for the plane number.
C	MAPKEY=0
C	PLANEKEY=1
C	CALL OPENFITS(LU_IN,.TRUE.,'FILE=',FILENAME_IN,MAPKEY,PLANEKEY,NMAPS,
C	1NPLNS,NCOLS,NROWS,PRGFMAT,STATUS)
C  Simple case of one map
C	MAPS_IN(1)=IMNUM
C	NMAPS_IN=1
C
C  Instead, in this example, we allow one or more maps in the input file
C  to be operated on
C
	CALL GET_INPUT_FITS_MAPS(LU_IN,FILENAME_IN,MAPS_IN,NMAPS_IN,STATUS)
	IF (STATUS.NE.0) THEN
	  CALL CLOSEFITS(LU_IN)
	  RETURN
	END IF
C
C  Obtain the coordinates which define the cropped area, i.e., the lower left
C  and the upper right.
C
	CALL MESSAGE('Define the cropped area')
	CALL USRINP(LOWER_LEFT,2,'I',4,0,'LOWER_LEFT=','Enter the column '//
	1'and row of the lower left corner:')
	CALL USRINP(UPPER_RIGHT,2,'I',4,0,'UPPER_RIGHT=','Enter the column '//
	1'and row of the upper right corner:')
	NCOLS_OUT=UPPER_RIGHT(1)-LOWER_LEFT(1)+1
	NROWS_OUT=UPPER_RIGHT(2)-LOWER_LEFT(2)+1
C
C  Check that the requested boundaries do not exceed the limits of the input
C  maps
C
	DO I=1,NMAPS_IN
	  IM=MAPS_IN(I)
	  IF (LOWER_LEFT(1).GT.NAXIS1(IM,LU_IN).OR.LOWER_LEFT(2).GT.
	1   NAXIS2(IM,LU_IN).OR.UPPER_RIGHT(1).GT.NAXIS1(IM,LU_IN).OR.
	2   UPPER_RIGHT(2).GT.NAXIS2(IM,LU_IN)) THEN
	    CALL ERROR(-3,'A crop boundary exceeded a map size')
	    RETURN
	  END IF
	END DO
C
C  Load the arrays for the output sizes.  In this simple case, all the
C  edited maps will be the same size.
C
	DO I=1,NMAPS_IN
	  NCOLS_OUT_ARRAY(I)=NCOLS_OUT
	  NROWS_OUT_ARRAY(I)=NROWS_OUT
	END DO
C
C  Create a temporary file.  This allows for several options when we save our
C  results.
C
	LU_TEMP=12
	CALL OPEN_TEMP_FITS(LU_IN,LU_TEMP,FILENAME_IN,MAPS_IN,NMAPS_IN,
	1NCOLS_OUT_ARRAY,NROWS_OUT_ARRAY,STATUS)
C
C  Loop over the maps
	DO I=1,NMAPS_IN
	  IM=MAPS_IN(I)
	  FILEFMT=CHFITSMHV(LU_IN,IM,'BITPIX','R4')
	  IF (FILEFMT.EQ.'I1'.OR.FILEFMT.EQ.'I2'.OR.FILEFMT.EQ.'I4'.OR.
	1 FILEFMT.EQ.'R4'.OR.FILEFMT.EQ.'C8') THEN
C  Loop over the planes
	    DO IP=1,NAXIS3(IM,LU_IN)
C  Read the area from the input file, and then write to the temporary file
	      CALL RDFITSXY(LU_IN,IM,IP,DATAXY,MAXSIZE,MAXSIZE,
	1     LOWER_LEFT(1),NCOLS_OUT,LOWER_LEFT(2),NROWS_OUT)
	      CALL WRFITSXY(LU_TEMP,IM,IP,DATAXY,MAXSIZE,MAXSIZE,
	1     1,NCOLS_OUT,1,NROWS_OUT)
	    END DO
	  ELSE IF (FILEFMT.EQ.'R8'.OR.FILEFMT.EQ.'C16') THEN
C  Loop over the planes
	    DO IP=1,NAXIS3(IM,LU_IN)
C  Read the area from the input file, and then write to the temporary file
	      CALL RDDPFITSXY(LU_IN,IM,IP,DATADPXY,MAXSIZE,MAXSIZE,
	1     LOWER_LEFT(1),NCOLS_OUT,LOWER_LEFT(2),NROWS_OUT)
	      CALL WRDPFITSXY(LU_TEMP,IM,IP,DATAXY,MAXSIZE,MAXSIZE,
	1     1,NCOLS_OUT,1,NROWS_OUT)
	    END DO
	  END IF
	END DO
	LU_OUT=13
	CALL SAVE_SAVE_AS(LU_IN,LU_TEMP,LU_OUT,MAPS_IN,NMAPS_IN,STATUS)
C  The files are closed by the above call to save_save_as.
	RETURN
	END
