      SUBROUTINE FITS_MAPS_TO_PLANES
C
C       This subroutine will write selected maps from a FITS file to a
C  single FITS map with multiple planes.
C				Version 1U, Donald H. Gudehus, August 7, 2006
C
	INCLUDE 'miipspar.for'
	INCLUDE 'fitsfacts.for'
C
	CHARACTER*3 PRGFMAT
	CHARACTER*80 FILENAME_IN,FILENAME_OUT
C
	INTEGER MAPKEY,PLANEKEY
	INTEGER MAPS_IN(MAPMAX),PLANES_IN(MAPMAX,NPLANEMAX),
	1NPLANES_IN(MAPMAX),TEMP(NPLANEMAX),STATUS
	INTEGER NMAPS_OUT,NCOLS_OUT,NROWS_OUT,NPLNS_OUT
	INTEGER BITPIX_OUT
	LOGICAL BITPIX_DIFFERS,BSCALE_BZERO_DIFFERS,WANT_COMPLEX
C
	REAL*4 DATA(MAXSIZE)
	REAL*8 DATADP(MAXSIZE)
	REAL*8 BSCALE_OUT,BZERO_OUT,BS_BZ(2)
C
	CHARACTER*80 DNAME
	COMMON /DFILE/ DNAME
C
	COMMON /MAPN/ IMNUM
C
C  Open the input file and select the maps
C
	LU_IN=11
	FILENAME_IN=' ' 
	CALL GET_INPUT_FITS_MAPS(LU_IN,FILENAME_IN,MAPS_IN,NMAPS_IN,STATUS)
	IF (STATUS.NE.0) THEN
	  CALL CLOSEFITS(LU_IN)
	  RETURN
	END IF
C
C  Check that all the selected maps are the same size
C
	NCOLS_OUT=NAXIS1(MAPS_IN(1),LU_IN)
	NROWS_OUT=NAXIS2(MAPS_IN(1),LU_IN)
	DO N=2,NMAPS_IN
	  IF (NCOLS_OUT.NE.NAXIS1(MAPS_IN(N),LU_IN).OR.
	1 NROWS_OUT.NE.NAXIS2(MAPS_IN(N),LU_IN)) THEN
	    CALL ERROR(-3,'THE INPUT MAPS DIFFER IN SIZE')
	    RETURN
	  END IF
	END DO
C
C  Check the data formats of the maps
C
	BITPIX_OUT=BITPIX(MAPS_IN(1),LU_IN)
	BITPIX_DIFFERS=.FALSE.
	DO N=2,NMAPS_IN
	  IF (BITPIX(MAPS_IN(N),LU_IN).NE.BITPIX_OUT)
	1 BITPIX_DIFFERS=.TRUE.
	END DO
	IF (BITPIX_DIFFERS) THEN
	  CALL MESSAGE('The data formats of the input maps are not '//
	1 'all the same')
	  CALL MESSAGE('Map Number     BITPIX')
	  DO N=1,NMAPS_IN
	    WRITE (6,1000) MAPS_IN(N),BITPIX(MAPS_IN(N),LU_IN)
1000	    FORMAT(' ',I4,11X,I4)
	    WRITE (7,1000) MAPS_IN(N),BITPIX(MAPS_IN(N),LU_IN)
	  END DO
	  CALL USRINP(BITPIX_OUT,1,'I',4,0,'BITPIX_OUT=',
	1 'Enter the desired output bitpix:')
	END IF
C
C  Check if some maps have multiple planes, and if so ask the
C  user which to use.  Also get the total of all the planes
C  in all the maps
C
	NTOTAL_PLANES=0
	DO N=1,NMAPS_IN
	  PLANES_IN(N,1)=1
	  NPLANES_IN(N)=1
	  IF (NAXIS3(MAPS_IN(N),LU_IN).GT.1) THEN
	    WRITE (6,1100) MAPS_IN(N),NAXIS3(MAPS_IN(N),LU_IN)
1100	    FORMAT(' ','Map number ',I3,' has ',I3,' planes')
	    WRITE (7,1100) MAPS_IN(N),NAXIS3(MAPS_IN(N),LU_IN)
	    DO M=1,NPLANEMAX
	      J=ISETEND(TEMP(M))
	    END DO
	    TEMP(1)=1
	    CALL USRINP(TEMP,NPLANEMAX,'I',4,1,'PLANE_NUMBERS=',
	1   'Enter the input plane numbers (default = 1):')
	    NPLANES_IN(N)=INEL(TEMP,NPLANEMAX)
	    DO NP=1,NPLANES_IN(N)
	      PLANES_IN(N,NP)=TEMP(NP)
	    END DO
	  END IF
	  NTOTAL_PLANES=NTOTAL_PLANES+NPLANES_IN(N)
	END DO
C
C  Check if BSCALE and BZERO are all the same
C
	BSCALE_OUT=BSCALE(1,LU_IN)
	BZERO_OUT=BZERO(1,LU_IN)
	BSCALE_BZERO_DIFFERS=.FALSE.
	DO N=2,NMAPS_IN
	  IF (BSCALE(MAPS_IN(N),LU_IN).NE.BSCALE_OUT.OR.
	1 BZERO(MAPS_IN(N),LU_IN).NE.BZERO_OUT)
	1 BSCALE_BZERO_DIFFERS=.TRUE.
	END DO
	IF (BSCALE_BZERO_DIFFERS) THEN
	  CALL MESSAGE('BSCALE and BZERO of the input maps are not '//
	1 'all the same')
	  CALL USRINP(BS_BZ,2,'F',8,1,'BSCALE_BZERO=',
	1 'Enter the desired output BSCALE and BZERO (default = 1.0 0.0)')
	  BSCALE_OUT=BS_BZ(1)
	  BZERO_OUT=BS_BZ(2)
	END IF
C
C  Open the output file
C
	LU_OUT=12
	FILENAME_OUT=' ' !  Lets the user choose
	MAPKEY=0
	PLANEKEY=NTOTAL_PLANES
	NMAPS_OUT=1
	NPLNS_OUT=NTOTAL_PLANES
	IF (BITPIX_OUT.EQ.8) THEN
	  PRGFMAT='I1'
	ELSE IF (BITPIX_OUT.EQ.16) THEN
	  PRGFMAT='I2'
	ELSE IF (BITPIX_OUT.EQ.32) THEN
	  PRGFMAT='I4'
	ELSE IF (BITPIX_OUT.EQ.-32) THEN
	  PRGFMAT='R4'
	ELSE IF (BITPIX_OUT.EQ.-64) THEN
	  PRGFMAT='R8'
	END IF
C  Inquire whether the user wants complex output format
	IF (NPLNS_OUT.EQ.2.AND.(BITPIX_OUT.EQ.-32.OR.BITPIX_OUT.EQ.-64)) THEN
	  WANT_COMPLEX=.FALSE.
	  CALL MESSAGE('Do you want the two output planes to be '//
	1 'considered as complex data?')
	  CALL USRINP(WANT_COMPLEX,1,'L',4,1,'WANT_COMPLEX=','(default = N)')
	  IF (WANT_COMPLEX) THEN
	    IF (PRGFMAT.EQ.'R4') PRGFMAT='C8'
	    IF (PRGFMAT.EQ.'R8') PRGFMAT='C16'
	  END IF
	END IF
	CALL MESSAGE('Open the output file')
	CALL OPENFITS(LU_OUT,.FALSE.,'FILE_OUT=',FILENAME_OUT,MAPKEY,
	1PLANEKEY,NMAPS_OUT,NPLNS_OUT,NCOLS_OUT,NROWS_OUT,PRGFMAT,STATUS)
C
C  Add some keywords
C
	IF (BSCALE_OUT.NE.1.0D0) THEN
	  CALL WRFITSMHV(LU_OUT,IMNUM,'BSCALE',BSCALE_OUT,'R8',
	1 'Scaling factor',.FALSE.)
	END IF
	IF (BZERO_OUT.NE.0.0D0) THEN
	  CALL WRFITSMHV(LU_OUT,IMNUM,'BZERO',BZERO_OUT,'R8',
	1 'Zero offset',.FALSE.)
	END IF
C  Update the COMMON blocks if necessary
	IF (BSCALE_OUT.NE.1.0D0.OR.BZERO_OUT.NE.0.0D0)
	1CALL GET_FITS_INFO(LU_OUT)
C
C  Carry out the transfer, looping over maps, planes, and rows
C
	IMNUM_OUT=IMNUM !  The ouput map number (from common block)
	IPNUM_OUT=1 !  The ouput plane number
	DO N=1,NMAPS_IN !  Loops throught the input maps
	  IMNUM_IN=MAPS_IN(N) !  The input map number
	  DO IP=1,NPLANES_IN(N) !  Loops through the selected planes in the
				!  input map
	    IPNUM_IN=PLANES_IN(N,IP)
	    IF (PRGFMAT.EQ.'I1'.OR.PRGFMAT.EQ.'I2'.OR.PRGFMAT.EQ.'I4'.OR.
	1   PRGFMAT.EQ.'R4'.OR.PRGFMAT.EQ.'C8') THEN
	      DO IR=1,NROWS_OUT
	        CALL RDFITSROW(LU_IN,IMNUM_IN,IPNUM_IN,DATA,MAXSIZE,IR)
	        CALL WRFITSROW(LU_OUT,IMNUM_OUT,IPNUM_OUT,DATA,MAXSIZE,IR)
	      END DO
	    ELSE IF (PRGFMAT.EQ.'R8'.OR.PRGFMAT.EQ.'C16') THEN
	      DO IR=1,NROWS_OUT
	        CALL RDDPFITSROW(LU_IN,IMNUM_IN,IPNUM_IN,DATADP,MAXSIZE,
	1       IR)
	        CALL WRDPFITSROW(LU_OUT,IMNUM_OUT,IPNUM_OUT,DATADP,
	1       MAXSIZE,IR)
	      END DO
	    END IF
	    IPNUM_OUT=IPNUM_OUT+1
	  END DO
	END DO
	CALL CLOSEFITS(LU_IN)
	CALL CLOSEFITS(LU_OUT)
	RETURN
	END
