      SUBROUTINE LINEPL_FITS(STATUS)
C
C       This subroutine will plot a cut through a two-dimensional file.
C  One can plot for one or more maps, one or more rows or one or more
C  columns on the same graph.  For a single map one can plot a cut at
C  any angle and send the output to the graphics package or the
C  Printronix.  Cursor input from the TV is also possible for
C  choosing the central location.  When plotting more than one row or
C  column, one has the choice of plotting the average of all of them.
C			Donald H. Gudehus, March, 1983
C			Version 2, Donald H. Gudehus, November, 1988
C			Added code for Trapix
C			Version 3, Donald H. Gudehus, May, 1989
C			Added code for plotting average of several
C			rows or columns.
C			Version 4, Donald H. Gudehus, September, 1993
C			Added code for SAOimge
C			Version 4U, Donald H. Gudehus, June 18, 2005
C			Ported code to Unix;  removed references to
C			Grinnell and Trapix.
C			Version 5, D. Gudehus, July 25, 2006
C			Incorporated new saofacts.for
C			Version 6, D. Gudehus, Jan. 15, 2008
C			Modified code for new openfits.for
C			Version 7, D. Gudehus, May 9, 2009
C			Modified code to allow for NAXIS=1
C			Version 8, Donald H. Gudehus, September 1, 2013
C			Added support for 64-bit processors.
C
	PARAMETER (MAXPLOTS_SMALL=10)
	INCLUDE 'miipspar.for'
	INCLUDE 'fitsfacts.for'
	INCLUDE 'saofacts.for'
C
	INTEGER STATUS
C
	BYTE OUTLINE(134),WORK(134,54)
	COMMON /BYTE_SCRATCH_BLOCK/ OUTLINE,WORK
C
	INTEGER*2 N,FLAG1,FLAG2,IXN,IYN,TVXSTART,TVYSTART,TVXEND,TVYEND,
	1STATUS_TR,IX(5),IY(5)
C
	COMMON /INTEGER2_SCRATCH_BLOCK/ N,FLAG1,FLAG2,IXN,IYN,TVXSTART,
	1TVYSTART,TVXEND,TVYEND,STATUS_TR,IX,IY
C
	INTEGER*4 MAPS(MAPMAX),NPNS(MAPMAX),PLANES(MAPMAX),
	2LENGTH,SIZE,ROWS(MAXSIZE),IPIX(2),COLUMNS(MAXSIZE),
	3ACTION,CURSOR_TYPE,COLOR_TYPE,COLOR(MAXPLOTS_SMALL),
	4NPLOTPOINTS(MAXPLOTS_SMALL),LINVECT(MAXPLOTS_SMALL),
	4INC(MAXPLOTS_SMALL),LINTYPE(MAXPLOTS_SMALL),
	5NSYMBOL(MAXPLOTS_SMALL),NOOFMAPS,NMAPS,NPLANES,NI,NJ,
	6NROWS,NCOLUMNS
C
	COMMON /INTEGER4_SCRATCH_BLOCK/ MAPS,NPNS,PLANES,LENGTH,SIZE,
	1ROWS,COLUMNS,IPIX,ACTION,CURSOR_TYPE,COLOR_TYPE,COLOR,LINVECT,INC,
	2LINTYPE,NSYMBOL,NOOFMAPS,NMAPS,NPLANES,NI,NJ,NROWS,NCOLUMNS
C
	INTEGER*4 ADDRESS1_4,ADDRESS2_4,ADDRESS3_4,ADDRESS4_4,ADDRESS5_4,
	1ADDRESS6_4,ADDRESS7_4,ADDRESS8_4
	INTEGER*8 ADDRESS1_8,ADDRESS2_8,ADDRESS3_8,ADDRESS4_8,ADDRESS5_8,
	1ADDRESS6_8,ADDRESS7_8,ADDRESS8_8
C
	LOGICAL CURSOR_ON(5),ZOOM,PAN,LUT,HELP
	LOGICAL FIRSTTIME,MORE,INTERACT,INVERT(2),CROSSHAIRS,SAO_MESSAGE,
	1PLOTAVERAGE,IOFL
C
	COMMON /LOGICAL_SCRATCH_BLOCK/ CURSOR_ON,ZOOM,PAN,LUT,HELP,
	1FIRSTTIME,MORE,INTERACT,INVERT,CROSSHAIRS,SAO_MESSAGE,
	2PLOTAVERAGE,IOFL
C
	DIMENSION DAT(MAXPLOTSIZE,MAXPLOTS_SMALL),DAT2(1,MAXPLOTSIZE)
	EQUIVALENCE (DAT,DAT2)
	REAL*4 XARRAY(MAXPLOTSIZE,MAXPLOTS_SMALL),CENTER(2),PARAMETRS(2)
C
	COMMON /REAL4_SCRATCH_BLOCK/ XARRAY,DAT
C
	INTEGER SYSTEM_NUMBER_BYTES
	COMMON /SYS_NUM_BYTES/ SYSTEM_NUMBER_BYTES
C
	CHARACTER*2 COMMAND
	CHARACTER*3 PRGFMAT,DATFMT,DEVICES(10)
	CHARACTER*40 FILEKEYWORD
	CHARACTER*50 BIN,CHFITSMHV,CHFITSMHC,COMMENT
	CHARACTER*50 XLABEL,YLABEL,TITLE
	CHARACTER*63 MESS
	CHARACTER*80 FILENAME
	COMMON /CHAR_SCRATCH_BLOCK/ COMMAND,PRGFMAT,DATFMT,
	1DEVICES,FILEKEYWORD,BIN,COMMENT,XLABEL,YLABEL,TITLE,
	2MESS,FILENAME
C
	CHARACTER*80 CNAME
	COMMON /CFILE/ CNAME
C
	CHARACTER*80 DNAME
	COMMON /DFILE/ DNAME
C
	LOGICAL GRINNELL_270,TRAPIX,RAMTEK,SAOIMAGE
	COMMON /DISPLAY_AVAILABILITY/ GRINNELL_270,TRAPIX,RAMTEK,SAOIMAGE
C
	CALL MESSAGE ('LINEPL_FITS, Version 4U')
	LU=11
	FIRSTTIME=.TRUE.
  40	COMMAND=' '
	DO WHILE (COMMAND.NE.'R'.AND.COMMAND.NE.'C'.AND.COMMAND.NE.'A'
	1.AND.COMMAND.NE.'AS'.AND.COMMAND.NE.'S')
	  COMMAND='R'
	  CALL MESSAGE('Enter a command:')
	  CALL MESSAGE('     R = Plots one or more rows (default)')
	  CALL MESSAGE('     C = Plots one or more columns')
	  CALL MESSAGE('     A = Plots at any angle (manual input of '//
	1 'position)')
	  IF (SAOIMAGE) THEN
	  CALL MESSAGE('    AS = Plots at any angle (SAOimage cursor '//
	1  'input;  SAOimage already loaded')
	  CALL MESSAGE('     S = Plots at any angle (SAOimage cursor '//
	1  'input;  need to load SAOimage')
	  END IF
	  CALL USRINP(%REF(COMMAND),1,'A',2,1,'COMMAND=',' ')
	  CALL ALPHA(%REF(COMMAND),1,2,NC,NF,NR,2)
	END DO
	IF (FIRSTTIME) THEN
	  IOFL=.TRUE.
	  FILENAME=' '
	  PRGFMAT=' '
	  CALL OPENFITS(LU,IOFL,'FILE=',FILENAME,1,1,NMPS,NPNS,NCLS,NRWS,
	1 PRGFMAT,STATUS)
	  IF (STATUS.NE.0) RETURN
	END IF
	FIRSTTIME=.FALSE.
	CNAME=DNAME
	NOOFMAPS=NMPS !  Number of maps in file
C  Check for a valid file
	DO M=1,NOOFMAPS
	  IF (NAXIS(M,LU).NE.1.AND.NAXIS(M,LU).NE.2.AND.
	1 NAXIS(M,LU).NE.3) THEN
	    CALL ERROR(-3,'Sorry, NAXIS must be 1 or 2 or 3')
	    STATUS=2
	    RETURN
	  END IF
	END DO
C
	NMAPS=1 !  Default number of maps to be plotted
	DO I=1,NOOFMAPS
	  MAPS(I)=I !  Sets up map array for possible plotting
	END DO
	DO I=1,MAPMAX
	  PLANES(I)=1 !  Default plane number for plotting
	END DO
C  Begin getting information from the user about the plot(s)
	NPLANES=1 !  Only one plane per map can be plotted now
	IF (NOOFMAPS.EQ.1) THEN
	  IPNUM=1
	  IF (NPNS(1).GT.1) THEN
	    CALL USRINP(IPNUM,1,'I',4,1,'PLANE=',
	1   'Enter the plane number:  (default = 1)')
C  No checking on plane number here yet
	  END IF
	  PLANES(1)=IPNUM
	ELSE IF (NOOFMAPS.NE.1.AND.(COMMAND.EQ.'R'.OR.COMMAND.EQ.'C')) THEN
	  CALL USRINP(MAPS,NOOFMAPS,'I',4,1,'MAPS=',
	1 'Enter the map numbers:  (default = all)')
	  NMAPS=INEL(MAPS,NOOFMAPS)
	  IF (NMAPS.GT.1) THEN
C  Check for the minimum number of planes in any map in the chosen set 
	    IPMINNUM=9999
	    DO I=1,NMAPS
	      IF (NAXIS3(MAPS(I),LU).LT.IPMINNUM) IPMINNUM=NAXIS3(MAPS(I),LU)
	    END DO
	    WRITE (6,1010) IPMINNUM
1010	    FORMAT(' ','The minimum number of planes for any map '//
	1   'in the set is ',I4)
	  ELSE
	    WRITE (6,1011) NAXIS3(MAPS(1),LU)
1011	    FORMAT(' ','There are ',I4,' planes in this map')
	    WRITE (7,1011) NAXIS3(MAPS(1),LU)
	  END IF
	  IPNUM=1
	  CALL USRINP(IPNUM,1,'I',4,1,'PLANE=',
	1 'Enter the plane number:  (default =1)')
C  Need to check if the plane number exceeds the highest available 
C  number
	  DO I=1,NMAPS
	    PLANES(I)=IPNUM
	  END DO
	ELSE IF (NOOFMAPS.NE.1.AND.(COMMAND.NE.'R'.AND.COMMAND.NE.'C'))
	1THEN !  Only one map allowed to be plotted here at present
	  CALL USRINP(MAPS,1,'I',4,1,'MAP=',
	1 'Enter the map number:  (default = 1)')
	  IF (NAXIS3(MAPS(1),LU).GT.1) THEN
	    IPNUM=1
	    CALL USRINP(IPNUM,1,'I',4,1,'PLANE=',
	1   'Enter the plane number:  (default =1)')
	    PLANES(1)=IPNUM
	  END IF
	END IF
	CALL MESSAGE('Reading from the file')
	DO I=1,NMAPS
	  DATFMT=CHFITSMHV(LU,MAPS(I),'BITPIX','?')
	  IF (DATFMT.EQ.'?') CALL ERROR(-3,'Error getting data format')
	  ENCODE(63,1251,MESS) MAPS(I),NRWS,NCLS,DATFMT(1:3)
1251	  FORMAT('Map number ',I4,' contains ',I4,' rows of ',I4,
	1 ' pixels in ',A3,' format')
	  CALL MESSAGE(MESS)
	END DO
	NI=NCLS
	NJ=NRWS
	YLABEL='ADU/Square pixel'
	NROWS=1
	NCOLUMNS=1
C  In reading from the file, we ignore whether the data type is REAL*8
C  because the plotting subroutine only accepts REAL*4 data, and the
C  usual plot devices do not benefit from the extra precision.
	IF (COMMAND.EQ.'R') THEN
	  XLABEL='Column'
	  IF (NJ.GT.1) THEN
	    CALL USRINP(ROWS,NJ,'I',4,0,'ROWS=','Enter the row numbers:')
	    NROWS=INEL(ROWS,NJ)
	  ELSE
	    ROWS(1)=1
	    NROWS=1
	  END IF
	  IPIX(1)=1
	  IPIX(2)=NI
	  CALL USRINP(IPIX,2,'I',4,1,'COLUMNS=',
	1 'Enter the start-column and the end-column:  (default = '//
	2 'entire range)')
	  IF (NROWS.EQ.1) CALL RDFITSROW(LU,MAPS(1),PLANES(1),DAT,NI,ROWS(1))
	ELSE IF (COMMAND.EQ.'C') THEN
	  XLABEL='Row'
	  CALL USRINP(COLUMNS,NI,'I',4,0,'COLUMNS=','Enter the column '//
	1 'numbers:')
	  NCOLUMNS=INEL(COLUMNS,NI)
	  IPIX(1)=1
	  IPIX(2)=NJ
	  CALL USRINP(IPIX,2,'I',4,1,'ROWS=',
	1 'Enter the start-row and the end-row:  (default = entire '//
	2 'range)')
	  IF (NCOLUMNS.EQ.1) CALL RDFITSXY(LU,MAPS(1),PLANES(1),DAT2,
	1 1,NJ,COLUMNS(1),1,1,NJ)
	ELSE IF (COMMAND.EQ.'A') THEN
	  CALL MESSAGE('Enter the column and row for the center of the plot:')
	  CALL USRINP(CENTER,2,'F',4,0,'CENTER=',
	1 '(fractional values allowed)')
	  XMN=CENTER(1)
	  YMN=CENTER(2)
	ELSE IF (COMMAND.EQ.'S') THEN
	  NPIC_SAO=1
	  IF (CHAN_OUT_SAO.EQ.0) THEN
	    CALL SAOSTART
	    CALL SAOFIRST
	  ELSE
	    CALL SAOERAS
	  END IF
	  CALL SAODELS(LU,1,0)
	END IF
	IF (COMMAND.EQ.'AS'.OR.COMMAND.EQ.'S') THEN
 155	  CALL MESSAGE('Be sure SAOimage has focus, position the cursor,'//
	1 ' and then press <CR>')
	  SAO_MESSAGE=.TRUE.
	  CALL SAO_CURN(NPICN,IXN,IYN,XN,YN,XMN,YMN,KEY,SAO_MESSAGE)
	  IF (KEY.EQ.13) THEN
	    IF (NPICN.EQ.0.OR.(NPICN.NE.0.AND.TV_FILENAME(NPICN)
	1   .NE.DNAME)) THEN
	      CALL MESSAGE('Try again')
	      GO TO 155
	    END IF
	  ELSE
	    CALL BELL
	    CALL MESSAGE('You must press the <CR>!')
	    GO TO 155
	  END IF
	  CENTER(1)=INT(XMN)+0.5  ! This offsets to the center of the pixel
	  CENTER(2)=INT(YMN)+0.5
	  WRITE (6,1000) CENTER(1),CENTER(2)
1000	  FORMAT(' ','The coordinates in the map are:  X = ',F7.1,', Y = ',
	1 F7.1)
	  WRITE (7,1000) CENTER(1),CENTER(2)
	END IF
	IF (COMMAND.EQ.'A'.OR.COMMAND.EQ.'AS'.OR.COMMAND.EQ.'S') THEN
C  This could eventually include the capabiity to do many maps
	  XLABEL='Pixel'
	  CALL USRINP(PARAMETRS,2,'F',4,0,'PARAMETERS=',
	1 'Enter the angle (degrees) and the length of the plot '//
	2 '(pixels):')
	  THETA=PARAMETRS(1)
	  PLENGTH=PARAMETRS(2)
	  SLENGTH=PLENGTH/2.0
	  ELENGTH=SLENGTH
C  Allow for XSTART to be > XEND, ie., (90 < THETA < 270), as well as
C  XSTART < XEND  and decrease each half of the length until the
C  boundaries of the map are not violated.
	  CS=COSD(THETA)
	  SN=SIND(THETA)
 200	  XSTART=CENTER(1)-CS*SLENGTH
	  YSTART=CENTER(2)-SN*SLENGTH
	  IF (XSTART.LT.1.0.OR.XSTART.GE.FLOAT(NI+1)
	1 .OR.YSTART.LT.1.0.OR.YSTART.GE.FLOAT(NJ+1)) THEN
	    SLENGTH=SLENGTH-1.0
	    GO TO 200
	  END IF
 220	  XEND=CENTER(1)+CS*ELENGTH
	  YEND=CENTER(2)+SN*ELENGTH
	  IF (XEND.LT.1.0.OR.XEND.GE.FLOAT(NI+1)
	1 .OR.YEND.LT.1.0.OR.YEND.GE.FLOAT(NJ+1)) THEN
	    ELENGTH=ELENGTH-1.0
	    GO TO 220
	  END IF
	  PLENGTH=SLENGTH+ELENGTH
	  IXHI=MAX1(XSTART,XEND)
	  IXLO=MIN1(XSTART,XEND)
	  IYHI=MAX1(YSTART,YEND)
	  IYLO=MIN1(YSTART,YEND)
	  IF (IXHI.GT.NI) IXHI=NI
	  IF (IXLO.LT.1) IXLO=1
	  IF (IYHI.GT.NJ) IYHI=NJ
	  IF (IYLO.LT.1) IYLO=1
	  ILX=IXHI-IXLO+1
	  ILY=IYHI-IYLO+1
C  Draw the line on SAOimage
	  IF (COMMAND.EQ.'AS'.OR.COMMAND.EQ.'S') THEN
C  Draw the erasable circle cursor on SAOimage
	    ACTION=1
	    CURSOR_TYPE=768  ! Rectangle
	    COLOR_TYPE=204  ! Red
	    NPIC1=NPICN
	    a=SLENGTH
	    b=0.0
	    CALL SAO_DRAW(ACTION,CURSOR_TYPE,COLOR_TYPE,CENTER(1),CENTER(2),
	1   NPIC1,a,b,THETA,EXTRA,0)
	  END IF
	  LENGTH=ILX*ILY+1
	  IF (SYSTEM_NUMBER_BYTES.EQ.4) THEN
	    CALL DYNAMIC(ADDRESS1_4,LENGTH,'  R4',.TRUE.)
	    CALL LINEPL1_FITS(%VAL(ADDRESS1_4),IXLO,XSTART,ILX,IYLO,
	1   YSTART,ILY,THETA,PLENGTH,DAT,MAPS(1),PLANES(1))
C  DAT now contains the data to be plotted.  If many maps are to be plotted,
C  the call to XYPLOT should be put in LINEPL1_FITS
	    CALL DYNAMIC(ADDRESS1_4,LENGTH,'  R4',.FALSE.)
	  ELSE IF (SYSTEM_NUMBER_BYTES.EQ.8) THEN
	    CALL DYNAMIC(ADDRESS1_8,LENGTH,'  R4',.TRUE.)
	    CALL LINEPL1_FITS(%VAL(ADDRESS1_8),IXLO,XSTART,ILX,IYLO,
	1   YSTART,ILY,THETA,PLENGTH,DAT,MAPS(1),PLANES(1))
	    CALL DYNAMIC(ADDRESS1_8,LENGTH,'  R4',.FALSE.)
	  END IF
	  IPIX(1)=1
	  IPIX(2)=PLENGTH
	END IF
C  Calculate the number of plotted points for the case of one plot
	NPLOTPOINTS(1)=IPIX(2)-IPIX(1)+1
	NPLOTPOINTS(1)=MIN(NPLOTPOINTS(1),MAXPLOTSIZE-5)
	SIZE=MAXPLOTSIZE
C  Get set up for making the plot on one of the plot devices
C  Initialize XARRAY and YARRAY since the scaling will be done by XYPLOT
	XARRAY(SIZE-3,1)=0.0
	DAT(SIZE-3,1)=0.0
	INTERACT=.FALSE.
	INVERT(1)=.FALSE.
	INVERT(2)=.FALSE.
	CROSSHAIRS=.TRUE.
	TITLE='SURFACE BRIGHTNESS' !  Default title
C  Get the labels
	IF (COMMAND.EQ.'R') THEN
C  Special labels
	  CALL BINVALFITS(LU,MAPS(1),BIN,XST,XND,XINC,.TRUE.,1)
	  IF (BIN.EQ.'WAVELENGTH'.OR.BIN.EQ.'WAVELENGTH(A)'.OR.
	1 BIN.EQ.'WAVEL(A)') THEN
	    XLABEL='WAVELENGTH (Angstroms)'
	  ELSE IF (BIN.EQ.'LOG(WAVEL(A))') THEN
	    XLABEL='log(Wavelength (Angstroms))'
	  ELSE IF (BIN.EQ.'SPATIAL FREQUENCY') THEN
	    COMMENT=CHFITSMHC(LU,MAPS(1),'CDELT1','?')
	    CALL ALPHA(%REF(COMMENT),1,50,NC,NF,NR,1)
	    XLABEL='SPATIAL FREQUENCY ('//COMMENT(1:NC)//')'
C  Ordinary FITS labels
	  ELSE
	    XLABEL=CHFITSMHV(LU,MAPS(1),'CTYPE1','?')
	  END IF
	ELSE IF (COMMAND.EQ.'C') THEN  ! Column
	  CALL BINVALFITS(LU,MAPS(1),BIN,XST,XND,XINC,.TRUE.,2)
C  Special
	  IF (BIN.EQ.'SPATIAL FREQUENCY') THEN
	    COMMENT=CHFITSMHC(LU,MAPS(1),'CDELT2','?')
	    CALL ALPHA(%REF(COMMENT),1,50,NC,NF,NR,1)
	    XLABEL='SPATIAL FREQUENCY ('//COMMENT(1:NC)//')'
C  Ordinary
	  ELSE
	    XLABEL=CHFITSMHV(LU,MAPS(1),'CTYPE2','?')
	  END IF
C  For "any angle", use basic units
	ELSE !  At any angle
	  XLABEL='Pixel'
	END IF
        IF (BIN.EQ.'SPATIAL FREQUENCY') THEN
	  YLABEL='POWER'
	ELSE
	  YLABEL=CHFITSMHV(LU,MAPS(1),'BUNIT','ADU')
	END IF
	TITLE=CHFITSMHV(LU,MAPS(1),'OBJECT','UNDEF')
	IF (TITLE.EQ.'UNDEF') THEN
	  TITLE=CHFITSMHV(LU,MAPS(1),'COMMENT','UNDEF')
	  IF (TITLE.EQ.'UNDEF'.OR.TITLE.EQ.'         ') THEN
	    TITLE=CHFITSMHV(LU,MAPS(1),'RUN',' ')
	  END IF
	END IF
C
C  Call XYPLOT to make the plot(s)
C  Simple case first
	IF (NMAPS.EQ.1.AND.NROWS.EQ.1.AND.NCOLUMNS.EQ.1) THEN
	  NPLOTS=1
	  LINVECT(1)=0 !  Linewidth will default to 0.020 inches since we
	 	       !  don't specify it.
	  INC(1)=1
	  LINTYPE(1)=0
	  NSYMBOL(1)=0
	  COLOR(1)=16776960  ! Yellow
C  Load the array
	  IF (COMMAND.EQ.'R'.OR.COMMAND.EQ.'C') THEN
	    DO I=1,NPLOTPOINTS(1)
	      XARRAY(I,1)=XST+XINC*(I+IPIX(1)-2)
	    END DO
	  ELSE !  Any angle
	    DO I=1,NPLOTPOINTS(1)
	      XARRAY(I,1)=I+IPIX(1)-0.5
	    END DO
	  END IF
	  DEVICES(1)=' '
	  CALL XYPLOT(XARRAY,DAT,
	1 SIZE,NPLOTS,NPLOTPOINTS,LINVECT,INC,
	2 LINTYPE,NSYMBOL,INVERT,COLOR,INTERACT,CROSSHAIRS,XLABEL,
	3 YLABEL,TITLE,DEVICES,.FALSE.)
	  CALL TEKPAU(0.,5.7,0)
C  Case of multiple rows or columns
	ELSE
	  NPLOTS=MAX(NMAPS*NROWS,NMAPS*NCOLUMNS)
	  PLOTAVERAGE=.FALSE.
	  IF (NPLOTS.NE.1) THEN
	    IF (COMMAND.EQ.'R') THEN
	      CALL USRINP(PLOTAVERAGE,1,'L',4,1,'PLOT_AVERAGE=',
	1     'Do you want to plot only the average of all the rows?  '//
	2     '(default = F)')
	    ELSE IF (COMMAND.EQ.'C') THEN
	      CALL USRINP(PLOTAVERAGE,1,'L',4,1,'PLOT_AVERAGE=',
	1     'Do you want to plot only the average of all the columns?  '//
	2     '(default = F)')
	    END IF
	  END IF
	  IF (SYSTEM_NUMBER_BYTES.EQ.4) THEN
	    CALL DYNAMIC(ADDRESS1_4,SIZE*NPLOTS,'  R4',.TRUE.) !  XARRAY
	    CALL DYNAMIC(ADDRESS2_4,SIZE*NPLOTS,'  R4',.TRUE.) !  YARRAY
	    CALL DYNAMIC(ADDRESS3_4,NPLOTS,'  I4',.TRUE.) !  NPLOTPOINTS
	    CALL DYNAMIC(ADDRESS4_4,NPLOTS,'  I4',.TRUE.) !  LINVECT
	    CALL DYNAMIC(ADDRESS5_4,NPLOTS,'  I4',.TRUE.) !  INC
	    CALL DYNAMIC(ADDRESS6_4,NPLOTS,'  I4',.TRUE.) !  LINTYPE
	    CALL DYNAMIC(ADDRESS7_4,NPLOTS,'  I4',.TRUE.) !  NSYMBOL
	    CALL DYNAMIC(ADDRESS8_4,NPLOTS,'  I4',.TRUE.) !  COLOR
	    CALL LINEPL2_FITS(MAPS,NMAPS,PLANES,NPLANES,ROWS,NROWS,COLUMNS,
	1   NCOLUMNS,NI,NJ,IPIX,COMMAND,%VAL(ADDRESS1_4),%VAL(ADDRESS2_4),
	2   SIZE,NPLOTS,%VAL(ADDRESS3_4),%VAL(ADDRESS4_4),%VAL(ADDRESS5_4),
	3   %VAL(ADDRESS6_4),%VAL(ADDRESS7_4),INVERT,%VAL(ADDRESS8_4),
	4   INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE,BIN,XST,XINC,PLOTAVERAGE)
	    CALL DYNAMIC(ADDRESS1_4,SIZE*NPLOTS,'  R4',.FALSE.)
	    CALL DYNAMIC(ADDRESS2_4,SIZE*NPLOTS,'  R4',.FALSE.)
	    CALL DYNAMIC(ADDRESS3_4,NPLOTS,'  I4',.FALSE.)
	    CALL DYNAMIC(ADDRESS4_4,NPLOTS,'  I4',.FALSE.)
	    CALL DYNAMIC(ADDRESS5_4,NPLOTS,'  I4',.FALSE.)
	    CALL DYNAMIC(ADDRESS6_4,NPLOTS,'  I4',.FALSE.)
	    CALL DYNAMIC(ADDRESS7_4,NPLOTS,'  I4',.FALSE.)
	    CALL DYNAMIC(ADDRESS8_4,NPLOTS,'  I4',.FALSE.)
	  ELSE IF (SYSTEM_NUMBER_BYTES.EQ.8) THEN
	    CALL DYNAMIC(ADDRESS1_8,SIZE*NPLOTS,'  R4',.TRUE.) !  XARRAY
	    CALL DYNAMIC(ADDRESS2_8,SIZE*NPLOTS,'  R4',.TRUE.) !  YARRAY
	    CALL DYNAMIC(ADDRESS3_8,NPLOTS,'  I4',.TRUE.) !  NPLOTPOINTS
	    CALL DYNAMIC(ADDRESS4_8,NPLOTS,'  I4',.TRUE.) !  LINVECT
	    CALL DYNAMIC(ADDRESS5_8,NPLOTS,'  I4',.TRUE.) !  INC
	    CALL DYNAMIC(ADDRESS6_8,NPLOTS,'  I4',.TRUE.) !  LINTYPE
	    CALL DYNAMIC(ADDRESS7_8,NPLOTS,'  I4',.TRUE.) !  NSYMBOL
	    CALL DYNAMIC(ADDRESS8_8,NPLOTS,'  I4',.TRUE.) !  COLOR
	    CALL LINEPL2_FITS(MAPS,NMAPS,PLANES,NPLANES,ROWS,NROWS,COLUMNS,
	1   NCOLUMNS,NI,NJ,IPIX,COMMAND,%VAL(ADDRESS1_8),%VAL(ADDRESS2_8),
	2   SIZE,NPLOTS,%VAL(ADDRESS3_8),%VAL(ADDRESS4_8),%VAL(ADDRESS5_8),
	3   %VAL(ADDRESS6_8),%VAL(ADDRESS7_8),INVERT,%VAL(ADDRESS8_8),
	4   INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE,BIN,XST,XINC,PLOTAVERAGE)
	    CALL DYNAMIC(ADDRESS1_8,SIZE*NPLOTS,'  R4',.FALSE.)
	    CALL DYNAMIC(ADDRESS2_8,SIZE*NPLOTS,'  R4',.FALSE.)
	    CALL DYNAMIC(ADDRESS3_8,NPLOTS,'  I4',.FALSE.)
	    CALL DYNAMIC(ADDRESS4_8,NPLOTS,'  I4',.FALSE.)
	    CALL DYNAMIC(ADDRESS5_8,NPLOTS,'  I4',.FALSE.)
	    CALL DYNAMIC(ADDRESS6_8,NPLOTS,'  I4',.FALSE.)
	    CALL DYNAMIC(ADDRESS7_8,NPLOTS,'  I4',.FALSE.)
	    CALL DYNAMIC(ADDRESS8_8,NPLOTS,'  I4',.FALSE.)
	  END IF	
	  CALL TEKPAU(0.,5.7,0)
	END IF
	CALL RESETTEK
	CALL PLOT9999QUES
	MORE=.FALSE.
	CALL USRINP(MORE,1,'L',4,1,'MORE=',
	1'More plotting with this file and command?  (default = N)')
	IF (COMMAND.EQ.'AS'.OR.COMMAND.EQ.'S') THEN
	  CALL SAO_ERASE_CURSOR(0)
	END IF
	IF (MORE) GO TO 40
	CALL CLOSEFITS(LU)
	STATUS=0
	RETURN
	END
