	SUBROUTINE GET_OUTPUT_FITS_MAPS(LU_OUT,FILENAME_OUT,MAPS_OUT,NMAPS_OUT,
	1NPLNS_OUT,NCOLS_OUT,NROWS_OUT,PRGFMAT_OUT,STATUS)
C
C       This subroutine opens a FITS file for output and selects the output
C  maps.  This subroutine is used when, in creating multiple maps, each new
C  map is desired to have an independent number of planes, columns, and
C  rows, and an independent data format.  In this implentation, in order to
C  guarantee not writing over existing maps the user does not select the
C  maps, the subroutine does.
C			Donald H. Gudehus, July, 2006
C			Version 1UM, D. Gudehus, Aug. 23, 2006
C			Modified USRINP for Absoft compiler on Mac OS X
C
C	LU_OUT		(C)	The logical unit for opening the output file
C	FILENAME_OUT	(C/R)	The output file name.  If it is blank in the
C				call, it is prompted for and returned.
C	MAPS_OUT	(R)	An array giving the map numbers.  In the
C				current implementation, the map numbers
C				are returned.
C	NMAPS_OUT	(C)	The number of output maps desired
C	NPLNS_OUT	(C)	An array giving the number of planes desired
C				in each map
C	NCOLS_OUT	(C)	An array giving the number of columns desired
C				in each map
C	NROWS_OUT	(C)	An array giving the number of rows desired
C				in each map
C	PRGMAT_OUT	(C)	An array giving the data format desired for
C				each map; i.e., I1, I2, I4, R4, R8, C8, and
C				C16.
C	STATUS		(R)	The returned status.  If all is alright,
C				zero is returned.
C
	PARAMETER (LUMIN=11,LUMAX=20,MAPMAX=256)
C
	CHARACTER*(*) FILENAME_OUT
	CHARACTER*3 PRGFMAT_OUT(MAPMAX)
C
	INTEGER NCOLS_OUT(MAPMAX),NROWS_OUT(MAPMAX),MAPS_OUT(MAPMAX),
	1NPLNS_OUT(MAPMAX),STATUS
C
	INTEGER PLANEKEY
C
	LOGICAL FILE_EXISTS,FILE_IS_OPEN
C
	PLANEKEY=1
	N=LEN(FILENAME_OUT)
	IF (FILENAME_OUT.EQ.' ') THEN
	  CALL MESSAGE('Open the output file')
	  CALL USRINP_CHAR(1,'A',N,0,'FILE_OUT=',
	1 'Enter the filename:',%REF(FILENAME_OUT))
	  CALL ALPHA(%REF(FILENAME_OUT),1,N,NCHAR,NFULL,NREST,2)
	END IF
C
	INQUIRE(FILE=FILENAME_OUT,EXIST=FILE_EXISTS,OPENED=FILE_IS_OPEN)
	IF (FILE_IS_OPEN) THEN
	  CALL ERROR(-3,'The file is already open')
	  STATUS=1
	  RETURN
	END IF
	IF (.NOT.FILE_EXISTS) THEN
C  Create the file and all the necessary maps
	  IOFL=.FALSE.
	  DO N=1,NMAPS_OUT
	    CALL MESSAGE(' ')
	    MAPKEY=N !  Used for additional maps in the newly created file
	    MAPS_OUT(N)=MAPKEY
	    NMAPS=1 !  Used for the first map in the new file
	    PLANEKEY=NPLNS_OUT(N)
	    NPLNS=PLANEKEY
	    CALL OPENFITS(LU_OUT,IOFL,'FILE_OUT=',FILENAME_OUT,MAPKEY,
	1   PLANEKEY,NMAPS,NPLNS,NCOLS_OUT(N),NROWS_OUT(N),PRGFMAT_OUT(N),
	2   STATUS)
	    IF (STATUS.NE.0) RETURN
C  Close the file before opening it again to add another map at the end.
	    IF (N.NE.NMAPS_OUT) CLOSE (LU_OUT)
	  END DO
	ELSE
	  IOFL=.FALSE.
C  Find out how many maps already exist in the file.  Then add the new maps
C  one at a time at the end of the file.
C  For future development:  allow the user to select the output maps, and if
C  some already exist, check that they have the required size and data type.
	  OPEN(LU_OUT,STATUS='OLD',ACCESS='DIRECT',RECL=2880,
	1 FORM='UNFORMATTED',FILE=FILENAME_OUT)
	  CALL GET_FITS_INFO(LU_OUT)
	  NMAPS_EXISTING=NO_OF_FITS_MAPS(LU_OUT)
	  CLOSE(LU_OUT)
	  DO N=1,NMAPS_OUT
	    CALL MESSAGE(' ')
	    MAPKEY=N+NMAPS_EXISTING !  Used for additional maps in 
				    !  the existing file
	    MAPS_OUT(N)=MAPKEY
	    PLANEKEY=NPLNS_OUT(N)
	    CALL OPENFITS(LU_OUT,IOFL,'FILE_OUT=',FILENAME_OUT,MAPKEY,
	1   PLANEKEY,NMAPS,NPLNS,NCOLS_OUT(N),NROWS_OUT(N),PRGFMAT_OUT(N),
	2   STATUS)
	    IF (STATUS.NE.0) RETURN
	    IF (N.NE.NMAPS_OUT) CLOSE (LU_OUT)
	  END DO
	END IF
	STATUS=0
	RETURN
	END
