	SUBROUTINE OPEN_TEMP_FITS(LU_IN,LU_TEMP,FILENAME_IN,MAPS_IN,NMAPS_IN,
	1NCOLS,NROWS,STATUS)
C
C	This subroutine copies an already opened FITS file as another FITS file
C  to allow for more options when saving an edited FITS file.  This subroutine 
C  should be called after calling OPENFITS for the input file.  Editing changes
C  are then made to the unit number that this temporary file was opened on.
C  The possibilities for saving then become:  1) Save:  the temporary file with
C  possibly resized maps that have been edited, overwrites the original file.
C  2)  Save As:  the maps that were possibly resized and edited are written to
C  the end of the original file, to the end of another exisiting file, or to
C  a new file with one or more maps.  The saving operation is not done by this
C  subroutine but rather SAVE_SAVE_AS.
C			Donald H. Gudehus, January 11-17, 2008
C			Version 2, Donald H. Gudehus, February 21, 2008
C			Made NCOLS and NROWS arrays so that each map can be
C			an independent size.  Also added code to restore IMNUM
C			and IPNUM to their initial values before returning.  
C			They can be set to 1 if OPENFITS is called.  Previously
C			operations on maps higher than 1 were not being 
C			performed correctly when OPEN_TEMP_FITS was used.
C
C	LU_IN	     (C)    The logical unit that the input file was opened on
C	LU_TEMP	     (C)    Logical unit for I/O.  This must be different than
C			    the unit that the original file was opened on.
C	FILENAME_IN  (C)    The name of the original file
C	MAPS_IN	     (C)    The input maps that will be operated on by the
C			    calling routine
C	NMAPS_IN     (C)    The number of the above input maps 
C	NCOLS	     (C)    An array containing the number of columns selected
C			    by the calling routine for each map that will be
C			    edited.
C	NROWS	     (C)    An array containing the number of rows selected
C			    by the calling routine for each map that will be
C			    edited.
C	STATUS	     (R)    A status return.  If there are no errors, the
C			    returned value is 0;  if the file is already
C			    open, a 1 is returned;  if there was a problem
C			    in opening the file, a 2 is returned.
C
	INCLUDE 'fitsfacts.for'
C
	CHARACTER*(*) FILENAME_IN
	INTEGER MAPS_IN(MAPMAX),NCOLS(MAPMAX),NROWS(MAPMAX),STATUS
C
	BYTE PHYSREC(3000),PHYSREC_0(3000)
	CHARACTER*2 FORMAT
	CHARACTER*3 PRGFMAT
	CHARACTER*8 CTIME
	CHARACTER*20 KEYWORD,COMMENT
	CHARACTER*80 TEMP_FILENAME,FILENAME_SAVE
	CHARACTER*120 COMMAND
	INTEGER I4,PLANEKEY,NUMBER_MAP_RECORDS_TEMP(MAPMAX)
	LOGICAL IOFL,SAME_SIZE,SIZE_CHANGED(MAPMAX),DUPLICATE
	REAL*8 VALUEDP
C
	CHARACTER*80 DNAME
	COMMON /DFILE/ DNAME
C
	COMMON /MAPN/ IMNUM
	COMMON /PLANEN/ IPNUM
C
	IBLKLEN=2880
C
C  Check if any of the maps in the temporary file is to have a size different
C  than the map of the same number in the input file.
C  Initialize
	DO I=1,NUMMAPS(LU_IN)
	  SIZE_CHANGED(I)=.FALSE.
	END DO
	SAME_SIZE=.TRUE.
	DO I=1,NMAPS_IN
	  IF (NCOLS(I).NE.NAXIS1(MAPS_IN(I),LU_IN).OR.NROWS(I).NE.
	1 NAXIS2(MAPS_IN(I),LU_IN)) THEN
	    SAME_SIZE=.FALSE.
	    SIZE_CHANGED(MAPS_IN(I))=.TRUE.
	  ELSE
	    SIZE_CHANGED(MAPS_IN(I))=.FALSE.
	  END IF
	END DO
C  Create the temporary file name
	CALL GET_LEN_NONBLANK(FILENAME_IN,LI)
	CALL TIME(CTIME)
C  Determine if the input file is in another directory
	ISLASH=0
	DO I=LI,1,-1
	  IF (FILENAME_IN(I:I).EQ.'/') THEN
	    ISLASH=I
	    GO TO 100
	  END IF
	END DO
 100	IF (ISLASH.NE.0) THEN
	  TEMP_FILENAME=FILENAME_IN(1:ISLASH)//CTIME//FILENAME_IN(ISLASH+1:LI)
	ELSE
	  TEMP_FILENAME=CTIME//FILENAME_IN(1:LI)
	END IF
C  Alternately, we could just append the time to the right end of the file name
C  for either case, e.g., TEMP_FILENAME=FILENAME_IN(1:LI)//CTIME
	IF (SAME_SIZE) THEN !  Simple case
	  COMMAND='cp '//FILENAME_IN(1:LI)//' '//TEMP_FILENAME(1:8+LI)
	  I4=SYSTEM(COMMAND(1:12+2*LI))
	  QUIET(LU_TEMP)=.TRUE.
	  IOFL=.TRUE.
	  MAPKEY=1 !  Prevents a prompt for the map number
	  PLANEKEY=1 !  Prevents a prompt for the plane number
	  NROWS_TEMP=0
	  NCOLS_TEMP=0
	  FILENAME_SAVE=DNAME
	  IMNUM_SAVE=IMNUM
	  IPNUM_SAVE=IPNUM
	  CALL OPENFITS(LU_TEMP,IOFL,' ',TEMP_FILENAME(1:8+LI),MAPKEY,PLANEKEY,
	1 NMAPS,NPLNS,NCOLS_TEMP,NROWS_TEMP,PRGFMAT,STATUS)
C
	ELSE !  Not the same size
C
C  Open the temporary file as a new file
	  OPEN(LU_TEMP,STATUS='NEW',ACCESS='DIRECT',RECL=2880,
	1 FORM='UNFORMATTED',FILE=TEMP_FILENAME(1:8+LI),IOSTAT=ISTAT)
C  Be sure to put the name into the FITSFACTS common block when opening
C  a file with just FORTRAN OPEN.
	  FITSFILENAME(LU_TEMP)=TEMP_FILENAME(1:8+LI)
	  IF (ISTAT.NE.0) THEN
	    CALL ERROR(-3,'Occurred while opening the temporary file')
	    STATUS=2
	  END IF
C  Calculate how many records each of the size-modified maps will contain.
	  DO I=1,NMAPS_IN
	    NPLNS=NAXIS3(MAPS_IN(I),LU_IN)
	    NPIXELSPERMAP=NPLNS*NROWS(I)*NCOLS(I)
	    NBYTESPERMAP=ABS(BITPIX(MAPS_IN(I),LU_IN))/8*NPIXELSPERMAP
	    NUMBER_MAP_RECORDS_TEMP(MAPS_IN(I))=(NBYTESPERMAP-1)/IBLKLEN+1
						!  Excludes the header
	  END DO
C  Start copying records from the input file to the temporary file.  When a
C  resized map is encountered, write a corrected number of records.
C  If the input file has only one map, start at header 1;  if greater than one map,
C  start at header 0.
C
C  Initialize an array for writing zeros
	  DO I=1,IBLKLEN
	    PHYSREC_0(I)=0
	  END DO
	  IF (NUMMAPS(LU_IN).EQ.1) THEN
	    IR_TEMP=1 !  Temporary file is new
	    IM=1 !  Only one map
C  Do the single header
	    DO IRH=HEADERRECORDS(1,IM,LU_IN),HEADERRECORDS(2,IM,LU_IN)
	      READ(LU_IN,REC=IRH,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	      IF (ISTAT.NE.0) THEN
	        CALL ERROR(-3,'Error in reading physical record')
	        STATUS=2
	      END IF
	      WRITE(LU_TEMP,REC=IR_TEMP,IOSTAT=ISTAT)
	1     (PHYSREC(I),I=1,IBLKLEN)
	      IF (ISTAT.NE.0) THEN
	        CALL ERROR(-3,'Error in writing physical record')
	        STATUS=2
	      END IF
	      IR_TEMP=IR_TEMP+1
	    END DO
C  Do the single map
	    IF (SIZE_CHANGED(IM)) THEN
C  Just write zeros since the calling program will write the data
	      DO IRD=1,NUMBER_MAP_RECORDS_TEMP(1)
	        WRITE(LU_TEMP,REC=IR_TEMP,IOSTAT=ISTAT)
	1       (PHYSREC_0(I),I=1,IBLKLEN)
	        IF (ISTAT.NE.0) THEN
	          CALL ERROR(-3,'Error in writing physical record')
	          STATUS=2
	        END IF
	        IR_TEMP=IR_TEMP+1
	      END DO
	    ELSE !  Same size map.  Logically, this section should not be
		 !  reached
	      DO IRD=MAPRECORDS(1,IM,LU_IN),MAPRECORDS(2,IM,LU_IN)
	        READ(LU_IN,REC=IRD,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	        IF (ISTAT.NE.0) THEN
	          CALL ERROR(-3,'Error in reading physical record')
	          STATUS=2
	        END IF
	        WRITE(LU_TEMP,REC=IR_TEMP,IOSTAT=ISTAT)
	1       (PHYSREC(I),I=1,IBLKLEN)
	        IF (ISTAT.NE.0) THEN
	          CALL ERROR(-3,'Error in writing physical record')
	          STATUS=2
	        END IF
	        IR_TEMP=IR_TEMP+1
	      END DO
	    END IF	      
C  Update the header keywords
C  In order to update keywords, there has to be some information in the
C  common blocks about the the headers, maps, and records.
	    CALL GET_FITS_INFO(LU_TEMP)
	    KEYWORD='NAXIS1'
	    VALUEDP=NCOLS(1)*1.0D0
	    FORMAT='I4'
	    COMMENT=' '
	    DUPLICATE=.FALSE.
	    CALL WRFITSMHV(LU_TEMP,IM,KEYWORD,VALUEDP,FORMAT,COMMENT,
	1   DUPLICATE)
	    KEYWORD='NAXIS2'
	    VALUEDP=NROWS(1)*1.0D0
	    CALL WRFITSMHV(LU_TEMP,IM,KEYWORD,VALUEDP,FORMAT,COMMENT,
	1   DUPLICATE)
	  ELSE !  NUMMAPS > 1
	    IR_TEMP=1 !  Temporary file is new
	    IM=0 !  Primary header
C  Write the primary header
	    DO IRH=HEADERRECORDS(1,IM,LU_IN),HEADERRECORDS(2,IM,LU_IN)
	      READ(LU_IN,REC=IRH,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	      IF (ISTAT.NE.0) THEN
	        CALL ERROR(-3,'Error in reading physical record')
	        STATUS=2
	      END IF
	      WRITE(LU_TEMP,REC=IR_TEMP,IOSTAT=ISTAT)
	1     (PHYSREC(I),I=1,IBLKLEN)
	      IF (ISTAT.NE.0) THEN
	        CALL ERROR(-3,'Error in writing physical record')
	        STATUS=2
	      END IF
	      IR_TEMP=IR_TEMP+1
	    END DO
C  Loop through all the maps and write the remaining records
	    DO IM=1,NUMMAPS(LU_IN)
	      DO IRH=HEADERRECORDS(1,IM,LU_IN),HEADERRECORDS(2,IM,LU_IN)
	        READ(LU_IN,REC=IRH,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	        IF (ISTAT.NE.0) THEN
	          CALL ERROR(-3,'Error in reading physical record')
	          STATUS=2
	        END IF
	        WRITE(LU_TEMP,REC=IR_TEMP,IOSTAT=ISTAT)
	1       (PHYSREC(I),I=1,IBLKLEN)
	        IF (ISTAT.NE.0) THEN
	          CALL ERROR(-3,'Error in writing physical record')
	          STATUS=2
	        END IF
	        IR_TEMP=IR_TEMP+1
	      END DO
	      IF (SIZE_CHANGED(IM)) THEN
C  Just write zeros since the calling program will write the data
	        DO IRD=1,NUMBER_MAP_RECORDS_TEMP(IM)
	          WRITE(LU_TEMP,REC=IR_TEMP,IOSTAT=ISTAT)
	1         (PHYSREC_0(I),I=1,IBLKLEN)
	          IF (ISTAT.NE.0) THEN
	            CALL ERROR(-3,'Error in writing physical record')
	            STATUS=2
	          END IF
	          IR_TEMP=IR_TEMP+1
	        END DO
	      ELSE
	        DO IRD=MAPRECORDS(1,IM,LU_IN),MAPRECORDS(2,IM,LU_IN)
	          READ(LU_IN,REC=IRD,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	          IF (ISTAT.NE.0) THEN
	            CALL ERROR(-3,'Error in reading physical record')
	            STATUS=2
	          END IF
	          WRITE(LU_TEMP,REC=IR_TEMP,IOSTAT=ISTAT)
	1         (PHYSREC(I),I=1,IBLKLEN)
	          IF (ISTAT.NE.0) THEN
	            CALL ERROR(-3,'Error in writing physical record')
	            STATUS=2
	          END IF
	          IR_TEMP=IR_TEMP+1
	        END DO
	      END IF
	    END DO !  Ends loop on all the maps in LU_IN
C  Update the header keywords for those maps that were resized.
C  In order to update keywords, there has to be some information in the
C  common blocks about the the headers, maps, and records.
	    CALL GET_FITS_INFO(LU_TEMP)
	    DO I=1,NMAPS_IN
	      IM=MAPS_IN(I)
	      IF (SIZE_CHANGED(IM)) THEN
	        VALUEDP=NCOLS(I)*1.0D0
	        CALL WRFITSMHV(LU_TEMP,IM,'NAXIS1',VALUEDP,'I4',' ',.FALSE.)
	        VALUEDP=NROWS(I)*1.0D0
	        CALL WRFITSMHV(LU_TEMP,IM,'NAXIS2',VALUEDP,'I4',' ',.FALSE.)
	      END IF
	    END DO
C
	  END IF !  Ends loop for NUMMAPS being 1 or greater than 1
	END IF !  Ends loop for SAMESIZE
C  Update the FITSFACTS common block to reflect the changes in NAXIS1 and
C  NAXIS2.
	CALL GET_FITS_INFO(LU_TEMP)
C  Don't use the temporary filename as the default filename
	DNAME=FILENAME_SAVE
C  Restore IPNUM and IMNUM which may have gotten reset by the call to OPENFITS.
	IMNUM=IMNUM_SAVE
	IPNUM=IPNUM_SAVE
C
	RETURN
	END
