	SUBROUTINE ALOGAXIS(X,Y,TEXT,AXLENGTH,ANGLE,FIRSTDECVAL,
	1DECADES,FLAG)
C
C  Note:  This is the Unix version.
C
C       This subroutine draws an axis and its labels with the annotation
C  representing the antilog of the plotted values.
C				Donald H. Gudehus, August, 1985
C			Version 2, Donald Gudehus, January 8, 2000
C			Removed LU argument;  changed AXISVAL and AXISVALINIT
C			to REAL*8.
C			Version 2U, Donald Gudehus, January 9, 2000
C			Modified code for Unix
C			Version 3, D. H. Gudehus, February 28, 2004
C			Changed INT to NINT for LOG10(AXISVAL) to avoid
C			errors.
C			Version 4, D. H. Gudehus, February 12, 2005
C			Introduced FIRSTTIME
C
C	X	    (C)	The starting x position of the axis measured in
C			inches assuming that there is no inversion of.
C			axis.
C	Y	    (C)	Like X but in the y direction
C	TEXT	    (C)	The label for the axis
C	AXLENGTH    (C)	The axis length in inches
C	ANGLE	    (C)	The angle of the axis from the horizontal
C	FIRSTDECVAL (C)	The value corresponding to the beginning of the axis
C			coded as log.antilog where log is the integer part of
C			the plotted value (which is a log), and antilog
C			represents the first displayed tick mark of that decade
C			(which is an antilog).  The antilog of the physical
C			value at the first tick mark may or may not be printed
C			by the program.
C	DECADES     (C)	The number of decade cycles per axis length starting
C			at the beginning of the decade corresponding to
C			FIRSTDECVAL coded as n.antilog where n is the integer
C			part of the number of decades, and antilog represents
C			the rightmost displayed tick mark of an additional
C			decade.  For example, if FIRSTDECVAL is -1.5 and
C			DECADES is 3.2, then the first tick mark will be at
C			X=0.5 (log = -0.301 = -1 + 0.699) and the last tick
C			mark will be at X=200 (log = 2.301).  Note that
C			fractional parts of .0 and .1 behave the same way.
C	FLAG(1)	    (C)	If +1, the tick marks appear on the counter-
C			clockwise side of the axis;  if -1, they appear
C			on the clockwise side.
C	FLAG(2)	    (C)	Like FLAG(1), but for the label
C	FLAG(3)	    (C)	If 0, the numbers by the tick marks are parallel
C			to the axis;  if 1, they are perpendicular.
C	FLAG(4)	    (C)	If 0, the numbers and label are omitted;  if 1,
C			they are displayed
C       FLAG(5)	    (C)	If 0, the axes are displayed normally;  if 1,
C			they are inverted
C
	BYTE TEXT(1)
	INTEGER*2 FLAG(5)
	LOGICAL PRINTALL,FIRSTTIME
	REAL NUMBHT,INCHESPERDECADE,LASTDECVALMAX0,LASTDECVALMAX1,
	1LASTDECVAL
	REAL*8 AXISVAL,AXISVALINIT
C
	INCLUDE 'plotpar.for'
C
	NUMBHT=0.12
	CHARHT=0.15
	SPACE=0.08
	TICKLENGTH=0.06
	EXPANS=1.5
	FIRSTDECVALMIN0=-1.8
	FIRSTDECVALMIN1=-4.0
	LASTDECVALMAX0=2.8
	LASTDECVALMAX1=4.0
	LASTDECVAL=FIRSTDECVAL+DECADES
	INCHESPERDECADE=AXLENGTH/(INT(DECADES)+LOG10(10.0*
	1MAX(0.1,DECADES-INT(DECADES)))
	1-LOG10(10.0*MAX(0.1,ABS(FIRSTDECVAL-INT(FIRSTDECVAL)))))
C
	IF (((DECADES-FIRSTDECVAL.LE.4.AND.FLAG(3).EQ.0).AND.FIRSTDECVAL.GE.
	2FIRSTDECVALMIN0.AND.LASTDECVAL.LE.LASTDECVALMAX0.AND.DECADES.LE.2.1)
	3.OR.(DECADES-FIRSTDECVAL.LE.5.AND.FLAG(3).EQ.1.AND.FIRSTDECVAL.GE.
	4FIRSTDECVALMIN1.AND.LASTDECVAL.LE.LASTDECVALMAX1)) THEN
	  PRINTALL=.TRUE. !  Numbers are printed at every tick mark without
			  !  exponential notation
C  Adjust the number height to avoid overlapping
	  IF (FLAG(3).EQ.0) THEN !  Numbers parallel
	    IF (AXLENGTH.LT.7.5) NUMBHT=NUMBHT*AXLENGTH/7.5
	  ELSE IF (FLAG(3).EQ.1) THEN !  Numbers perpendicular
C  Be sure that the 9 and 10 do not overlap
	    TEST=INCHESPERDECADE*(1.0-LOG10(9.0))
	    DO WHILE (TEST.LT.NUMBHT*1.05)
	      NUMBHT=NUMBHT*0.99
	    END DO
	  END IF
	ELSE
	  PRINTALL=.FALSE.
	  NUMBHT=NUMBHT*EXPANS
		!  The number height is made larger to allow room
		!  for the exponent in the positioning calculation.
		!  The actual number size is written as before.
	END IF
	EPSILON=1E-4
C
C  Find on which side of the axis the tick marks are to be produced
C
	SIDET=SIGN(1,FLAG(1))
C
C  Find which side of the axis the label and numbers are to be plotted.
C
	SIDEA=SIGN(1,FLAG(2))
	CALL ALPHA(TEXT,1,50,NC,NF,NR,1)
	CALL ALPHA(TEXT,1,50,NCLABEL,NF,NR,3)
C
C  Find sine and cosine of ANGLE
C
	SINA=SIND(ANGLE)
	COSA=COSD(ANGLE)
C
C  Get the incremental positions of the annotations from the axis.  The
C  notation is consistent with a horizontal axis.
C
C  Get the antilog value for the start of the first decade
	AXISVALINIT=10.0**(INT(FIRSTDECVAL))
	FIRSTALOGV=10.0**ALOGDECODE(FIRSTDECVAL)
	IF (FLAG(4).EQ.0) GO TO 999 !  Omits numbers and labels
C
	DELTYNNORM=ABS(SIDET+SIDEA)/2*TICKLENGTH*2*SIDET+(0.5*NUMBHT+SPACE)*
	1SIDEA-0.5*NUMBHT
	DELTYLNORM=ABS(SIDET+SIDEA)/2*TICKLENGTH*2*SIDET+(0.5*CHARHT+NUMBHT
	1+2.3*SPACE)*SIDEA-0.5*CHARHT !  Valid for numbers parallel to axis
	CALL COLORMODE(16766720) !  Gold
	DELTYL2MAX=0.0
	I=1
	FI=1.0
	AXISVAL=AXISVALINIT !  An antilog value
	DELTYL=DELTYLNORM
	IF (PRINTALL) THEN !  Numbers at every tick mark
	  DO WHILE(AXISVAL.LE.AXISVALINIT*10.0*(MAX(0.1,DECADES-INT(DECADES)))*
	1 10.0**INT(DECADES)+EPSILON*AXISVAL)
C  Compute the postion adjustments of the numbers for the number of digits.
C  Adjust for the number of digits when the numbers are perpendicular to the
C  axis and on the counterclockwise side.  We also
C  find the maximum shift left for all the numbers in order
C  to position the label correctly.
C  Compute the locations for each number
C  For less than DECADESMAX cycles, print every number
	    IF (AXISVAL+AXISVAL*EPSILON.GE.FIRSTALOGV) THEN
	      IF (AXISVAL.LT.1.0) THEN
	        NDECI=1+INT(ABS(LOG10(AXISVAL))) !  Same formula as above for
						 !  NDECI for label
	      ELSE
	        NDECI=-1 !  Eliminates the decimal point
	      END IF
	      IF (FLAG(3).EQ.1) THEN !  Numbers perpendicular to axis
	        IF (NDECI.EQ.-1) THEN
	          NDIGITS=INT(LOG10(AXISVAL))+1
	        ELSE
	          NDIGITS=2+NDECI
	        END IF
	        IF (FLAG(2).EQ.1) THEN !  Annotation on counterclockwise side
	          DELTXNTOT=-0.5*NUMBHT
	          DELTYNTOT=DELTYNNORM+0.836*NUMBHT*(NDIGITS-0.2)
	          DELTYL2=0.836*NUMBHT*(NDIGITS-1.2)
	        ELSE
	          DELTXNTOT=-0.5*NUMBHT
	          DELTYNTOT=DELTYNNORM+0.836*NUMBHT
	          DELTYL2=0.836*NUMBHT*(NDIGITS-1.2)
	        END IF
	      ELSE
	        IF (NDECI.EQ.-1) THEN
	          NDIGITS=INT(LOG10(AXISVAL))+1
	        ELSE
	          NDIGITS=2+NDECI
	        END IF
	        DELTXNTOT=(0.3-NDIGITS)*NUMBHT*0.5*0.836
	        DELTYNTOT=DELTYNNORM
	        DELTYL2=0.0
	      END IF
	      DELTYL2MAX=MAX(DELTYL2MAX,DELTYL2)
	      IF (FLAG(5).EQ.0) THEN
	        XPOS=X+DELTXNTOT*COSA-DELTYNTOT*SINA+(LOG10(AXISVAL)-
	1       LOG10(FIRSTALOGV))*INCHESPERDECADE*COSA
	        YPOS=Y+DELTYNTOT*COSA+DELTXNTOT*SINA+(LOG10(AXISVAL)-
	1       LOG10(FIRSTALOGV))*INCHESPERDECADE*SINA
	      ELSE IF (FLAG(5).EQ.1) THEN
	        XPOS=X+DELTXNTOT*COSA-DELTYNTOT*SINA+AXLENGTH*COSA-
	1       (LOG10(AXISVAL)-LOG10(FIRSTALOGV))*INCHESPERDECADE*COSA
	        YPOS=Y+DELTYNTOT*COSA+DELTXNTOT*SINA+AXLENGTH*SINA-
	1       (LOG10(AXISVAL)-LOG10(FIRSTALOGV))*INCHESPERDECADE*SINA
	      END IF
C
	      IF (FLAG(3).EQ.0) THEN !  Numbers parallel to axis
	        CALL NUMBER(XPOS,YPOS,NUMBHT,AXISVAL,ANGLE,NDECI)
	      ELSE IF (FLAG(3).EQ.1) THEN
	        CALL NUMBER(XPOS,YPOS,NUMBHT,AXISVAL,ANGLE-90.0,NDECI)
	      END IF
	    END IF
	    I=I+1
	    IF (I.EQ.10) THEN
	      I=1
	      FI=FI*10.0
	    END IF
	    AXISVAL=AXISVALINIT*I*FI
	  END DO
	ELSE IF (.NOT.PRINTALL) THEN !  Exponential notation at decades
	  NDECI=-1 !  Eliminates the decimal point
	  IF (FLAG(3).EQ.1) THEN !  Numbers perpendicular to axis
	    DELTXNTOT=-0.5*NUMBHT
	    IF (FLAG(2).EQ.1) THEN !  Annotation on counterclockwise side
	      DELTYN=DELTYNNORM+0.836*NUMBHT*(NDECI+1.8)
	      DELTYL=DELTYLNORM+0.836*NUMBHT*(NDECI+1)
	    ELSE
	      DELTYN=DELTYNNORM+0.836*NUMBHT
	      DELTYL=DELTYLNORM-0.836*NUMBHT*(NDECI+1)
	    END IF
	  ELSE
	    DELTYN=DELTYNNORM
	    DELTYL=DELTYLNORM
	  END IF
	  DO WHILE(AXISVAL.LE.AXISVALINIT*10.0*(MAX(0.1,DECADES-INT(DECADES)))*
	1 10.0**INT(DECADES)+EPSILON*AXISVAL)
C  The present code allows for future expansion where NDIGITS might take
C  account of the size of the exponent.
	    IF (AXISVAL*(1.0+EPSILON).GE.FIRSTALOGV) THEN
	      IF (FLAG(3).EQ.1) THEN !  Numbers perpendicular to axis
	        NDIGITS=3
	        DELTYN2=0.838*NUMBHT*(NDIGITS-1)
	      ELSE
	        NDIGITS=3+NDECI
	        DELTXNTOT=(0.3-NDIGITS)*NUMBHT*0.5*0.836
	        DELTYN2=0.0
	      END IF
	      DELTYL2MAX=MAX(DELTYL2MAX,DELTYN2)
	      IF (FLAG(2).EQ.1) THEN
	        DELTYNTOT=DELTYN+DELTYN2
	      ELSE
	        DELTYNTOT=DELTYN
	      END IF
	      IF (FLAG(5).EQ.0) THEN
	        XPOS=X+DELTXNTOT*COSA-DELTYNTOT*SINA+(LOG10(AXISVAL)-
	1       LOG10(FIRSTALOGV))*INCHESPERDECADE*COSA
	        YPOS=Y+DELTYNTOT*COSA+DELTXNTOT*SINA+(LOG10(AXISVAL)-
	1       LOG10(FIRSTALOGV))*INCHESPERDECADE*SINA
	      ELSE IF (FLAG(5).EQ.1) THEN
	        XPOS=X+DELTXNTOT*COSA-DELTYNTOT*SINA+AXLENGTH*COSA-
	1       (LOG10(AXISVAL)-LOG10(FIRSTALOGV))*INCHESPERDECADE*COSA
	        YPOS=Y+DELTYNTOT*COSA+DELTXNTOT*SINA+AXLENGTH*SINA-
	1       (LOG10(AXISVAL)-LOG10(FIRSTALOGV))*INCHESPERDECADE*SINA
	      END IF
	      IF (FLAG(3).EQ.0) THEN !  Numbers parallel to axis
C  Print a 10
	        CALL NUMBER(XPOS,YPOS,NUMBHT/EXPANS,10.0D0,ANGLE,NDECI)
	        XPOS=XPOS+(2.2*COSA-0.8*SINA)*0.836*NUMBHT/EXPANS
	        YPOS=YPOS+(2.2*SINA+0.8*COSA)*0.836*NUMBHT/EXPANS
C  Print the exponent
	        CALL NUMBER(XPOS,YPOS,0.7*NUMBHT/EXPANS,
	1       DBLE(NINT(LOG10(AXISVAL))),ANGLE,NDECI)
	      ELSE IF (FLAG(3).EQ.1) THEN
	        CALL NUMBER(XPOS,YPOS,NUMBHT/EXPANS,10.0D0,ANGLE-90.0,NDECI)
	        XPOS=XPOS+(2.2*SINA+0.8*COSA)*0.836*NUMBHT/EXPANS
	        YPOS=YPOS+(-2.2*COSA+0.8*SINA)*0.836*NUMBHT/EXPANS
	        CALL NUMBER(XPOS,YPOS,0.7*NUMBHT/EXPANS,
	1       DBLE(NINT(LOG10(AXISVAL))),ANGLE-90.0,NDECI)
	      END IF
	    END IF
	    FI=FI*10.0
	    AXISVAL=AXISVALINIT*I*FI
	  END DO
	END IF
	DELTYL=DELTYL+SIDEA*DELTYL2MAX
C
C  Label the axis
	CALL COLORMODE(65280) !  Green
	DELTXL=0.5*AXLENGTH-0.5*0.825*CHARHT*NCLABEL
	XPOS=X+DELTXL*COSA-DELTYL*SINA
	YPOS=Y+DELTYL*COSA+DELTXL*SINA
	CALL SYMBOL(XPOS,YPOS,CHARHT,TEXT,ANGLE,NC)
C
C  Draw the axis and ticks
C
 999	CALL COLORMODE(255) !  Blue
	DELTX=-TICKLENGTH*SIDET*SINA
	DELTY=TICKLENGTH*SIDET*COSA
	IF (FLAG(5).EQ.0) THEN
	  CALL PLOT(X,Y,3)
	ELSE IF (FLAG(5).EQ.1) THEN
	  CALL PLOT(X+AXLENGTH*COSA,Y+AXLENGTH*SINA,3)
	END IF
	I=1
	FI=1
	FIRSTTIME=.TRUE.
	AXISVAL=AXISVALINIT
	DO WHILE(AXISVAL.LE.AXISVALINIT*10.0*MAX(0.1,(DECADES-INT(DECADES)))*
	110.0**INT(DECADES)+EPSILON*AXISVAL)
	  IF (AXISVAL.GE.FIRSTALOGV-EPSILON*AXISVAL) THEN
	    IF (FLAG(5).EQ.0) THEN
	      XPOS=X+(LOG10(AXISVAL)-LOG10(FIRSTALOGV))*INCHESPERDECADE*COSA
	      YPOS=Y+(LOG10(AXISVAL)-LOG10(FIRSTALOGV))*INCHESPERDECADE*SINA
	    ELSE IF (FLAG(5).EQ.1) THEN
	      XPOS=X+AXLENGTH*COSA-(LOG10(AXISVAL)-LOG10(FIRSTALOGV))*
	1     INCHESPERDECADE*COSA
	      YPOS=Y+AXLENGTH*SINA-(LOG10(AXISVAL)-LOG10(FIRSTALOGV))*
	1     INCHESPERDECADE*SINA
	    END IF
	    IF (.NOT.FIRSTTIME) CALL PLOT(XPOS,YPOS,2)
	    FIRSTTIME=.FALSE.
	    IF (I.EQ.1) THEN
	      CALL PLOT(XPOS+2.0*DELTX,YPOS+2.0*DELTY,2)
	    ELSE
	      CALL PLOT(XPOS+DELTX,YPOS+DELTY,2)
	    END IF
	    CALL PLOT(XPOS,YPOS,3) !  Keeps from going to point plot mode
	  END IF
	  I=I+1
	  IF (I.EQ.10) THEN
	    I=1
	    FI=FI*10
	  END IF
	  AXISVAL=AXISVALINIT*I*FI
	END DO
C  Finish drawing the axis if necessary
	IF (FLAG(5).EQ.0) THEN
	  IF (SQRT((XPOS-X)**2+(YPOS-Y)**2).LT.AXLENGTH) THEN
	    XPOS=X+AXLENGTH*COSA
	    YPOS=Y+AXLENGTH*SINA
	    CALL PLOT(XPOS,YPOS,2)
	  END IF
	ELSE IF (FLAG(5).EQ.1) THEN
	  IF (SQRT((AXLENGTH*COSA+X-XPOS)**2+(AXLENGTH*SINA+Y-YPOS)**2)
	1 .LT.AXLENGTH) THEN
	    XPOS=X
	    YPOS=Y
	    CALL PLOT(XPOS,YPOS,2)
	  END IF
	END IF
C
	RETURN
	END
