	SUBROUTINE APPLE(IX,IY,IPEN)
C
C  Note:  This is the Unix version
C
C       This subroutine writes plot commands for the Apple Laserwriter
C  to a file which can then be sent the Laserwriter for plotting.  
C  When drawing vectors, positions are sent as absolute coordinates (to
C  avoid cumlative errors) in units of pixels (600ths of an inch).
C			Donald H. Gudehus, August ,1987
C			Version 2, Donald H. Gudehus, July 23, 1995
C			Changed code so as to be device resolution
C			independent.
C			Version 3, Donald H. Gudehus, June 6, 1999
C			Modified dashing to be similar to Xterm.
C			Version 4, Donald H. Gudehus, June 7, 1999
C			Added code for encapsulated postscript.
C			Version 4U, Donald H. Gudehus, February 5, 2000
C			Modified code for Unix
C			Version 5, Donald H. Gudehus, April 16, 2005
C			Made the default bounding box size for eps smaller.
C			Adjusted the postscript starting location slightly.
C			Version 6, Donald H. Gudehus, May 19, 2009
C			Added gsave after grestore to preserve graphic state
C			when multiple plots are written to a file.  This was
C			failing after the 2nd plot.
C
C	IX	(C)	The x location in pixels
C	IY	(C)	The y location in pixels
C	IPEN	(C)	0  Initializes the Current Transformation Matrix
C			1  Goes to point plot mode
C			2  Sends the new position for a point, a
C			   vector, or a rectangle.
C			3  Moves to a spot with the pen up 
C			4  Resets the device to a simple default setup
C			5  Sets the line style
C			8  Sets the data level:  0 = dots turned on,
C			   1 = dots turned off, 2 = dots toggled/
C			   complemented
C			9  Goes to rectilinear graphics mode
C		       10  Sets the cursor position and goes to the
C			   alphagraphics mode
C		       11  Sets the filling style
C		       12  Sets the pen width
C		       15  Causes the page to be printed
C
C	CHARACTER CBUF3*2
	CHARACTER*5 CBUF1,CBUF2
	CHARACTER*4 CBUF4
	CHARACTER*9 DAYO
	CHARACTER*30 CHARBUF
	INTEGER FILL,WIDTH
	LOGICAL POINTPLOT,RECTILINEAR
	INCLUDE 'plotpar.for'
C
	COMMON /PLOT_CHAN/ NCHAN,NDEV
	COMMON /PLOT_LINESTYLE/ MODE
	COMMON /PLOT_DATALEVEL/ LEVEL
	COMMON /PLOT_FILLSTYLE/ FILL
	COMMON /PLOT_PENWIDTH/ WIDTH
C
	LOGICAL LASERGRAFIX,APPLE0,APPLE2,APPLE3,APPLE4,APPLE5,APPLE6,
	1LPS40,EPS,POSTSCRIPT,IMAGEN0
	COMMON /LIB_/ LASERGRAFIX,APPLE0,APPLE2,APPLE3,APPLE4,APPLE5,APPLE6,
	1LPS40,EPS,POSTSCRIPT,IMAGEN0

	DATA NCOUNT/0/
C
	IF (IPEN.EQ.0.AND..NOT.EPS) THEN
	  POINTPLOT=.FALSE.
	  RECTILINEAR=.FALSE.
C  Notify the spooler that the text is in PostScript
	  CHARBUF='%!-PS-Adobe-'
	  NA=12
	  WRITE (53,1000) CHARBUF
1000	  FORMAT(A<NA>)
C  Save the initial state
	  CHARBUF='gsave'
	  NA=5
	  WRITE (53,1000) CHARBUF
C  Define the units as 600ths of an inch so as to make the code independent
C  of the resolution of the device.
	  CHARBUF='/nibs {0.12 mul} def'
	  NA=20
	  WRITE (53,1000) CHARBUF
C  Initialize the transformation matrix.  This sets the scale, translation,
C  and rotation to the default of the device.
	  CHARBUF='initmatrix'
	  NA=10
	  WRITE (53,1000) CHARBUF
C  Make the orientation landscape on the LaserWriter
C	  CHARBUF='5100 nibs 0 nibs translate'
	  CHARBUF='4900 nibs 0 nibs translate'
		!  Puts the plot close to the right edge
	  NA=26
	  WRITE (53,1000) CHARBUF
	  CHARBUF='90 rotate'
	  NA=9
	  WRITE (53,1000) CHARBUF
C  Set the pen width to the default of 20 thousandths of an inch or 6 pixels
C  (if 300 dpi).  Since Postscript wants units of points (1/72 inch) 
	  CHARBUF='20 0.072 mul setlinewidth'
	  NA=25
	  WRITE (53,1000) CHARBUF
C  Set the ends of the vectors to round
	  CHARBUF='1 setlinecap'
	  NA=12
	  WRITE (53,1000) CHARBUF
C  Set the joining of the vectors to round
	  CHARBUF='1 setlinejoin'
	  NA=13
	  WRITE (53,1000) CHARBUF
C  Store the setup.  We save this so that if there are several plots in a file
C  they will all appear in the same orientation and scale when displayed with
C  the plot command.
	  NA=5
	  CHARBUF='gsave'
	  WRITE (53,1000) CHARBUF
C  The Apple Laserwriter is now ready to draw bright vectors.
C
	ELSE IF (IPEN.EQ.0.AND.EPS) THEN
	  POINTPLOT=.FALSE.
	  RECTILINEAR=.FALSE.
C  Notify the spooler that the text is in Encapsulated PostScript
	  CHARBUF='%!PS-Adobe-2.0 EPSF-1.2'
	  NA=23
	  WRITE (53,1000) CHARBUF
	  CHARBUF='%%Title:  EPS for TeX'
	  NA=21
	  WRITE (53,1000) CHARBUF
	  CALL DATE(DAYO)
	  CHARBUF='%%CreationDate:  '
	  CHARBUF(18:26)=DAYO
	  NA=26
	  WRITE (53,1000) CHARBUF
	  CHARBUF='%%Pages:  1'
	  NA=11
	  WRITE (53,1000) CHARBUF
C	  CHARBUF='%%BoundingBox:  0 0 1100 1000'
	  CHARBUF='%%BoundingBox:  0 0 550 420'
C	  NA=29
	  NA=27
	  WRITE (53,1000) CHARBUF
	  CHARBUF='%%EndComments:'
	  NA=14
	  WRITE (53,1000) CHARBUF
	  CHARBUF='%%EndProlog:'
	  NA=12
	  WRITE (53,1000) CHARBUF
C  Define the units as 600ths of an inch so as to make the code independent
C  of the resolution of the device.
	  CHARBUF='/nibs {0.12 mul} def'
	  NA=20
	  WRITE (53,1000) CHARBUF
C  Make the orientation portrait on the LaserWriter
	  CHARBUF='0 nibs 0 nibs translate'
	  NA=23
	  WRITE (53,1000) CHARBUF
C  Set the pen width to the default of 20 thousandths of an inch or 6 pixels
C  (if 300 dpi).  Since Postscript wants units of points (1/72 inch) 
	  CHARBUF='20 0.072 mul setlinewidth'
	  NA=25
	  WRITE (53,1000) CHARBUF
C  Set the ends of the vectors to round
	  CHARBUF='1 setlinecap'
	  NA=12
	  WRITE (53,1000) CHARBUF
C  Set the joining of the vectors to round
	  CHARBUF='1 setlinejoin'
	  NA=13
	  WRITE (53,1000) CHARBUF
	  CHARBUF='%%Page:  1'
	  NA=10
	  WRITE (53,1000) CHARBUF
C  The file is set up for EPS
C
C  Enter the point plot mode
C
	ELSE IF (IPEN.EQ.1) THEN
	  POINTPLOT=.TRUE.
C
C  Draw a bright vector
C
	ELSE IF (IPEN.EQ.2) THEN
	  IF (POINTPLOT) THEN
C  First draw a dark vector
	    ENCODE(5,1005,CBUF1) IX
	    ENCODE(5,1005,CBUF2) IY
1005	    FORMAT (I5)
	    CHARBUF=CBUF1(1:5)//' nibs '//CBUF2(1:5)//' nibs moveto'
	    NA=28
	    WRITE (53,1000) CHARBUF
C  Put the pen down to make the point
	    CHARBUF=CBUF1(1:5)//' nibs '//CBUF2(1:5)//' nibs lineto'
	    NA=28
	    WRITE (53,1000) CHARBUF
	    NCOUNT=NCOUNT+2
	  ELSE IF (RECTILINEAR) THEN  !  Filled rectangle
C  Draw a rectangle.  The pen has already been placed at the lower left.
C  Because the Apple does not automatically draw a rectangle, we must do it.
C  Send a "stroke" to print any vectors with preexisting linestyles.
	    CHARBUF='stroke'
	    NA=6
	    WRITE (53,1000) CHARBUF
C  Originally the following code drew a rectangle in lieu of being able
C  to create a filled rectangle.  However this overwrote dashes, so at
C  present it is commented out.  DHG Feb., 1991
C  Reestablish the path since "stroke" erases it
C	    ENCODE(5,1005,CBUF1) IXOLD
C	    ENCODE(5,1005,CBUF2) IYOLD
C	    CHARBUF=CBUF1(1:5)//' '//CBUF2(1:5)//' moveto'
C	    NA=18
C	    WRITE (53,1000) CHARBUF
C  Get the filling style
C	    ENCODE(2,1002,CBUF3) FILL
1002	    FORMAT(I2)
C	    ENCODE(5,1005,CBUF1) IX
C	    ENCODE(5,1005,CBUF2) IYOLD
C	    CHARBUF=CBUF1(1:5)//' '//CBUF2(1:5)//' lineto'
C	    NA=18
C	    WRITE (53,1000) CHARBUF
C	    ENCODE(5,1005,CBUF2) IY
C	    CHARBUF=CBUF1(1:5)//' '//CBUF2(1:5)//' lineto'
C	    NA=18
C	    WRITE (53,1000) CHARBUF
C	    ENCODE(5,1005,CBUF1) IXOLD
C	    CHARBUF=CBUF1(1:5)//' '//CBUF2(1:5)//' lineto'
C	    NA=18
C	    WRITE (53,1000) CHARBUF
C	    ENCODE(5,1005,CBUF2) IYOLD
C	    CHARBUF=CBUF1(1:5)//' '//CBUF2(1:5)//' lineto'
C	    NA=18
C	    WRITE (53,1000) CHARBUF
C	    CHARBUF='stroke'
C	    NA=6
C	    WRITE (53,1000) CHARBUF
C  Reestablish the path since "stroke" erases it.  Set it equal to the
C  upper right
	    ENCODE(5,1005,CBUF1) IX
	    ENCODE(5,1005,CBUF2) IY
	    CHARBUF=CBUF1(1:5)//' nibs '//CBUF2(1:5)//' nibs moveto'
	    NA=28
	    WRITE (53,1000) CHARBUF
	    NCOUNT=0
	  ELSE  ! Not rectilinear;  draws a bright vector
	    ENCODE(5,1005,CBUF1) IX
	    ENCODE(5,1005,CBUF2) IY
	    CHARBUF=CBUF1(1:5)//' nibs '//CBUF2(1:5)//' nibs lineto'
	    NA=28
	    WRITE (53,1000) CHARBUF
	    NCOUNT=NCOUNT+1
	  END IF
	  IXOLD=IX
	  IYOLD=IY
	  RECTILINEAR=.FALSE.
C
C  Draw a dark vector
C
	ELSE IF (IPEN.EQ.3) THEN
	  ENCODE(5,1005,CBUF1) IX
	  ENCODE(5,1005,CBUF2) IY
	  CHARBUF=CBUF1(1:5)//' nibs '//CBUF2(1:5)//' nibs moveto'
	  NA=28
	  WRITE (53,1000) CHARBUF
	  POINTPLOT=.FALSE.
	  IXOLD=IX
	  IYOLD=IY
	  NCOUNT=NCOUNT+1
C
C  Reset
C
	ELSE IF (IPEN.EQ.4.AND..NOT.EPS) THEN
C  Restore the graphics state
	  CHARBUF='grestore'
	  NA=8
	  WRITE (53,1000) CHARBUF
C  and save it again in case there are multiple plots being written to the file
	  CHARBUF='gsave'
	  NA=5  
	  WRITE (53,1000) CHARBUF
C
C  Set the type of lines that are indicated for the vector
C
	ELSE IF (IPEN.EQ.5) THEN
C  Send a "stroke" to print any vectors with preexisting linestyles.
	  CHARBUF='stroke'
	  NA=6
	  WRITE (53,1000) CHARBUF
	  IF (MODE.EQ.0) THEN
	    CHARBUF='[] 0 setdash'
	    NA=12
	  ELSE IF (MODE.EQ.1) THEN
	    CHARBUF='[3] 0 setdash'
	    NA=13
	  ELSE IF (MODE.EQ.2) THEN
	    CHARBUF='[6 2.5 6 4] 0 setdash'
	    NA=21
	  ELSE IF (MODE.EQ.3) THEN
	    CHARBUF='[6 4] 0 setdash'
	    NA=15
	  ELSE IF (MODE.EQ.4) THEN
	    CHARBUF='[6 12] 0 setdash'
	    NA=16
	  ELSE IF (MODE.EQ.5) THEN
	    CHARBUF='[12 12] 0 setdash'
	    NA=17
	  ELSE IF (MODE.EQ.6) THEN
	    CHARBUF='[12 4] 0 setdash'
	    NA=16
	  ELSE IF (MODE.EQ.7) THEN
	    CHARBUF='[5 4 12 4] 0 setdash'
	    NA=20
	  ELSE
	    CHARBUF='[] 0 setdash'
	    NA=12
	  END IF
	  WRITE (53,1000) CHARBUF
C  Reestablish the path since "stroke" erases it
	  ENCODE(5,1005,CBUF1) IXOLD
	  ENCODE(5,1005,CBUF2) IYOLD
	  CHARBUF=CBUF1(1:5)//' nibs '//CBUF2(1:5)//' nibs moveto'
	  NA=28
	  WRITE (53,1000) CHARBUF
	  NCOUNT=0
C
C  Set the data level
C
	ELSE IF (IPEN.EQ.8) THEN
C
C  Go to the rectilinear mode
C
	ELSE IF (IPEN.EQ.9) THEN
	  POINTPLOT=.FALSE.
	  RECTILINEAR=.TRUE.
C
C  Go to the alphagraphics mode
C
	ELSE IF (IPEN.EQ.10) THEN
C
C  Reset the pen width.  WIDTH is measured in thousandths of an inch.
C
	ELSE IF (IPEN.EQ.12) THEN
C  Send a "stroke" to print any vectors with preexisting linewidths.
	  CHARBUF='stroke'
	  NA=6
	  WRITE (53,1000) CHARBUF
	  IF (WIDTH.EQ.0) THEN
C  Take the default if it was not given
	    WIDTH=20  ! Mills
	  END IF
	  ENCODE(4,1004,CBUF4) WIDTH
1004	  FORMAT(I4)
	  CHARBUF=CBUF4//' 0.072 mul setlinewidth'
	  NA=27
	  WRITE (53,1000) CHARBUF
C  Reestablish the path since "stroke" erases it
	  ENCODE(5,1005,CBUF1) IXOLD
	  ENCODE(5,1005,CBUF2) IYOLD
	  CHARBUF=CBUF1(1:5)//' nibs '//CBUF2(1:5)//' nibs moveto'
	  NA=28
	  WRITE (53,1000) CHARBUF
	  NCOUNT=0
C
C  Enable the page for printing
C
	ELSE IF (IPEN.EQ.15.AND..NOT.EPS) THEN
	  IF (NDEV.GT.0) THEN !  Not overlaying a TeX file
	    CHARBUF='stroke showpage'
	    NA=15
	  ELSE
	    CHARBUF='stroke'
	    NA=6
	  END IF
	  WRITE (53,1000) CHARBUF
	ELSE IF (IPEN.EQ.15.AND.EPS) THEN
	  CHARBUF='stroke'
	  NA=6
	  WRITE (53,1000) CHARBUF
	  CHARBUF='%%Trailer'
	  NA=9
	  WRITE (53,1000) CHARBUF
	  CHARBUF='%%Pages:  1'
	  NA=11
	  WRITE (53,1000) CHARBUF
	END IF
	IF (NCOUNT.GE.500) THEN  ! Flushes the buffer before it overflows
	  CHARBUF='stroke'
	  NA=6
	  WRITE (53,1000) CHARBUF
	  NCOUNT=0
C  Reestablish the path since "stroke" erases it
	  ENCODE(5,1005,CBUF1) IXOLD
	  ENCODE(5,1005,CBUF2) IYOLD
	  CHARBUF=CBUF1(1:5)//' nibs '//CBUF2(1:5)//' nibs moveto'
	  NA=28
	  WRITE (53,1000) CHARBUF
	END IF
	  
C
	RETURN
	END
