	SUBROUTINE AXIS(X,Y,TEXT,AXLEN,ANGLE,FIRSTV,DELTAV,
	1N_INTERVALS,N_INTERVALS_OFFSET_L,FLAG)
C
C  Note:  This is the Unix version
C
C			Donald H. Gudehus, September, 1982
C			Version 2, James W. Haywood, March, 1984
C			Modified calculation for numbers which use
C			exponential notation
C			Version 2U, Donald H. Gudehus, January 8, 2000
C			Modified code for Unix
C			Version 3, Donald H. Gudehus, May 3, 2000
C			Introducted N_INTERVALS_OFFSET_L
C			Version 4, D. H. G., February 28, 2004
C			Adjusted loop of with upper limit of N_INTERVALS+1
C			by subtracting N_INTERVALS_OFFSET_L.  This allowed
C			the offsets to be increased from 0.49 to 0.99
C
C	X		     (C) The starting x position of the axis measured in
C				 inches assuming that there is no inversion of 
C				 the axis.
C	Y		     (C) Like X but in the y direction
C	TEXT	 	     (C) The label for the axis.  If the first two 
C				 bytes are blank, the annotation will be 
C				 supressed.
C	AXLEN	 	     (C) The axis length in inches
C	ANGLE	 	     (C) The angle of the axis from the horizontal
C	FIRSTV	 	     (C) The printed value to appear at the first tick
C				 mark
C	DELTAV	 	     (C) The number of physical units per interval
C	N_INTERVALS	     (C) The total number of intervals per axis length, 
C				 including any fractional parts at the 
C				 beginning and end of the axis.
C	N_INTERVALS_OFFSET_L (C) The fractional number of intervals at the 
C				 beginning of the axis, before any annotation
C				 is written.
C	FLAG(1)		     (C) If +1, the tick marks appear on the counter-
C				 clockwise side of the axis;  if -1, they appear
C				 on the clockwise side.
C	FLAG(2)	 	     (C) Like FLAG(1), but for the label
C	FLAG(3)	 	     (C) If 0, the numbers by the tick marks are
C				 parallel to the axis;  if 1, they are
C				 perpendicular.
C	FLAG(4)	 	     (C) If 0, the numbers and label are omitted;  if 1,
C				 they are displayed
C       FLAG(5)	 	     (C) If 0, the axes are displayed normally;  if 1,
C				 they are inverted
C
	BYTE TEXT(1)
	INTEGER*2 FLAG(5)
	REAL X,Y,AXLEN,ANGLE,FIRSTV,DELTAV,N_INTERVALS,N_INTERVALS_OFFSET_L
C
	CHARACTER*4 CTEXT
	LOGICAL SUPRESS_ANNOTATION
	REAL RTEXT,CHARHT,NUMBHT
	REAL*8 AXISVAL,AXISDV
	EQUIVALENCE (RTEXT,CTEXT)
	INCLUDE 'plotpar.for'
C
	NUMBHT=0.11
	CHARHT=0.15
	SPACE=0.08  ! The space between ticks or the axis and the numbers
	TICKLENGTH=0.07
C  Establish a default line width to use.  This width is adjustable
C  by the PLOT command when plotting from a file.
	CALL PENMODE(20)  ! 20 thousandths of an inch
C
C  Find out if the annotation is to be supressed
C
	RTEXT=TEXT(1)
	IF ((CTEXT(1:1).EQ.' '.AND.CTEXT(2:2).EQ.' ').OR.
	1FLAG(4).EQ.0) THEN
	  SUPRESS_ANNOTATION=.TRUE.
	ELSE IF ((CTEXT(1:1).NE.' '.OR.CTEXT(2:2).NE.' ').OR.
	1FLAG(4).EQ.1) THEN
	  SUPRESS_ANNOTATION=.FALSE.
	END IF
C
C  Find which side of the axis the tick marks are to be produced
C
	SIDET=SIGN(1,FLAG(1))
C
C  Find which side of the axis the label and numbers are to be plotted.
C
	SIDEA=SIGN(1,FLAG(2))
	CALL ALPHA(TEXT,1,50,NC,NF,NR,1)
	CALL ALPHA(TEXT,1,50,NCLABEL,NF,NR,3)
C
C  Find DELTAV exponent
C
	AXISDV=ABS(DELTAV)
	IF (AXISDV.EQ.0.0) IEXP=0
	IF (AXISDV.GT.0.0.AND.AXISDV.LT.0.00099999) IEXP=
	1NINT(LOG10(AXISDV))
	IF (AXISDV.GE.0.00099999.AND.AXISDV.LT.100000.0) IEXP=0
	IF (AXISDV.GE.10000.0) IEXP=NINT(LOG10(AXISDV))
	WRITE(7,1000) IEXP,AXISDV
1000	FORMAT(I4,G14.6)
C
C  Normalize FIRSTV and DELTAV
C
	AXISVAL=FIRSTV*(10.0D0**(-IEXP))
	AXISDV=DELTAV*(10.0D0**(-IEXP))
C
C  Adjust IEXP, AXISVAL and AXISDV so that there is only one digit to
C  the left of the decimal point if an exponent is required
C
C	IF (IEXP.NE.0) THEN
C	  I=0
C	  LOOP=0
C	  DO WHILE (LOOP.EQ.0)
C	    IF (ABS(AXISVAL).LT.10.**(I-1)) THEN
C	      IEXP=IEXP-(2-I)
C	      AXISVAL=AXISVAL*10.**(2-I)
C	      AXISDV=AXISDV*10.**(2-I)
C	      LOOP=1
C	    END IF
C	    I=I+1
C	  END DO
C
C  Make sure that some numbers will not have two digits to the left
C  of the decimal point if an exponent is required
C
	IF (IEXP.NE.0) THEN
	  DELTA=ABS(AXISDV)*INT(AXLEN)
	  IF (DELTA.GE.10.) THEN
	    IEXP=IEXP+1
	    AXISVAL=AXISVAL*.1
	    AXISDV=AXISDV*.1
	  END IF
C
C  Make sure changes in the displayed numbers occur in at least the
C  thousandth's place if an exponent is required
C
	  DO WHILE (ABS(AXISDV).LT..001)
	    IEXP=IEXP-1
	    AXISVAL=AXISVAL*10.
	    AXISDV=AXISDV*10.
	  END DO
	END IF
C
C  Find sine and cosine of ANGLE
C
	SINA=SIND(ANGLE)
	COSA=COSD(ANGLE)
C
C  Get the offsets in the X and Y directions based on N_INTERVALS_OFFSET_L.
C  These are in units of inches
C
	X_OFFSET=N_INTERVALS_OFFSET_L*AXLEN/N_INTERVALS*COSA
	Y_OFFSET=N_INTERVALS_OFFSET_L*AXLEN/N_INTERVALS*SINA
C
C  Find out how many decimal places to use for the numbers
C
	FRACTAXISDV=ABS(AXISDV-NINT(AXISDV))
	FRACTAXISVAL=ABS(AXISVAL-NINT(AXISVAL))
	NDECI=0
	DO I=0,2
	  IF (ABS(10**I*FRACTAXISDV-NINT(10**I*FRACTAXISDV))
	1 .GT.1E-5*ABS(AXISDV).OR.
	2 ABS(10**I*FRACTAXISVAL-NINT(10**I*FRACTAXISVAL))
	3 .GT.1E-5*ABS(AXISVAL))
	4 NDECI=NDECI+1
	END DO
	IF (NDECI.EQ.0) NDECI=-1  ! Eliminates the decimal point
C	IF (IEXP.NE.0) NDECI=2
C
C  Get the incremental positions of the annotations from the axis.  The
C  notation is consistent with a horizontal axis.
C
	IF (.NOT.SUPRESS_ANNOTATION) THEN
	  DELTYNNORM=ABS(SIDET+SIDEA)/2*TICKLENGTH*SIDET+(0.5*NUMBHT+SPACE)*
	1 SIDEA-0.5*NUMBHT
	  DELTYLNORM=ABS(SIDET+SIDEA)/2*TICKLENGTH*SIDET+(0.5*CHARHT+NUMBHT+
	1 2.3*SPACE)*SIDEA-0.5*CHARHT
C  At this point in the program, only take account of the space used by the
C  decimal point and digits to the right of the decimal point.
	  IF (FLAG(3).EQ.1) THEN  ! Numbers perpendicular
	    IF (FLAG(2).EQ.1) THEN  ! Annotation on counterclockwise side
	      DELTX=-0.5*NUMBHT
	      DELTYN=DELTYNNORM+0.836*NUMBHT*(NDECI+1.8)
	      DELTYL=DELTYLNORM+0.836*NUMBHT*(NDECI+1)
	    ELSE
	      DELTX=-0.5*NUMBHT
	      DELTYN=DELTYNNORM+0.836*NUMBHT
	      DELTYL=DELTYLNORM-0.836*NUMBHT*(NDECI+1)
	    END IF
	  ELSE
	    DELTYN=DELTYNNORM
	    DELTYL=DELTYLNORM
	  END IF
C
C  Annotate the axis
C
	  CALL COLORMODE(16766720) !  Gold1
C  For the special case of FLAG(2)=-1 and FLAG(3)=1, we must determine the
C  the maximum number of digits which will be used before writing any numbers
C  in loop 300 below.
	  IF (FLAG(2).EQ.-1.AND.FLAG(3).EQ.1) THEN
	    DELTY2SPECIALMAX=0.0
	    AXISVALSPECIAL=AXISVAL
	    DO I=1,N_INTERVALS+1
	      IF (ABS(AXISVALSPECIAL).GT.0.1) THEN
	        NLEFT=INT(LOG10(ABS(AXISVALSPECIAL)))+1
	      ELSE
	        NLEFT=1
	      END IF
	      IF (AXISVALSPECIAL.LT.0.0) NLEFT=NLEFT+1
	      DELTY2=(NLEFT-1)*NUMBHT*0.836
	      DELTY2SPECIALMAX=MAX(DELTY2SPECIALMAX,DELTY2)
	      AXISVALSPECIAL=AXISVALSPECIAL+AXISDV
	    END DO
	  END IF
C
	  DELTY2MAX=0.0
	  DO 300 I=1,N_INTERVALS-N_INTERVALS_OFFSET_L+1
C  Make a zero value for AXISVAL not subject to roundoff errors
	    IF (ABS(AXISVAL).LT.0.0001) AXISVAL=0.0
C  Adjust for the number of places to the left of the decimal point
C  when the numbers are perpendicular to the axis and on the
C  counterclockwise side.  We also
C  find the maximum shift left for all the numbers in order
C  to position the label correctly.
	    IF (FLAG(3).EQ.1) THEN  ! Numbers perpendicular
	      IF (ABS(AXISVAL).GT.0.1) THEN
	        NLEFT=INT(LOG10(ABS(AXISVAL*1.000001)))+1
			! Avoids roundoff error
	      ELSE
	        NLEFT=1
	      END IF
	      IF (AXISVAL.LT.0.0) NLEFT=NLEFT+1
	      DELTY2=(NLEFT-1)*NUMBHT*0.836
	    ELSE
	      IF (ABS(AXISVAL).GT.0.1) THEN
	        NDIGITS=INT(LOG10(ABS(AXISVAL)))+2+NDECI
	      ELSE
	        NDIGITS=2+NDECI
	      END IF
	      IF (AXISVAL.LT.0.0) NDIGITS=NDIGITS+1
	      DELTX=(0.3-NDIGITS)*NUMBHT*0.5*0.836
	      DELTY2=0.0
	    END IF
	    DELTY2MAX=MAX(DELTY2MAX,DELTY2)
	    IF (FLAG(2).EQ.1) THEN
	      DELTY2N=DELTYN+DELTY2
	    ELSE
	      IF (FLAG(3).EQ.1) THEN
	        DELTY2N=DELTYN-(DELTY2SPECIALMAX-DELTY2)
	      ELSE
	        DELTY2N=DELTYN
	      END IF
	    END IF
C  Compute the locations for each number
	    IF (FLAG(5).EQ.0) THEN
	      XPOS=X+X_OFFSET+DELTX*COSA-DELTY2N*SINA+
	1     (I-1)*AXLEN/(N_INTERVALS)*COSA
	      YPOS=Y+Y_OFFSET+DELTY2N*COSA+DELTX*SINA+
	1     (I-1)*AXLEN/(N_INTERVALS)*SINA
	    ELSE IF (FLAG(5).EQ.1) THEN
	      XPOS=X-X_OFFSET+DELTX*COSA-DELTY2N*SINA+AXLEN*COSA-
	1     (I-1)*AXLEN/N_INTERVALS*COSA
	      YPOS=Y-Y_OFFSET+DELTY2N*COSA+DELTX*SINA+AXLEN*SINA-
	1     (I-1)*AXLEN/N_INTERVALS*SINA
	    END IF
C
	    IF (FLAG(3).EQ.0) THEN
	      CALL NUMBER(XPOS,YPOS,NUMBHT,AXISVAL,ANGLE,NDECI)
	    ELSE IF (FLAG(3).EQ.1) THEN
	      CALL NUMBER(XPOS,YPOS,NUMBHT,AXISVAL,ANGLE-90.0,NDECI)
	    END IF
	    AXISVAL=AXISVAL+AXISDV
 300	  CONTINUE
	  DELTYL=DELTYL+SIDEA*DELTY2MAX
C
C  Label the axis
	  CALL COLORMODE(65280)  ! Green
	  IF (IEXP.EQ.0) THEN
	    DELTX=0.5*AXLEN-0.5*0.825*CHARHT*NCLABEL
	  ELSE
	    DELTX=0.5*AXLEN-0.5*0.825*CHARHT*(NCLABEL+6)
	  END IF
	  XPOS=X+DELTX*COSA-DELTYL*SINA
	  YPOS=Y+DELTYL*COSA+DELTX*SINA
	  CALL SYMBOL(XPOS,YPOS,CHARHT,TEXT,ANGLE,NC)
	  IF (IEXP.GT.0) THEN
	    CALL SYMBOL(999.,999.,CHARHT,%REF(' / 10'),ANGLE,5)
	    XPOS=XPOS+((NCLABEL+5)*COSA-0.8*SINA)*0.845*CHARHT
	    YPOS=YPOS+((NCLABEL+5)*SINA+0.8*COSA)*0.845*CHARHT
	    CALL NUMBER(XPOS,YPOS,0.07,REAL(IEXP)*1.0D0,ANGLE,-1)
	  ELSE IF (IEXP.LT.0) THEN
	    IEXP=-IEXP
	    CALL SYMBOL(999.,999.,CHARHT,%REF(' * 10'),ANGLE,5)
	    XPOS=XPOS+((NCLABEL+5)*COSA-0.8*SINA)*0.845*CHARHT
	    YPOS=YPOS+((NCLABEL+5)*SINA+0.8*COSA)*0.845*CHARHT
	    CALL NUMBER(XPOS,YPOS,0.07,REAL(IEXP)*1.0D0,ANGLE,-1)
	  END IF
 	END IF !  Ends block for annotation
C
C  Draw the axis and ticks
C
	CALL COLORMODE(255)  ! Blue
	DELTX=-TICKLENGTH*SIDET*SINA
	DELTY=TICKLENGTH*SIDET*COSA
	IF (N_INTERVALS_OFFSET_L.GT.0.0) THEN
	  IF (FLAG(5).EQ.0) THEN
	    CALL PLOT(X,Y,3)
	    CALL PLOT(X+X_OFFSET,Y+Y_OFFSET,2)
	  ELSE IF (FLAG(5).EQ.1) THEN
	    CALL PLOT(X+AXLEN*COSA,Y+AXLEN*SINA,3)
	    CALL PLOT(X+AXLEN*COSA-X_OFFSET,Y+AXLEN*SINA-Y_OFFSET,2)
	  END IF
	END IF
	IF (FLAG(5).EQ.0) THEN
	  CALL PLOT(X+DELTX+X_OFFSET,Y+Y_OFFSET+DELTY,3)
	  CALL PLOT(X+X_OFFSET,Y+Y_OFFSET,2)
	ELSE IF (FLAG(5).EQ.1) THEN
	  CALL PLOT(X+AXLEN*COSA-X_OFFSET+DELTX,
	1 Y+AXLEN*SINA-Y_OFFSET+DELTY,3)
	  CALL PLOT(X+AXLEN*COSA-X_OFFSET,Y+AXLEN*SINA-Y_OFFSET,2)
	END IF
	DO 400 I=2,(N_INTERVALS-N_INTERVALS_OFFSET_L)+1
	  IF (FLAG(5).EQ.0) THEN
	    XPOS=X+X_OFFSET+(I-1)*COSA*AXLEN/N_INTERVALS
	    YPOS=Y+Y_OFFSET+(I-1)*SINA*AXLEN/N_INTERVALS
	  ELSE IF (FLAG(5).EQ.1) THEN
	    XPOS=X+AXLEN*COSA-X_OFFSET-(I-1)*COSA*AXLEN/N_INTERVALS
	    YPOS=Y+AXLEN*SINA-Y_OFFSET-(I-1)*SINA*AXLEN/N_INTERVALS
	  END IF
	  CALL PLOT(XPOS,YPOS,2)
	  CALL PLOT(XPOS+DELTX,YPOS+DELTY,2)
	  CALL PLOT(XPOS,YPOS,3)
400	CONTINUE
C  Finish drawing the axis if N_INTERVALS was not an integer
	IF (FLAG(5).EQ.0) THEN !  Normal
	  IF (SQRT((XPOS-X)**2+(YPOS-Y)**2).LT.AXLEN) THEN
	    XPOS=X+AXLEN*COSA
	    YPOS=Y+AXLEN*SINA
	    CALL PLOT(XPOS,YPOS,2)
	  END IF
	ELSE IF (FLAG(5).EQ.1) THEN !  Inverted
	  IF (SQRT((AXLEN*COSA+X-XPOS)**2+(AXLEN*SINA+Y-YPOS)**2)
	1 .LT.AXLEN) THEN
	    XPOS=X
	    YPOS=Y
	    CALL PLOT(XPOS,YPOS,2)
	  END IF
	END IF
	RETURN
	END
