	SUBROUTINE ERRORBAR(XARRAY,YARRAY,SIZE,NPOINTS,INC,
	1XOFFSET,YOFFSET,XAXIS,YAXIS,X_INTERVALS,Y_INTERVALS,
	2X_INTRVLS_OFF,Y_INTRVLS_OFF,NSYMBOL,INVERT)
C
C  Note:  This is the Unix version.
C
C       This subroutine draws error bars or residual vectors on a plot
C  device.  The data for the locations of the ends of the lines are arranged
C  in pairs, stored in array locations which differ by one in the second index
C  in XARRAY and YARRAY.  Alternately, if NSYMBOL contains a flag value, i.e.,
C  100 added to the previous value, then instead of pairs of end-point
C  locations, one can use a central location and deviation.  For example,
C  for a vertical errorbar located at (1.0,2.0) with a deviation of 0.1 we 
C  could have XARRAY(1,1)=1.0, YARRAY(1,1)=2.0, XARRAY(1,2)=1.0, 
C  YARRAY(1,2)=0.1.  Note that XARRAY(1,2) can either repeat the value for
C  XARRAY(1,1) or be set equal to zero.  The increment mode can only be used
C  for drawing errorbars, not residual vectors.  This subroutine is called 
C  by XYPLOT.
C
C				Donald H. Gudehus, October, 1984
C			Version 2, Donald H. Gudehus, May, 1995
C			Guarded against invalid argument to math library
C			function ATAN2D.
C			Version 3, Donald H. Gudehus, January 11, 2000
C			Removed LU argument.
C			Version 3U, Donald H. Gudehus, January 11, 2000
C			Modified code for Unix.
C			Version 4, Donald H. Gudehus, February 19, 2000
C			Put in check for END value.
C			Version 5, Donald H. Gudehus, February 24, 2000
C			Introduced increment method of drawing the bars
C			Version 6, D. H. G., January 1, 2003
C			Introduced X_INTRVLS_OFF and Y_INTRVLS_OFF
C			Version 7, D. H. G., September 4, 2004
C			Modified code to better handle cases of errorbar data
C			being present in form of increments
C			Version 8, D. H. G., March 2, 2008
C			Added check for case of sigma = 0.
C
C	XARRAY	      (C)  The array containing either the X positions of the
C			   ends of the bar, or the central position and the
C			   deviation measured in physical value
C			   for the end points of the error bars.
C	YARRAY	      (C)  Like XARRAY, but for the Y positions.
C	SIZE	      (C)  The size of each of the above arrays.
C			   This should be at least two larger than the
C			   maximum number of points in the array.
C	NPOINTS	      (C)  The number of data points in the arrays for each
C			   plot including points skipped over if INC>1
C	INC	      (C)  An array containing the increment to be used in
C			   stepping through XARRAY and YARRAY when plotting
C	XOFFSET	      (C)  The starting X location on the plot device
C			   measured in inches.
C	YOFFSET	      (C)  The starting Y location on the plot device
C			   measured in inches.
C	XAXIS	      (C)  The length of the x axis in inches
C	YAXIS	      (C)  The length of the y axis in inches
C	X_INTERVALS   (C)  The number of tick mark intervals along the X
C			   axis, including any offsets at the start and end
C	Y_INTERVALS   (C)  Like X_INTERVALS but along the Y axis
C	X_INTRVLS_OFF (C)  The fractional offset at the X axis start
C			   measured in units of intervals
C	Y_INTRVLS_OFF (C)  The fractional offset at the Y axis start
C	NSYMBOL	      (C)  An array which in the first byte gives the style
C		           for each end of the error bar.  A zero gives
C			   the usual short tick at that end;  a +1
C			   gives nothing at that end;  a +2 gives an arrow-
C			   head at that end.  If 100 is added to the first
C			   byte, the X or Y locations contain the central
C			   location and the deviation to the low and
C			   high directions, respectively, for the
C			   adjacent array locations.
C	INVERT	      (C)  A two dimensional array determining whether
C			   the X and/or Y coordinates are to be inverted.
C
	INTEGER SIZE
 	DIMENSION XARRAY(SIZE,2),YARRAY(SIZE,2),
	1NPOINTS(2),INC(2),NSYMBOL(2)
	LOGICAL INVERT(2)
C
	CHARACTER*1 BAR_END(2),ORIENTATION
	CHARACTER*9  METHOD
	LOGICAL OUTSIDE_S,OUTSIDE_F
	REAL LENGTH
	INCLUDE 'plotpar.for'
	DATA END/'FF7FFFFF'X/
C
	LENGTH=0.06  ! Length of arrowhead
C
	KEY=PLT$M_FIRSTBYTE.AND.NSYMBOL(1)
	IF (KEY.GE.100) THEN
	  METHOD='INCREMENT'
	  KEY=KEY-100
	ELSE 
	  METHOD='ABSOLUTE'
	END IF
	IF (KEY.EQ.0) THEN
	  BAR_END(1)='T' !  Tick
	ELSE IF (KEY.EQ.1) THEN
	  BAR_END(1)='N' !  Nothing
	ELSE IF (KEY.EQ.2) THEN
	  BAR_END(1)='A' !  Arrowhead
	END IF
	KEY=PLT$M_FIRSTBYTE.AND.NSYMBOL(2)
	IF (KEY.GE.100) KEY=KEY-100
	IF (KEY.EQ.0) THEN
	  BAR_END(2)='T' !  Tick
	ELSE IF (KEY.EQ.1) THEN
	  BAR_END(2)='N' !  Nothing
	ELSE IF (KEY.EQ.2) THEN
	  BAR_END(2)='A' !  Arrowhead
	END IF
C
	DO N=1,2
	  IF (NPOINTS(N).LE.0) THEN
	    CALL PRTMSG(PLT$S_NOPOINTS,'F'X)
	    RETURN
	  END IF
	END DO
	XSTARTVAL=XARRAY(SIZE-1,1)
	YSTARTVAL=YARRAY(SIZE-1,1)
	XDV=XARRAY(SIZE,1)*X_INTERVALS/XAXIS
	YDV=YARRAY(SIZE,1)*Y_INTERVALS/YAXIS
	DO INDEX=1,NPOINTS(1),INC(1)
	  IF (XARRAY(INDEX,1).EQ.END.OR.XARRAY(INDEX,2).EQ.END.OR.
	1 YARRAY(INDEX,1).EQ.END.OR.YARRAY(INDEX,2).EQ.END) GO TO 300
	  DELTAX=0.0
	  DELTAY=0.0
	  IF (METHOD.EQ.'ABSOLUTE') THEN
	    IF (XARRAY(INDEX,1).EQ.XARRAY(INDEX,2)) THEN  ! Vertical
	      DELTAX=0.02
	      ORIENTATION='V'
	    ELSE IF (YARRAY(INDEX,1).EQ.YARRAY(INDEX,2)) THEN !  Horizontal
	      DELTAY=0.02
	      ORIENTATION='H'
	    END IF
	  ELSE IF (METHOD.EQ.'INCREMENT') THEN
	    IF (XARRAY(INDEX,2).EQ.0.0.AND.YARRAY(INDEX,2).NE.0.0) THEN
	      DELTAX=0.02
	      ORIENTATION='V'
	    ELSE IF (YARRAY(INDEX,2).EQ.0.0.AND.XARRAY(INDEX,2).NE.0.0) THEN 
	      DELTAY=0.02
	      ORIENTATION='H'
	    END IF
C  Original older way of designating errorbars when using increments
	    IF (XARRAY(INDEX,1).EQ.XARRAY(INDEX,2).AND.
	1   YARRAY(INDEX,2).NE.0.0) THEN
	      DELTAX=0.02
	      ORIENTATION='V'
	    ELSE IF (YARRAY(INDEX,1).EQ.YARRAY(INDEX,2).AND.
	1   XARRAY(INDEX,2).NE.0.0) THEN
	      DELTAY=0.02
	      ORIENTATION='H'
	    END IF
	  END IF
C  Get the locations of the error bars
	  X1=XARRAY(INDEX,1)
	  IF (METHOD.EQ.'INCREMENT'.AND.ORIENTATION.EQ.'H')
	1 X1=X1+XARRAY(INDEX,2)
	  Y1=YARRAY(INDEX,1)
	  IF (METHOD.EQ.'INCREMENT'.AND.ORIENTATION.EQ.'V')
	1 Y1=Y1+YARRAY(INDEX,2)
	  X2=XARRAY(INDEX,2)
	  IF (METHOD.EQ.'INCREMENT'.AND.ORIENTATION.EQ.'H')
	1 X2=X1-2*XARRAY(INDEX,2)
	  Y2=YARRAY(INDEX,2)
	  IF (METHOD.EQ.'INCREMENT'.AND.ORIENTATION.EQ.'V')
	1 Y2=Y1-2*YARRAY(INDEX,2)
C  Check for an impossible case.  This can arise if a sigma of 0
C  is encountered is one coordinate.
	  IF (X2.EQ.0.0.AND.Y2.EQ.0.0) RETURN
C  Handle cases where XARRAY(INDEX,2) or YARRAY(INDEX,2) could be 0.
	  IF (METHOD.EQ.'INCREMENT'.AND.ORIENTATION.EQ.'H') THEN
	    Y2=YARRAY(INDEX,1)
	  ELSE IF (METHOD.EQ.'INCREMENT'.AND.ORIENTATION.EQ.'V') THEN
	    X2=XARRAY(INDEX,1)
	  END IF
C
	  IF (INVERT(1)) THEN
	    X1=XAXIS-(X1-XSTARTVAL)/XDV+XOFFSET-X_INTRVLS_OFF*XAXIS/
	1   X_INTERVALS
	    X2=XAXIS-(X2-XSTARTVAL)/XDV+XOFFSET-X_INTRVLS_OFF*XAXIS/
	1   X_INTERVALS
	  ELSE
	    X1=(X1-XSTARTVAL)/XDV+XOFFSET+X_INTRVLS_OFF*XAXIS/
	1   X_INTERVALS
	    X2=(X2-XSTARTVAL)/XDV+XOFFSET+X_INTRVLS_OFF*XAXIS/
	1   X_INTERVALS
	  END IF
	  IF (INVERT(2)) THEN
	    Y1=YAXIS-(Y1-YSTARTVAL)/YDV+YOFFSET-Y_INTRVLS_OFF*YAXIS/
	1   Y_INTERVALS
	    Y2=YAXIS-(Y2-YSTARTVAL)/YDV+YOFFSET-Y_INTRVLS_OFF*YAXIS/
	1   Y_INTERVALS
	  ELSE
	    Y1=(Y1-YSTARTVAL)/YDV+YOFFSET+Y_INTRVLS_OFF*YAXIS/
	1   Y_INTERVALS
	    Y2=(Y2-YSTARTVAL)/YDV+YOFFSET+Y_INTRVLS_OFF*YAXIS/
	1   Y_INTERVALS
	  END IF
	  Y12=Y1-Y2
C  We must guard against 0/0 in the call to the ATAN2D function
	  IF (Y12.NE.0.0) THEN
	    ANGLE1=ATAN2D(Y1-Y2,X1-X2)
	  ELSE
	    ANGLE1=0.0
	  END IF
	  ANGLE2=ANGLE1+180.0
C  Draw the tick mark or arrowhead at the starting end
	  OUTSIDE_S=.FALSE.
	  IF (X1-DELTAX.GT.XAXIS+XOFFSET) OUTSIDE_S=.TRUE.
	  IF (X1-DELTAX.LT.XOFFSET) OUTSIDE_S=.TRUE.
	  IF (Y1-DELTAY.GT.YAXIS+YOFFSET) OUTSIDE_S=.TRUE.
	  IF (Y1-DELTAY.LT.YOFFSET) OUTSIDE_S=.TRUE.
	  CALL PLOT(MAX(XOFFSET,MIN(XAXIS+XOFFSET,X1-DELTAX)),
	1 MAX(YOFFSET,MIN(YAXIS+YOFFSET,Y1-DELTAY)),3)
	  OUTSIDE_F=.FALSE.
	  IF (X1+DELTAX.GT.XAXIS+XOFFSET) OUTSIDE_F=.TRUE.
	  IF (X1+DELTAX.LT.XOFFSET) OUTSIDE_F=.TRUE.
	  IF (Y1+DELTAY.GT.YAXIS+YOFFSET) OUTSIDE_F=.TRUE.
	  IF (Y1+DELTAY.LT.YOFFSET) OUTSIDE_F=.TRUE.
	  IF (.NOT.(OUTSIDE_S.AND.OUTSIDE_F)) THEN
	    IF (BAR_END(1).EQ.'T') THEN
	      CALL PLOT(MAX(XOFFSET,MIN(XAXIS+XOFFSET,X1+DELTAX)),
	1     MAX(YOFFSET,MIN(YAXIS+YOFFSET,Y1+DELTAY)),2)
	    ELSE IF (BAR_END(1).EQ.'A') THEN
	      CALL ARROWHEAD(X1,Y1,ANGLE1,LENGTH)
	    END IF
	  END IF
C  Draw the bar
	  OUTSIDE_S=.FALSE.
	  IF (X1.GT.XAXIS+XOFFSET) OUTSIDE_S=.TRUE.
	  IF (X1.LT.XOFFSET) OUTSIDE_S=.TRUE.
	  IF (Y1.GT.YAXIS+YOFFSET) OUTSIDE_S=.TRUE.
	  IF (Y1.LT.YOFFSET) OUTSIDE_S=.TRUE.
	  CALL PLOT(MAX(XOFFSET,MIN(XAXIS+XOFFSET,X1)),
	1 MAX(YOFFSET,MIN(YAXIS+YOFFSET,Y1)),3)
C  Now go to the other end
	  OUTSIDE_F=.FALSE.
	  IF (X2.GT.XAXIS+XOFFSET) OUTSIDE_F=.TRUE.
	  IF (X2.LT.XOFFSET) OUTSIDE_F=.TRUE.
	  IF (Y2.GT.YAXIS+YOFFSET) OUTSIDE_F=.TRUE.
	  IF (Y2.LT.YOFFSET) OUTSIDE_F=.TRUE.
	  IF (.NOT.(OUTSIDE_S.AND.OUTSIDE_F)) THEN
	    CALL PLOT(MAX(XOFFSET,MIN(XAXIS+XOFFSET,X2)),
	1   MAX(YOFFSET,MIN(YAXIS+YOFFSET,Y2)),2)
	  END IF
C  Draw the tick mark or arrowhead
	  OUTSIDE_S=.FALSE.
	  IF (X2-DELTAX.GT.XAXIS+XOFFSET) OUTSIDE_S=.TRUE.
	  IF (X2-DELTAX.LT.XOFFSET) OUTSIDE_S=.TRUE.
	  IF (Y2-DELTAY.GT.YAXIS+YOFFSET) OUTSIDE_S=.TRUE.
	  IF (Y2-DELTAY.LT.YOFFSET) OUTSIDE_S=.TRUE.
	  CALL PLOT(MAX(XOFFSET,MIN(XAXIS+XOFFSET,X2-DELTAX)),
	1 MAX(YOFFSET,MIN(YAXIS+YOFFSET,Y2-DELTAY)),3)
	  OUTSIDE_F=.FALSE.
	  IF (X2+DELTAX.GT.XAXIS+XOFFSET) OUTSIDE_F=.TRUE.
	  IF (X2+DELTAX.LT.XOFFSET) OUTSIDE_F=.TRUE.
	  IF (Y2+DELTAY.GT.YAXIS+YOFFSET) OUTSIDE_F=.TRUE.
	  IF (Y2+DELTAY.LT.YOFFSET) OUTSIDE_F=.TRUE.
	  IF (.NOT.(OUTSIDE_S.AND.OUTSIDE_F)) THEN
	    IF (BAR_END(2).EQ.'T') THEN
	      CALL PLOT(MAX(XOFFSET,MIN(XAXIS+XOFFSET,X2+DELTAX)),
	1     MAX(YOFFSET,MIN(YAXIS+YOFFSET,Y2+DELTAY)),2)
	    ELSE IF (BAR_END(2).EQ.'A') THEN
	      CALL ARROWHEAD(X2,Y2,ANGLE2,LENGTH)
	    END IF
	  END IF
 300	END DO
	RETURN
	END
