	SUBROUTINE HISTOGRAM(XARRAY,YARRAY,SIZE,NPLOTS,ISTART,NPOINTS,
	1LINVECT,INC,BARTYPE,NFILL,INVERT,COLOR,XOFFSET,YOFFSET,
	2XAXIS,YAXIS,X_INTERVALS,Y_INTERVALS,X_INTERVALS_OFF,
	3Y_INTERVALS_OFF)
C
C  Note:  This is the Unix version
C
C       This subroutine plots one or more  histograms on a plot device.
C  It is called by XYPLOT.  Either vertically stacked or horizontally
C  displaced bars can be drawn.
C				Donald H. Gudehus, May, 1984
C			Version 2, Donald Gudehus, January 8, 2000
C			Removed LU argument
C			Version 2U, Donald H. Gudehus, January 22, 2000
C			Modified code for Unix
C			Version 3, Donald Gudehus, January 11, 2000
C			Declared HNUM as INTEGER
C			Version 4, Donald H. Gudehus, May 6, 2000
C			Introduced X_INTERVALS_OFF and Y_INTERVALS_OFF
C
C	XARRAY		(C)  The array containing the X positions for the
C			     center of each bar for each histogram.  The
C			     X coordinates for any curves to be plotted
C			     by XYPLOT are also in this array.
C	YARRAY		(C)  The array containing the Y positions
C	SIZE		(C)  The size of each of the above arrays.
C			     This should be at least four larger than the
C			     maximum number of points in the array.
C	NPLOTS		(C)  The number of plots to be made, including
C			     curves and histograms, and one of the dimensions
C			     of XARRAY and YARRAY
C	ISTART		(C)  The starting index in XARRAY and YARRAY for
C			     data to be plotted.
C	NPOINTS		(C)  The number of data points in the arrays for each
C			     plot including points skipped over if INC>1 and
C			     starting at the index ISTART and ending at
C			     ISTART+NPOINTS
C	LINVECT		(C)  An array giving the vector type for each plot.
C	INC		(C)  An array containing the increment to be used in
C			     stepping through XARRAY and YARRAY when plotting
C	BARTYPE		(C)  An array controlling the type of bar to be
C			     drawn for each histogram.  If equal to 998, the
C			     bars are stacked vertically, and if equal to
C			     999, they are placed next to each other in the
C			     horizontal direction.
C	NFILL		(C)  The number for the filling style for each plot
C			     (on a Visual 550)
C	INVERT		(C)  A two dimensional array determining whether
C			     the X and/or Y coordinates are to be inverted.
C	COLOR		(C)  An array containing the color for each plot.  If
C			     COLOR(I)=0, that histogram is not drawn.
C	XOFFSET		(C)  The starting X location on the plot device
C			     measured in inches.
C	YOFFSET		(C)  The starting Y location on the plot device
C			     measured in inches.
C	XAXIS		(C)  The length of the x axis in inches
C	YAXIS		(C)  The length of the y axis in inches
C	X_INTERVALS	(C)  The number of tick mark intervals along the X
C			     axis
C	Y_INTERVALS	(C)  Like X_INTERVALS but along the Y axis
C	X_INTERVALS_OFF	(C)  The fractional offset at the X axis start
C			     measured in units of intervals
C	Y_INTERVALS_OFF (C)  The fractional offset at the Y axis start
C
	INTEGER SIZE,BARTYPE
 	DIMENSION XARRAY(SIZE,NPLOTS),YARRAY(SIZE,NPLOTS),
	1NPOINTS(NPLOTS),LINVECT(NPLOTS),INC(NPLOTS),BARTYPE(NPLOTS),
	2NFILL(NPLOTS)
	LOGICAL INVERT(2)
C
	INTEGER COLOR(NPLOTS)
C
	INTEGER HINDEX(50),HNUM
	COMMON /PLOT_SCRATCH_BLOCK/ HINDEX
C
	INCLUDE 'plotpar.for'
C
C	Y, YLOW  Measured in physical units
C	YLOWIN, YHIGHIN, YLAST   Measured in inches
C
	DO I=1,NPLOTS
	  IF (BARTYPE(I).EQ.998.OR.BARTYPE(I).EQ.999) THEN
	    IF (NPOINTS(I).LT.1) THEN
	      CALL PRTMSG(PLT$S_NOPOINTS,'F'X)
	      RETURN
	    END IF
	  END IF
	END DO
C
	DO N=1,50
	  J=ISETEND(HINDEX(N))
	END DO
	HNUM=0
	DO N=1,NPLOTS
	  IF(BARTYPE(N).EQ.998.OR.BARTYPE(N).EQ.999) THEN
	    HNUM=HNUM+1
	    HINDEX(HNUM)=N
	  END IF
	END DO
	NHISTOGRAMS=INEL(HINDEX,50)
	DO N=1,NHISTOGRAMS
	  CALL COLORMODE(COLOR(HINDEX(N)))
	  IF (COLOR(HINDEX(N)).NE.0) THEN
	    DELTALEFT=0.5*XAXIS  ! Default in case only one point exists
	    XSTARTVAL=XARRAY(SIZE-1,HINDEX(N))
	    YSTARTVAL=YARRAY(SIZE-1,HINDEX(N))
C  Get the physical increment/inch
	    XDV=XARRAY(SIZE,HINDEX(N))*X_INTERVALS/XAXIS
	    YDV=YARRAY(SIZE,HINDEX(N))*Y_INTERVALS/YAXIS
C  The comments below which explain the operation of the plotting are
C  based on the presumption of no inversion of axes.
C  Send filled rectangles if the device is the Visual 550 or the
C  Grinnell;  the VT100 will ignore the commands.
	    DO INDEX=ISTART,NPOINTS(HINDEX(N))+ISTART-1,INC(HINDEX(N))
C  Send a filled rectangle (for the Visual 550)
C  Get the half-width of the histogram bar
	      IF (INDEX-ISTART+1.NE.NPOINTS(HINDEX(N))-MOD(NPOINTS
	1     (HINDEX(N)),INC(HINDEX(N)))) THEN
	        DELTARIGHT=0.5*(XARRAY(INDEX+INC(HINDEX(N)),HINDEX(N))
	1       -XARRAY(INDEX,HINDEX(N)))
	      ELSE  ! The last bar
	        DELTARIGHT=DELTALEFT998  ! Uses the last value
	      END IF
	      IF (INDEX-ISTART+1.NE.1) THEN
	        DELTALEFT=PRIORDELTARIGHT998
	      ELSE
	        DELTALEFT=DELTARIGHT
	      END IF
	      DELTARIGHT998=DELTARIGHT
	      DELTALEFT998=DELTALEFT
	      IF (BARTYPE(HINDEX(N)).EQ.999) THEN
	        DELTARIGHT=0.8*DELTARIGHT  ! Leaves a gap between bar
					   ! groups
	        DELTALEFT=0.8*DELTALEFT
	        DELTALR=DELTALEFT+DELTARIGHT
	        DELTARIGHT=-DELTALEFT+FLOAT(N)/NHISTOGRAMS*DELTALR
	        DELTALEFT=DELTALEFT-FLOAT(N-1)/NHISTOGRAMS*DELTALR
	      END IF
	      IF (INVERT(1)) THEN
	        X=XAXIS-(XARRAY(INDEX,HINDEX(N))-XSTARTVAL-DELTALEFT)/XDV+
	1       XOFFSET-X_INTERVALS_OFF*XAXIS/X_INTERVALS
	      ELSE
	        X=(XARRAY(INDEX,HINDEX(N))-XSTARTVAL-DELTALEFT)/XDV+
	1       XOFFSET+X_INTERVALS_OFF*XAXIS/X_INTERVALS
	      END IF
C  YLOW is the bottom Y coordinate of the rectangles for each plot
C  measured in physical value
	      IF (BARTYPE(HINDEX(N)).EQ.998) THEN
	        YLOW=0.0
	        DO NN=1,N-1
	          YLOW=YLOW+YARRAY(INDEX,HINDEX(NN))
	        END DO
	      ELSE IF (BARTYPE(HINDEX(N)).EQ.999) THEN
	        YLOW=0.0
	      END IF
	      Y=(YLOW-YSTARTVAL)/YDV
	      IF (INVERT(2)) THEN
	        Y=YAXIS-Y+YOFFSET-Y_INTERVALS_OFF*YAXIS/Y_INTERVALS
	      ELSE
	        Y=Y+YOFFSET+Y_INTERVALS_OFF*YAXIS/Y_INTERVALS
	      END IF
	      XLEFTIN=MIN(X,XAXIS+XOFFSET)
	      XLEFTIN=MAX(XLEFTIN,XOFFSET)
	      YLOWIN=MIN(Y,YAXIS+YOFFSET)
	      YLOWIN=MAX(YLOWIN,YOFFSET)
	      X=(XARRAY(INDEX,HINDEX(N))-XSTARTVAL+DELTARIGHT)/XDV
	      IF (INVERT(1)) THEN
	        X=XAXIS-X+XOFFSET-X_INTERVALS_OFF*XAXIS/X_INTERVALS
	      ELSE
	        X=X+XOFFSET+X_INTERVALS_OFF*XAXIS/X_INTERVALS
	      END IF
	      IF (BARTYPE(HINDEX(N)).EQ.998) THEN
	        YHIGH=0
	        DO NN=1,N
	          YHIGH=YHIGH+YARRAY(INDEX,HINDEX(NN))
	        END DO
	      ELSE IF (BARTYPE(HINDEX(N)).EQ.999) THEN
	        YHIGH=YARRAY(INDEX,HINDEX(N))
	      END IF
	      Y=(YHIGH-YSTARTVAL)/YDV
	      IF (INVERT(2)) THEN
	        Y=YAXIS-Y+YOFFSET-Y_INTERVALS_OFF*YAXIS/Y_INTERVALS
	      ELSE
	        Y=Y+YOFFSET+Y_INTERVALS_OFF*YAXIS/Y_INTERVALS
	      END IF
	      XRIGHTIN=MIN(X,XAXIS+XOFFSET)
	      XRIGHTIN=MAX(XRIGHTIN,XOFFSET)
	      YHIGHIN=MIN(Y,YAXIS+YOFFSET)
	      YHIGHIN=MAX(YHIGHIN,YOFFSET)
	      IF (YHIGHIN.NE.YLOWIN) THEN
	        CALL PLOT(XLEFTIN,YLOWIN,3)  ! Lower left
	        CALL FILLMODE(NFILL(HINDEX(N)))
	        CALL PLOT(0.0,0.0,9)  ! Activates rectilinear
				      ! graphics
	        CALL PLOT(XRIGHTIN,YHIGHIN,2)  ! Upper right
	      END IF
	      PRIORDELTARIGHT998=DELTARIGHT998
	    END DO
	  END IF
C  Flush the buffer for Xterm and Xgterm
	  CALL PLOT(0.0,0.0,18)
	END DO
C  Draw an unfilled rectangle
	DO N=1,NHISTOGRAMS
	  CALL NEWPEN
	  CALL VECTMODE(PLT$M_FIRSTBYTE.AND.LINVECT(HINDEX(N)))
	  CALL PENMODE((PLT$M_SECONDBYTE.AND.LINVECT(HINDEX(N)))/256)
	  IF (COLOR(HINDEX(N)).NE.0) THEN
	    DELTALEFT=0.5*XAXIS  ! Default in case only one point exists
	    XSTARTVAL=XARRAY(SIZE-1,HINDEX(N))
	    YSTARTVAL=YARRAY(SIZE-1,HINDEX(N))
C  Get the physical increment per inch
	    XDV=XARRAY(SIZE,HINDEX(N))*X_INTERVALS/XAXIS
	    YDV=YARRAY(SIZE,HINDEX(N))*Y_INTERVALS/YAXIS
	    CALL COLORMODE(7982541)  ! Makes the outline DarkSlateGray3
	    DO INDEX=ISTART,NPOINTS(HINDEX(N))+ISTART-1,INC(HINDEX(N))
	      IF (INDEX-ISTART+1.EQ.1) THEN
	        YLAST=MIN(0.0,YAXIS+YOFFSET)
	        YLAST=MAX(YLAST,YOFFSET)
	      END IF
	      IF (INDEX-ISTART+1.NE.NPOINTS(HINDEX(N))-MOD(NPOINTS
	1     (HINDEX(N)),INC(HINDEX(N)))) THEN
	        DELTARIGHT=0.5*(XARRAY(INDEX+INC(HINDEX(N)),HINDEX(N))
	1       -XARRAY(INDEX,HINDEX(N)))
	      ELSE  ! The last bar
	        DELTARIGHT=DELTALEFT998  ! Use the last value
	      END IF
	      IF (INDEX-ISTART+1.NE.1) THEN
	        DELTALEFT=PRIORDELTARIGHT998
	      ELSE
	        DELTALEFT=DELTARIGHT
	      END IF
	      DELTARIGHT998=DELTARIGHT
	      DELTALEFT998=DELTALEFT
	      IF (BARTYPE(HINDEX(N)).EQ.999) THEN
	        DELTARIGHT=0.8*DELTARIGHT  ! Leaves a gap between bar
					   ! groups
	        DELTALEFT=0.8*DELTALEFT
	        DELTALR=DELTALEFT+DELTARIGHT
	        DELTARIGHT=-DELTALEFT+FLOAT(N)/NHISTOGRAMS*DELTALR
	        DELTALEFT=DELTALEFT-FLOAT(N-1)/NHISTOGRAMS*DELTALR
	      END IF
	      IF (INVERT(1)) THEN
	        X=XAXIS-(XARRAY(INDEX,HINDEX(N))-XSTARTVAL-DELTALEFT)/XDV+
	1       XOFFSET-X_INTERVALS_OFF*XAXIS/X_INTERVALS
	      ELSE
	        X=(XARRAY(INDEX,HINDEX(N))-XSTARTVAL-DELTALEFT)/XDV+
	1       XOFFSET+X_INTERVALS_OFF*XAXIS/X_INTERVALS
	      END IF
C  YLOW is the bottom Y coordinate of the rectangles for each plot
C  measured in physical value.  Its is computed from the sum of the
C  values of all previous plots for each point.
	      IF (BARTYPE(HINDEX(N)).EQ.998) THEN
	        YLOW=0.0
	        DO NN=1,N-1
	          YLOW=YLOW+YARRAY(INDEX,HINDEX(NN))
	        END DO
	      ELSE IF (BARTYPE(HINDEX(N)).EQ.999) THEN
	        YLOW=0.0
	      END IF
	      Y=(YLOW-YSTARTVAL)/YDV
	      IF (INVERT(2)) THEN
	        Y=YAXIS-Y+YOFFSET-Y_INTERVALS_OFF*YAXIS/Y_INTERVALS
	      ELSE
	        Y=Y+YOFFSET+Y_INTERVALS_OFF*YAXIS/Y_INTERVALS
	      END IF
	      XLEFTIN=MIN(X,XAXIS+XOFFSET)
	      XLEFTIN=MAX(XLEFTIN,XOFFSET)
	      YLOWIN=MIN(Y,YAXIS+YOFFSET)
	      YLOWIN=MAX(YLOWIN,YOFFSET)
	      X=(XARRAY(INDEX,HINDEX(N))-XSTARTVAL+DELTARIGHT)/XDV
	      IF (INVERT(1)) THEN
	        X=XAXIS-X+XOFFSET-X_INTERVALS_OFF*XAXIS/X_INTERVALS
	      ELSE
	        X=X+XOFFSET+X_INTERVALS_OFF*XAXIS/X_INTERVALS
	      END IF
	      IF (BARTYPE(HINDEX(N)).EQ.998) THEN
	        YHIGH=0
	        DO NN=1,N
	          YHIGH=YHIGH+YARRAY(INDEX,HINDEX(NN))
	        END DO
	      ELSE IF (BARTYPE(HINDEX(N)).EQ.999) THEN
	        YHIGH=YARRAY(INDEX,HINDEX(N))
	      END IF
	      Y=(YHIGH-YSTARTVAL)/YDV
	      IF (INVERT(2)) THEN
	        Y=YAXIS-Y+YOFFSET-Y_INTERVALS_OFF*YAXIS/Y_INTERVALS
	      ELSE
	        Y=Y+YOFFSET+Y_INTERVALS_OFF*YAXIS/Y_INTERVALS
	      END IF
	      XRIGHTIN=MIN(X,XAXIS+XOFFSET)
	      XRIGHTIN=MAX(XRIGHTIN,XOFFSET)
	      YHIGHIN=MIN(Y,YAXIS+YOFFSET)
	      YHIGHIN=MAX(YHIGHIN,YOFFSET)
C  Draw the left side of the histogram bar
C  Get the starting position
	      IF (BARTYPE(HINDEX(N)).EQ.998) THEN
	        IF (INVERT(2)) THEN
C  Prevent overwriting the right side of the previous bar of the same
C  plot or the left side of the current bar of the previous plot.
	          YLOWLEFTIN=MIN(YLAST,YLOWIN)
	        ELSE
	          YLOWLEFTIN=MAX(YLAST,YLOWIN)
	        END IF
	      ELSE IF (BARTYPE(HINDEX(N)).EQ.999) THEN
	        YLOWLEFTIN=YLOWIN
	      END IF
	IF (N.EQ.1) THEN
C	WRITE (7,3333) INDEX,XLEFTIN,YLOWLEFTIN,YHIGHIN,YLOW,Y,YLAST,YLOWIN
3333	FORMAT(' ',I5,7F8.4)
	END IF
	      CALL PLOT(XLEFTIN,YLOWLEFTIN,3)
	      IF (INVERT(2).AND.YHIGHIN.LT.YLOWLEFTIN.OR..NOT.INVERT(2)
	1     .AND.YHIGHIN.GT.YLOWLEFTIN) THEN
	        CALL PLOT(XLEFTIN,YHIGHIN,2)
	      ELSE
	        CALL PLOT(XLEFTIN,YHIGHIN,3)
	      END IF
C  Draw the top of the histogram bar
	      CALL PLOT(XRIGHTIN,YHIGHIN,2)
C  Draw the right side of the histogram bar
	      IF (BARTYPE(HINDEX(N)).EQ.998) THEN
	        IF (INDEX-ISTART+1.NE.NPOINTS(HINDEX(N))-MOD(NPOINTS
	1       (HINDEX(N)),INC(HINDEX(N)))) THEN  ! Not the last point
	          YLOWRIGHT=0.0
	          DO NN=1,N-1
	            YLOWRIGHT=YLOWRIGHT+YARRAY(INDEX+INC(HINDEX(NN)),
	1           HINDEX(NN))
	          END DO
	        ELSE
	         YLOWRIGHT=YLOW
	        END IF
	        YLOWRIGHT=MAX(YLOWRIGHT,YLOW)
C  Prevent overwriting the adjacent bar of the previous plot
	        IF (YLOWRIGHT.LT.YHIGH) THEN
	          IPEN=2
	        ELSE
	          IPEN=3
	        END IF
	      ELSE IF (BARTYPE(HINDEX(N)).EQ.999) THEN
	        IF (N.NE.NHISTOGRAMS) THEN
	          YLOWRIGHT=MIN(YHIGH,YARRAY(INDEX,HINDEX(N+1)))
	        ELSE
	          YLOWRIGHT=YLOW
	        END IF
	        IPEN=2
	      END IF
	      Y=(YLOWRIGHT-YSTARTVAL)/YDV
	      IF (INVERT(2)) THEN
	        Y=YAXIS-Y+YOFFSET-Y_INTERVALS_OFF*YAXIS/Y_INTERVALS
	      ELSE
	        Y=Y+YOFFSET+Y_INTERVALS_OFF*YAXIS/Y_INTERVALS
	      END IF
	      YLOWRIGHTIN=MIN(Y,YAXIS+YOFFSET)
	      YLOWRIGHTIN=MAX(YLOWRIGHTIN,YOFFSET)
	      CALL PLOT(XRIGHTIN,YLOWRIGHTIN,IPEN)
	      PRIORDELTARIGHT998=DELTARIGHT998
	      YLAST=YHIGHIN
	    END DO
	  END IF
	END DO
	RETURN
	END
