	SUBROUTINE LINE(X,Y,SIZE,NPTS,INC,XOFFSET,YOFFSET,XAXIS,YAXIS,
	1X_INTRVLS,Y_INTRVLS,X_INTRVLS_OFF,Y_INTRVLS_OFF,LINTYP,NSYMBL,
	2SYMBLHT,INVERT)
C
C  Note:  This is the Unix version
C
C       This subroutine plots vectors between the given X, Y
C  pairs of data points and/or plots symbols at their locations.
C			Donald H. Gudehus, September, l983
C			Version 2, Donald H. Gudehus, January 7, 1987
C			Excluded values equal to OMIT from being plotted.
C			Version 3, Donald H. Gudehus, June, 1995
C			Corrected misspelling of XOFFSET in code.
C			Changed logic so that section for drawing
C			lines only is only called with LINETYP=0.
C			Version 4, D. Gudehus, January 8, 2000
C			Removed argument LU
C			Version 5, D. Gudehus, January 9, 2000
C			Changed OMIT from 7FFFFFFF to FF7FFFFF so as to
C			be compatible with the Unix version.
C			Version 5U, D. Gudehus, January 9, 2000
C			Modified code for Unix
C			Version 6, D. Gudehus, May 3, 2000
C			Introduced X_INTRVLS_OFF and Y_INTRVLS_OFF
C
C	X	      (C)	The array containing the X positions.  The last
C				position should contain the physical value per
C				interval;  the next to last position should 
C				contain	the starting physical value.
C	Y	      (C)	The array containing the Y positions
C	SIZE	      (C)	The size of each of the above arrays.
C				This should be at least two larger than the
C				maximum number of points in the array.
C	NPTS	      (C)	The number of data points in the arrays
C	INC	      (C)	The increment to use in stepping through the
C				arrays.
C	XOFFSET	      (C)	The starting X location on the plot device
C				measured in inches.
C	YOFFSET	      (C)	The starting Y location on the plot device
C				measured in inches.
C	XAXIS	      (C)	The length of the x axis in inches
C	YAXIS	      (C)	The length of the y axis in inches
C	X_INTRVLS     (C)	The number of intervals per axis length.  If
C				there is to be one tick mark per inch, then
C				X_INTRVLS=XAXIS.  Note that X_INTRVLS can have
C				fractional values.
C	Y_INTRVLS     (C)	Like X_INTRVLS but in the Y direction
C	X_INTRVLS_OFF (C)	The fractional offset at the X axis start 
C				measured in units of intervals
C	Y_INTRVLS_OFF (C)	The fractional offset at the Y axis start
C	LINTYP	      (C)	A variable controlling the type of line to be
C				drawn through the data points.  The magnitude 
C				of LINTYP controls the frequency of plotted
C				symbols:  e.g., if LINTYP equals 4 a special
C				symbol drawn at every 4th data point.  If
C				LINTYP is zero, no symbols are plotted.  If
C				LINTYP is positive, a straight line connects 
C				the data points.  If LINTYP is negative, no
C				connecting lines are drawn;  only the symbols
C				are plotted.
C	NSYMBL	      (C)	The number of the plotting symbol to be 
C				centered at each data point.  For the special 
C				symbols, the symbol number should be placed 
C				in the first byte location;  for any ASCII 
C				symbol, the ASCII code should be placed in 
C				the second byte location.  NSYMBL has meaing 
C				only when LINTYP is non-zero.
C	SYMBLHT	      (C)	The height of a symbol in inches
C	INVERT	      (C)	A two dimensional array determining whether
C				the X and/or Y coordinates are to be inverted.
C
	BYTE TEXT
	INTEGER SIZE,SYMCOUNT
	LOGICAL INVERT(2),PRIOR_POINT_OUTSIDE
	REAL X(SIZE),Y(SIZE)
	INCLUDE 'plotpar.for'
	DATA OMIT/'FF7FFFFF'X/
C
	EPSI=3E-3
	XSTARTVAL=X(SIZE-1)
	YSTARTVAL=Y(SIZE-1)
	XDV=X(SIZE)*X_INTRVLS/XAXIS  ! Physical value/inch
	YDV=Y(SIZE)*Y_INTRVLS/YAXIS
C
C  Move the pen to the first point
C
	I=1
	DO WHILE((X(I).EQ.OMIT.OR.Y(I).EQ.OMIT).AND.I.LE.NPTS)
	  I=I+INC
	END DO
	IF (INVERT(1)) THEN
	  X1=XAXIS-(X(I)-XSTARTVAL)/XDV+XOFFSET-X_INTRVLS_OFF*XAXIS/X_INTRVLS
	ELSE
	  X1=(X(I)-XSTARTVAL)/XDV+XOFFSET+X_INTRVLS_OFF*XAXIS/X_INTRVLS
	END IF
	IF (INVERT(2)) THEN
	  Y1=YAXIS-(Y(I)-YSTARTVAL)/YDV+YOFFSET-Y_INTRVLS_OFF*YAXIS/Y_INTRVLS
	ELSE
	  Y1=(Y(I)-YSTARTVAL)/YDV+YOFFSET+Y_INTRVLS_OFF*YAXIS/Y_INTRVLS
	END IF
	X1F=MIN(X1,XAXIS+XOFFSET)
	X1F=MAX(X1F,XOFFSET)
	Y1F=MIN(Y1,YAXIS+YOFFSET)
	Y1F=MAX(Y1F,YOFFSET)
	CALL PLOT(X1F,Y1F,3)
	IF (X1F.LE.(XAXIS+XOFFSET+EPSI).AND.X1F.GE.(XOFFSET-EPSI).AND.
	1Y1F.LE.(YAXIS+YOFFSET+EPSI).AND.Y1F.GE.(YOFFSET-EPSI)) THEN
	  PRIOR_POINT_OUTSIDE=.FALSE.
	ELSE
	  PRIOR_POINT_OUTSIDE=.TRUE.
	END IF
C
	IF (LINTYP.EQ.0) THEN  ! Plots lines only
	  DO 300 I=1+INC,NPTS,INC
	    IF (X(I).EQ.OMIT.OR.Y(I).EQ.OMIT) GO TO 300
	    XINCH=(X(I)-XSTARTVAL)/XDV
	    YINCH=(Y(I)-YSTARTVAL)/YDV
	    IF (INVERT(1)) THEN
	      X1=XAXIS-XINCH+XOFFSET-X_INTRVLS_OFF*XAXIS/X_INTRVLS
	    ELSE
	      X1=XINCH+XOFFSET+X_INTRVLS_OFF*XAXIS/X_INTRVLS
	    END IF
	    IF (INVERT(2)) THEN
	      Y1=YAXIS-YINCH+YOFFSET-Y_INTRVLS_OFF*YAXIS/Y_INTRVLS
	    ELSE
	      Y1=YINCH+YOFFSET+Y_INTRVLS_OFF*YAXIS/Y_INTRVLS
	    END IF
	    IF (X1.LE.(XAXIS+XOFFSET+EPSI).AND.X1.GE.(XOFFSET-EPSI).AND.
	1   Y1.LE.(YAXIS+YOFFSET+EPSI).AND.Y1.GE.(YOFFSET-EPSI)) THEN
	      IF (PRIOR_POINT_OUTSIDE.EQ..FALSE.) THEN
	        CALL PLOT(X1,Y1,2)
	      ELSE
	        X1=MIN(X1,XAXIS+XOFFSET)
	        X1=MAX(X1,XOFFSET)
	        Y1=MIN(Y1,YAXIS+YOFFSET)
	        Y1=MAX(Y1,YOFFSET)
	        CALL PLOT(X1,Y1,3)
	        PRIOR_POINT_OUTSIDE=.FALSE.
	      END IF
	    ELSE
	      X1=MIN(X1,XAXIS+XOFFSET)
	      X1=MAX(X1,XOFFSET)
	      Y1=MIN(Y1,YAXIS+YOFFSET)
	      Y1=MAX(Y1,YOFFSET)
	      CALL PLOT(X1,Y1,3)
	      PRIOR_POINT_OUTSIDE=.TRUE.
	    END IF
300	  CONTINUE
	END IF
C
C  If LINTYP=0, we are done.  If LINTYP is not zero, we are already at the
C  first point.
C  Plot symbols, possibly with connecting lines, if requested
	IF (ABS(LINTYP).GT.0)  THEN
	  IF (LINTYP.GT.0) ILINE=-2 !  Connecting lines.  Pen down.
	  IF (LINTYP.LT.0) ILINE=-1 !  No connecting lines.  Pen up.
	  SYMCOUNT=0
	  DO 400 I=1,NPTS,INC !  Starts at 1 because a symbol will always be
			      !  plotted there
	    IF (X(I).EQ.OMIT.OR.Y(I).EQ.OMIT) GO TO 400
	    XINCH=(X(I)-XSTARTVAL)/XDV
	    YINCH=(Y(I)-YSTARTVAL)/YDV
	    IF (INVERT(1)) THEN
	      X1=XAXIS-XINCH+XOFFSET-X_INTRVLS_OFF*XAXIS/X_INTRVLS
	    ELSE
	      X1=XINCH+XOFFSET+X_INTRVLS_OFF*XAXIS/X_INTRVLS
	    END IF
	    IF (INVERT(2)) THEN
	      Y1=YAXIS-YINCH+YOFFSET-Y_INTRVLS_OFF*YAXIS/Y_INTRVLS
	    ELSE
	      Y1=YINCH+YOFFSET+Y_INTRVLS_OFF*YAXIS/Y_INTRVLS
	    END IF
	    IF (X1.LE.(XAXIS+XOFFSET+EPSI).AND.X1.GE.(XOFFSET-EPSI).AND.
	1   Y1.LE.(YAXIS+YOFFSET+EPSI).AND.Y1.GE.(YOFFSET-EPSI)) THEN
	      SYMCOUNT=SYMCOUNT+1
	      IF (SYMCOUNT.EQ.IABS(LINTYP)) THEN
	        TEXT=(PLT$M_SECONDBYTE.AND.NSYMBL)/256
	        IF (TEXT.EQ.0) THEN
	          TEXT=PLT$M_FIRSTBYTE.AND.NSYMBL
	          CALL SYMBOL(X1,Y1,SYMBLHT,TEXT,
	1         0.0,ILINE) !  Plots a symbol (ILINE is negative)
	        ELSE
	          TEXT=(PLT$M_SECONDBYTE.AND.NSYMBL)/256
	          CALL SYMBOL(X1-0.2857*SYMBLHT,Y1-0.5*SYMBLHT,SYMBLHT,TEXT,
	1         0.0,1) !  Plots a character
	        END IF
	        SYMCOUNT=0
	      END IF
	      IF (SYMCOUNT.EQ.IABS(LINTYP)) SYMCOUNT=0
	    END IF
 400	  CONTINUE
	END IF
	RETURN
	END
