	SUBROUTINE SCALE_AXIS (ARRAY,SIZE,NPLOTS,NPOINTS,NINTRVLS,INC,LINTYPE,
	1NSYMBOL,COLOR,STACKED)
C 
C  Note:  This is the Unix version
C
C       This subroutine operates on ARRAY and loads the last 4 locations
C  of it with information needed to scale an axis when plotting.  This
C  subroutine is called by XYPLOT.
C
C	ARRAY	 (C)	An array of unscaled data.
C	SIZE	 (C)	The dimension of ARRAY.  This should be at least 4
C			larger than the maximum number of data points
C			because the starting physical value and the
C			physical value/interval are stored in the last
C			two points.
C	NPLOTS	 (C)	The second dimension of ARRAY.  It is equal to 
C			the number of plots.
C	NPOINTS	 (C)	The number of data points for each plot in ARRAY.
C	NINTRVLS (C)	The number of tick mark intervals.
C			Fractional values are allowed.
C	INC	 (C)	The increment used when indexing through ARRAY
C			In this subroutine it is for now the same for
C			each plot.
C	LINTYPE	 (C)	An array controlling the type of line to be
C			drawn through the data points.  The magnitude of
C			LINTYPE controls the frequency of plotted
C			symbols:  e.g., if LINTYPE equals 4 a special
C			symbol drawn at every 4'th data point.  If
C			LINTYPE is zero, no symbols are plotted.  If
C			LINTYPE is positive, a straight line connects
C			the data points.  If LINTYPE is negative, no
C			connecting lines are drawn;  only the symbols
C			are plotted.  If LINTYPE = 997, error bars
C			are drawn;  if LINTYPE = 998, a histogram is
C			drawn with the bars stacked vertically if
C			multiple plots are made;  and if LINTYPE = 999,
C			a histogram is drawn with the bars placed next
C			to each other if multiple plots are made.
C	NSYMBOL	 (C)	An array giving the symbol for each plot.  Here
C			we are only concerned whether 100 has been added
C			to the first byte.  In that case adjacent 
C			positions in ARRAY, for the direction representing
C			the errorbar's deviation, contain the central
C			location and the deviation to the low and high
C			directions.  For the perpendicular direction,
C			adjacent positions in ARRAY contain either the
C			central location and zero, or two occurances
C			of the central location (older style).  If 100
C			has not been added to the first byte, adjacent
C			positions contain the actual locations of the low
C			and high endpoints.
C	COLOR	 (C)	An array containing the color for each plot.
C			A value of zero causes that particular plot
C			not to be drawn.
C	STACKED	 (C)	If a stacked histogram is in the direction of
C			this axis, it is .TRUE., otherwise false.  Has
C			no effect if there are no histograms.
C
C		
C		Version 1, JRD, 01/01/75
C		Version 2, RSW, 02/02/76   
C		REVISED FOR HP 21 DOS 
C		Version 3, Donald H. Gudehus, November 30, 1983
C		Added SIZE to allow for multiple plots of different
C		numbers of points
C		Version 4, Donald H. Gudehus, September, 1984
C		Added NINTRVLS to the argument list
C		Version 5, Donald H. Gudehus, January, 1987
C		Added exclusion of values equal to OMIT from scaling
C		operation
C		Version 6, Donald H. Gudehus, February, 1992
C		Added NPLOTS to the argument list so that all plots
C		will be searched so that the scaling will not exclude
C		any plots.  NPOINTS was made into an array also.
C		Version 7, Donald H. Gudehus, January 9, 2000
C		Changed OMIT from 7FFFFFFF to FF7FFFFF so as to be 
C		compatible with the Unix version.
C		Version 8, Donald H. Gudehus, September 3, 2004
C		Introduced LINTYPE, NSYMBOL, and COLOR to the argument list,
C		and introduced their associated code.
C		Version 8UM, Donald H. Gudehus, February 8, 2005
C		Modified IOMIT for Absoft compiler on Mac OS.
C		Version 9, Donald H. Gudehus, February 24, 2005
C		Fixed bug where under certain conditions of number
C		sequences, either the minimum or the maximum but
C		not both would be found.  The correction involved
C		removing several ELSE statements.
C		Version 10, Donald H. Gudehus, April 9,2005
C		Added STACKED to the argument list to better handle
C		histograms.
C
	INTEGER SIZE
	DIMENSION ARRAY(SIZE,NPLOTS),NPOINTS(NPLOTS),LINTYPE(NPLOTS),
	1NSYMBOL(NPLOTS)
	DIMENSION UNITS(7)
	INTEGER COLOR(NPLOTS)
	LOGICAL STACKED
	REAL NINTRVLS
	INCLUDE 'plotpar.for'
	INTEGER IOMIT
	EQUIVALENCE (IOMIT,OMIT)
	DATA UNITS /1.0,2.0,4.0,5.0,8.0,10.0,20.0/
	DATA OMIT /'FF7FFFFF'X/
C 
C  Find out how errorbars, if any, are written
C  Initialize the minimum and maximum values of ARRAY
	K=IABS(INC)
	AMIN=1.0E37
	AMAX=-1.0E37
C  Initialize the sums for vertically stacked histograms
	VSUM_MIN=1.0E37
	VSUM_MAX=-1.0E37
C  Obtain the minimum and maximum values of ARRAY
	J=1
	DO WHILE (J.LE.NPLOTS)
C  Handle the cases of no errorbars or residuals, and no vertically 
C  stacked histogram
	  IF (LINTYPE(J).NE.997.AND.
	1 .NOT.(LINTYPE(J).EQ.998.AND.STACKED)) THEN
	    IF (COLOR(J).NE.0) THEN
	      DO I=1,NPOINTS(J),K
 	        IF (ARRAY(I,J).LT.AMIN.AND.ARRAY(I,J).NE.OMIT)
	1       AMIN=ARRAY(I,J)
	        IF (ARRAY(I,J).GT.AMAX.AND.ARRAY(I,J).NE.OMIT)
	1       AMAX=ARRAY(I,J)
	      END DO
	    END IF
C  Handle the cases of errorbars or residuals
	  ELSE IF (LINTYPE(J).EQ.997) THEN
	    IF (COLOR(J).NE.0) THEN
C  Find out whether the increment or absolute method is used
	      KEY=PLT$M_FIRSTBYTE.AND.NSYMBOL(J)
	      IF (KEY.GE.100) THEN !  Increment method (for errorbars)
	        DO I=1,NPOINTS(J),K
C  Case of ARRAY containing information in direction of errorbar
	          IF (ARRAY(I,J+1).NE.0.0.AND.(ARRAY(I,J).NE.ARRAY(I,J+1)))
	1         THEN
	            IF ((ARRAY(I,J)-ARRAY(I,J+1)).LT.AMIN.AND.(ARRAY(I,J).NE.
	1           OMIT)) AMIN=ARRAY(I,J)-ARRAY(I,J+1)
	            IF ((ARRAY(I,J)+ARRAY(I,J+1)).GT.AMAX.AND.
	1           (ARRAY(I,J).NE.OMIT)) AMAX=ARRAY(I,J)+ARRAY(I,J+1)
C  Case of ARRAY containing information in direction perpendicular to
C  errorbar
	          ELSE IF (ARRAY(I,J+1).EQ.0.0.OR.(ARRAY(I,J).EQ.
	1         ARRAY(I,J+1))) THEN
	            IF (ARRAY(I,J).LT.AMIN.AND.ARRAY(I,J).NE.OMIT)
	1           AMIN=ARRAY(I,J)
	            IF (ARRAY(I,J).GT.AMAX.AND.ARRAY(I,J).NE.OMIT)
	1           AMAX=ARRAY(I,J)
	          END IF
	        END DO
	        J=J+1
	      ELSE !  Absolute method (for errorbars and residuals)
	        DO I=1,NPOINTS(J),K
	          IF (ARRAY(I,J).LT.AMIN.AND.ARRAY(I,J).NE.OMIT)
	1         AMIN=ARRAY(I,J)
	          IF (ARRAY(I,J).GT.AMAX.AND.ARRAY(I,J).NE.OMIT)
	1         AMAX=ARRAY(I,J)
	        END DO
	      END IF
	    END IF
	  END IF
	  J=J+1
	END DO
C  Handle a stacked histogram separately.
	IF (STACKED) THEN
C  First get the maximum number of points in any histogram plot
	  NPOINTS_MAX=0
	  DO J=1,NPLOTS
	    IF (LINTYPE(J).EQ.998.AND.COLOR(J).NE.0.AND.NPOINTS(J).GT.
	1   NPOINTS_MAX) NPOINTS_MAX=NPOINTS(J)
	  END DO
C  Now get the minimum and maximum of any stack by cycling through the plots
C  for each point.
	  I=1
	  DO WHILE (I.LE.NPOINTS_MAX)
	    SUM=0.0
	    DO J=1,NPLOTS
	      IF (LINTYPE(J).EQ.998.AND.COLOR(J).NE.0.AND.I.LE.NPOINTS(J)) THEN
	        IF (ARRAY(I,J).NE.OMIT) SUM=SUM+ARRAY(I,J)
	      END IF
	    END DO
	    IF (SUM.LT.VSUM_MIN) VSUM_MIN=SUM
	    IF (SUM.GT.VSUM_MAX) VSUM_MAX=SUM
	    I=I+1
	  END DO
C  Bring AMIN, AMAX, VSUM_MIN, and VSUM_MAX into agreement
	  AMIN=MIN(AMIN,VSUM_MIN)
	  AMAX=MAX(AMAX,VSUM_MAX*1.01) !  So stack is not up against top
	END IF
C 
C  Compute the physical increment per interval
	DV=(AMAX-AMIN)/NINTRVLS
	IF (DV.LE.0.0) THEN
C  Error - AMAX & AMIN not in correct range 
	  DV=ABS((AMIN+AMIN)/NINTRVLS)+1.0
	END IF
C  Compute tens power of 'DV' value 
	A=10.0**(IFIX(ALOG10(DV)+1000.0)-1000)
C 
C  Compute normalized 'DV' value (1.0<=DV+0.01<10.0) 
	DV=DV/A-0.01
C 
C  Locate closest "UNIT" DV value (normalized)
	DO I=1,6 
	  IF (UNITS(I)-DV) 80,90,90 
  80	END DO
C
C  Set normalized 'SIGNIFANCE' direction
  90	TMIN=AMIN 
	SGNF=0.01 
	IF (AMIN) 100,110,110 
 100	SGNF=-0.99 
C 
C  Expand unit 'DV' to floating 
 110	DV=UNITS(I)*A 
C 
C  Compute "UNIT" minimum based on "UNIT" DV
	TMIN=DV * AINT(AMIN/DV+SGNF)
C 
C  Check to see if adjusted "UNIT" scale will still fit axis length 
	IF ((TMIN+(NINTRVLS+0.01)*DV)-AMAX) 120,140,140
 120	TMIN=AINT(AMIN/A+SGNF)*A
	IF ((TMIN+(NINTRVLS+0.01)*DV)-AMAX) 130,140,140
 130	I=I+1 
	GO TO 110 
C 
C  Compute final adjusted minimum
 140	TMIN=TMIN-DV*AINT((NINTRVLS+(TMIN-AMAX)/DV)/2.0) 
	IF ((AMIN/A)*(TMIN/A)) 150,150,160  ! Divides AMIN and TMIN by A
C		! in order to prevent a floating overflow or underflow
 150	TMIN=0.0
 160	IF (INC) 170,170,180
C  Reverse direction of scale (Use 'AMAX' and '-DV')
 170	TMIN=TMIN+DV*AINT(NINTRVLS+0.5)
	DV=-DV
C 
C  Load the values in ARRAY
 180	ARRAY(SIZE-1,1)=TMIN 
	ARRAY(SIZE,1)=DV 
C 
 199	RETURN
	END 
