	FUNCTION PLOTDEVQUES(PLOTDEVICE,MODE)
C
C       This function returns a logical value depending on whether a
C  plot device or a member of its class of devices is actually being used.
C  For example, if PLOTDEVICE is equal to G* and MODE equals 1, the function
C  checks DEVICES in the COMMON block PLOT_CHAR_DEVICES to see if any 
C  graphics terminal or emulator is being used as the first one in the list.
C  The first one in the list is always the one for which interactive graphics
C  is enabled in subroutines such as XYPLOT, CONTOURPLOT, MESHPLOT, etc.
C  For MODE=1 and MODE=0, where the entire list is checked, it is necessary
C  that the subroutine PLOTDEVICES first be called.  If alternately, the
C  lower level subroutine PLTDEV, is first called, MODE must be set equal to 2,
C  which causes NDEV in the COMMON block PLOT_CHAN to be checked.
C
C					Donald H. Gudehus, July, 1987
C				Version 2, Donald H. Gudehus, October, 1994
C				Introduced the argument MODE.
C				Version 3, Donald H. Gudehus, May 6, 2000
C				Added 1WT
C				Version 4, Donald H. Gudehus, March 25, 2004
C				Added MODE = 2
C
C	PLOTDEVICE	(C)	The plotdevice abreviation which is used
C				in PLOTDEVICES.FOR or a special symbol 
C				recognized by this subroutine such as G*
C				(any interactive graphics device) or GR
C				(the Grinnell; GR or -GR).
C				Special Symbols         Devices
C				G*		Any interactive graphics
C						display device
C				GD*		Any interactive graphics
C						display device where text is
C						normally input and output on
C						a separate window without any
C						special	action necessary, 
C						e.g., SAOimage, Grinnell,
C						Trapix.
C				GT		Any TEK style interactice
C						graphics device
C				1WT		A TEK device which only
C						can show one window at 
C						a time.
C				2WT		A 2-windows TEK device,
C						e.g. VersaTerm Pro, Xterm
C				D*		Any graphics device which
C						displays vectors so slowly
C						that it is faster to delay
C						display until all have been
C						written, e.g. SAOimage
C				GR*		The Grinnell (GR, -GR)
C				S*		SAOimage (S, -S)
C				XG		Xgterm (only on Unix)
C				NOI		Devices that do not create
C						an extra interrupt when
C						a key is pressed.
C	MODE		(C)	The mode of operation.  If equal to 0,
C				all devices in the list of possible devices
C				in the COMMON block PLOT_CHAR_DEVICES are
C				checked.  If equal to 1, only the first
C				one in the list (the one which interactive
C				graphics is enabled for) is checked.  If 
C				equal to 2, then instead of checking the list
C				of devices, the number of the device in the
C				COMMON block PLOT_CHAN is checked.
C
C
	LOGICAL PLOTDEVQUES
C
	CHARACTER*(*) PLOTDEVICE
C
	INTEGER CHANNELS(10),DEVICENUMBERS(10)
	COMMON /PLOT_DEVICES/ NUMBERDEVICES,CHANNELS,DEVICENUMBERS,NDEVICE,
	1NLOWER
C
	CHARACTER*3 DEVICES(10)
	COMMON /PLOT_CHAR_DEVICES/ DEVICES
C
	COMMON /PLOT_CHAN/ NCHAN,NDEV
C
	PLOTDEVQUES=.FALSE.
	IF (MODE.EQ.0.OR.MODE.EQ.1) THEN
	  IF (MODE.EQ.0) THEN
	    INUMBERDEVICES=NUMBERDEVICES
	  ELSE IF (MODE.EQ.1) THEN
	    INUMBERDEVICES=1
	  END IF
	  DO N=1,INUMBERDEVICES
	    IF (PLOTDEVICE.EQ.'G*') THEN
	      IF (DEVICES(N).EQ.'G'.OR.DEVICES(N).EQ.'G1'.OR.
	1     DEVICES(N).EQ.'G2'.OR.DEVICES(N).EQ.'G3'.OR.
	2     DEVICES(N).EQ.'G4'.OR.
	3     DEVICES(N).EQ.'GR'.OR.DEVICES(N).EQ.'-GR'.OR.
	3     DEVICES(N).EQ.'TR'.OR.DEVICES(N).EQ.'-TR'.OR.
	4     DEVICES(N).EQ.'X'.OR.DEVICES(N).EQ.'XG'.OR.
	5     DEVICES(N).EQ.'S'.OR.DEVICES(N).EQ.'-S')
	4     PLOTDEVQUES=.TRUE.
	    ELSE IF (PLOTDEVICE.EQ.'GD*') THEN
	      IF (DEVICES(N).EQ.'GR'.OR.DEVICES(N).EQ.'-GR'.OR.
	1     DEVICES(N).EQ.'TR'.OR.DEVICES(N).EQ.'-TR'.OR.
	2     DEVICES(N).EQ.'S'.OR.DEVICES(N).EQ.'-S')
	3     PLOTDEVQUES=.TRUE.
	    ELSE IF (PLOTDEVICE.EQ.'GT') THEN
	      IF (DEVICES(N).EQ.'G'.OR.DEVICES(N).EQ.'G1'.OR.
	1     DEVICES(N).EQ.'G2'.OR.DEVICES(N).EQ.'G3'.OR.
	2     DEVICES(N).EQ.'G4'.OR.DEVICES(N).EQ.'X'.OR.
	3     DEVICES(N).EQ.'XG')
	4     PLOTDEVQUES=.TRUE.
	    ELSE IF (PLOTDEVICE.EQ.'1WT') THEN
	      IF (DEVICES(N).EQ.'G'.OR.DEVICES(N).EQ.'G1'.OR.
	1     DEVICES(N).EQ.'G2'.OR.DEVICES(N).EQ.'G3')
	2     PLOTDEVQUES=.TRUE.
	    ELSE IF (PLOTDEVICE.EQ.'2WT') THEN
	      IF (DEVICES(N).EQ.'G4'.OR.DEVICES(N).EQ.'X'.OR.
	1     DEVICES(N).EQ.'XG') PLOTDEVQUES=.TRUE. 
	    ELSE IF (PLOTDEVICE.EQ.'D*') THEN
	      IF (DEVICES(N).EQ.'S'.OR.DEVICES(N).EQ.'-S') 
	1     PLOTDEVQUES=.TRUE.
	    ELSE IF (PLOTDEVICE.EQ.'GR*') THEN
	      IF (DEVICES(N).EQ.'GR'.OR.DEVICES(N).EQ.'-GR')
	1     PLOTDEVQUES=.TRUE.
	    ELSE IF (PLOTDEVICE.EQ.'S*') THEN
	      IF (DEVICES(N).EQ.'S'.OR.DEVICES(N).EQ.'-S')
	1     PLOTDEVQUES=.TRUE.
	    ELSE IF (PLOTDEVICE.EQ.'XG') THEN
	      IF (DEVICES(N).EQ.'XG') PLOTDEVQUES=.TRUE.
	    ELSE IF (PLOTDEVICE.EQ.'NOI') THEN
	      IF (DEVICES(N).EQ.'GR'.OR.DEVICES(N).EQ.'-GR'.OR.
	1     DEVICES(N).EQ.'TR'.OR.DEVICES(N).EQ.'-TR'.OR.
	2     DEVICES(N).EQ.'X'.OR.DEVICES(N).EQ.'XG'.OR.
	3     DEVICES(N).EQ.'S'.OR.DEVICES(N).EQ.'-S')
	4     PLOTDEVQUES=.TRUE.
	    ELSE
	      IF (DEVICES(N).EQ.PLOTDEVICE) PLOTDEVQUES=.TRUE.
	    END IF
	  END DO
	ELSE IF (MODE.EQ.2) THEN
	  IF (PLOTDEVICE.EQ.'G*') THEN
	    IF (NDEV.EQ.1.OR.NDEV.EQ.101.OR.
	1   NDEV.EQ.201.OR.NDEV.EQ.301.OR.
	2   NDEV.EQ.401.OR.
	3   NDEV.EQ.8.OR.NDEV.EQ.-8.OR.
	4   NDEV.EQ.10001.OR.NDEV.EQ.20001.OR.
	5   NDEV.EQ.12.OR.NDEV.EQ.-12.OR.
	6   NDEV.EQ.13.OR.NDEV.EQ.-13)
	7   PLOTDEVQUES=.TRUE.
	  ELSE IF (PLOTDEVICE.EQ.'GD*') THEN
	    IF (NDEV.EQ.8.OR.NDEV.EQ.-8.OR.
	1   NDEV.EQ.12.OR.NDEV.EQ.-12.OR.
	2   NDEV.EQ.13.OR.NDEV.EQ.-13)
	3   PLOTDEVQUES=.TRUE.
	  ELSE IF (PLOTDEVICE.EQ.'GT') THEN
	    IF (NDEV.EQ.1.OR.NDEV.EQ.101.OR.
	1   NDEV.EQ.201.OR.NDEV.EQ.301.OR.
	2   NDEV.EQ.401.OR.NDEV.EQ.10001.OR.
	3   NDEV.EQ.20001)
	4   PLOTDEVQUES=.TRUE.
	  ELSE IF (PLOTDEVICE.EQ.'1WT') THEN
	    IF (NDEV.EQ.1.OR.NDEV.EQ.101.OR.
	1   NDEV.EQ.201.OR.NDEV.EQ.301)
	2   PLOTDEVQUES=.TRUE.
	  ELSE IF (PLOTDEVICE.EQ.'2WT') THEN
	    IF (NDEV.EQ.401.OR.NDEV.EQ.10001.OR.
	1   NDEV.EQ.20001) PLOTDEVQUES=.TRUE. 
	  ELSE IF (PLOTDEVICE.EQ.'D*') THEN
	    IF (NDEV.EQ.13.OR.NDEV.EQ.-13) 
	1   PLOTDEVQUES=.TRUE.
	  ELSE IF (PLOTDEVICE.EQ.'GR*') THEN
	    IF (NDEV.EQ.8.OR.NDEV.EQ.-8)
	1   PLOTDEVQUES=.TRUE.
	  ELSE IF (PLOTDEVICE.EQ.'S*') THEN
	    IF (NDEV.EQ.13.OR.NDEV.EQ.-13)
	1   PLOTDEVQUES=.TRUE.
	  ELSE IF (PLOTDEVICE.EQ.'XG') THEN
	    IF (NDEV.EQ.20001) PLOTDEVQUES=.TRUE.
	  ELSE IF (PLOTDEVICE.EQ.'NOI') THEN
	    IF (NDEV.EQ.8.OR.NDEV.EQ.-8.OR.
	1   NDEV.EQ.12.OR.NDEV.EQ.-12.OR.
	2   NDEV.EQ.10001.OR.NDEV.EQ.20001.OR.
	3   NDEV.EQ.13.OR.NDEV.EQ.-13)
	4   PLOTDEVQUES=.TRUE.
	  END IF
	END IF
	RETURN
	END
