	SUBROUTINE SAO(IX,IY,IPEN)
C
C	Note:  This is the Unix version.
C
C       This subroutine does graphic output to SAOimage.
C			Donald H. Gudehus, October, 1994
C			Version 2, D. H. G., June 23, 1999
C			Added IPEN=17 for overlaying on pictures,
C			and added call to ERROR.
C			Version 2U, Donald Gudehus, January ,2000
C			Converted code to work under Unix.
C			Version 3, Donald Gudehus, January 22, 2000
C			Added IPEN=19 for changing the background color.
C			Version 4, D. Gudehus, July 25, 2006
C			Incorporated new saofacts.for
C
C	WHERE:
C		IX=	X  Raster coordinate
C		IY=	Y  Raster coordinate
C		IPEN = 	0  Erases screen, resets, goes to vector mode, 
C			   sets the drawing action to immediate display,
C			   and loads the default lookup table.
C			1  Point plot mode
C			2  Moves to new position
C			3  Moves to spot without drawing vector
C			   and goes to vector mode
C			6  Sets color
C			8  Sets data level
C			9  Sets rectilinear graphics mode.  Since SAOimage
C			   does not do this at present, we simply filter
C			   this out.
C			13 Sets the drawing action to immediate display
C			14 Sets the drawing action to delayed display
C			15 Displays the vectors which have been previously
C			   sent to SAOimage with delayed display
C			16 Like IPEN=0, but screen is not erased
C			17 Like IPEN=16, but default lookup table is not
C			   loaded.
C			18 In use for TEK to flush the buffer
C			19 Sets color of background (first entry in lookup
C			   table)
C
	INTEGER*4 IX,IY
	REAL XSTART,YSTART,XEND,YEND
C
	LOGICAL POINTPLOT
C
	COMMON /PLOT_CHAN/ NCHAN,NDEV
C
	COMMON /PLOT_LINESTYLE/ MODE
C
	INTEGER LCOLOR,LBCOLOR
	COMMON /PLOT_COLOR/ LCOLOR,LBCOLOR
C
	COMMON /PLOT_DATALEVEL/ LEVEL
C
	INCLUDE 'saofacts.for'
C
	INCLUDE 'plotpar.for'
C
	INTEGER ACTION
	LOGICAL RECTILINEAR,PROCEED
	INTEGER RED(200),GREEN(200),BLUE(200)
	INCLUDE 'sao_default_vlt.for' !  This must be last in 
				      !  the list
C
C  For IPEN=0 erase the screen and set up the coordinate system.  The frame 
C  buffer size is declared in PLTDEV as being 800X800.  If an overlay is 
C  made while in MIIPS, however, the frame buffer can be a different size.  
C  In that case, the coordinate system is already estblished and a call to 
C  SAO with IPEN=0 should not be made.  If the PLOT command is being used
C  to display several plots at the same time, the screen should not be 
C  erased.  In that case, SAO with IPEN=16 is called.  Next, load the 
C  default color look-up table.
C
	IF (IPEN.EQ.0.OR.IPEN.EQ.16.OR.IPEN.EQ.17)THEN
	  POINTPLOT=.FALSE.
	  IF (IPEN.EQ.0) CALL SAOFIRST
C  Get some parameters set up.  
	  NPIC_SAO=1 !  Pictures run from 1 to 16
	  NFRAME_SAO=0 !  Frames run from 0 to 3.  Only one frame is supported
		       !  on SAOimage
C  Establish the starting location on the frame.  We don't want to be too
C  close to the frame buffer boundary so we chose offsets of 50 and 100. 
C  These can be further adjusted by a call to PLOT(X,Y,-3) if desired.  
C  This is in fact done in CONTOURPLOT where the offsets of (1.4,0.3) 
C  inches are added to the pixel offsets here.  However when overlaying
C  the contour on a picture, SAO(X,Y,0) is not called and SAODELS establishes
C  the values of XLOC_SAO and YLOC_SAO.  The offsets XLOC_SAO and YLOC_SAO can
C  always be changed by resetting XLOC_SAO and YLOC_SAO and then calling
C  SAOCAPTION.
	  XLOC_SAO(NPIC_SAO)=50
	  YLOC_SAO(NPIC_SAO)=MAXFB_ROWS_SAO-100
	  TV_FILENAME(NPIC_SAO)=' '
C  Send these offsets to SAOimage
	  CALL SAOCAPTION
C  Establish the starting location in the array
	  IMSX_SAO(NFRAME_SAO,NPIC_SAO)=1
	  IMSY_SAO(NFRAME_SAO,NPIC_SAO)=1
C  Establish the number of pixels in the picture (assumed to be as
C  big as the frame buffer)
	  NPTSX_SAO(NPIC_SAO)=MAXFB_COLS_SAO
	  NPTSY_SAO(NPIC_SAO)=MAXFB_ROWS_SAO
C  Set the drawing action to immediate
	  ACTION=3
C
	  XSTART=0
	  YSTART=0
C  Load the default color look-up table except when overlaying on pictures
C  in which case a different lookup table has already been loaded.  The
C  default LUT has black at the first position.
	  IF (IPEN.NE.17) THEN
	    DO I=1,N_SAO_VLT_COLORS
	      RED(I)=IAND(PLT$M_THIRDBYTE,SAO_VLT_COLORS(I))/256**2
	      GREEN(I)=IAND(PLT$M_SECONDBYTE,SAO_VLT_COLORS(I))/256
	      BLUE(I)=IAND(PLT$M_FIRSTBYTE,SAO_VLT_COLORS(I))
	    END DO
C  Fill the rest of the look-up table with nonoffensive numbers
C  If the numbers are all the same, say 0, when the SAOimage LUT palette
C  window is brought up, SAOiamge will crash.
	    ISEED=12345
	    DO I=N_SAO_VLT_COLORS+1,200
C  The full value would be obained by multiplying by 255, but we keep it
C  less than that so that the LUT display mainly shows the nonrandom
C  values.  Note that multipying by 5 causes a crash on Linux when the
C  LUT palette is displayed.
	      RED(I)=RAN(ISEED)*25
	      GREEN(I)=RAN(ISEED)*25
	      BLUE(I)=RAN(ISEED)*25
	    END DO
	    CALL SAO_LOAD_VLT(BLUE,GREEN,RED)
	  END IF
	ELSE IF (IPEN.EQ.1) THEN
	  POINTPLOT=.TRUE.
C
C  Go to the new position
C
	ELSE IF (IPEN.EQ.2) THEN
	  IF (POINTPLOT) THEN
	    XSTART=IX
	    YSTART=IY
	  END IF
	  XEND=IX
	  YEND=IY
	  IF (RECTILINEAR) THEN
	    RECTILINEAR=.FALSE.
	  ELSE 
	    CALL SAO_VECT(ACTION,ICOLOR,XSTART,YSTART,XEND,YEND,NPIC_SAO)
	  END IF
	  XSTART=XEND
	  YSTART=YEND
	ELSE IF (IPEN.EQ.3) THEN
	  POINTPLOT=.FALSE.
	  XSTART=IX
	  YSTART=IY
	ELSE IF (IPEN.EQ.6) THEN  
C
C       The default table of colors (24 bits) is included in this file
C  upon compilation.  The look-up table colors are downloaded to SAOimage 
C  when that plot device is first initialized (IPEN=0).  See COLORMODE.FOR 
C  for a list of the available  colors.   When any 24-bit color is requested 
C  by a call to COLORMODE, we first search the 24-bit color default table 
C  for that color.  If it is found we only need to determine its index in the 
C  table and set ICOLOR to the index minus 1.  The allowable values of ICOLOR 
C  range from 0 to 199 when used with the look-up table.  We therefore 
C  must scale the 24-bit requested color to a range of 200 values.
C       SAOimage also allows overlays.  The overlay colors lie higher than
C  24-bit values and so can be detected here by that fact.  Overlay colors
C  are only available in green, black, white, red, blue and yellow.  When
C  an overlay color is requested, we must not load the look-up table, but
C  simply set the value of ICOLOR to it, after scaling.
	  IF (LCOLOR.LE.16777215)  THEN  ! 24-bit value valid for look-up table
	    I=1 !  Indexes start at 1
	    PROCEED=.TRUE.
	    DO WHILE (I.LE.N_SAO_VLT_COLORS.AND.PROCEED)
	      IF (LCOLOR.EQ.SAO_VLT_COLORS(I)) THEN
	        ICOLOR=I-1  ! Range for LUT is 0 to 199
	        PROCEED=.FALSE.
	      END IF
	      I=I+1
	    END DO
C Check if we need to load a new color, and if there is room to do so
	    IF (PROCEED.AND.N_SAO_VLT_COLORS.LT.200) THEN
	      N_SAO_VLT_COLORS=N_SAO_VLT_COLORS+1
	      RED(N_SAO_VLT_COLORS)=IAND(PLT$M_THIRDBYTE,LCOLOR)/256**2
	      GREEN(N_SAO_VLT_COLORS)=IAND(PLT$M_SECONDBYTE,LCOLOR)/256
	      BLUE(N_SAO_VLT_COLORS)=IAND(PLT$M_FIRSTBYTE,LCOLOR)
	      SAO_VLT_COLORS(N_SAO_VLT_COLORS)=LCOLOR
	      ICOLOR=I-1
	      CALL SAO_LOAD_VLT(BLUE,GREEN,RED)
	    END IF
	  ELSE  ! Overlay colors
	    ICOLOR=LCOLOR-16777215
	  END IF
	ELSE IF (IPEN.EQ.8) THEN
	  IF (LEVEL.EQ.0) THEN
	    IF (ICOLOR_SAVE.NE.0) ICOLOR=ICOLOR_SAVE
	  ELSE IF (LEVEL.EQ.1) THEN
C  Simply set the color equal to black
	    ICOLOR_SAVE=ICOLOR
	    ICOLOR=0
	  END IF
	ELSE IF (IPEN.EQ.9) THEN
	  RECTILINEAR=.TRUE.
	ELSE IF (IPEN.EQ.13) THEN
	  ACTION=3
	ELSE IF (IPEN.EQ.14) THEN
	  ACTION=4
	ELSE IF (IPEN.EQ.15) THEN
	  CALL SAO_DISPLAY_VECT
C  IPEN=16 and IPEN=17 are found above
C  IPEN=18 is reserved for TEK
	ELSE IF (IPEN.EQ.19) THEN !  Changes only the first entry (background)
	  SAO_VLT_COLORS(1)=LBCOLOR
	  DO I=1,N_SAO_VLT_COLORS
	    RED(I)=IAND(PLT$M_THIRDBYTE,SAO_VLT_COLORS(I))/256**2
	    GREEN(I)=IAND(PLT$M_SECONDBYTE,SAO_VLT_COLORS(I))/256
	    BLUE(I)=IAND(PLT$M_FIRSTBYTE,SAO_VLT_COLORS(I))
	  END DO
C  Fill the rest of the look-up table with nonoffensive numbers
C  If the numbers are all the same, say 0, when the SAOimage LUT palette
C  window is brought up, SAOiamge will crash.
C	  ISEED=12345
C	  DO I=N_SAO_VLT_COLORS+1,200
C	    RED(I)=RAN(ISEED)*5
C	    GREEN(I)=RAN(ISEED)*5
C	    BLUE(I)=RAN(ISEED)*5
C	  END DO
	  CALL SAO_LOAD_VLT(BLUE,GREEN,RED)
C  The following is an example of putting a caption on SAOimage
	ELSE IF (IPEN.EQ.20) THEN
	  NPIC_SAO=1
	  TV_FILENAME(NPIC_SAO)='TEST'
	  NPIC_SAO=1
	  CALL SAOCAPTION
	END IF
	RETURN
	END
