      SUBROUTINE TEKPAU(XD,YD,KEY)
C
C  Note:  This is the Unix version
C
C       This subroutine enables pausing in the plot mode of a graphics
C  terminal.  Alphanumeric data may then be sent to and from the
C  terminal.  The plot mode can be resumed by a call to TEKCON.  In one
C  mode of operation, the calling program will supply an initial X and Y, both
C  being positive.  TEKPAU will then position the cursor for writing at 
C  that location, with the restriction that the resulting x location will 
C  only be at one of two margins.  In another mode of operation, the 
C  calling program will not be concerned with the location;  each write
C  by the calling program will result in the Y location being automatically 
C  decremented, starting from the initial location.  The method is not
C  foolproof since it doesn't keep track of how many lines have been
C  written, and therefore may not know what margin is current.
C				Donald H. Gudehus, May, 1984
C		Version 2, Donald H. Gudehus, October, 1992
C		Version 3, D. H. G., January 1, 1998
C		Corrected misspelling of LEFT_MARGIN
C		Version 3U, Donald H. Gudehus, January 21, 2000
C		Converted the code to work under Unix
C		Version 4, Donald H. Gudehus, March 28, 2004
C		Brought OpenVMS and Unix versions closer together
C
C	XD	   (C)	The starting x position measured in inches
C			of the beginning of the text from the left side
C			of the screen.  Currently, the values will
C			be rounded off to the margins at 0.0 and 3.75.
C	YD	   (C)	The starting y position measured in inches
C			of the beginning of the text from the bottom of 
C			the screen.  The values are quantized to
C			increments of 0.16 inches.
C	KEY	   (C)	A key such that when KEY=0, the X and Y
C			in the call are used to determine where to
C			place the cursor;  when KEY=1, the y position
C			is decremented from the previously used value;
C
	COMMON /PLOT_CHAN/ NCHAN,NDEV
	COMMON /PLOT_TEXT/ ALPHAGRAPHICS,LEFT_MARGIN
C
	BYTE BUF(11),TAG(4),BYTE
	LOGICAL ALPHAGRAPHICS,LEFT_MARGIN
	BYTE BEL,CAN,CR,EM,ESC,FF,FS,GS,HT,LF,US,IO$_WRITEVBLK
	INTEGER*2 IO$M_NOFORMAT
	PARAMETER (BEL='07'X,CAN='18'X,CR='0D'X,EM='19'X,ESC='1B'X,
	1FF='0C'X,FS='1C'X,GS='1D'X,HT='09'X,LF='0A'X,US='1F'X,
	2IO$_WRITEVBLK='30'X,
	3IO$M_NOFORMAT='100'X)
C
	INCLUDE 'plotpar.for'
C
	DATA TAG/'20'X,'60'X,'20'X,'40'X/
	DATA LEFT_MARGIN /.TRUE./
C
C  Define a function to set up the bit pattern for a vector or point

	BYTE(ICOORD,N)=(ICOORD/32**(N-1).AND.'1F'X)
C
	ALPHAGRAPHICS=.TRUE.
	X=XD
	Y=YD
	IF (NDEV.EQ.1.OR.NDEV.EQ.4) THEN
	  Y=MOD(Y,5.55) !  5.54 is the highest value that works without
			!  producing a margin change.
	ELSE IF (NDEV.EQ.101.OR.NDEV.EQ.201.OR.NDEV.EQ.301) THEN
	  Y=MOD(Y,5.55) !  An assumption since it hasn't been tested.
C  On VersaTerm Pro, the highest practical Y is 5.44.  Higher values
C  cause overwriting but don't cause a margin change.
C  On Xterm, the highest practical value is 5.60.  Higher values cause
C  overwriting but don't cause a margin change.
	END IF
C  Some terminals have a finite spacing between lines.  For example, Xterm
C  has a spacing of about 0.16 inch.
	STEPSIZE=0.16  !  As observed on Xterm
	IF (KEY.EQ.0) THEN
	  BUF(1)=GS
	  II=PUTC(BUF(1))
	  IX=X*PLT$C_TEKPIX*PLT$C_NIBS
C  Make IY slightly higher and then later send a LF.  Unix is different than
C  OpenVMS.
	  IY=(Y+STEPSIZE+0.01)*PLT$C_TEKPIX*PLT$C_NIBS
	  BUF(1)=BYTE(IY,2).OR.TAG(1)
	  BUF(2)=BYTE(IY,1).OR.TAG(2)
	  BUF(3)=BYTE(IX,2).OR.TAG(3)
	  BUF(4)=BYTE(IX,1).OR.TAG(4)
	  DO I=1,4
	   II=PUTC(BUF(I))
	  END DO
C  If the other margin has been requested, get there via a
C  column of blank writes.
	  IF (X.GE.3.75.AND.LEFT_MARGIN) THEN
	    IF (NDEV.EQ.1.OR.NDEV.EQ.4.OR.NDEV.EQ.401) THEN
	      N=34
	    ELSE
	      N=35
	    END IF
	    DO I=1,N
	      WRITE(6,1000)
1000	      FORMAT(' ')
	    END DO
	    LEFT_MARGIN=.FALSE.
	  ELSE IF (X.LT.3.75.AND..NOT.LEFT_MARGIN) THEN
	    IF (NDEV.EQ.1.OR.NDEV.EQ.4.OR.NDEV.EQ.401) THEN
	      N=34
	    ELSE
	      N=35
	    END IF
	    DO I=1,N
	      WRITE(6,1000)
	    END DO
	    LEFT_MARGIN=.TRUE.
	  END IF
C  Make an attempt to keep track of margin changes.  This only works
C  under Unix.
	  IF (Y.LT.0.05) THEN
	    IF (LEFT_MARGIN) THEN
	      LEFT_MARGIN=.FALSE.
	    ELSE
	      LEFT_MARGIN=.TRUE.
	    END IF
	  END IF
	  CALL TEK (0,0,10)
C  Send the LF promised above
	  IF (NDEV.NE.401) THEN
	    BUF(1)=LF
	    II=PUTC(BUF(1))
	  END IF
	ELSE IF (KEY.EQ.1) THEN
	  CALL TEK (0,0,10)
	END IF
	RETURN
        END
