	SUBROUTINE XYPLOT(XARRAY,YARRAY,SIZE,NPLOTS,NPOINTS,LINVECT,
	1INC,LINTYPE,NSYMBOL,INVERT,COLOR,INTERACT,CROSSHAIRS,XLABEL,
	2YLABEL,TITLE,DEVICES,ADDENDA)
C
C  Note:  This is the Unix version.
C
C       This subroutine is a generalized plotting routine which can plot
C  several plots and/or histograms on one or more plot devices at the same
C  time with a variety of colors, symbols, vector line styles, pen
C  widths, and filling styles.  The scaling of the axes can be done
C  automatically or controlled by the user through the calling program
C  or interactivley by responding to prompts.  Linear and antilog annotation
C  of the axes is possible.  Additional axis annotation in different units can
C  be added on the right and upper sides of the plot.  By turning on the
C  cursor one can read back the physical values from the plot, where the
C  values are put into a COMMON block for use by the calling program.
C  The user can replot subsections of the plot defined by the cursor and
C  modify the values of the data through simple arithmetic operations.
C  Text, including Greek characters, can be added interactively or through
C  the calling program.  The object code belongs in the PAN library.
C			Donald H. Gudehus, November, 1983
C			Version 2, D. Gudehus, April, 1993
C			The maximum and minimum for command R
C			is now determined from all the plots instead
C			of just the first.
C			Version 3, D. Gudehus, July, 1993
C			Prevented erasing of screen when using H
C			command, if a graphics device with a separate
C			TEK window exists.
C			Version 4, D. Gudehus, April, 1995
C			Increased size of XCOORD and YCOORD from 68
C			to 1000.
C			Version 5, D. Gudehus, June, 1995
C			Added interactive command 's' to modify
C			symbols.
C			Version 6, D. Gudehus, July, 1998
C			Added interactive command 'm' to modify
C			individual plot
C			Version 6U, D. Gudehus, January, 2000
C			Modified code to work under Unix.
C			Version 7, D. Gudehus, January 22, 2000
C			Added interactive command 'b' to change
C			background color.
C			Version 8, D. Gudehus, May 6, 2000
C			Added code for X_INTERVALS_OFFSET and
C			Y_INTERVALS_OFFSET
C			Version 9, D. Gudehus, August 17, 2000
C			Allowed for nonplotted points in XARRAY
C			and YARRAY when replotting with commands
C			A, R, M, m, and P
C			Version 10, D. Gudehus, Jan. 1, 2003
C			Added X_INTERVALS_OFFSET and
C			Y_INTERVALS_OFFSET in call to ERRORBARS.
C			Version 11, D. Gudehus, Mar. 1, 2004
C			Fixed typo which affected commands B and R
C			if left X axis offsets had previously been
C			used.
C			Version 12, D. Gudehus, Mar. 19, 2004
C			Added DATA statement for OMIT which had been
C			left out.  Improved code in interactive
C			section.
C			Version 13, D. Gudehus, April 2, 2009
C			Fixed bug for interactive command "v" where
C			vector was not being drawn.
C			Version 14, Donald H. Gudehus, September 1, 2013
C			Added support for 64-bit processors.
C
C	XARRAY	   (C)	The array holding the data for the x postions of
C			each point for each plot.  A linear scaling of the
C			axes is automatically done from the range of
C			values of all the plots in the array, provided
C			that XARRAY(SIZE-3) equals 0.  If XARRAY(SIZE-3)
C			equals -1, an antilog scaling of the axes is
C			automatically done.  The linear scaling can be manually
C			controlled from the calling program by filling the
C			last 4 locations with the axis length, number of
C			intervals (coded), starting physical value, and physical
C			increment per interval.  The antilog scaling can be
C			manually controlled  from the calling program by
C			filling the last 4 locations with the axis length,
C			number of decades, starting physical value, and
C			zero.  Note that the fractional parts of the decades,
C			and starting value are antilogs.  Alternately, the
C			scaling can be done interactively if CROSSHAIRS is
C			.TRUE. and I is typed.  It is sometimes convenient to
C			have points in the array which should not be plotted.
C			This can be done by setting their value equal to
C			'FF7FFFFF'X.  In order to draw error bars or residual
C			vectors, the data for the locations of the ends of 
C			the lines are arranged in pairs, stored in array 
C			locations which differ by one in the second index
C			in XARRAY and YARRAY.  Alternately, if NSYMBOL 
C			contains a flag value, i.e., 100 added to the 
C			previous value, then instead of pairs of end-point
C			locations, one can use a central location and 
C			deviation.  For example, for a vertical errorbar 
C			located at (1.0,2.0) with a deviation of 0.1 we
C			could have XARRAY(1,1)=1.0, YARRAY(1,1)=2.0, 
C			XARRAY(1,2)=1.0, YARRAY(1,2)=0.1.  Note that 
C			XARRAY(1,2) can either repeat the value for
C			XARRAY(1,1) or be set equal to zero.  The increment 
C			mode can only be used for drawing errorbars, not 
C			residual vectors.
C	YARRAY	   (C)	Like XARRAY but for the y positions.
C	SIZE	   (C)	One of the dimensions of XARRAY and YARRAY.
C			It should be at least 4 larger than the
C			maximum number of points in any plot.
C	NPLOTS	   (C)	The number of plots to be simultaneously
C			produced and one the dimensions of XARRAY
C			and YARRAY.  If error bars are to be produced,
C			NPLOTS must be increased by 2;  the values
C			for the end points of the error bars should be
C			placed in XARRAY and YARRAY.
C	NPOINTS	   (C)	An array containing the number of data points
C			for each pair of arrays for each plot.
C	LINVECT	   (C)	An array giving the vector type (solid, dashed,
C			etc.) for each plot.  The numbers range from
C			0 to 4 for the VT100 and Visual 550, from 0 to 7
C			for the GraphOn, and from 0 to 15 for the
C			Lasergrafix.  See [MIIPS.PLOT]VECTMODE.FOR for 
C			some examples of the vector line stles.  One can
C			optionally control the pen width (only on the
C			Lasergrafix and Apple Laser Writer) by including the
C			width in thousandths of an inch in the
C			second byte of LINVECT.  If the pen width number
C			is not present, a default pen width (equal to
C			6 pixels on the Apple and 7 pixels on the Lasergrafix)
C			is selected.
C	INC	   (C)	An array containing the increment to be used in
C			stepping through XARRAY and YARRAY when plotting
C	LINTYPE	   (C)	An array controlling the type of line to be
C			drawn through the data points.  The magnitude of
C			LINTYPE controls the frequency of plotted
C			symbols:  e.g., if LINTYPE equals 4 a special
C			symbol drawn at every 4'th data point.  If
C			LINTYPE is zero, no symbols are plotted.  If
C			LINTYPE is positive, a straight line connects
C			the data points.  If LINTYPE is negative, no
C			connecting lines are drawn;  only the symbols
C			are plotted.  If LINTYPE = 997, error bars
C			are drawn;  if LINTYPE = 998, a histogram is
C			drawn with the bars stacked vertically if
C			multiple plots are made;  and if LINTYPE = 999,
C			a histogram is drawn with the bars placed next
C			to each other if multiple plots are made.
C	NSYMBOL	   (C)	An array giving the symbol for each plot.
C			Values from 0 to 13 are allowed.  If a character
C			is to be plotted instead of a symbol, the ASCII
C			code of the character should be placed in the
C			second byte of NSYMBOL.  If a histogram is to be
C			drawn, the value in the first byte gives the
C			the filling style (on the Visual 550, the GraphOn,
C			and the Lasergrafix 1200).  If error bars are to be
C			drawn, the value in the first byte gives the
C			style of the end of the error bar.  A zero gives
C			the usual short tick at that end;  a +1
C			gives nothing at that end;  a +2 gives an arrow-
C			head at that end.  If 100 is added to the first
C			byte, the X or Y locations contain the central
C			location and the deviation to the low and
C			high directions, respectively, for the adjacent
C			array locations.  The value in the third byte
C			specifies the symbol height in hundredths of an
C			inch.  If the value is not present, a default
C			of 0.14 inch is chosen.
C	INVERT	   (C)	An array controling the inversion of data
C			plotted in the x and y directions, respectively.
C	COLOR	   (C)	An array containing the color for each plot.
C			A value of zero causes that particular plot
C			not to be drawn.
C	INTERACT   (C)	A logical variable controlling whether the user 
C			will be prompted for plotting parameters.  If
C			.TRUE. the user is asked for whether the axes
C			are to be inverted, the lowest physical value
C			and increment, the axes' labels, the title, 
C			and the plot device.  In general this should
C			be set to .FALSE. and interaction done as
C			described below.
C	CROSSHAIRS (C)	A logical variable controlling whether the
C			crosshairs can be turned on and used to read
C			data values from the plot and redo the plot.
C			If .TRUE., then the following is true:  
C			Typing (not entering) A will turn on the crosshairs 
C			and by pressing the <CR> at chosen X limits, the mean
C			Y value will be calculated for each plot.
C			Typing a allows the user to enter
C			annotation at the location of the crosshairs
C			with the color most recently selected:  CNTRL I
C			(CNTRL L on Mac OS) toggles to the Greek alphabet and
C			special symbols and back to English, CNTRL H (CNTRL B
C			on Mac OS) gives a backspace, and CNTRL K toggles to
C			subscript, superscript, and back to normal.  Refer
C			to symgengr.for for the locations of the Greek
C			characters and special symbols on the keyboard.
C			CNTRL A prompts user for the height of the annotation
C			Typing B and defining a box with the crosshairs
C			will replot with user defined scaling.
C			Typing b allows the user to be prompted for the
C			color of the background.
C			Typing C turns on the crosshairs, pressing the
C			<CR> prints the values at that location and puts
C			the values in a common block, typing - deletes
C			the last pair of values from the common block,
C			and typing Q turns off the crosshairs.
C			Typing c allows the user to be prompted for the
C			color of a vector.
C			Typing F allows the user to alter the scaling factor
C			for the current plot device(s).
C			Typing f allows the user to change the fill style for
C			each plot.
C			Typing I allows the user to be prompted for
C			several of the plotting parameters before a
C			replot is done.
C			Typing L allows the user to take the log.
C			Typing l allows the user to take the natural log.
C			Typing M allows the user to modify the X and Y values
C			by adding increments and adjusting by scaling factors.
C			Typing m allows the user to modify the X and Y values 
C			by adding increments and adjusting by scaling factors
C			for a selected plot.
C			Typing O redoes the original plot.  If keys X, x, Y, or
C			y were pressed, additional axes will be plotted.
C			Typing o redoes the original plot.  If keys X, x, Y, or
C			y were pressed, that information is cancelled.
C			Typing P allows the user to take powers.
C			Typing p allows the user to determine the color of
C			each plot
C			Typing Q or q will return.
C			Typing R will turn on the crosshairs and by
C			pressing the <CR> at chosen X limits, replotting
C			with automatic scaling will be done.
C			Typing S allows the user to specify the style of vector
C			Typing s allows the user to specify symbols for sets
C			of points.
C			Typing V allows cursor input for drawing a solid
C			vector of the color and line style most recently
C			selected.
C			Typing v allows the user to be prompted for the
C			starting and ending physical values of a vector.
C			Typing X or Y allows plotting additional linearly
C			scaled axes in different units.
C			Typing x or y allows plotting additional antilog
C			scaled axes in different units.
C	XLABEL	   (C)	The label to be displayed along the x axis.
C			Control I (control L on Mac OS) toggles between the
C			Greek and English alphabets; 
C			control K toggles between subscripts, superscripts,
C			and normal type;  and
C			control H (control B on Mac OS) will insert a 
C			backspace.  Two spaces at the beginning will supress
C			the annotation of the axis.
C	YLABEL	   (C)	The label to be displayed along the y axis.
C	TITLE	   (C)	The title to be displayed in the upper right of the
C			plot.
C	DEVICES	   (C)	The plotting device(s).  If it is blank in the
C			in the call, the user is prompted for it.  See
C			PLOTDEVICES.FOR for the available devices.
C	ADDENDA	   (C)	An argument which if .TRUE. will
C			prevent a call to PLOT with IPEN = 999, thus
C			allowing further plotting and annotation by
C			the calling program.  The calling program should
C			call  PLOT with IPEN = 999 when all additional
C			plotting is completed.  All coordinates for
C			annotation are measured in inches from the lower
C			left corner of the plot.
C
	PARAMETER (NSYMBOL_PAR1=4,NSYMBOL_PAR2=100)
	CHARACTER*(*) XLABEL,YLABEL,TITLE,DEVICES
	CHARACTER*3 DUMDEVICES(10)
	CHARACTER*30 DUMDEVICES2
	CHARACTER*50 OXLABEL,OYLABEL,OTITLE,
	1DUMXLABEL,DUMYLABEL,DUMTITLE,XXLABEL,YYLABEL
	CHARACTER*1 XSCALEMODE,YSCALEMODE,OXSCALEMODE,OYSCALEMODE,
	1XXSCALEMODE,YYSCALEMODE,OXXSCALEMODE,OYYSCALEMODE
	CHARACTER*132 STRING
c	EXTERNAL CLI$M_NOCLISYM,CLI$M_NOLOGNAM,CLI$M_NOKEYPAD
	INTEGER SIZE,DUMSIZE,PLOT_NUMBER
C	INTEGER SYS$NUMARGS,MASK,STATUS,LIB$SPAWN
C	INTEGER ONDEVICE
	INTEGER*2 FLAG(5)
	INTEGER COLOR(NPLOTS),TEMPCOLOR,
	1SYMBOL_PAR(NSYMBOL_PAR1,NSYMBOL_PAR2)
 	DIMENSION XARRAY(SIZE,NPLOTS),YARRAY(SIZE,NPLOTS),
	1NPOINTS(NPLOTS),LINVECT(NPLOTS),INC(NPLOTS),LINTYPE(NPLOTS),
	2NSYMBOL(NPLOTS),XDUMARRAY(4),YDUMARRAY(4),FACTORS(2),
	3INCREMENTS(2),SYMBLHT(1000),OSYMBLHT(1000),VECTPARAM(4)
	INTEGER*4 ADDRESS1_4
	INTEGER*8 ADDRESS1_8
	LOGICAL INVERT(2),OINVERT(2),CROSSHAIRS,INTERACT,I_INTERACT,
	1ADDENDA,
	2SYMBOLS,SUBSCRIPTFLAG,SUPERSCRIPTFLAG,FWASTYPED,TAKELOG,TAKEPOWER,
	3xxALTYPED,yyALTYPED,XXTYPED,YYTYPED,PLOTDEVQUES
	REAL INCREMENTS
	INCLUDE 'plotpar.for'
C
	COMMON /PLOT_BLOCK/ XCOORD(1000),YCOORD(1000)
C
	COMMON /PLOT_CHAN/ NCHAN,NDEV
C
	INTEGER SYSTEM_NUMBER_BYTES
	COMMON /SYS_NUM_BYTES/ SYSTEM_NUMBER_BYTES
C
	DATA OMIT /'FF7FFFFF'X/
C
	DO I=1,1000
	  J=SETEND(XCOORD(I))
	  J=SETEND(YCOORD(I))
	END DO
C  Determine if symbols are to be plotted
	SYMBOLS=.FALSE.
	DO I=1,NPLOTS
	  IF (ABS(LINTYPE(I)).GT.0.AND.(LINTYPE(I).NE.997.AND.LINTYPE(I)
	1 .NE.998.AND.LINTYPE(I).NE.999)) SYMBOLS=.TRUE.
	END DO
	XAXIS_OFFSET=1.0
	YAXIS_OFFSET=0.6
	ISTART=1  ! Only 1 is supported now
	ANNOTATION_HEIGHT=0.14
	I_SYM_PAR=0  ! Counter for changes to symbol parameters
	DO IS=1,NSYMBOL_PAR2
	  SYMBOL_PAR(1,IS)=0
	END DO
	DUMSIZE=SIZE-ISTART+1
C  Determine the scaling mode and obtain the original values for a replot
	IF (XARRAY(DUMSIZE-3,1).EQ.0) THEN
	  XSCALEMODE='A'  ! Bottom
	  XXSCALEMODE='A'  ! Top
C  The original XDELTAV and XFIRSTV are not available at this point in the
C  program for this XSCALEMODE.
	  OX_INTERVALS=6.0
	  OXAXIS_LENGTH=6.0
	ELSE IF (XARRAY(DUMSIZE-3,1).NE.0.0.AND.XARRAY(DUMSIZE,1).NE.0.0) THEN
	  XSCALEMODE='B'
	  XXSCALEMODE='B'
	  OXDELTAV=XARRAY(DUMSIZE,1)  ! Physical value/interval
	  OXFIRSTV=XARRAY(DUMSIZE-1,1)
	  OX_INTERVALS=XARRAY(DUMSIZE-2,1)
	  OXAXIS_LENGTH=XARRAY(DUMSIZE-3,1)
	ELSE IF (XARRAY(DUMSIZE-3,1).EQ.-1) THEN
	  XSCALEMODE='C'
	  XXSCALEMODE='C'
C  The original X_DECADES and XFIRSTDECVAL are not available at this point in
C  the program for this XSCALEMODE.
	  OX_INTERVALS=6.0
	  OXAXIS_LENGTH=6.0
	ELSE IF (XARRAY(DUMSIZE-3,1).NE.0.0.AND.XARRAY(DUMSIZE,1).EQ.0.0) THEN
	  XSCALEMODE='D'
	  XXSCALEMODE='D'
	  OX_INTERVALS=6.0
	  OXFIRSTDECVAL=XARRAY(DUMSIZE-1,1)
	  OX_DECADES=XARRAY(DUMSIZE-2,1)
	  OXAXIS_LENGTH=XARRAY(DUMSIZE-3,1)
	END IF
	IF (YARRAY(DUMSIZE-3,1).EQ.0) THEN
	  YSCALEMODE='A'  ! Left
	  YYSCALEMODE='A'  ! Right
	  OY_INTERVALS=5.0
	  OYAXIS_LENGTH=5.0
	ELSE IF (YARRAY(DUMSIZE-3,1).NE.0.0.AND.YARRAY(DUMSIZE,1).NE.0.0) THEN
	  YSCALEMODE='B'
	  YYSCALEMODE='B'
	  OYDELTAV=YARRAY(DUMSIZE,1)
	  OYFIRSTV=YARRAY(DUMSIZE-1,1)
	  OY_INTERVALS=YARRAY(DUMSIZE-2,1)
	  OYAXIS_LENGTH=YARRAY(DUMSIZE-3,1)
	ELSE IF (YARRAY(DUMSIZE-3,1).EQ.-1) THEN
	  YSCALEMODE='C'
	  YYSCALEMODE='C'
	  OY_INTERVALS=5.0
	  OYAXIS_LENGTH=5.0
	ELSE IF (YARRAY(DUMSIZE-3,1).NE.0.0.AND.YARRAY(DUMSIZE,1).EQ.0.0) THEN
	  YSCALEMODE='D'
	  YYSCALEMODE='D'
	  OY_INTERVALS=5.0
	  OYFIRSTDECVAL=YARRAY(DUMSIZE-1,1)
	  OY_DECADES=YARRAY(DUMSIZE-2,1)
	  OYAXIS_LENGTH=YARRAY(DUMSIZE-3,1)
	END IF
	OXLABEL=XLABEL
	OYLABEL=YLABEL
	OTITLE=TITLE
	DUMXLABEL=XLABEL
	DUMYLABEL=YLABEL
	DUMTITLE=TITLE
	OINVERT(1)=INVERT(1)
	OINVERT(2)=INVERT(2)
	OXSCALEMODE=XSCALEMODE
	OYSCALEMODE=YSCALEMODE
	OXXSCALEMODE=XXSCALEMODE
	OYYSCALEMODE=YYSCALEMODE
	DO I=1,NPLOTS
	  IF ((PLT$M_THIRDBYTE.AND.NSYMBOL(I)).NE.0) THEN
	    SYMBLHT(I)=(PLT$M_THIRDBYTE.AND.NSYMBOL(I))/6553600.0
	  ELSE
	    SYMBLHT(I)=0.14
	  END IF
	  OSYMBLHT(I)=SYMBLHT(I)
	END DO
	FWASTYPED=.FALSE.
	XFACTOR=0.0  ! For top axis with linear scaling
	YFACTOR=0.0  ! For right axis with linear scaling
	XINCREMENT=0.0  ! For top axis with antilog scaling
	YINCREMENT=0.0  ! For right axis with antilog scaling
	xxALTYPED=.FALSE.
	yyALTYPED=.FALSE.
	XXTYPED=.FALSE.
	YYTYPED=.FALSE.
C
   5	IF (INTERACT.OR.I_INTERACT) THEN !  Return point for typing I, but not
					 !  other letters

	  IF (PLOTDEVQUES('GD',1)) CALL MESSAGE(' ')
	  IF (PLOTDEVQUES('2WT',1)) THEN
	    CALL TEK(0,0,4)
	    CALL MESSAGE(' ')
	  ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	    CALL TEKPAU(0.0,0.0,1)
	  END IF

	  INVERT(1)=.FALSE.
	  CALL USRINP(INVERT(1),1,'L',4,1,'INVERT=',
	1 'Do you wish to invert the X axis?  (default = N)')
	  INVERT(2)=.FALSE.
	  CALL USRINP(INVERT(2),1,'L',4,1,'INVERT=',
	1 'Do you wish to invert the Y axis?  (default = N)')
   6	  XSCALEMODE='A'
	  CALL MESSAGE('Enter the X scale mode:')
	  CALL MESSAGE('     A = Linear automatic (default)')
	  CALL MESSAGE('     B = Linear manual')
	  CALL MESSAGE('     C = Antilog automatic')
	  CALL USRINP(%REF(XSCALEMODE),1,'A',1,1,'X_SCALE_MODE=',
	1 '     D = Antilog manual')
	  IF (XSCALEMODE.NE.'A'.AND.XSCALEMODE.NE.'B'.AND.XSCALEMODE.NE.
	1 'C'.AND.XSCALEMODE.NE.'D') GO TO 6
C  Reset XXSCALEMODE in case "I" was typed
	  IF (.NOT.XXTYPED.AND..NOT.xxALTYPED) XXSCALEMODE=XSCALEMODE
	  IF (XSCALEMODE.EQ.'B') THEN
	    CALL USRINP(XARRAY(DUMSIZE-1,1),1,'F',4,0,'X_LOWEST_VALUE=',
	1  'Enter the lowest labeled physical value for the X axis:')
	    CALL USRINP(XARRAY(DUMSIZE,1),1,'F',4,0,'DELTA_X=',
	1   'Enter the X physical value/interval:')
	    XARRAY(DUMSIZE-2,1)=6.0
	    CALL USRINP(XARRAY(DUMSIZE-2,1),1,'F',4,1,'X_INTERVALS=',
	1   'Enter the number of X intervals (coded):  (default = 6.0)')
	    XARRAY(DUMSIZE-3,1)=6.0
	    CALL USRINP(XARRAY(DUMSIZE-3,1),1,'F',4,1,'X_AXIS_LENGTH=',
	1   'Enter the X axis length:  (default = 6.0)')
	  ELSE IF (XSCALEMODE.EQ.'D') THEN
	    CALL USRINP(XARRAY(DUMSIZE-1,1),1,'F',4,0,'X_LOWEST_VALUE=',
	1  'Enter the lowest value (coded log.antilog) for the X axis:')
	    CALL USRINP(XARRAY(DUMSIZE-2,1),1,'F',4,0,'X_DECADES=',
	1   'Enter the number of X decades (coded log.antilog):')
	    XARRAY(DUMSIZE-3,1)=6.0
	    CALL USRINP(XARRAY(DUMSIZE-3,1),1,'F',4,1,'X_AXIS_LENGTH=',
	1   'Enter the X axis length:  (default = 6.0)')
	  END IF
   7	  YSCALEMODE='A'
	  CALL MESSAGE('Enter the Y scale mode:')
	  CALL MESSAGE('     A = Linear automatic (default)')
	  CALL MESSAGE('     B = Linear manual')
	  CALL MESSAGE('     C = Antilog automatic')
	  CALL USRINP(%REF(YSCALEMODE),1,'A',1,1,'Y_SCALE_MODE=',
	1 '     D = Antilog manual')
	  IF (YSCALEMODE.NE.'A'.AND.YSCALEMODE.NE.'B'.AND.YSCALEMODE.NE.
	1 'C'.AND.YSCALEMODE.NE.'D') GO TO 7
C  Reset YYSCALEMODE in case "I" was typed
	  IF (.NOT.YYTYPED.AND..NOT.yyALTYPED) YYSCALEMODE=YSCALEMODE
	  IF (YSCALEMODE.EQ.'B') THEN
	    CALL USRINP(YARRAY(DUMSIZE-1,1),1,'F',4,0,'Y_LOWEST_VALUE=',
	1   'Enter the lowest labeled physical value for the Y axis:')
	    CALL USRINP(YARRAY(DUMSIZE,1),1,'F',4,0,'DELTA_Y=',
	1   'Enter the Y physical value/interval:')
	    YARRAY(DUMSIZE-2,1)=5.0
	    CALL USRINP(YARRAY(DUMSIZE-2,1),1,'F',4,1,'Y_INTERVALS=',
	1   'Enter the number of Y intervals (coded):  (default = 5.0)')
	    YARRAY(DUMSIZE-3,1)=5.0
	    CALL USRINP(YARRAY(DUMSIZE-3,1),1,'F',4,1,'Y_AXIS_LENGTH=',
	1   'Enter the Y axis length:  (default = 5.0)')
	  ELSE IF (YSCALEMODE.EQ.'D') THEN
	    CALL USRINP(YARRAY(DUMSIZE-1,1),1,'F',4,0,'Y_LOWEST_VALUE=',
	1   'Enter the lowest value (coded log.antilog) for the Y axis:')
	    CALL USRINP(YARRAY(DUMSIZE-2,1),1,'F',4,0,'Y_DECADES=',
	1   'Enter the number of Y decades (coded log.antilog):')
	    YARRAY(DUMSIZE-3,1)=5.0
	    CALL USRINP(YARRAY(DUMSIZE-3,1),1,'F',4,1,'Y_AXIS_LENGTH=',
	1   'Enter the Y axis length:  (default = 5.0)')
	  END IF
	  IF (SYMBOLS) THEN
	    CALL USRINP(SYMBLHT,NPLOTS,'F',4,1,'SYMBOL_HEIGHT=',
	1   'Enter the symbol heights:  (defaults = values in call)')
	  END IF
	  IF (INTERACT) THEN
	    DUMXLABEL='X AXIS'
	    CALL USRINP(%REF(DUMXLABEL),1,'A',50,1,'X_LABEL=',
	1   'Enter the label for the X axis:  (default = X AXIS)')
	    DUMYLABEL='Y AXIS'
	    CALL USRINP(%REF(DUMYLABEL),1,'A',50,1,'Y_LABEL=',
	1   'Enter the label for the Y axis:  (default = Y AXIS)')
	    DUMTITLE=' '
	    CALL USRINP(%REF(DUMTITLE),1,'A',50,1,'TITLE=',
	1   'Enter the title:  (default = blank)')
	  ELSE IF (I_INTERACT) THEN
	    DUMXLABEL=OXLABEL
	    CALL USRINP(%REF(DUMXLABEL),1,'A',50,1,'X_LABEL=',
	1   'Enter the label for the X axis:  (default = old label)')
	    DUMYLABEL=OYLABEL
	    CALL USRINP(%REF(DUMYLABEL),1,'A',50,1,'Y_LABEL=',
	1   'Enter the label for the Y axis:  (default = old label)')
	    DUMTITLE=OTITLE
	    CALL USRINP(%REF(DUMTITLE),1,'A',50,1,'TITLE=',
	1   'Enter the title:  (default = old title)')
	  END IF
	END IF
	DUMDEVICES2=DEVICES
	LL=1
	DO J=1,10
	  DO I=1,3
	    DUMDEVICES(J)(I:I)=DUMDEVICES2(LL:LL)
	    LL=LL+1
	  END DO
	END DO
	CALL ALPHA(%REF(DUMDEVICES),10,3,NC,NF,NR,4)
	NUMBERDEVICES=NF
	IF (NUMBERDEVICES.EQ.0.OR.INTERACT.OR.I_INTERACT) THEN
	  DUMDEVICES(1)=' '
	  CALL PLOTDEVICES(DUMDEVICES)
	  I_INTERACT=.FALSE.  ! We are done with I_INTERACT this time around
	END IF
C	ONDEVICE=NDUMDEVICE
C  Return point for commands B, R, O, or o
  10	CALL ALPHA(%REF(DUMTITLE),1,50,NCT,NF,NR,1)
	CALL ALPHA(%REF(DUMTITLE),1,50,NCTIT,NF,NR,3)
	IF (XSCALEMODE.EQ.'A') THEN
	  XAXIS_LENGTH=6.0
	  XARRAY(DUMSIZE-3,1)=XAXIS_LENGTH
	  X_INTERVALS=6.0 !  No need to decode
	  X_INTERVALS_OFFSET_L=0.0
	  X_INTERVALS_OFFSET_R=0.0 !  Not currently used
	  XARRAY(DUMSIZE-2,1)=X_INTERVALS
	  CALL SCALE_AXIS(XARRAY(ISTART,1),DUMSIZE,NPLOTS,NPOINTS,
	1 X_INTERVALS,1,LINTYPE,NSYMBOL,COLOR,.FALSE.)
	  XFIRSTV=XARRAY(DUMSIZE-1,1)
	  XDELTAV=XARRAY(DUMSIZE,1)  ! Physical value/interval
	ELSE IF (XSCALEMODE.EQ.'B') THEN
	  XAXIS_LENGTH=XARRAY(DUMSIZE-3,1)
	  CALL DECODE_INTERVALS(XARRAY(DUMSIZE-2,1),X_INTERVALS,
	1 X_INTERVALS_OFFSET_L,X_INTERVALS_OFFSET_R)
	  XFIRSTV=XARRAY(DUMSIZE-1,1)
	  XDELTAV=XARRAY(DUMSIZE,1)  ! Physical value/interval
	ELSE IF (XSCALEMODE.EQ.'C') THEN
	  XAXIS_LENGTH=6.0
	  XARRAY(DUMSIZE-3,1)=XAXIS_LENGTH
	  X_INTERVALS=6.0  ! Preparation for call to SCALE
	  X_INTERVALS_OFFSET_L=0.0
	  X_INTERVALS_OFFSET_R=0.0 !  Not currently used
	  XARRAY(DUMSIZE-2,1)=X_INTERVALS
	  CALL SCALE_AXIS(XARRAY(ISTART,1),DUMSIZE,NPLOTS,NPOINTS,
	1 X_INTERVALS,1,LINTYPE,NSYMBOL,COLOR,.FALSE.)
C  Make an adjustment to the results of the call to SCALE
C  Note that the integer part will be a log and the fractional part is an
C  antilog
	  X_DECADES=ALOGDECCODE(XARRAY(DUMSIZE-1,1),
	1 X_INTERVALS*XARRAY(DUMSIZE,1),2)
	  XFIRSTDECVAL=ALOGCODE(XARRAY(DUMSIZE-1,1),1) !  Needed for ALOGAXIS
	  XARRAY(DUMSIZE,1)=(ALOGDECODE(X_DECADES)-
	1 ALOGDECODE(XFIRSTDECVAL-INT(XFIRSTDECVAL)))/X_INTERVALS
	  XARRAY(DUMSIZE-1,1)=ALOGDECODE(XFIRSTDECVAL) !  For LINE, etc.
C  Get a value for XFIRSTV in case the cursor is used
	  XFIRSTV=XARRAY(DUMSIZE-1,1)
C  Get a value for XDELTAV in case the cursor is used
	  XDELTAV=XARRAY(DUMSIZE,1)
	ELSE IF (XSCALEMODE.EQ.'D') THEN
	  XAXIS_LENGTH=XARRAY(DUMSIZE-3,1)
	  X_INTERVALS=6.0
	  X_INTERVALS_OFFSET_L=0.0
	  X_INTERVALS_OFFSET_R=0.0 !  Not currently used
	  X_DECADES=XARRAY(DUMSIZE-2,1)
	  XFIRSTDECVAL=XARRAY(DUMSIZE-1,1)
	  XARRAY(DUMSIZE-1,1)=ALOGDECODE(XFIRSTDECVAL)
C  Get a value for XFIRSTV in case the cursor is used
	  XFIRSTV=XARRAY(DUMSIZE-1,1)
	  XARRAY(DUMSIZE,1)=(ALOGDECODE(X_DECADES)-
	1 ALOGDECODE(XFIRSTDECVAL-INT(XFIRSTDECVAL)))/X_INTERVALS
C  Get a value for XDELTAV in case the cursor is used
	  XDELTAV=XARRAY(DUMSIZE,1)
	END IF
C  For eventual automatic operation of top antilog axis
	IF (xxALTYPED) THEN
C	  IF (XSCALEMODE.EQ.'C'.OR.XSCALEMODE.EQ.'D') THEN
C	    XX_DECADES=X_DECADES
C	    XXFIRSTDECVAL=XFIRSTDECVAL
C	  ELSE 
C	    XX_DECADES=ALOGDECCODE(XARRAY(DUMSIZE-1,1),
C	1   X_INTERVALS*XARRAY(DUMSIZE,1),2)
C	    XXFIRSTDECVAL=ALOGCODE(XARRAY(DUMSIZE-1,1),1) !  Needed for ALOGAXIS
C	  END IF
	END IF
	IF (YSCALEMODE.EQ.'A') THEN
	  YAXIS_LENGTH=5.0
	  YARRAY(DUMSIZE-3,1)=YAXIS_LENGTH
	  Y_INTERVALS=5.0 !  No need to decode
	  Y_INTERVALS_OFFSET_L=0.0
	  Y_INTERVALS_OFFSET_R=0.0 !  Not currently used
	  YARRAY(DUMSIZE-2,1)=Y_INTERVALS
	  CALL SCALE_AXIS(YARRAY(ISTART,1),DUMSIZE,NPLOTS,NPOINTS,
	1 Y_INTERVALS,1,LINTYPE,NSYMBOL,COLOR,.TRUE.)
	  YFIRSTV=YARRAY(DUMSIZE-1,1)
	  YDELTAV=YARRAY(DUMSIZE,1)
	ELSE IF (YSCALEMODE.EQ.'B') THEN
	  YAXIS_LENGTH=YARRAY(DUMSIZE-3,1)
	  CALL DECODE_INTERVALS(YARRAY(DUMSIZE-2,1),Y_INTERVALS,
	1 Y_INTERVALS_OFFSET_L,Y_INTERVALS_OFFSET_R)
	  YFIRSTV=YARRAY(DUMSIZE-1,1)
	  YDELTAV=YARRAY(DUMSIZE,1)
	ELSE IF (YSCALEMODE.EQ.'C') THEN
	  YAXIS_LENGTH=5.0
	  YARRAY(DUMSIZE-3,1)=YAXIS_LENGTH
	  Y_INTERVALS=5.0
	  Y_INTERVALS_OFFSET_L=0.0
	  Y_INTERVALS_OFFSET_R=0.0 !  Not currently used
	  YARRAY(DUMSIZE-2,1)=Y_INTERVALS
	  CALL SCALE_AXIS(YARRAY(ISTART,1),DUMSIZE,NPLOTS,NPOINTS,
	1 Y_INTERVALS,1,LINTYPE,NSYMBOL,COLOR,.TRUE.)
C  Make an adjustment to the results of the call to SCALE
C  Note that the integer part is a log and the fractional part is an antilog
	  Y_DECADES=ALOGDECCODE(YARRAY(DUMSIZE-1,1),
	1 Y_INTERVALS*YARRAY(DUMSIZE,1),2)
	  YFIRSTDECVAL=ALOGCODE(YARRAY(DUMSIZE-1,1),1) !  Needed for ALOGAXIS
	  YARRAY(DUMSIZE,1)=(ALOGDECODE(Y_DECADES)-
	1 ALOGDECODE(YFIRSTDECVAL-INT(YFIRSTDECVAL)))/Y_INTERVALS
	  YARRAY(DUMSIZE-1,1)=ALOGDECODE(YFIRSTDECVAL) !  For LINE, etc.
C  Set YFIRSTV in case the cursor is used
	  YFIRSTV=YARRAY(DUMSIZE-1,1)
C  Set YDELTAV in case the cursor is used
	  YDELTAV=YARRAY(DUMSIZE,1)
	ELSE IF (YSCALEMODE.EQ.'D') THEN
	  YAXIS_LENGTH=YARRAY(DUMSIZE-3,1)
	  Y_INTERVALS=5.0
	  Y_INTEVALS_OFFSET=0.0
	  Y_DECADES=YARRAY(DUMSIZE-2,1)
	  YFIRSTDECVAL=YARRAY(DUMSIZE-1,1)
	  YARRAY(DUMSIZE-1,1)=ALOGDECODE(YFIRSTDECVAL)
C  Get a value for YFIRSTV in case the cursor is used
	  YFIRSTV=YARRAY(DUMSIZE-1,1)
	  YARRAY(DUMSIZE,1)=(ALOGDECODE(Y_DECADES)-
	1 ALOGDECODE(YFIRSTDECVAL-INT(YFIRSTDECVAL)))/Y_INTERVALS
C  Get a value for YDELTAV in case the cursor is used
	  YDELTAV=YARRAY(DUMSIZE,1)
	END IF
	IF (YYSCALEMODE.EQ.'C') THEN
C	  YY_DECADES=ALOGDECCODE(YARRAY(DUMSIZE-1,1),
C	1 Y_INTERVALS*YARRAY(DUMSIZE,1),2)
C	  YYFIRSTDECVAL=ALOGCODE(YARRAY(DUMSIZE-1,1),1) !  Needed for ALOGAXIS
	END IF
C
	CALL PLOTDEVICES2(DUMDEVICES)
C  Allow devices which are slow when sending vectors to do so in a
C  delayed (block) mode 
	IF (PLOTDEVQUES('D*',0)) CALL PLOT(0,0,14)
	IF (FWASTYPED) THEN
	  CALL FACTOR_LU(FACT,7)
	  FWASTYPED=.FALSE.
	END IF
	CALL PLOT(XAXIS_OFFSET,YAXIS_OFFSET,-3)
C  Initialize the line count for cursor read values
	LINECOUNT=1
	FLAG(1)=1
	FLAG(2)=-1
	FLAG(3)=0
	FLAG(4)=1
	IF (INVERT(1)) THEN
	  FLAG(5)=1
	ELSE
	  FLAG(5)=0
	END IF
C  Initialize the line style to solid lines
	CALL VECTMODE(0)
	IF (XSCALEMODE.EQ.'A'.OR.XSCALEMODE.EQ.'B') THEN
	  CALL AXIS(0.0,0.0,%REF(DUMXLABEL),XAXIS_LENGTH,0.0,XFIRSTV,
	1 XDELTAV,X_INTERVALS,X_INTERVALS_OFFSET_L,FLAG)
	ELSE IF (XSCALEMODE.EQ.'C'.OR.XSCALEMODE.EQ.'D') THEN
	  CALL ALOGAXIS(0.0,0.0,%REF(DUMXLABEL),XAXIS_LENGTH,0.0,XFIRSTDECVAL,
	1 X_DECADES,FLAG)
	END IF
	FLAG(1)=-1
	FLAG(2)=1
	FLAG(3)=1
	IF (INVERT(2)) THEN
	  FLAG(5)=1
	ELSE
	  FLAG(5)=0
	END IF
	IF (YSCALEMODE.EQ.'A'.OR.YSCALEMODE.EQ.'B') THEN
	  CALL AXIS(0.0,0.0,%REF(DUMYLABEL),YAXIS_LENGTH,90.0,YFIRSTV,
	1 YDELTAV,Y_INTERVALS,Y_INTERVALS_OFFSET_L,FLAG)
	ELSE IF (YSCALEMODE.EQ.'C'.OR.YSCALEMODE.EQ.'D') THEN
	  CALL ALOGAXIS(0.0,0.0,%REF(DUMYLABEL),YAXIS_LENGTH,90.0,
	1 YFIRSTDECVAL,Y_DECADES,FLAG)
	END IF
	FLAG(1)=1
	IF (YYSCALEMODE.EQ.'A'.OR.YYSCALEMODE.EQ.'B') THEN
	  IF (YFACTOR.EQ.0.0.AND.(YSCALEMODE.EQ.'A'.OR.YSCALEMODE.EQ.'B')) THEN
	    FLAG(4)=0
	    CALL AXIS(XAXIS_LENGTH,0.0,%REF(' '),YAXIS_LENGTH,90.0,
	1   YFIRSTV,YDELTAV,Y_INTERVALS,Y_INTERVALS_OFFSET_L,FLAG)
	  ELSE
	    FLAG(2)=-1
	    FLAG(4)=1
	    CALL AXIS(XAXIS_LENGTH,0.0,%REF(YYLABEL),YAXIS_LENGTH,90.0,
	1   YFIRSTV*YFACTOR,YYDELTAV,
	2   Y_INTERVALS*YDELTAV*YFACTOR/YYDELTAV,
	3   Y_INTERVALS_OFFSET_L*YDELTAV*YFACTOR/YYDELTAV,FLAG)
	  END IF
	ELSE IF (YYSCALEMODE.EQ.'C'.OR.YYSCALEMODE.EQ.'D') THEN
	  IF (.NOT.yyALTYPED.AND.(YSCALEMODE.EQ.'C'.OR.YSCALEMODE.EQ.'D'))
	1 THEN
	    FLAG(4)=0
	    CALL ALOGAXIS(XAXIS_LENGTH,0.0,%REF(' '),YAXIS_LENGTH,90.0,
	1   YFIRSTDECVAL,Y_DECADES,FLAG)
	  ELSE
	    FLAG(2)=-1
	    FLAG(4)=1
	    CALL ALOGAXIS(XAXIS_LENGTH,0.0,%REF(YYLABEL),YAXIS_LENGTH,90.0,
	1   YYFIRSTDECVAL+YINCREMENT,YY_DECADES,FLAG)
	  END IF
	END IF
	FLAG(1)=-1
	IF (INVERT(1)) THEN
	  FLAG(5)=1
	ELSE
	  FLAG(5)=0
	END IF
	IF (XXSCALEMODE.EQ.'A'.OR.XXSCALEMODE.EQ.'B') THEN
	  IF (XFACTOR.EQ.0.0.AND.(XSCALEMODE.EQ.'A'.OR.XSCALEMODE.EQ.'B')) THEN
	    FLAG(4)=0
	    CALL AXIS(0.0,YAXIS_LENGTH,%REF(' '),XAXIS_LENGTH,0.0,
	1   XFIRSTV,XDELTAV,X_INTERVALS,X_INTERVALS_OFFSET_L,FLAG)
	  ELSE
	    FLAG(2)=1
	    FLAG(3)=0
	    FLAG(4)=1
	    CALL AXIS(0.0,YAXIS_LENGTH,%REF(XXLABEL),XAXIS_LENGTH,0.0,
	1   XFIRSTV*XFACTOR,XXDELTAV,
	2   X_INTERVALS*XDELTAV*XFACTOR/XXDELTAV,
	3   X_INTERVALS_OFFSET_L*XDELTAV*XFACTOR/XXDELTAV,FLAG)
	  END IF
	ELSE IF (XXSCALEMODE.EQ.'C'.OR.XXSCALEMODE.EQ.'D') THEN
	  IF (.NOT.xxALTYPED.AND.(XSCALEMODE.EQ.'C'.OR.XSCALEMODE.EQ.'D'))
	1 THEN
	    FLAG(4)=0
	    CALL ALOGAXIS(0.0,YAXIS_LENGTH,%REF(' '),XAXIS_LENGTH,0.0,
	1   XFIRSTDECVAL,X_DECADES,FLAG)
	  ELSE
	    FLAG(2)=1
	    FLAG(3)=0
	    FLAG(4)=1
	    CALL ALOGAXIS(0.0,YAXIS_LENGTH,%REF(XXLABEL),XAXIS_LENGTH,0.0,
	1   XXFIRSTDECVAL+XINCREMENT,XX_DECADES,FLAG)
	  END IF
	END IF
C  This needs to be amended for antilog axes
	DO I=1,NPLOTS  ! This sets up the values for the call to
		       ! to HISTOGRAM and ERRORBAR also
	  XARRAY(DUMSIZE-1,I)=XFIRSTV
	  XARRAY(DUMSIZE,I)=XDELTAV
	  YARRAY(DUMSIZE-1,I)=YFIRSTV
	  YARRAY(DUMSIZE,I)=YDELTAV
	END DO
C  Display any vectors which haven't yet been displayed, for those devices
C  which are using block mode, such as SAOimage.
	IF (PLOTDEVQUES('D*',0)) CALL PLOT(0,0,15)
C  Plot the histogram first so that call to LINE will write over the
C  bars
	CALL HISTOGRAM(XARRAY,YARRAY,DUMSIZE,NPLOTS,ISTART,
	1NPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	20.0,0.0,XAXIS_LENGTH,YAXIS_LENGTH,X_INTERVALS,Y_INTERVALS,
	3X_INTERVALS_OFFSET_L,Y_INTERVALS_OFFSET_L)
	I=1
	DO WHILE (I.LE.NPLOTS)
	  IF (LINTYPE(I).NE.998.AND.LINTYPE(I).NE.999) THEN
	    IF (I.GT.1.AND.LINTYPE(I).NE.997) CALL NEWPEN
	    CALL VECTMODE(PLT$M_FIRSTBYTE.AND.LINVECT(I))
	    CALL PENMODE((PLT$M_SECONDBYTE.AND.LINVECT(I))/256)
	    CALL COLORMODE(COLOR(I))
	    IF (COLOR(I).NE.0) THEN
	      IF (LINTYPE(I).NE.997) THEN
	        IF (I_SYM_PAR.EQ.0) THEN
	          CALL LINE(XARRAY(ISTART,I),YARRAY(ISTART,I),DUMSIZE,
	1         NPOINTS(I),INC(I),0.0,0.0,XAXIS_LENGTH,YAXIS_LENGTH,
	2         X_INTERVALS,Y_INTERVALS,X_INTERVALS_OFFSET_L,
	3         Y_INTERVALS_OFFSET_L,LINTYPE(I),NSYMBOL(I),
	4         SYMBLHT(I),INVERT)
	        ELSE
	          CALL LINE_SYMBL(XARRAY(ISTART,I),YARRAY(ISTART,I),DUMSIZE,
	1         NPOINTS(I),INC(I),0.0,0.0,XAXIS_LENGTH,YAXIS_LENGTH,
	2         X_INTERVALS,Y_INTERVALS,X_INTERVALS_OFFSET_L,
	3         Y_INTERVALS_OFFSET_L,LINTYPE(I),NSYMBOL(I),
	4         SYMBLHT(I),INVERT,SYMBOL_PAR,I)
	        END IF
C  Plot the error bars
	      ELSE
	        CALL ERRORBAR(XARRAY(ISTART,I),YARRAY(ISTART,I),DUMSIZE,
	1       NPOINTS(I),INC(I),0.0,0.0,XAXIS_LENGTH,YAXIS_LENGTH,
	2       X_INTERVALS,Y_INTERVALS,X_INTERVALS_OFFSET_L,
	3       Y_INTERVALS_OFFSET_L,NSYMBOL(I),INVERT)
	        I=I+1
	      END IF
	    END IF
	  END IF
	  I=I+1
	END DO
C  Plot the title
	IF (NPLOTS.GT.1) CALL NEWPEN(0)
	CALL VECTMODE(0)
	CALL PENMODE(20)  ! 7 pixels on the Lasergrafix and 6 pixels on the
			  ! Apple Laser Writer or 20/1000 inches 
	CALL COLORMODE(16711680)  ! Red
	CALL ISSUBSUP(%REF(DUMTITLE),NCT,SUBSCRIPTFLAG,SUPERSCRIPTFLAG)
	IF (SUPERSCRIPTFLAG) THEN
	  DELY=0.1
	ELSE
	  DELY=0.0
	END IF
	CALL SYMBOL(XAXIS_LENGTH-0.05-(NCTIT)*.2475,YAXIS_LENGTH-0.42
	1-DELY,0.3,%REF(DUMTITLE),0.0,NCT)
C  Display any vectors which haven't yet been displayed, for those devices
C  which are using block mode, such as SAOimage.
	IF (PLOTDEVQUES('D*',0)) CALL PLOT(0,0,15)
C  Flush the buffer, as required for certain displays such as xterm
C  and xgterm under Unix.
	CALL PLOT(0.0,0.0,18)
C  Activate the crosshairs if they were requested and readback is 
C  supported on the plot device
	IF (CROSSHAIRS.AND.PLOTDEVQUES('G*',1)) THEN
	  ICHAR=0
	  IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	    CALL KEYBOARD(ICHAR)
	  ELSE 
	    CALL WHERE(XNOW,YNOW,ICHAR)
	  END IF
C  Set the display mode to immediate for those devices which can switch
C  between immediate and delayed, such as SAOimage
	  IF (PLOTDEVQUES('D*',0)) CALL PLOT(0,0,13)
C  Start the alphagraphics mode in the upper left corner.  For SAOimage
C  this would however create occasional garbage characters.
	  IF (.NOT.PLOTDEVQUES('GD*',1)) CALL TEKPAU(0.0,5.44,0)
	  DO WHILE (ICHAR.NE.10.AND.ICHAR.NE.13.AND.ICHAR.NE.81.AND.
	1 ICHAR.NE.113)
	! A <LF>, <CR>, or the letter Q or q for quit.  Unix gives a
	! <LF> for a carriage return press except in SAOimage where a
	! <CR> is produced
	    IF (ICHAR.EQ.65) THEN !  The letter A for average
	      IF (PLOTDEVQUES('GD*',1)) CALL MESSAGE('Average Y requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Average Y requested')
	        CALL TRANSPARENT_TO_TEK
	      END IF
	      CALL WHERE(XNOW,YNOW,ICHAR1)
	      IF (INVERT(1)) THEN
	        XEND=(XAXIS_LENGTH+XAXIS_OFFSET-XNOW)*XDELTAV*
	1       X_INTERVALS/XAXIS_LENGTH+XFIRSTV-X_INTERVALS_OFFSET_L*XDELTAV
	      ELSE
	        XSTART=(XNOW-XAXIS_OFFSET)*XDELTAV*X_INTERVALS/
	1       XAXIS_LENGTH+XFIRSTV-X_INTERVALS_OFFSET_L*XDELTAV
	      END IF
	      CALL WHERE(XNOW,YNOW,ICHAR1)
	      IF (INVERT(1)) THEN
	        XSTART=(XAXIS_LENGTH+XAXIS_OFFSET-XNOW)*XDELTAV*
	1       X_INTERVALS/XAXIS_LENGTH+XFIRST-X_INTERVALS_OFFSET_L*XDELTAV
	      ELSE
	        XEND=(XNOW-XAXIS_OFFSET)*XDELTAV*X_INTERVALS/
	1       XAXIS_LENGTH+XFIRSTV-X_INTERVALS_OFFSET_L*XDELTAV
	      END IF
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      CALL MESSAGE('  XSTART         XEND     YAVERAGE')
	      DO N=1,NPLOTS
	        YAVERAGE=0.0
	        NCOUNT=0   
	        DO NN=1,NPOINTS(N)
	          IF (XARRAY(NN,N).GE.XSTART.AND.XARRAY(NN,N).LE.XEND.AND.
	1         XARRAY(NN,N).NE.OMIT.AND.YARRAY(NN,N).NE.OMIT) THEN
	            YAVERAGE=YAVERAGE+YARRAY(NN,N)
	            NCOUNT=NCOUNT+1
	          END IF
	        END DO
	        IF (NCOUNT.GT.0) THEN
	          YAVERAGE=YAVERAGE/NCOUNT
	          WRITE(6,1000) XSTART,XEND,YAVERAGE
	          WRITE(7,1000) XSTART,XEND,YAVERAGE
1000	          FORMAT(3G13.6)
	        END IF
	      END DO
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	    ELSE IF (ICHAR.EQ.97) THEN !  The letter a for annotation
	      IF (PLOTDEVQUES('GD*',1)) CALL MESSAGE('Annotation requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Annotation requested')
	        CALL TRANSPARENT_TO_TEK
	      END IF
	      CALL WHERE(XNOW,YNOW,ICHAR1)
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
C  For a graphics terminal connected to a Unix system, the call to TEKPAU
C  is essential.
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      STRING=' '
	      CALL USRINP(%REF(STRING),1,'A',132,0,'STRING=',
	1     'Enter a string:')
	      CALL ALPHA(%REF(STRING),1,132,NC,NF,NR,1)
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	      CALL SYMBOL(XNOW-XAXIS_OFFSET,YNOW-YAXIS_OFFSET,
	1     ANNOTATION_HEIGHT,%REF(STRING),0.0,NC)
	    ELSE IF (ICHAR.EQ.1) THEN !  Control A for height of annotation
	      ANNOTATION_HEIGHT=0.14
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Annotation height requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Annotation height requested')
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      CALL USRINP(ANNOTATION_HEIGHT,1,'F',4,1,'HEIGHT=',
	1     'Enter the annotation height in inches:  (default = 0.14)')
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	    ELSE IF (ICHAR.EQ.98) THEN !  The letter b for background color
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Background color requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Background color requested')
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      CALL COLOR_INPUT(TEMPCOLOR,'BACKGROUND_COLOR=',
	1     'Enter the background color (singlet or triplet), e.g., '//
	2     'for SAOimage:')
	      CALL COLORMODE_BACKGROUND(TEMPCOLOR)
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	    ELSE IF (ICHAR.EQ.67) THEN !  The letter C for coordinates
	      IF (PLOTDEVQUES('GD*',1)) CALL MESSAGE('Coordinates requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Coordinates requested')
	        CALL TRANSPARENT_TO_TEK
	      END IF
	      ICHAR1=0
	      I=1
	      DO WHILE (ICHAR1.NE.81.AND.ICHAR1.NE.113)
				!  The letter Q or q for quitting coordinates
	        CALL WHERE(XNOW,YNOW,ICHAR1)
	        IF (INVERT(1)) THEN
	          XDATA=(XAXIS_LENGTH+XAXIS_OFFSET-XNOW)*XDELTAV*
	1         X_INTERVALS/XAXIS_LENGTH+XFIRSTV-X_INTERVALS_OFFSET_L*XDELTAV
	        ELSE
	          XDATA=(XNOW-XAXIS_OFFSET)*XDELTAV*X_INTERVALS/
	1         XAXIS_LENGTH+XFIRSTV-X_INTERVALS_OFFSET_L*XDELTAV
	        END IF
	        IF (INVERT(2)) THEN
	          YDATA=(YAXIS_LENGTH+YAXIS_OFFSET-YNOW)*YDELTAV*
	1         Y_INTERVALS/YAXIS_LENGTH+YFIRSTV-Y_INTERVALS_OFFSET_L*YDELTAV
	        ELSE
	          YDATA=(YNOW-YAXIS_OFFSET)*YDELTAV*Y_INTERVALS/
	1         YAXIS_LENGTH+YFIRSTV-Y_INTERVALS_OFFSET_L*YDELTAV
	        END IF
	        IF (ICHAR1.NE.81.AND.ICHAR1.NE.113) THEN
			!  Prevents write upon quitting
	          IF (ICHAR1.NE.45) THEN !  If not deleting the last
					 !  values
	            XCOORD(I)=XDATA
	            YCOORD(I)=YDATA
C  Go to alphagraphics mode at the current location.
	            IF (PLOTDEVQUES('2WT',1)) THEN
	              CALL TEK(0,0,4)
	            ELSE IF (PLOTDEVQUES('1WT',1)) THEN
C  Compute the y position for output so that non-VT100/Retrographics
C  terminals will have the correct position.  For example, on terminals
C  such as the Visual 550 and GraphOn, the alphagraphics cursor is hardware
C  controlled by the crosshair location.  Successive FORTRAN writes then
C  will result in overlapping numbers.  
	              IF (NDEV.EQ.301) THEN
	                CALL TEKPAU(0.0,5.50-0.16*LINECOUNT,0)  
	              ELSE
	                CALL TEKPAU(0.0,5.60-0.16*LINECOUNT,0)  
	              END IF
	            END IF
	            WRITE (6,3000) XDATA,YDATA
3000	            FORMAT(' ','X=',G15.7,'Y=',G15.7)
	            WRITE (7,3000) XDATA,YDATA
	            LINECOUNT=LINECOUNT+1
	            IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	              IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	              IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	            END IF
	          ELSE !  A minus sign for deleting the last value
	            J=SETEND(XCOORD(I))
	            J=SETEND(YCOORD(I))
	            I=I-1
	          END IF
	        END IF
	        I=I+1
	      END DO
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Coordinates request canceled')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Coordinates request canceled')
	        CALL TRANSPARENT_TO_TEK
	      END IF
	    ELSE IF (ICHAR.EQ.99) THEN !  The letter c for color
	      IF (PLOTDEVQUES('GD*',1)) CALL MESSAGE('Color requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Color requested')
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      CALL COLOR_INPUT(TEMPCOLOR,'COLOR=',
	1     'Enter the color (singlet or triplet):')
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	      CALL COLORMODE(TEMPCOLOR)
	    ELSE IF (ICHAR.EQ.66.OR.ICHAR.EQ.82) THEN !  B for box plot
						      !  R for replot
	      IF (PLOTDEVQUES('GD*',1)) CALL MESSAGE('Subplot requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Subplot requested')
	        CALL TRANSPARENT_TO_TEK
	      END IF
C  If the letter B is typed, the plot is redone with the scales
C  determined by the crosshair box;  if the letter R is typed the
C  plot is redone with automatic scaling (only the X coordinates are
C  are read in this case).  The previous values of the axes lengths and
C  numbers of intervals are used.
	      CALL PLOT(0.0,0.0,999)
	      CALL WHERE(XNOW,YNOW,ICHAR1)
	      IF (INVERT(1)) THEN
	        XEND=(XAXIS_LENGTH+XAXIS_OFFSET-XNOW)*XDELTAV*
	1       X_INTERVALS/XAXIS_LENGTH+XFIRSTV-X_INTERVALS_OFFSET_L*XDELTAV
	      ELSE
	        XSTART=(XNOW-XAXIS_OFFSET)*XDELTAV*X_INTERVALS/
	1       XAXIS_LENGTH+XFIRSTV-X_INTERVALS_OFFSET_L*XDELTAV
	      END IF
	      IF (INVERT(2)) THEN
	        YEND=(YAXIS_LENGTH+YAXIS_OFFSET-YNOW)*YDELTAV*
	1       Y_INTERVALS/YAXIS_LENGTH+YFIRSTV-Y_INTERVALS_OFFSET_L*YDELTAV
	      ELSE
	        YSTART=(YNOW-YAXIS_OFFSET)*YDELTAV*Y_INTERVALS/
	1       YAXIS_LENGTH+YFIRSTV-Y_INTERVALS_OFFSET_L*YDELTAV
	      END IF
	      CALL WHERE(XNOW,YNOW,ICHAR1)
	      IF (INVERT(1)) THEN
	        XSTART=(XAXIS_LENGTH+XAXIS_OFFSET-XNOW)*XDELTAV*
	1       X_INTERVALS/XAXIS_LENGTH+XFIRSTV-X_INTERVALS_OFFSET_L*XDELTAV
	      ELSE
	        XEND=(XNOW-XAXIS_OFFSET)*XDELTAV*X_INTERVALS/
	1       XAXIS_LENGTH+XFIRSTV-X_INTERVALS_OFFSET_L*XDELTAV
	      END IF
	      IF (INVERT(2)) THEN
	        YSTART=(YAXIS_LENGTH+YAXIS_OFFSET-YNOW)*YDELTAV*
	1       Y_INTERVALS/YAXIS_LENGTH+YFIRSTV-Y_INTERVALS_OFFSET_L*YDELTAV
	      ELSE
	        YEND=(YNOW-YAXIS_OFFSET)*YDELTAV*Y_INTERVALS/
	1       YAXIS_LENGTH+YFIRSTV-Y_INTERVALS_OFFSET_L*YDELTAV
	      END IF
	      IF (ICHAR.EQ.66) THEN !  B
	        XARRAY(DUMSIZE-1,1)=XSTART
	        XARRAY(DUMSIZE,1)=(XEND-XSTART)/XAXIS_LENGTH
	        YARRAY(DUMSIZE-1,1)=YSTART
	        YARRAY(DUMSIZE,1)=(YEND-YSTART)/YAXIS_LENGTH
	      ELSE IF (ICHAR.EQ.82) THEN !  R
	        XDUMARRAY(1)=XSTART
	        XDUMARRAY(2)=XEND
	        CALL SCALE_AXIS(XDUMARRAY,4,1,2,X_INTERVALS,1,LINTYPE,
	1       NSYMBOL,COLOR,.FALSE.)
	        XARRAY(DUMSIZE-1,1)=XDUMARRAY(3)
	        XARRAY(DUMSIZE,1)=XDUMARRAY(4)
C  Get the range in Y from the data in all the plots for this range 
C  in X read from the crosshairs.  Pay attention to points that are
C  not to be plotted.
	        YMIN=YARRAY(1,1)
	        YMAX=YARRAY(1,1)
	        I=1
	        DO WHILE(YARRAY(I,1).EQ.OMIT.AND.I.LE.NPOINTS(1))
	          I=I+1
	          YMIN=YARRAY(I,1)
	          YMAX=YMIN
	        END DO
	        DO J=1,NPLOTS
	          DO I=1,NPOINTS(J)
	            IF (YARRAY(I,J).LT.YMIN.AND.YARRAY(I,J).NE.OMIT) THEN
	              YMIN=YARRAY(I,J)
	            ELSE IF (YARRAY(I,J).GT.YMAX.AND.YARRAY(I,J).NE.OMIT) THEN
	              YMAX=YARRAY(I,J)
	            END IF
	          END DO
	        END DO
C  We so far have the maximum and minimum Y for all the plots, inside 
C  and outside the selected area.
	        YSTART=YMAX
	        YEND=YMIN
	        DO J=1,NPLOTS
	          DO I=1,NPOINTS(J)
	            IF (YARRAY(I,J).LE.YSTART.AND.XARRAY(I,J).GE.XSTART
	1           .AND.XARRAY(I,J).LE.XEND.AND.YARRAY(I,J).NE.OMIT)
	2           YSTART=YARRAY(I,J)
	            IF (YARRAY(I,J).GE.YEND.AND.XARRAY(I,J).GE.XSTART
	1           .AND.XARRAY(I,J).LE.XEND.AND.YARRAY(I,J).NE.OMIT)
	2           YEND=YARRAY(I,J)
	          END DO
	        END DO
	        YDUMARRAY(1)=YSTART
	        YDUMARRAY(2)=YEND
	        CALL SCALE_AXIS(YDUMARRAY,4,1,2,Y_INTERVALS,1,LINTYPE,
	1       NSYMBOL,COLOR,.TRUE.)
	        YARRAY(DUMSIZE-1,1)=YDUMARRAY(3)
	        YARRAY(DUMSIZE,1)=YDUMARRAY(4)
	      END IF
	      WRITE (7,3000) XSTART,YSTART
	      WRITE (7,3000) XEND,YEND
	      IF (XSCALEMODE.EQ.'A'.OR.XSCALEMODE.EQ.'B') THEN
	        XSCALEMODE='B'
	      ELSE
	        XSCALEMODE='D'
	        IF (ICHAR.EQ.66) THEN !  B
	          XARRAY(DUMSIZE-1,1)=ALOGCODE(XSTART,0)
	          XARRAY(DUMSIZE-2,1)=ALOGDECCODE(XSTART,
	1         X_INTERVALS*XARRAY(DUMSIZE,1),0)
	        ELSE IF (ICHAR.EQ.82) THEN !  R
	          XARRAY(DUMSIZE-1,1)=ALOGCODE(XARRAY(DUMSIZE-1,1),1)
	          XARRAY(DUMSIZE-2,1)=ALOGDECCODE(XARRAY(DUMSIZE-1,1),
	1         X_INTERVALS*XARRAY(DUMSIZE,1),2)
	        END IF
	      END IF
	      IF (YSCALEMODE.EQ.'A'.OR.YSCALEMODE.EQ.'B') THEN
	        YSCALEMODE='B'
	      ELSE
	        YSCALEMODE='D'
	        IF (ICHAR.EQ.66) THEN !  B
	          YARRAY(DUMSIZE-1,1)=ALOGCODE(YSTART,0)
	          YARRAY(DUMSIZE-2,1)=ALOGDECCODE(YSTART,
	1         Y_INTERVALS*YARRAY(DUMSIZE,1),0)
	        ELSE IF (ICHAR.EQ.82) THEN !  R
	          YARRAY(DUMSIZE-1,1)=ALOGCODE(YARRAY(DUMSIZE-1,1),1)
	          YARRAY(DUMSIZE-2,1)=ALOGDECCODE(YARRAY(DUMSIZE-1,1),
	1         Y_INTERVALS*YARRAY(DUMSIZE,1),2)
	        END IF
	      END IF
C  Turn off the cursor.  No problem with xterm displaying
C  garbage characters here (under Unix).
	      IF (.NOT.PLOTDEVQUES('GD*',1)) CALL TEKCON
	      GO TO 10
	    ELSE IF (ICHAR.EQ.70) THEN !  F for applying a factor
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Scaling factor requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Scaling factor requested')
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      CALL USRINP(FACT,1,'F',4,1,'FACTOR=','Enter the factor:')
	      FWASTYPED=.TRUE.
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	    ELSE IF (ICHAR.EQ.102) THEN !  The letter f for fill style
	      IF (PLOTDEVQUES('GD*',1)) CALL MESSAGE('Fill style requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Fill style requested')
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      IF (SYSTEM_NUMBER_BYTES.EQ.4) THEN
	        CALL DYNAMIC(ADDRESS1_4,NPLOTS,'  I4',.TRUE.)
	        CALL XYPLOT_F(%VAL(ADDRESS1_4),NPLOTS,NSYMBOL)
	        CALL DYNAMIC(ADDRESS1_4,NPLOTS,'  I4',.FALSE.)
	      ELSE IF (SYSTEM_NUMBER_BYTES.EQ.8) THEN
	        CALL DYNAMIC(ADDRESS1_8,NPLOTS,'  I4',.TRUE.)
	        CALL XYPLOT_F(%VAL(ADDRESS1_8),NPLOTS,NSYMBOL)
	        CALL DYNAMIC(ADDRESS1_8,NPLOTS,'  I4',.FALSE.)
	      END IF
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	    ELSE IF (ICHAR.EQ.72.OR.ICHAR.EQ.104) THEN
				!  The letters H or h for help
	      IF (PLOTDEVQUES('GD*',1)) CALL MESSAGE('Help requested')
C  If a graphics terminal having a separate TEK window exists, don't erase
C  the screen;  otherwise erase in order to display the help commands.
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Help requested')
	      ELSE IF (PLOTDEVQUES('GT',1)) THEN
	        CALL TEK(0,0,0)
	        CALL TEK(0,0,4)
	      END IF
	      CALL MESSAGE('A = Turns on the crosshairs and by'//
	1     ' pressing the <CR> at chosen X limits,')
	      CALL MESSAGE('    the mean Y value will be calculated for'//
	1     ' each plot')
	      CALL MESSAGE('a = Allows the user to enter annotation at the'//
	1     ' location of the crosshairs')
	      CALL MESSAGE('    with the color most recently selected')
	      CALL MESSAGE('    ^H (^B on Mac OS) gives a backspace')
	      CALL MESSAGE('    ^I (^L on Mac OS) toggles to the Greek'//
	1     ' alphabet and special symbols')
	      CALL MESSAGE('    and back to English')
	      CALL MESSAGE('    ^K toggles to subscript, superscript, and'//
	1     ' back to normal')
	      CALL MESSAGE('^A = Prompts user for the height of the '//
	1     'annotation')
	      CALL MESSAGE('B = Defines a box with the crosshairs placed'//
	1     ' at the lower left and upper right')
	      CALL MESSAGE('    Will replot with this scaling')
	      CALL MESSAGE('b = Allows the user to be prompted for the '//
	1     'background color, e.g., on SAOimage')
	      CALL MESSAGE('C = Turns on the crosshairs and by pressing the'//
	1     ' <CR>, prints the values at')
	      CALL MESSAGE('    that location and puts the values in a '//
	1     'common block;  typing - deletes the')
	      CALL MESSAGE('    last pair of values from the common block;'//
	1     '  and typing Q turns off the')
	      CALL MESSAGE('    crosshairs.')
	      CALL MESSAGE('c = Allows the user to be prompted for the '//
	1     'vector color')
	      CALL MESSAGE('F = Allows the user to alter the scaling factor'//
	1     ' on the current plot device(s)')
	      CALL MESSAGE('f = Allows the user to change the fill style'//
	1     ' for each plot')
	      CALL MESSAGE('H = Gives help')
	      CALL MESSAGE('h = Gives help')
	      CALL MESSAGE('I = Allows the user to be prompted for several'//
	1     ' of the plotting parameters')
	      CALL MESSAGE('    before a replot is done')
	      CALL MESSAGE('L = Allows the user to take log(X) and log(Y)')
	      CALL MESSAGE('l = Allows the user to take ln(X) and ln(Y)')
	      CALL MESSAGE('M = Allows the user to modify the X and Y values'//
	1     ' by adding increments and')
	      CALL MESSAGE('    adjusting by scaling factors')
	      CALL MESSAGE('m = Allows the user to modify the X and Y values'//
	1     ' by adding increments and')
	      CALL MESSAGE('    adjusting by scaling factors for a selected'//
	1     ' plot')
	      CALL MESSAGE('O = Redoes the original plot with any new'//
	1     ' axes, colors, and symbols')
	      CALL MESSAGE('o = Redoes the original plot without any new'//
	1     ' axes or symbols')
	      CALL MESSAGE('P = Allows the user to take powers')
	      CALL MESSAGE('p = Allows the user to specify the color of'//
	1     ' each plot')
	      CALL MESSAGE('Q = Returns')
	      CALL MESSAGE('q = Returns')
	      CALL MESSAGE('R = Turns on the crosshairs and by pressing the'//
	1     ' <CR> at chosen X limits')
	      CALL MESSAGE('    replotting with automatic scaling will be'//
	1     ' done')
	      CALL MESSAGE('S = Allows the user to specify the style of'//
	1     ' vector')
	      CALL MESSAGE('s = Allows the user to specify the symbol for'//
	1     ' a set of points')
	      CALL MESSAGE('V = Draws a vector of the color and vector'//
	1     ' style most recently selected')
	      CALL MESSAGE('v = Allows the user to be prompted for the'//
	1     ' starting and ending physical')
	      CALL MESSAGE('    values of a vector')
	      CALL MESSAGE('X = Allows plotting an additional X axis in'//
	1     ' different linear units')
	      CALL MESSAGE('x = Allows plotting an additional X axis in'//
	1     ' different antilog units')
	      CALL MESSAGE('Y = Allows plotting an additional Y axis in'//
	1     ' different linear units')
	      CALL MESSAGE('y = Allows plotting an additional Y axis in'//
	1     ' different antilog units')
C  Return control to the TEK windows if a multi-window device is being used.
	      IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	      IF (PLOTDEVQUES('1WT',1))
	1     CALL MESSAGE('Press "O" or "o" to continue')
	    ELSE IF (ICHAR.EQ.73.OR.ICHAR.EQ.76.OR.ICHAR.EQ.108
	1   .OR.ICHAR.EQ.77.OR.ICHAR.EQ.109.OR.ICHAR.EQ.79.OR.
	2   ICHAR.EQ.111.OR.ICHAR.EQ.80) THEN
C  If the letter I is typed, one can interact;  if L or l is typed one can
C  take logs;  if M is typed, one can modify the values;  if m is typed,
C  one can modify the values of a selected plot;  if O is typed,
C  the original plot is redone with any new axes;  if o is typed, the
C  original plot is redone without any new axes.
	      ISTART=1
	      CALL PLOT(0.0,0.0,999)
	      IF (ICHAR.EQ.73) THEN !  I for interact
	        IF (PLOTDEVQUES('GD*',1))
	1       CALL MESSAGE('Interaction requested')
	        I_INTERACT=.TRUE.
	        IF (PLOTDEVQUES('2WT',1)) THEN
	          CALL TEK(0,0,4)
	          CALL MESSAGE('Interaction requested')
	        ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	          CALL TEKPAU(0.0,0.0,1)
	        END IF
	        GO TO 5
	      ELSE IF (ICHAR.EQ.76.OR.ICHAR.EQ.108.OR.ICHAR.EQ.77.OR.ICHAR
	1     .EQ.109.OR.ICHAR.EQ.79.OR.ICHAR.EQ.111.OR.ICHAR.EQ.80) THEN
			     !  L or l or M or m or O or o or P
	        IF (ICHAR.EQ.76.OR.ICHAR.EQ.108.OR.ICHAR.EQ.77.OR.ICHAR.EQ.
	1       109.OR.ICHAR.EQ.80) THEN
	          IF (ICHAR.EQ.76) THEN !  L for taking logs
	            IF (PLOTDEVQUES('GD*',1))
	1           CALL MESSAGE('Logs requested')
	            IF (PLOTDEVQUES('2WT',1)) THEN
	              CALL TEK(0,0,4)
	              CALL MESSAGE('Logs requested')
	            ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	              CALL TEKPAU(0.0,0.0,1)
	            END IF
	            TAKELOG=.TRUE.
	            CALL USRINP(TAKELOG,1,'L',4,1,'TAKE_LOG_X=',
	1           'Do you want to take log(x)?  (default = Y)')
	            IF (TAKELOG) THEN
	              DO N=1,NPLOTS
	                DO M=1,NPOINTS(N)
	                  IF (XARRAY(M,N).NE.OMIT)
	1                 XARRAY(M,N)=LOG10(XARRAY(M,N))
	                END DO
	              END DO
	            END IF
	            TAKELOG=.TRUE.
	            CALL USRINP(TAKELOG,1,'L',4,1,'TAKE_LOG_Y=',
	1           'Do you want to take log(y)?  (default = Y)')
	            IF (TAKELOG) THEN
	              DO N=1,NPLOTS
	                DO M=1,NPOINTS(N)
	                  IF (YARRAY(M,N).NE.OMIT)
	1                 YARRAY(M,N)=LOG10(YARRAY(M,N))
	                END DO
	              END DO
	            END IF
	            IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	              IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	              IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	            END IF
	          ELSE IF (ICHAR.EQ.108) THEN !  l for taking natural logs
	            IF (PLOTDEVQUES('GD*',1))
	1           CALL MESSAGE('Natural logs requested')
	            IF (PLOTDEVQUES('2WT',1)) THEN
	              CALL TEK(0,0,4)
	              CALL MESSAGE('Natural logs requested')
	            ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	              CALL TEKPAU(0.0,0.0,1)
	            END IF
	            TAKELOG=.TRUE.
	            CALL USRINP(TAKELOG,1,'L',4,1,'TAKE_LN_X=',
	1           'Do you want to take ln(x)?  (default = Y)')
	            IF (TAKELOG) THEN
	              DO N=1,NPLOTS
	                DO M=1,NPOINTS(N)
	                  IF (XARRAY(M,N).NE.OMIT)
	1                 XARRAY(M,N)=LOG(XARRAY(M,N))
	                END DO
	              END DO
	            END IF
	            TAKELOG=.TRUE.
	            CALL USRINP(TAKELOG,1,'L',4,1,'TAKE_LN_Y=',
	1           'Do you want to take ln(y)?  (default = Y)')
	            IF (TAKELOG) THEN
	              DO N=1,NPLOTS
	                DO M=1,NPOINTS(N)
	                  IF (YARRAY(M,N).NE.OMIT)
	1                 YARRAY(M,N)=LOG(YARRAY(M,N))
	                END DO
	              END DO
	            END IF
	            IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	              IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	              IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	            END IF
	          ELSE IF (ICHAR.EQ.77) THEN !  M for modifying 
	            IF (PLOTDEVQUES('GD*',1))
	1           CALL MESSAGE('Modification of values requested')
	            IF (PLOTDEVQUES('2WT',1)) THEN
	              CALL TEK(0,0,4)
	              CALL MESSAGE('Modification of values requested')
	            ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	              CALL TEKPAU(0.0,0.0,1)
	            END IF
	            CALL USRINP(INCREMENTS,2,'F',4,0,'INCREMENTS=',
	1           'Enter the increments for the X and Y values:')
	            CALL USRINP(FACTORS,2,'F',4,0,'FACTORS=',
	1           'Enter the factors for the X and Y values:')
	            DO N=1,NPLOTS
	              DO M=1,NPOINTS(N)
	                IF (XARRAY(M,N).NE.OMIT)
	1               XARRAY(M,N)=FACTORS(1)*(XARRAY(M,N)+INCREMENTS(1))
	                IF (YARRAY(M,N).NE.OMIT)
	1               YARRAY(M,N)=FACTORS(2)*(YARRAY(M,N)+INCREMENTS(2))
	              END DO
	            END DO
	            IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	              IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	              IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	            END IF
	          ELSE IF (ICHAR.EQ.109) THEN !  m for modifying by plot
	            IF (PLOTDEVQUES('GD*',1))
	1           CALL MESSAGE('Modification of values requested')
	            IF (PLOTDEVQUES('2WT',1)) THEN
	              CALL TEK(0,0,4)
	              CALL MESSAGE('Modification of values requested')
	            ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	              CALL TEKPAU(0.0,0.0,1)
	            END IF
	            CALL USRINP(PLOT_NUMBER,1,'I',4,0,'PLOT_NUMBER=',
	1           'Enter the plot number:')
	            CALL USRINP(INCREMENTS,2,'F',4,0,'INCREMENTS=',
	1           'Enter the increments for the X and Y values:')
	            CALL USRINP(FACTORS,2,'F',4,0,'FACTORS=',
	1           'Enter the factors for the X and Y values:')
	            N=PLOT_NUMBER
	            DO M=1,NPOINTS(N)
	              IF (XARRAY(M,N).NE.OMIT)
	1             XARRAY(M,N)=FACTORS(1)*(XARRAY(M,N)+INCREMENTS(1))
	              IF (YARRAY(M,N).NE.OMIT)
	1             YARRAY(M,N)=FACTORS(2)*(YARRAY(M,N)+INCREMENTS(2))
	            END DO
	            IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	              IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	              IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	            END IF
	          ELSE IF (ICHAR.EQ.80) THEN !  P  for taking powers
	            IF (PLOTDEVQUES('GD*',1))
	1           CALL MESSAGE('Powers requested')
	            IF (PLOTDEVQUES('2WT',1)) THEN
	              CALL TEK(0,0,4)
	              CALL MESSAGE('Powers requested')
	            ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	              CALL TEKPAU(0.0,0.0,1)
	            END IF
	            TAKEPOWER=.TRUE.
	            CALL USRINP(TAKEPOWER,1,'L',4,1,'TAKE_POWER_X=',
	1           'Do you want to take a power of x?  (default = Y)')
	            IF (TAKEPOWER) THEN
	              CALL USRINP(POWER,1,'F',4,0,'POWER_X=',
	1             'Enter the power of x:')
	              DO N=1,NPLOTS
	                DO M=1,NPOINTS(N)
	                  IF (XARRAY(M,N).NE.OMIT)
	1                 XARRAY(M,N)=XARRAY(M,N)**POWER
	                END DO
	              END DO
	            END IF
	            TAKEPOWER=.TRUE.
	            CALL USRINP(TAKEPOWER,1,'L',4,1,'TAKE_POWER_Y=',
	1           'Do you want to take a power of y)?  (default = Y)')
	            IF (TAKEPOWER) THEN
	              CALL USRINP(POWER,1,'F',4,0,'POWER_Y=',
	1             'Enter the power of y:')
	              DO N=1,NPLOTS
	                DO M=1,NPOINTS(N)
	                  IF (YARRAY(M,N).NE.OMIT)
	1                 YARRAY(M,N)=YARRAY(M,N)**POWER
	                END DO
	              END DO
	            END IF
	            IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	              IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	              IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	            END IF
	          END IF
C  Best to do automatic scaling at the return
	          IF (XSCALEMODE.EQ.'A'.OR.XSCALEMODE.EQ.'B') THEN
	            XSCALEMODE='A'
	          ELSE
	            XSCALEMODE='C'
	          END IF
	          IF (YSCALEMODE.EQ.'A'.OR.YSCALEMODE.EQ.'B') THEN
	            YSCALEMODE='A'
	          ELSE
	            YSCALEMODE='C'
	          END IF
	        ELSE IF (ICHAR.EQ.79.OR.ICHAR.EQ.111) THEN !  O or o for
							   !  original
	          IF (PLOTDEVQUES('GD*',1))
	1         CALL MESSAGE('Original plot requested')
	          IF (PLOTDEVQUES('2WT',1)) THEN
	            CALL TEK(0,0,4)
	            CALL MESSAGE('Original plot requested')
	            CALL TRANSPARENT_TO_TEK
	          END IF
	          DO I=1,50
	            DUMXLABEL(I:I)=OXLABEL(I:I)
	            DUMYLABEL(I:I)=OYLABEL(I:I)
	            DUMTITLE(I:I)=OTITLE(I:I)
	          END DO
	          INVERT(1)=OINVERT(1)
	          INVERT(2)=OINVERT(2)
	          DO I=1,NPLOTS
	            SYMBLHT(I)=OSYMBLHT(I)
	          END DO
C	          NDUMDEVICE=ONDEVICE
	          XSCALEMODE=OXSCALEMODE
	          YSCALEMODE=OYSCALEMODE
	          XARRAY(DUMSIZE-3,1)=OXAXIS_LENGTH
	          XARRAY(DUMSIZE-2,1)=OX_INTERVALS
	          IF (XSCALEMODE.EQ.'D') XARRAY(DUMSIZE-2,1)=OX_DECADES
	          XARRAY(DUMSIZE-1,1)=OXFIRSTV
	          XARRAY(DUMSIZE,1)=OXDELTAV
	          YARRAY(DUMSIZE-3,1)=OYAXIS_LENGTH
	          YARRAY(DUMSIZE-2,1)=OY_INTERVALS
	          IF (YSCALEMODE.EQ.'D') YARRAY(DUMSIZE-2,1)=OY_DECADES
	          YARRAY(DUMSIZE-1,1)=OYFIRSTV
	          YARRAY(DUMSIZE,1)=OYDELTAV
	          IF (ICHAR.EQ.111) THEN !  o;  Right and Top instructions
					 !  cancelled
	            XINCREMENT=0.0
	            YINCREMENT=0.0
	            XFACTOR=0.0
	            YFACTOR=0.0
	            XXSCALEMODE=OXXSCALEMODE
	            YYSCALEMODE=OYYSCALEMODE
	            xxALTYPED=.FALSE.
	            yyALTYPED=.FALSE.
	            XXTYPED=.FALSE.
	            YYTYPED=.FALSE.
	            I_SYM_PAR=0 !  Resets counter for changes to symbols
	            DO IS=1,NSYMBOL_PAR2
	              SYMBOL_PAR(1,IS)=0
	            END DO
	          END IF
	        END IF
	        GO TO 10
	      END IF
	    ELSE IF (ICHAR.EQ.112) THEN !  The letter p for specifying
					!  colors of each plot
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Colors of plots requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Colors of plots requested')
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      CALL USRINP(COLOR,NPLOTS,'I',4,0,'COLOR=',
	1     'Enter the color(s) (singlets) of the plot(s):')
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	    ELSE IF (ICHAR.EQ.83) THEN !  The letter S for specifying the
				       !  style of vectors
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Vector styles requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Vector styles requested')
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      CALL USRINP(IVSTYLE,1,'I',4,0,'STYLE=',
	1     'Enter the vector line style')
	      CALL VECTMODE(IVSTYLE)
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	    ELSE IF (ICHAR.EQ.115) THEN !  The letter s for symbols of
					!  sets of points
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Symbols of set of points requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Symbols of set of points requested')
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      I_SYM_PAR=I_SYM_PAR+1
	      I_SYM_PAR=MIN(I_SYM_PAR,NSYMBOL_PAR2)
	      CALL USRINP(SYMBOL_PAR(1,I_SYM_PAR),4,'I',4,0,
	1     'SYMBOL_PARAMETERS=',
	1     'Enter the plot, starting point, number of points, and symbol:')
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	    ELSE IF (ICHAR.EQ.86) THEN !  The letter V for vector
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Vectors from cursor requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Vectors from cursor requested')
	        CALL TRANSPARENT_TO_TEK
	      END IF
	      CALL WHERE(XNOW,YNOW,ICHAR1)
	      CALL WHERE(XNOW1,YNOW1,ICHAR1)
	      CALL PLOT(XNOW-XAXIS_OFFSET,YNOW-YAXIS_OFFSET,3)
	      CALL PLOT(XNOW1-XAXIS_OFFSET,YNOW1-YAXIS_OFFSET,2)
C  Turn off the cursor so that the vector will be unobstructed.
	      IF (.NOT.PLOTDEVQUES('GD*',1)) CALL TEKCON
	    ELSE IF (ICHAR.EQ.118) THEN !  The letter v for vector in
					!  physical units
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Vectors from physical values requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Vectors from physical values requested')
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      CALL USRINP(VECTPARAM,4,'F',4,0,'PARAMETERS=',
	1     'Enter start x, y and end x, y in physical units:')
	      IF (INVERT(1)) THEN
	        XNOW=XAXIS_LENGTH-
	1       (VECTPARAM(1)-XFIRSTV+X_INTERVALS_OFFSET_L*XDELTAV)*
	2       XAXIS_LENGTH/XDELTAV/X_INTERVALS
	      ELSE
	        XNOW=(VECTPARAM(1)-XFIRSTV+X_INTERVALS_OFFSET_L*XDELTAV)*
	1       XAXIS_LENGTH/XDELTAV/X_INTERVALS
	      END IF
	      IF (INVERT(2)) THEN
	        YNOW=YAXIS_LENGTH-
	1       (VECTPARAM(2)-YFIRSTV+Y_INTERVALS_OFFSET_L*YDELTAV)*
	2       YAXIS_LENGTH/YDELTAV/Y_INTERVALS
	      ELSE
	        YNOW=(VECTPARAM(2)-YFIRSTV+Y_INTERVALS_OFFSET_L*YDELTAV)*
	1       YAXIS_LENGTH/YDELTAV/Y_INTERVALS
	      END IF
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	      CALL PLOT(XNOW,YNOW,3)
	      IF (INVERT(1)) THEN
	        XNOW=XAXIS_LENGTH-
	1       (VECTPARAM(3)-XFIRSTV+X_INTERVALS_OFFSET_L*XDELTAV)*
	2       XAXIS_LENGTH/XDELTAV/X_INTERVALS
	      ELSE
	        XNOW=(VECTPARAM(3)-XFIRSTV+X_INTERVALS_OFFSET_L*XDELTAV)*
	1       XAXIS_LENGTH/XDELTAV/X_INTERVALS
	      END IF
	      IF (INVERT(2)) THEN
	        YNOW=YAXIS_LENGTH-
	1       (VECTPARAM(4)-YFIRSTV+Y_INTERVALS_OFFSET_L*YDELTAV)*
	2       YAXIS_LENGTH/YDELTAV/Y_INTERVALS
	      ELSE
	        YNOW=(VECTPARAM(4)-YFIRSTV+Y_INTERVALS_OFFSET_L*YDELTAV)*
	1       YAXIS_LENGTH/YDELTAV/Y_INTERVALS
	      END IF
	      CALL PLOT(XNOW,YNOW,2)
	    ELSE IF (ICHAR.EQ.88) THEN !  The letter X for additional X axis
C  Typing the letter X will produce an additional linearly annotated axis on
C  the top of the plot after I or O is typed
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Additional X axis requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Additional X axis requested')
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      CALL USRINP(XFACTOR,1,'F',4,0,'X_FACTOR=',
	1     'Enter the factor to convert the data to the new unit:')
	      CALL USRINP(XXDELTAV,1,'F',4,0,'DELTA_X=',
	1     'Enter the physical value per interval:')
	      CALL USRINP(%REF(XXLABEL),1,'A',50,0,'X_LABEL=',
	1     'Enter the X label:')
	      XXSCALEMODE='A'
	      XXTYPED=.TRUE.
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	    ELSE IF (ICHAR.EQ.120) THEN !  The letter x for additional x
					!  antilog axis
C  Typing the letter x will produce an additional antilog annotated axis on the
C  top of the plot after I or O is typed
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Additional antilog labeled axis requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Additional antilog labeled axis requested')
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
C  For eventual automatic operation
C	      CALL USRINP(XINCREMENT,1,'F',4,0,'X_INCREMENT=','Enter the
C	1 increment in log to convert the data to the new unit:')
	      CALL USRINP(XXFIRSTDECVAL,1,'F',4,0,'X_LOWEST_VALUE=',
	1    'Enter the lowest value (coded log.antilog) for the X axis:')
	      CALL USRINP(XX_DECADES,1,'F',4,0,'X_DECADES=',
	1     'Enter the number of X decades (coded log.antilog):')
	      CALL USRINP(%REF(XXLABEL),1,'A',50,0,'X_LABEL=',
	1     'Enter the X label:')
	      XXSCALEMODE='C'
	      xxALTYPED=.TRUE.
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	    ELSE IF (ICHAR.EQ.89) THEN !  The letter Y for addition Y axis
C  Typing the letter Y will produce an additional linearly annotated axis on
C  the right side of the plot after I or O is typed
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Additional Y axis requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Additional Y axis requested')
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      CALL USRINP(YFACTOR,1,'F',4,0,'Y_FACTOR=',
	1     'Enter the factor to convert the data to the new unit:')
	      CALL USRINP(YYDELTAV,1,'F',4,0,'DELTA_Y=',
	1     'Enter the physical value per interval:')
	      CALL USRINP(%REF(YYLABEL),1,'A',50,0,'Y_LABEL=',
	1     'Enter the Y label:')
	      YYSCALEMODE='A'
	      YYTYPED=.TRUE.
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	    ELSE IF (ICHAR.EQ.121) THEN !  The letter y for addtional y
					!  antilog axis
C  Typing the letter y will produce an additional linearly annotated axis on
C  the right side of the plot after I or O is typed
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Additional Y antilog axis requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Additional Y antilog axis requested')
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
C  For eventual automatic operation
C	      CALL USRINP(YINCREMENT,1,'F',4,0,'Y_INCREMENT=','Enter the
C	1 increment in log to convert the data to the new unit:')
	      CALL USRINP(YYFIRSTDECVAL,1,'F',4,0,'Y_LOWEST_VALUE=',
	1     'Enter the lowest value (coded log.antilog) for the Y axis:')
	      CALL USRINP(YY_DECADES,1,'F',4,0,'Y_DECADES=',
	1     'Enter the number of Y decades (coded log.antilog):')
	      CALL USRINP(%REF(YYLABEL),1,'A',50,0,'Y_LABEL=',
	1     'Enter the Y label:')
	      YYSCALEMODE='C'
	      yyALTYPED=.TRUE.
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	    END IF
C  Flush the buffer
	    CALL PLOT(0.0,0.0,18)
	    IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	      CALL KEYBOARD(ICHAR)
	    ELSE 
	      CALL WHERE(XNOW,YNOW,ICHAR)
	    END IF
	  END DO
	END IF
C  On Unix, avoid garbage vectors by flushing the buffer.  Going to
C  alphagraphics with TEKPAU or calling RESETTEK before
C  quitting also prevents this.
	CALL PLOT(0.0,0.0,18)
C  Produce an end-of-plot command
	IF (.NOT.ADDENDA) THEN
	  CALL PLOT(-XAXIS_OFFSET,-YAXIS_OFFSET,-3)
	  CALL PLOT(0.0,0.0,999)
	END IF
	RETURN
	END
