	PROGRAM XYPLOT_ASCII
C
C  Note:  This is the Unix version.
C
C       This program is a general purpose interface for the SUBROUTINE
C  XYPLOT to allow plotting data from one or more character files.  Three
C  cases can be handled.  In the first case, the data for the abscissa and
C  ordinate are contained in one file having an arbitrary number of header
C  lines, an arbitrary number of columns, and an arbitrary number of rows of
C  data.  All columns must have the same number of rows however.  Multiple 
C  plots are possible by selecting more than one column for the abscissa and
C  ordinate.  The number of columns chosen for the abscissa and ordinate
C  must match.  In the second case, the data for the abscissa are contained
C  in a first file and the data for the ordinate are contained in a second
C  file.  The number of rows in both files, other than header rows, must 
C  match.  Multiple plots are again possible.  In the third case, the abscissa
C  and ordinate data for the first set of plots are contained in the first of
C  several files, and the abscissa and ordinate data for additional plots are
C  contained in additional files up to a maximum of fifteen files.
C       A setup file will be opened which sets up the line dashing, pen
C  width, line style, symbols, and colors.  By editing this setup file, many
C  different plots can be produced.  In the setup file, the name of the
C  control parameter is followed on successive lines by the values desired
C  for each plot.  The control parameters may be in any order.  The number of
C  values can be less than, equal to, or greater than the number of plots
C  desired.  The program will simply use the values up to the last one
C  available, and then roll the values over.
C       The following restriction applies:  maximum number of characters per
C  line in the files is 132.
C			Donald H. Gudehus, April 27, 2001
C			Version 1U, Donald H. Gudehus, April 27, 2001
C			Converted code to Unix
C			Version 2, Donald H. Gudehus, February 24, 2000
C			Eliminated requirement that data files must conform
C			to FORTRAN carriage control.  Fixed problem where
C			large setup files could cause buffer overflow.
C			Version 3, Donald H. Gudehus, March 13, 2008
C			Combined XYPLOT_PROGRAM and XYPLOT2_PROGRAM into
C			this program and added another option, plotting
C			from up to fifteen files where each file contains
C			the data for the abscissa and ordinate of several
C			plots.
C			Version 4, Donald H. Gudehus, September 1, 2013
C			Added support for 64-bit processors.
C
	PARAMETER (MAXNUNITS=15,MAXNPLOTS=200)
C
	BYTE BSTRING_D(132)
C
	CHARACTER*9 DAT
	CHARACTER*132 STRING_D
	EQUIVALENCE (BSTRING_D,STRING_D)
	CHARACTER*20 DATA(64) !  We assume 20 significant figures and 64
			      !  columns maximum
C
	INTEGER COLUMNS_ABSCISSA(64,MAXNUNITS),ITEMP(64),
	1COLUMNS_ORDINATE(64,MAXNUNITS),ADDRESS1,ADDRESS2,ADDRESS3,
	2ADDRESS4,ADDRESS5,ADDRESS6,ADDRESS7,ADDRESS8,ADDRESS9,
	3IUNIT(MAXNUNITS),N_LINES_HEADER3(MAXNUNITS),
	4N_LINES_DATA_PER_FILE(MAXNUNITS),N_LINES_DATA_PER_PLOT(MAXNPLOTS),
	5NPLOTS_PER_FILE(MAXNUNITS)
C
	INTEGER*4 ADDRESS1_4,ADDRESS2_4,ADDRESS3_4,ADDRESS4_4,ADDRESS5_4,
	1ADDRESS6_4,ADDRESS7_4,ADDRESS8_4,ADDRESS9_4
	INTEGER*8 ADDRESS1_8,ADDRESS2_8,ADDRESS3_8,ADDRESS4_8,ADDRESS5_8,
	1ADDRESS6_8,ADDRESS7_8,ADDRESS8_8,ADDRESS9_8
C
	LOGICAL PROCEED
C
	INTEGER SYSTEM_NUMBER_BYTES
	COMMON /SYS_NUM_BYTES/ SYSTEM_NUMBER_BYTES
C
        LOGICAL FILEUSED
        COMMON /PLOT_FILEUSED/ FILEUSED
C
        CHARACTER*80 FILENAME
        COMMON /CFILE/ FILENAME
C
	CALL GET_OS_BYTES
	FILENAME=' '
	NCHOICE=0
	DO WHILE(NCHOICE.NE.1.AND.NCHOICE.NE.2.AND.NCHOICE.NE.3)
	  NCHOICE=1
	  CALL MESSAGE('Enter a choice for the input data:')
	  CALL MESSAGE('     1 = Abscissa and ordinate data for one or '//
	1 'more plots in one file')
	  CALL MESSAGE('         (default)')
	  CALL MESSAGE('     2 = Abscissa data in one file and ordinate '//
	1 'data in another file for')
	  CALL MESSAGE('         one or more plots')
	  CALL MESSAGE('     3 = Abscissa and ordinate data for first '//
	1 'plot in first file,')
	  CALL MESSAGE('         additional data for additional plots '//
	1 'in successive files, up')
	  CALL USRINP(NCHOICE,1,'I',4,1,'CHOICE=',
	1 '         to a maximum of ten files')
	END DO
C  Open the log file
	OPEN (UNIT=7,NAME='XYPLOT_ASCII.LOG',STATUS='UNKNOWN')
	CALL DATE(DAT)
	WRITE (7,1000)DAT
1000	FORMAT('    XYPLOT_ASCII.LOG    ',9A)
C
	IF (NCHOICE.EQ.1) THEN
	  CALL MESSAGE('Open the file containing the abscissa and ordinate '//
	1 'data for one or')
	  CALL MESSAGE('more plots.')
	ELSE IF (NCHOICE.EQ.2) THEN
	  CALL MESSAGE('Open the file containing the abscissa data for one '//
	1 'or more plots.')
	ELSE IF (NCHOIDE.EQ.3) THEN
	  CALL MESSAGE('Open the file containing the abscissa and ordinate '//
	1 'data for the first plot.')
	END IF
	CALL OPENCF(21,3,'FILE=',0)
C
	IF (NCHOICE.EQ.1.OR.NCHOICE.EQ.2) THEN
	  N_LINES_HEADER1=1
	  CALL USRINP(N_LINES_HEADER1,1,'I',4,1,'LINES_HEADER_1=',
	1 'How many lines are in the header?  (default = 1)')
	  DO I=1,N_LINES_HEADER1
	    READ(21,1100) STRING_D
1100	    FORMAT(A)
C  Clear out control characters which might results from sending the file
C  by the Apple mail utility to a Unix system.
	    DO II=1,132
	      IF (BSTRING_D(II).LT.32) BSTRING_D(II)=32
	    END DO
	    WRITE (6,1100) STRING_D
	    WRITE (7,1100) STRING_D
	  END DO
	  MAX_NUM_COLUMNS=0
	  N_LINES_DATA1=0
	  DO WHILE(.TRUE.)
	    READ (21,1200,END=990) DATA
1200	    FORMAT(132A)
	    N_LINES_DATA1=N_LINES_DATA1+1
	    CALL ALPHA(%REF(DATA),64,20,NCHAR,NFULL,NREST,4)
	    MAX_NUM_COLUMNS=MAX(MAX_NUM_COLUMNS,NFULL)
	  END DO
 990	  CONTINUE
	  NL=INT(LOG10(FLOAT(N_LINES_DATA1)))+1
	  WRITE(6,1500) MAX_NUM_COLUMNS,N_LINES_DATA1
1500	  FORMAT(' ','There are ',I3,' columns of data, and ',I<NL>,
	1 ' lines of data.')
	  WRITE(7,1500) MAX_NUM_COLUMNS,N_LINES_DATA1
	  N_ABSCISSA=0
	  N_ORDINATE=1
	  DO WHILE(N_ABSCISSA.NE.N_ORDINATE)
	    CALL USRINP(COLUMNS_ABSCISSA,64,'I',4,0,'COLUMNS_ABSCISSA=',
	1   'Enter the column(s) to be plotted as abscissa values:')
	    N_ABSCISSA=INEL(COLUMNS_ABSCISSA,64)
	    IF (NCHOICE.EQ.1) THEN
	      CALL USRINP(COLUMNS_ORDINATE,64,'I',4,0,'COLUMNS_ORDINATE=',
	1     'Enter the column(s) to be plotted as ordinate values:')
	      N_ORDINATE=INEL(COLUMNS_ORDINATE,64)
	      IF (N_ABSCISSA.NE.N_ORDINATE) THEN
	        CALL MESSAGE('The number of columns used as abscissa values '//
	1       'must match the number')
	        CALL MESSAGE('used as ordinate values.')
	        STOP
	      END IF
	      N_LINES_DATA_PER_FILE(1)=N_LINES_DATA1
	    ELSE IF (NCHOICE.EQ.2) THEN
	      N_ORDINATE=N_ABSCISSA
	    END IF
	  END DO
	END IF
C
	IF (NCHOICE.EQ.2) THEN
	  CALL MESSAGE('Open the file for the ordinate')
	  CALL OPENCF(22,3,'FILE=',0)
	  N_LINES_HEADER2=1
	  CALL USRINP(N_LINES_HEADER2,1,'I',4,1,'LINES_HEADER_2=',
	1 'How many lines are in the header?  (default = 1)')
	  DO I=1,N_LINES_HEADER2
	    READ(22,1100) STRING_D
	    WRITE (6,1100) STRING_D
	    WRITE (7,1100) STRING_D
	  END DO
	  MAX_NUM_COLUMNS=0
	  N_LINES_DATA2=0
	  DO WHILE(.TRUE.)
	    READ (22,1200,END=992) DATA
	    N_LINES_DATA2=N_LINES_DATA2+1
	    CALL ALPHA(%REF(DATA),64,20,NCHAR,NFULL,NREST,4)
	    MAX_NUM_COLUMNS=MAX(MAX_NUM_COLUMNS,NFULL)
	  END DO
 992      CONTINUE
	  NL=INT(LOG10(FLOAT(N_LINES_DATA2)))+1
	  WRITE(6,1501) MAX_NUM_COLUMNS,N_LINES_DATA2
1501	  FORMAT(' ','There are ',I3,' columns of data, and ',I<NL>,
	1 ' lines of data'/' ','in the file for the ordinate')
	  WRITE(7,1501) MAX_NUM_COLUMNS,N_LINES_DATA2
	  N_ORDINATE=0
	  DO WHILE(N_ABSCISSA.NE.N_ORDINATE)
	    CALL USRINP(COLUMNS_ORDINATE,64,'I',4,0,'COLUMNS_ORDINATE=',
	1   'Enter the column(s) to be plotted as ordinate values:')
	    N_ORDINATE=INEL(COLUMNS_ORDINATE,64)
	    IF (N_ABSCISSA.NE.N_ORDINATE) THEN
	      CALL MESSAGE('The number of columns used as abscissa values '//
	1     'must match the number')
	      CALL MESSAGE('used as ordinate values.')
	      STOP
	    END IF
	  END DO
	  N_LINES_DATA_PER_FILE(1)=MIN(N_LINES_DATA1,N_LINES_DATA2)
	END IF
C
	IF (NCHOICE.EQ.3) THEN
	  N_FILES=1
	  NPLOTS=0
	  IUNIT(N_FILES)=21
	  PROCEED=.TRUE.
	  DO WHILE(PROCEED)
	    N_LINES_HEADER3(N_FILES)=1
	    CALL USRINP(N_LINES_HEADER3(N_FILES),1,'I',4,1,'LINES_HEADER=',
	1   'How many lines are in the header?  (default = 1)')
	    DO I=1,N_LINES_HEADER3(N_FILES)
	      READ(IUNIT(N_FILES),1100) STRING_D
C  Clear out control characters
	      DO II=1,132
	        IF (BSTRING_D(II).LT.32) BSTRING_D(II)=32
	      END DO
	      WRITE (6,1100) STRING_D
	      WRITE (7,1100) STRING_D
	    END DO
	    MAX_NUM_COLUMNS=0
	    N_LINES_DATA_PER_FILE(N_FILES)=0
	    DO WHILE(.TRUE.)
	      READ (IUNIT(N_FILES),1200,END=993) DATA
	      N_LINES_DATA_PER_FILE(N_FILES)=N_LINES_DATA_PER_FILE(N_FILES)+1
	      CALL ALPHA(%REF(DATA),64,20,NCHAR,NFULL,NREST,4)
	      MAX_NUM_COLUMNS=MAX(MAX_NUM_COLUMNS,NFULL)
	    END DO
 993	    CONTINUE
	    NL=INT(LOG10(FLOAT(N_LINES_DATA_PER_FILE(N_FILES))))+1
							!  For FORMAT
	    WRITE(6,1500) MAX_NUM_COLUMNS,N_LINES_DATA_PER_FILE(N_FILES)
	    WRITE(7,1500) MAX_NUM_COLUMNS,N_LINES_DATA_PER_FILE(N_FILES)
	    CALL USRINP(COLUMNS_ABSCISSA(1,N_FILES),64,'I',4,0,
	1   'COLUMNS_ABSCISSA=',
	2   'Enter the column(s) to be plotted as abscissa values:')
	    N_ABSCISSA=INEL(COLUMNS_ABSCISSA(1,N_FILES),64)
 994	    CALL USRINP(ITEMP,64,'I',4,0,'COLUMNS_ORDINATE=',
	1   'Enter the column(s) to be plotted as ordinate values:')
	    N_ORDINATE=INEL(ITEMP,64)
	    DO I=1,N_ORDINATE
	      COLUMNS_ORDINATE(I,N_FILES)=ITEMP(I)
	    END DO
	    IF (N_ABSCISSA.NE.N_ORDINATE) THEN
	      CALL MESSAGE('The number of columns used as abscissa values '//
	1     'must match the number')
	      CALL MESSAGE('used as ordinate values.')
	      GO TO 994
	    END IF
	    NPLOTS_PER_FILE(N_FILES)=N_ABSCISSA
	    NPLOTS=NPLOTS+N_ABSCISSA
	    CALL MESSAGE('Enter the name of the input file for an '//
	1   'additional plot;')
	    CALL USRINP(%REF(FILENAME),1,'A',80,0,'FILE=',
	1   'enter 0 to continue:')
	    CALL ALPHA(%REF(FILENAME),1,80,NC,NF,NR,2)
	    CALL ALPHA(%REF(FILENAME),1,80,NC,NF,NR,1)
	    IF (FILENAME.EQ.'0') THEN
	      PROCEED=.FALSE.
	    ELSE
	      N_FILES=N_FILES+1
	      IUNIT(N_FILES)=IUNIT(N_FILES-1)+1
	      CALL OPENCF(IUNIT(N_FILES),3,'FILE=',1)
	    END IF
	  END DO
	END IF
C
	IF (NCHOICE.EQ.1.OR.NCHOICE.EQ.2) THEN
	  NPLOTS=N_ABSCISSA
	  REWIND(21)
	  DO I=1,N_LINES_HEADER1
	    READ(21,1100) STRING_D
	  END DO
	  N_LINES_DATA_PER_PLOT_MAX=N_LINES_DATA_PER_FILE(1)
	  DO N=1,NPLOTS
	    N_LINES_DATA_PER_PLOT(N)=N_LINES_DATA_PER_PLOT_MAX
	  END DO
	END IF
	IF (NCHOICE.EQ.2) THEN
	  REWIND(22)
	  DO I=1,N_LINES_HEADER2
	    READ(22,1100) STRING_D
	  END DO
	ELSE IF (NCHOICE.EQ.3) THEN
	  N_LINES_DATA_PER_PLOT_MAX=N_LINES_DATA_PER_FILE(1)
	  NP=1
	  DO N=1,N_FILES
	    REWIND(IUNIT(N))
	    DO I=1,N_LINES_HEADER3(N)
	      READ(IUNIT(N),1100) STRING_D
	    END DO
	    N_LINES_DATA_PER_PLOT_MAX=
	1   MAX(N_LINES_DATA_PER_PLOT_MAX,N_LINES_DATA_PER_FILE(N))
	    DO IP=1,NPLOTS_PER_FILE(N)
	      N_LINES_DATA_PER_PLOT(NP)=N_LINES_DATA_PER_FILE(N)
	      NP=NP+1
	    END DO
	  END DO
	END IF
C  Open a file on unit 79 in case the plot is written to a file.  Since
C  PLTDEV doesn't open this unit, it is OK to do it here.  It is in fact
C  necessary to do it here because DYNAMIC is being called. If we don't
C  open the file before the call, the call to DYNAMIC with .FALSE. will
C  result in a page owner violation.
	OPEN(UNIT=79,STATUS='UNKNOWN',FORM='UNFORMATTED')
C  Get the setup parameters
	CALL XYPLOT_SETUP(MAXNUNITS+21)
	IF (SYSTEM_NUMBER_BYTES.EQ.4) THEN
	  CALL DYNAMIC(ADDRESS1_4,NPLOTS*(N_LINES_DATA_PER_PLOT_MAX+4),
	1'  R4',.TRUE.)
	  CALL DYNAMIC(ADDRESS2_4,NPLOTS*(N_LINES_DATA_PER_PLOT_MAX+4),
	1'  R4',.TRUE.)
	  CALL DYNAMIC(ADDRESS3_4,NPLOTS,'  I4',.TRUE.) !  NPLPOINTS
	  CALL DYNAMIC(ADDRESS4_4,NPLOTS,'  I4',.TRUE.) !  LINVECT
	  CALL DYNAMIC(ADDRESS5_4,NPLOTS,'  I4',.TRUE.) !  INC
	  CALL DYNAMIC(ADDRESS6_4,NPLOTS,'  I4',.TRUE.) !  LINTYPE
	  CALL DYNAMIC(ADDRESS7_4,NPLOTS,'  I4',.TRUE.) !  NSYMBOL
	  CALL DYNAMIC(ADDRESS8_4,NPLOTS,'  I4',.TRUE.) !  NSYMBOLHEIGHT
	  CALL DYNAMIC(ADDRESS9_4,NPLOTS,'  I4',.TRUE.) !  COLOR
	  CALL XYPLOT_ASCII1(NCHOICE,%VAL(ADDRESS1_4),%VAL(ADDRESS2_4),
	1 N_LINES_DATA_PER_PLOT,N_LINES_DATA_PER_PLOT_MAX,NPLOTS_PER_FILE,
	2 NPLOTS,%VAL(ADDRESS3_4),%VAL(ADDRESS4_4),%VAL(ADDRESS5_4),
	3 %VAL(ADDRESS6_4),%VAL(ADDRESS7_4),%VAL(ADDRESS8_4),
	4 %VAL(ADDRESS9_4),COLUMNS_ABSCISSA,COLUMNS_ORDINATE,IUNIT,N_FILES)
	  CALL DYNAMIC(ADDRESS1_4,NPLOTS*(N_LINES_DATA_PER_PLOT_MAX+4),
	1 '  R4',.FALSE.)
	  CALL DYNAMIC(ADDRESS2_4,NPLOTS*(N_LINES_DATA_PER_PLOT_MAX+4),
	1 '  R4',.FALSE.)
	  CALL DYNAMIC(ADDRESS3_4,NPLOTS,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS4_4,NPLOTS,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS5_4,NPLOTS,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS6_4,NPLOTS,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS7_4,NPLOTS,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS8_4,NPLOTS,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS9_4,NPLOTS,'  I4',.FALSE.)
	ELSE IF (SYSTEM_NUMBER_BYTES.EQ.8) THEN
	  CALL DYNAMIC(ADDRESS1_8,NPLOTS*(N_LINES_DATA_PER_PLOT_MAX+4),
	1'  R4',.TRUE.)
	  CALL DYNAMIC(ADDRESS2_8,NPLOTS*(N_LINES_DATA_PER_PLOT_MAX+4),
	1'  R4',.TRUE.)
	  CALL DYNAMIC(ADDRESS3_8,NPLOTS,'  I4',.TRUE.) !  NPLPOINTS
	  CALL DYNAMIC(ADDRESS4_8,NPLOTS,'  I4',.TRUE.) !  LINVECT
	  CALL DYNAMIC(ADDRESS5_8,NPLOTS,'  I4',.TRUE.) !  INC
	  CALL DYNAMIC(ADDRESS6_8,NPLOTS,'  I4',.TRUE.) !  LINTYPE
	  CALL DYNAMIC(ADDRESS7_8,NPLOTS,'  I4',.TRUE.) !  NSYMBOL
	  CALL DYNAMIC(ADDRESS8_8,NPLOTS,'  I4',.TRUE.) !  NSYMBOLHEIGHT
	  CALL DYNAMIC(ADDRESS9_8,NPLOTS,'  I4',.TRUE.) !  COLOR
	  CALL XYPLOT_ASCII1(NCHOICE,%VAL(ADDRESS1_8),%VAL(ADDRESS2_8),
	1 N_LINES_DATA_PER_PLOT,N_LINES_DATA_PER_PLOT_MAX,NPLOTS_PER_FILE,
	2 NPLOTS,%VAL(ADDRESS3_8),%VAL(ADDRESS4_8),%VAL(ADDRESS5_8),
	3 %VAL(ADDRESS6_8),%VAL(ADDRESS7_8),%VAL(ADDRESS8_8),
	4 %VAL(ADDRESS9_8),COLUMNS_ABSCISSA,COLUMNS_ORDINATE,IUNIT,N_FILES)
	  CALL DYNAMIC(ADDRESS1_8,NPLOTS*(N_LINES_DATA_PER_PLOT_MAX+4),
	1 '  R4',.FALSE.)
	  CALL DYNAMIC(ADDRESS2_8,NPLOTS*(N_LINES_DATA_PER_PLOT_MAX+4),
	1 '  R4',.FALSE.)
	  CALL DYNAMIC(ADDRESS3_8,NPLOTS,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS4_8,NPLOTS,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS5_8,NPLOTS,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS6_8,NPLOTS,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS7_8,NPLOTS,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS8_8,NPLOTS,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS9_8,NPLOTS,'  I4',.FALSE.)
	END IF
	CALL RESETTEK
	IF (FILEUSED) THEN
	  CALL PLOT(0.0,0.0,9999)
	ELSE
	  CLOSE(UNIT=79,STATUS='DELETE')
	END IF
	CLOSE(UNIT=7)
	CLOSE(UNIT=21)
	IF (NCHOICE.EQ.2) THEN
	  CLOSE(UNIT=22)
	ELSE IF (NCHOICE.EQ.3) THEN
	  DO N=2,NPLOTS
	    CLOSE(UNIT=IUNIT(N))
	  END DO
	END IF
	CLOSE(UNIT=MAXNUNITS+21)
	CLOSE(UNIT=49,STATUS='DELETE')
C
	END
C
	SUBROUTINE XYPLOT_ASCII1(NCHOICE,XARRAY,YARRAY,N_LINES_DATA_PER_PLOT,
	1N_LINES_DATA_PER_PLOT_MAX,NPLOTS_PER_FILE,NPLOTS,NPLPOINTS,LINVECT,
	2INC,LINTYPE,NSYMBOL,NSYMBOLHEIGHT,COLOR,COLUMNS_ABSCISSA,
	3COLUMNS_ORDINATE,IUNIT,N_FILES)
C
C       This subroutine helps XYPLOT_ASCII carry out its operations.
C			Donald H. Gudehus, July, 1992
C			Version 2, Donald H. Gudehus, February 23, 2000
C			Fixed bug where NSYMBOL wasn't being handled
C			correctly.
C			Version 3, Donald H. Gudehus, March 14, 2008
C			Updated code to handle three cases, and changed
C			the name from XYPLOT2_PROGRAM1.
C
	PARAMETER (MAXNUNITS=15,MAXNPLOTS=200)
C
	BYTE B_DATA(1280)
C
	CHARACTER*20 DATA(64) !  We assume 20 significant figures maximum
	EQUIVALENCE (B_DATA,DATA)
C
	INTEGER N_LINES_DATA_PER_FILE(MAXNUNITS),
	1N_LINES_DATA_PER_PLOT(MAXNPLOTS),NPLOTS_PER_FILE(MAXNUNITS),
	1NPLPOINTS(NPLOTS),LINVECT(NPLOTS),INC(NPLOTS),LINTYPE(NPLOTS),
	2NSYMBOL(NPLOTS),NSYMBOLHEIGHT(NPLOTS),COLOR(NPLOTS),IUNIT(NPLOTS)
	INTEGER COLUMNS_ABSCISSA(64,MAXNUNITS),COLUMNS_ORDINATE(64,MAXNUNITS)
C
	REAL*4 XARRAY(N_LINES_DATA_PER_PLOT_MAX+4,NPLOTS),
	1YARRAY(N_LINES_DATA_PER_PLOT_MAX+4,NPLOTS)
CC
	INTEGER SIZE
C
	LOGICAL INVERT(2),INTERACT,CROSSHAIRS
C
	COMMON /XYPLOT__SETUP/ INDEX_TOTAL,INDEX_LINVECT,INDEX_PENWIDTH,
	1INDEX_LINTYPE,INDEX_SYMBOL,INDEX_SYMBOLHEIGHT,INDEX_COLOR,
	2NUMBER_LINVECT,NUMBER_PENWIDTH,NUMBER_LINTYPE,
	3NUMBER_SYMBOL,NUMBER_SYMBOLHEIGHT,NUMBER_COLOR
C
	CHARACTER*3 DEVICES(10)
	CHARACTER*7 INVERT_X,INVERT_Y
	CHARACTER*50 XLABEL,YLABEL,TITLE
	COMMON /XYPLOT__SETUP_CHAR/ INVERT_X,INVERT_Y,XLABEL,YLABEL,TITLE
C
	CALL GET_DP_AVAILABILITY
	SIZE=N_LINES_DATA_PER_PLOT_MAX+4
C  Read in the data from the user's character file(s)
C  First do the abscissa and ordinate for choice 1, and the abscissa for
C  choice 2.
	IF (NCHOICE.EQ.1.OR.NCHOICE.EQ.2) THEN
	  I=0
	  DO WHILE(.TRUE.)
	    READ (21,1200,END=210) DATA
1200	    FORMAT(132A)
	    DO K=1,1280
	      IF (B_DATA(K).LT.32) B_DATA(K)=32
	    END DO
	    I=I+1
	    CALL ALPHA(%REF(DATA),64,20,NCHAR,NFULL,NREST,4)
	    DO J=1,NPLOTS
C  Count the number of characters for this column entry
	      CALL ALPHA(%REF(DATA(COLUMNS_ABSCISSA(J,1))),1,20,
	1     NCHAR,NFULL,NREST,1)
	      DECODE(NCHAR,2000,DATA(COLUMNS_ABSCISSA(J,1))) TEMP
2000	      FORMAT(F<NCHAR>.0) !  This works for decimal and integer input
				 !  numbers
	      XARRAY(I,J)=TEMP
	      IF (NCHOICE.EQ.1) THEN
	        CALL ALPHA(%REF(DATA(COLUMNS_ORDINATE(J,1))),1,20,
	1       NCHAR,NFULL,NREST,1)
	        DECODE(NCHAR,2000,DATA(COLUMNS_ORDINATE(J,1))) TEMP
	        YARRAY(I,J)=TEMP
	      END IF
	    END DO
	  END DO
 210	  CONTINUE
	END IF
C  Next do the ordinate for choice 2.
	IF (NCHOICE.EQ.2) THEN
	  I=0
	  DO WHILE(.TRUE.)
	    READ (22,1200,END=220) DATA
	    DO K=1,1280
	      IF (B_DATA(K).LT.32) B_DATA(K)=32
	    END DO
	    I=I+1
	    CALL ALPHA(%REF(DATA),64,20,NCHAR,NFULL,NREST,4)
	    DO J=1,NPLOTS
C  Count the number of characters for this column entry
	      CALL ALPHA(%REF(DATA(COLUMNS_ORDINATE(J,1))),1,20,
	1     NCHAR,NFULL,NREST,1)
	      DECODE(NCHAR,2000,DATA(COLUMNS_ORDINATE(J,1))) TEMP
	      YARRAY(I,J)=TEMP
	    END DO
	  END DO
 220	  CONTINUE
	END IF
C  Get the abscissa and ordinate for choice 3.
	IF (NCHOICE.EQ.3) THEN
	  NINC=0
	  DO N=1,N_FILES
	    I=0
	    DO WHILE(.TRUE.)
	      READ (IUNIT(N),1200,END=230) DATA
	      DO K=1,1280
	        IF (B_DATA(K).LT.32) B_DATA(K)=32
	      END DO
	      I=I+1
	      CALL ALPHA(%REF(DATA),64,20,NCHAR,NFULL,NREST,4)
	      DO J=1,NPLOTS_PER_FILE(N)
C  Count the number of characters for this column entry
	        CALL ALPHA(%REF(DATA(COLUMNS_ABSCISSA(J,N))),1,20,
	1       NCHAR,NFULL,NREST,1)
	        DECODE(NCHAR,2000,DATA(COLUMNS_ABSCISSA(J,N))) TEMP
	        XARRAY(I,J+NINC)=TEMP
	        CALL ALPHA(%REF(DATA(COLUMNS_ORDINATE(J,N))),1,20,
	1       NCHAR,NFULL,NREST,1)
	        DECODE(NCHAR,2000,DATA(COLUMNS_ORDINATE(J,N))) TEMP
	        YARRAY(I,J+NINC)=TEMP
	      END DO
	    END DO
 230	    CONTINUE
	    NINC=NINC+NPLOTS_PER_FILE(N)
	  END DO
	END IF
C Initialize
	XARRAY(SIZE-3,1)=0.0
	YARRAY(SIZE-3,1)=0.0
	DO I=1,NPLOTS
	  NPLPOINTS(I)=N_LINES_DATA_PER_PLOT(I)
	  INC(I)=1
	END DO
C  If xyplot_setup.dat was not found, do not try to read from
C  unit MAXNUNITS+21, i.e., 36;  instead, use simple defaults.
	IF (INDEX_TOTAL.EQ.0) THEN
	  DO I=1,NPLOTS
	    LINVECT(I)=0
	    LINTYPE(I)=-1
	    NSYMBOL(I)=3 !  + symbol
	    COLOR(I)=16711680 !  Red
	  END DO
	  INVERT(1)=.FALSE.
	  INVERT(2)=.FALSE.
	  XLABEL='X Label'
	  YLABEL='Y Label'
	  TITLE='Untitled'
	  GO TO 9999
	END IF
C  Load the parameters from the setup file, using rollover when
C  necessary.
	REWIND (MAXNUNITS+21)
	DO I=1,INDEX_LINVECT
	  READ (MAXNUNITS+21,3000,END=299) STRING
3000	  FORMAT (A)
	END DO
	DO I=1,MIN(NPLOTS,NUMBER_LINVECT)
	  READ (MAXNUNITS+21,4009,END=299) LINVECT(I)
4009	  FORMAT(I9)
	END DO
C  Assign LINE VECTOR STYLE values to each plot, by rolling over the
C  setup values.
 299	IF (NPLOTS.GT.NUMBER_LINVECT.AND.NUMBER_LINVECT.GT.0) THEN
	  DO I=1,NPLOTS
	    LINVECT(I)=LINVECT(MOD(I-1,NUMBER_LINVECT)+1)
	  END DO
	END IF
C
	REWIND (MAXNUNITS+21)
	DO I=1,INDEX_PENWIDTH
	  READ (MAXNUNITS+21,3000,END=399) STRING
	END DO
	DO I=1,MIN(NPLOTS,NUMBER_PENWIDTH)
	  READ (MAXNUNITS+21,4009,END=399) LINTYPE(I) 
				!  LINTYPE is used temporarily
	END DO
C  Assign PEN WIDTH values to each plot, by rolling over the setup values.
C  The pen width goes in the second byte of LINVECT
 399	IF (NPLOTS.GT.NUMBER_PENWIDTH.AND.NUMBER_PENWIDTH.GT.0) THEN
	  DO I=1,NPLOTS
	    LINVECT(I)=LINVECT(I)+256*LINTYPE(MOD(I-1,NUMBER_PENWIDTH)+1)
	  END DO
	END IF
C
	REWIND (MAXNUNITS+21)
	DO I=1,INDEX_LINTYPE
	  READ (MAXNUNITS+21,3000,END=499) STRING
	END DO
	DO I=1,MIN(NPLOTS,NUMBER_LINTYPE)
	  READ (MAXNUNITS+21,4009,END=499) LINTYPE(I)
	END DO
C  Assign LINE TYPE values to each plot, by rolling over the setup values.
 499	IF (NPLOTS.GT.NUMBER_LINTYPE.AND.NUMBER_LINTYPE.GT.0) THEN
	  DO I=1,NPLOTS
	    LINTYPE(I)=LINTYPE(MOD(I-1,NUMBER_LINTYPE)+1)
	  END DO
	END IF
C
	REWIND (MAXNUNITS+21)
	DO I=1,INDEX_SYMBOL
	  READ (MAXNUNITS+21,3000,END=599) STRING
	END DO
	DO I=1,MIN(NPLOTS,NUMBER_SYMBOL)
	  READ (MAXNUNITS+21,4009,END=599) NSYMBOL(I)
	END DO
C  Assign SYMBOL values to each plot, by rolling over the setup values.
 599	IF (NPLOTS.GT.NUMBER_SYMBOL.AND.NUMBER_SYMBOL.GT.0) THEN
	  DO I=1,NPLOTS
	    NSYMBOL(I)=NSYMBOL(MOD(I-1,NUMBER_SYMBOL)+1)
	  END DO
	END IF
C
	REWIND (MAXNUNITS+21)
	DO I=1,INDEX_SYMBOLHEIGHT
	  READ (MAXNUNITS+21,3000,END=699) STRING
	END DO
	DO I=1,MIN(NPLOTS,NUMBER_SYMBOLHEIGHT)
	  READ (MAXNUNITS+21,4009,END=699) NSYMBOLHEIGHT(I)
	END DO
C  Assign SYMBOL HEIGHT values to each plot, by rolling over the setup values.
 699	IF (NPLOTS.GT.NUMBER_SYMBOLHEIGHT.AND.NUMBER_SYMBOLHEIGHT.GT.0) 
	1THEN
	  DO I=1,NPLOTS
	    NSYMBOL(I)=256**2*NSYMBOLHEIGHT(MOD(I-1,NUMBER_SYMBOLHEIGHT)+1)+
	1   NSYMBOL(I)
	  END DO
	END IF
C
	REWIND (MAXNUNITS+21)
	DO I=1,INDEX_COLOR
	  READ (MAXNUNITS+21,3000,END=799) STRING
	END DO
	DO I=1,MIN(NPLOTS,NUMBER_COLOR)
	  READ (MAXNUNITS+21,4009,END=799) COLOR(I)
	END DO
C  Assign COLOR values to each plot, by rolling over the setup values.
 799	IF (NPLOTS.GT.NUMBER_COLOR) THEN
	  DO I=1,NPLOTS
	    COLOR(I)=COLOR(MOD(I-1,NUMBER_COLOR)+1)
	  END DO
	END IF
C
C  Get the values of INVERT for the X and Y axes from the COMMON block
	IF (INVERT_X.EQ.'NO'.OR.INVERT_X.EQ.'FALSE'.OR.INVERT_X.EQ.'.FALSE.')
	1INVERT(1)=.FALSE.
	IF (INVERT_X.EQ.'YES'.OR.INVERT_X.EQ.'TRUE'.OR.INVERT_X.EQ.'.TRUE.')
	1INVERT(1)=.TRUE.
	IF (INVERT_Y.EQ.'NO'.OR.INVERT_Y.EQ.'FALSE'.OR.INVERT_Y.EQ.'.FALSE.')
	1INVERT(2)=.FALSE.
	IF (INVERT_Y.EQ.'YES'.OR.INVERT_Y.EQ.'TRUE'.OR.INVERT_Y.EQ.'.TRUE.')
	1INVERT(2)=.TRUE.
C  Use XLABEL, YLABEL, and TITLE values from the COMMON block
9999	CROSSHAIRS=.TRUE.
	INTERACT=.FALSE.
	DEVICES(1)=' '
	CALL XYPLOT(XARRAY,YARRAY,SIZE,NPLOTS,NPLPOINTS,LINVECT,INC,
	1LINTYPE,NSYMBOL,INVERT,COLOR,INTERACT,CROSSHAIRS,XLABEL,
	2YLABEL,TITLE,DEVICES,.FALSE.)
	RETURN
	END
