      SUBROUTINE ROTATE_FITS_SIMPLE
C
C  Note:  This is the Unix version
C
C       This subroutine will rotate a FITS file by 90, 180, or 270
C  degrees.  If more than one map exists in the file, the user can
C  select which maps to rotate.  If a map has more than one plane in 
C  it, all the planes will be rotated.  In this vesion, the output
C  file cannot be the input file.
C		Version 1, Donald H. Gudehus, June 22, 2006
C		Ported code to Unix;  removed references to
C		Grinnell and Trapix.
C		Version 2, D. Gudehus, September 1, 2013
C		Modified code to allow for 64-bit systems
C
	INCLUDE 'miipspar.for'
	INCLUDE 'fitsfacts.for'
C
	CHARACTER*3 FILEFMT
	CHARACTER*3 PRGFMAT_OUT(MAPMAX)
	CHARACTER*50 CHFITSMHV
	CHARACTER*80 FILENAME_IN,FILENAME_OUT
C
	INTEGER ICOL_OUT(2),IROW_OUT(2)
	INTEGER MAPS_IN(MAPMAX),MAPS_OUT(MAPMAX),STATUS
	INTEGER NCOLS_OUT(MAPMAX),NROWS_OUT(MAPMAX),NPLNS_OUT(MAPMAX)
C
	INTEGER*4 ADDRESS1_4
	INTEGER*8 ADDRESS1_8
	INTEGER SYSTEM_NUMBER_BYTES
	COMMON /SYS_NUM_BYTES/ SYSTEM_NUMBER_BYTES
C
	REAL*4 DATA(MAXSIZE),DATA_ROTATE(MAXSIZE)
	REAL*8 DATADP(MAXSIZE),DATADP_ROTATE(MAXSIZE)
C
	CHARACTER*80 DNAME
	COMMON /DFILE/ DNAME
C
	ICOMMAND=0
	DO WHILE (ICOMMAND.NE.1.AND.ICOMMAND.NE.2.AND.ICOMMAND.NE.3)
	  ICOMMAND=1
	  CALL MESSAGE('Enter a command:')
	  CALL MESSAGE('     1 = Rotates by 90 degrees clockwise (default)')
	  CALL MESSAGE('     2 = Rotates by 180 degrees clockwise')
	  CALL MESSAGE('     3 = Rotates by 90 degrees counterclockwise')
	  CALL USRINP(ICOMMAND,1,'I',4,1,'COMMAND=',' ')
	END DO
C
C  Open the input file and select the maps
C
	LU_IN=11
	FILENAME_IN=' ' 
	CALL GET_INPUT_FITS_MAPS(LU_IN,FILENAME_IN,MAPS_IN,NMAPS_IN,STATUS)
	IF (STATUS.NE.0) THEN
	  CALL CLOSEFITS(LU_IN)
	  RETURN
	END IF
C
C  Calculate what size output maps are required for the various commands
C
	DO N=1,NMAPS_IN
	  NPLNS_OUT(N)=NAXIS3(MAPS_IN(N),LU_IN)
	  PRGFMAT_OUT(N)=CHFITSMHV(LU_IN,MAPS_IN(N),'BITPIX','R4')
	  IF (ICOMMAND.EQ.1.OR.ICOMMAND.EQ.3) THEN
	    NCOLS_OUT(N)=NAXIS2(MAPS_IN(N),LU_IN)
	    NROWS_OUT(N)=NAXIS1(MAPS_IN(N),LU_IN)
	  ELSE IF (ICOMMAND.EQ.2) THEN
	    NCOLS_OUT(N)=NAXIS1(MAPS_IN(N),LU_IN)
	    NROWS_OUT(N)=NAXIS2(MAPS_IN(N),LU_IN)
	  END IF
	END DO
C
C  Open the output file and select the maps
C
	LU_OUT=12
	FILENAME_OUT=' ' !  Lets the user choose
	NMAPS_OUT=NMAPS_IN
	CALL GET_OUTPUT_FITS_MAPS(LU_OUT,FILENAME_OUT,MAPS_OUT,NMAPS_OUT,
	1NPLNS_OUT,NCOLS_OUT,NROWS_OUT,PRGFMAT_OUT,STATUS)
	IF (STATUS.NE.0) THEN
	  CALL CLOSEFITS(LU_IN)
	  CALL CLOSEFITS(LU_OUT)
	  RETURN
	END IF
C
C  Carry out the rotation
C
	IF (ICOMMAND.EQ.1.OR.ICOMMAND.EQ.3) THEN
	  CALL MESSAGE(' ')
	  IF (ICOMMAND.EQ.1) CALL MESSAGE('   Rotating 90 Degrees Clockwise')
	  IF (ICOMMAND.EQ.3) CALL MESSAGE('Rotating 90 Degrees '//
	1 'Counterclockwise')
	  CALL MESSAGE('Input Map No.       Output Map No.')
	  DO N=1,NMAPS_OUT
	    IMNUM_IN=MAPS_IN(N)
	    IMNUM_OUT=MAPS_OUT(N)
	    WRITE(6,1000) IMNUM_IN,IMNUM_OUT
1000	    FORMAT(' ',6X,I4,17X,I4)
	    WRITE(7,1000) IMNUM_IN,IMNUM_OUT
	    NI=NAXIS1(MAPS_IN(N),LU_IN)
	    NJ=NAXIS2(MAPS_IN(N),LU_IN)
	    DO IPNUM=1,NPLNS_OUT(N)
	      IROW_OUT(1)=1
	      IROW_OUT(2)=NAXIS2(MAPS_OUT(N),LU_OUT)
	      ICOL_OUT(1)=1
	      ICOL_OUT(2)=NAXIS1(MAPS_OUT(N),LU_OUT)
	      FILEFMT=CHFITSMHV(LU_OUT,IMNUM_OUT,'BITPIX','R4')
	      IF (FILEFMT.EQ.'I1'.OR.FILEFMT.EQ.'I2'.OR.FILEFMT.EQ.'I4'.OR.
	1     FILEFMT.EQ.'R4'.OR.FILEFMT.EQ.'C8') THEN
	        IF (SYSTEM_NUMBER_BYTES.EQ.4) THEN
	          CALL DYNAMIC(ADDRESS1_4,NI*NJ,'  R4',.TRUE.)
	          CALL ROTATE_FITS_SIMPLE1SP(%VAL(ADDRESS1_4),NI,NJ,
	1         DATA_ROTATE,DATADP_ROTATE,MAXSIZE,ICOMMAND,
	2         LU_IN,LU_OUT,IMNUM_IN,IMNUM_OUT,IPNUM,ICOL_OUT,IROW_OUT)
	          CALL DYNAMIC(ADDRESS1_4,NI*NJ,'  R4',.FALSE.)
	        ELSE IF (SYSTEM_NUMBER_BYTES.EQ.8) THEN
	          CALL DYNAMIC(ADDRESS1_8,NI*NJ,'  R4',.TRUE.)
	          CALL ROTATE_FITS_SIMPLE1SP(%VAL(ADDRESS1_8),NI,NJ,
	1         DATA_ROTATE,DATADP_ROTATE,MAXSIZE,ICOMMAND,
	2         LU_IN,LU_OUT,IMNUM_IN,IMNUM_OUT,IPNUM,ICOL_OUT,IROW_OUT)
	          CALL DYNAMIC(ADDRESS1_8,NI*NJ,'  R4',.FALSE.)
	        END IF
	      ELSE IF (FILEFMT.EQ.'R8'.OR.FILEFMT.EQ.'C16') THEN
	        IF (SYSTEM_NUMBER_BYTES.EQ.4) THEN
	          CALL DYNAMIC(ADDRESS1_4,NI*NJ,'  R8',.TRUE.)
	          CALL ROTATE_FITS_SIMPLE1DP(%VAL(ADDRESS1_4),NI,NJ,
	1         DATA_ROTATE,DATADP_ROTATE,MAXSIZE,ICOMMAND,
	2         LU_IN,LU_OUT,IMNUM_IN,IMNUM_OUT,IPNUM,ICOL_OUT,IROW_OUT)
	          CALL DYNAMIC(ADDRESS1_4,NI*NJ,'  R8',.FALSE.)
	        ELSE IF (SYSTEM_NUMBER_BYTES.EQ.8) THEN
	          CALL DYNAMIC(ADDRESS1_8,NI*NJ,'  R8',.TRUE.)
	          CALL ROTATE_FITS_SIMPLE1DP(%VAL(ADDRESS1_8),NI,NJ,
	1         DATA_ROTATE,DATADP_ROTATE,MAXSIZE,ICOMMAND,
	2         LU_IN,LU_OUT,IMNUM_IN,IMNUM_OUT,IPNUM,ICOL_OUT,IROW_OUT)
	          CALL DYNAMIC(ADDRESS1_8,NI*NJ,'  R8',.FALSE.)
	        END IF
	      END IF
	    END DO
	  END DO
	ELSE IF (ICOMMAND.EQ.2) THEN
	  CALL MESSAGE(' ')
	  CALL MESSAGE('     Rotating 180 Degrees')
	  CALL MESSAGE('Input Map No.       Output Map No.')
	  DO N=1,NMAPS_OUT
	    IMNUM_IN=MAPS_IN(N)
	    IMNUM_OUT=MAPS_OUT(N)
	    WRITE(6,1000) IMNUM_IN,IMNUM_OUT
	    WRITE(7,1000) IMNUM_IN,IMNUM_OUT
	    DO IPNUM=1,NPLNS_OUT(N)
	      ICOL_OUT(1)=1
	      ICOL_OUT(2)=NAXIS1(MAPS_OUT(N),LU_OUT)
	      IROW_OUT(1)=1
	      IROW_OUT(2)=NAXIS2(MAPS_OUT(N),LU_OUT)
	      FILEFMT=CHFITSMHV(LU_OUT,IMNUM_OUT,'BITPIX','R4')
	      IF (FILEFMT.EQ.'I1'.OR.FILEFMT.EQ.'I2'.OR.FILEFMT.EQ.'I4'.OR.
	1     FILEFMT.EQ.'R4'.OR.FILEFMT.EQ.'C8') THEN
	        DO IR=IROW_OUT(1),IROW_OUT(2)
	          IROW_READ=IROW_OUT(2)-IR+1
	          CALL RDFITSROW(LU_IN,IMNUM_IN,IPNUM,DATA,MAXSIZE,IROW_READ)
	          DO I=ICOL_OUT(1),ICOL_OUT(2)
	            II=ICOL_OUT(2)-I+1
	            DATA_ROTATE(I)=DATA(II)
	          END DO
	          CALL WRFITSROW(LU_OUT,IMNUM_OUT,IPNUM,DATA_ROTATE,MAXSIZE,IR)
	        END DO
	      ELSE IF (FILEFMT.EQ.'R8'.OR.FILEFMT.EQ.'C16') THEN
	        DO IR=IROW_OUT(1),IROW_OUT(2)
	          IROW_READ=IROW_OUT(2)-IR+1
	          CALL RDDPFITSROW(LU_IN,IMNUM_IN,IPNUM,DATADP,MAXSIZE,
	1         IROW_READ)
	          DO I=ICOL_OUT(1),ICOL_OUT(2)
	            II=ICOL_OUT(2)-I+1
	            DATADP_ROTATE(I)=DATADP(II)
	          END DO
	          CALL WRDPFITSROW(LU_OUT,IMNUM_OUT,IPNUM,DATADP_ROTATE,
	1         MAXSIZE,IR)
	        END DO
	      END IF
	    END DO
	  END DO
	END IF
	CALL CLOSEFITS(LU_IN)
	CALL CLOSEFITS(LU_OUT)
	RETURN
	END

	SUBROUTINE ROTATE_FITS_SIMPLE1(DATAXY,NI,NJ,DATA_ROTATE,
	1DATADP_ROTATE,MAXSIZE,ICOMMAND,LU_IN,LU_OUT,
	2IMNUM_IN,IMNUM_OUT,IPNUM,ICOL_OUT,IROW_OUT)
C
C       This subroutine help ROTATE_FITS_SIMPLE in carrying out
C  a FITS file rotation.
C			Donald H. Gudehus, June, 2006
C
	INTEGER*4 ICOL_OUT(2),IROW_OUT(2)
	REAL*4 DATAXY(NI,NJ),DATA_ROTATE(MAXSIZE)
	REAL*8 DATADPXY(NI,NJ),DATADP_ROTATE(MAXSIZE)
C
	CHARACTER*3 PRGFMAT
C
	ENTRY ROTATE_FITS_SIMPLE1SP(DATAXY,NI,NJ,DATA_ROTATE,
	1DATADP_ROTATE,MAXSIZE,ICOMMAND,LU_IN,LU_OUT,
	2IMNUM_IN,IMNUM_OUT,IPNUM,ICOL_OUT,IROW_OUT)
	PRGFMAT='R4'
	GO TO 100
C
	ENTRY ROTATE_FITS_SIMPLE1DP(DATADPXY,NI,NJ,DATA_ROTATE,
	1DATADP_ROTATE,MAXSIZE,ICOMMAND,LU_IN,LU_OUT,
	2IMNUM_IN,IMNUM_OUT,IPNUM,ICOL_OUT,IROW_OUT)
	PRGFMAT='R8'
	GO TO 100
C
 100	CONTINUE
	IF (PRGFMAT.EQ.'R4') THEN
	  CALL RDFITSXY(LU_IN,IMNUM_IN,IPNUM,DATAXY,NI,NJ,
	1 1,NI,1,NJ)
	  IF (ICOMMAND.EQ.1) THEN !  90 Degrees Clockwise
C  Loop on the output rows
	    DO IR=IROW_OUT(1),IROW_OUT(2)
C  Compute the column in the array of the input map to read.
C  Start at the right side.
	      ICOL=NI-IR+1
C  Load the output row-array
	      DO I=ICOL_OUT(1),ICOL_OUT(2)
	        DATA_ROTATE(I)=DATAXY(ICOL,I)
	      END DO
	      CALL WRFITSROW(LU_OUT,IMNUM_OUT,IPNUM,DATA_ROTATE,MAXSIZE,IR)
	    END DO
	  ELSE IF (ICOMMAND.EQ.3) THEN !  90 Degrees Counterclockwise
C  Loop on the output rows
	    DO IR=IROW_OUT(1),IROW_OUT(2)
C  Compute the column in the array of the input map to read.
C  Start at the left side.
	      ICOL=IR
C  Load the output row-array
	      DO I=ICOL_OUT(1),ICOL_OUT(2)
	        II=ICOL_OUT(2)-I+1
	        DATA_ROTATE(I)=DATAXY(ICOL,II)
	      END DO
	      CALL WRFITSROW(LU_OUT,IMNUM_OUT,IPNUM,DATA_ROTATE,MAXSIZE,IR)
	    END DO
	  END IF
	ELSE IF (PRGFMAT.EQ.'R8') THEN
	  CALL RDDPFITSXY(LU_IN,IMNUM_IN,IPNUM,DATADPXY,NI,NJ,
	1 1,NI,1,NJ)
	  IF (ICOMMAND.EQ.1) THEN
C  Loop on the output rows
	    DO IR=IROW_OUT(1),IROW_OUT(2)
C  Compute the column in the array of the input map to read.
C  Start at the right side.
	      ICOL=NI-IR+1
C  Load the output row-array
	      DO I=ICOL_OUT(1),ICOL_OUT(2)
	        DATADP_ROTATE(I)=DATADPXY(ICOL,I)
	      END DO
	      CALL WRDPFITSROW(LU_OUT,IMNUM_OUT,IPNUM,DATADP_ROTATE,MAXSIZE,IR)
	    END DO
	  ELSE IF (ICOMMAND.EQ.3) THEN
C  Loop on the output rows
	    DO IR=IROW_OUT(1),IROW_OUT(2)
C  Compute the column in the array of the input map to read.
C  Start at the left side.
	      ICOL=IR
C  Load the output row-array
	      DO I=ICOL_OUT(1),ICOL_OUT(2)
	        II=ICOL_OUT(2)-I+1
	        DATADP_ROTATE(I)=DATADPXY(ICOL,II)
	      END DO
	      CALL WRDPFITSROW(LU_OUT,IMNUM_OUT,IPNUM,DATADP_ROTATE,MAXSIZE,IR)
	    END DO
	  END IF
	END IF
C
	RETURN
	END
