	SUBROUTINE RWFITSXY(LU,NMAP,NPLANE,DATA,LX,LY,ISX,ILX,ISY,ILY)
C
C       This subroutine reads from or writes to a FITS file, a rectangular
C  section of data.  One should not call RWFITSROW directly but rather one
C  of the various entries, e.g., RDFITSXY or RDDPFITSXY for reading, and
C  WRFITSXY or WRDPFITSXY, for writing.  Each of the entries can read
C  or write data for several different value of BITPIX, as contained
C  in the COMMON block FITSFACTS_INTEGER4.  The choice of which subroutine
C  one uses will affect whether the data array in the argument list is
C  REAL*4 or REAL*8.
C
C			Donald H. Gudehus, June 26, 2005
C			Version 2, D. H. Gudehus, January 28, 2008
C			Improved the accuracy when writing by taking
C			the nearest integer.
C
C	LU	(C)	Logical unit for I/O
C	NMAP	(C)	Number of map to be read or written to, counting
C			from 1
C	NPLANE	(C)	Number of plane to be read or written to, counting
C			from 1
C	DATA	(C/R)	A two-dimensional array to receive data on read or to 
C			provide data on write
C	LX	(C)	Dimension of data array in the X direction.
C	LY	(C)	Dimension of data array in the Y direction.
C	ISX	(C)	The first column in the section of the file to be read
C			or written to.  It is measured relative to the origin
C			of the image (1,1) and always corresponds to the origin
C			of the array DATA.
C	ILX	(C)	The number of columns in the section to be read or
C			written to.
C	ISY	(C)	The first row in the section of the file to be read
C			or written to.  Analogous to ISX.
C	ILY	(C)	The number of rows in the section to be read or
C			written to
C
	INTEGER LU
	REAL DATA(LX,LY)
	REAL*8 DATADP(LX,LY)
C
	INCLUDE 'fitsfacts.for'
C
	BYTE PHYSREC(3000)
	INTEGER*2 I2REC(1500)
	INTEGER I4REC(750),I4X2REC(1500)
	REAL*4 R4REC(750)
	REAL*8 R8REC(375),BS,BZ
	EQUIVALENCE (PHYSREC,I2REC,I4REC,I4X2REC,R4REC,R8REC)
	CHARACTER*3 PRGFMAT
	CHARACTER*4 CALL
	INTEGER BSWAP,WSWAP
C
	CHARACTER*1 SYSTEM_ENDIAN
	COMMON /SYS_ENDIAN/ SYSTEM_ENDIAN
C
	ENTRY RDFITSXY(LU,NMAP,NPLANE,DATA,LX,LY,ISX,ILX,ISY,ILY)
	PRGFMAT='R4'
	CALL='READ'
	GO TO 100
C
	ENTRY WRFITSXY(LU,NMAP,NPLANE,DATA,LX,LY,ISX,ILX,ISY,ILY)
	CALL='WRIT'
	PRGFMAT='R4'
	GO TO 100
C
	ENTRY RDDPFITSXY(LU,NMAP,NPLANE,DATADP,LX,LY,ISX,ILX,ISY,ILY)
	PRGFMAT='R8'
	CALL='READ'
	GO TO 100
C
	ENTRY WRDPFITSXY(LU,NMAP,NPLANE,DATADP,LX,LY,ISX,ILX,ISY,ILY)
	CALL='WRIT'
	PRGFMAT='R8'
	GO TO 100
C
 100	IBLKLEN=2880
	BSWAP=1
	WSWAP=1
C  We first loop on rows, calculating for each row, the start and end
C  records to read, and then transfer data to the array for each record.
C  An alternate way of accomplishing this would be to create dynamic
C  memory equal to the size of the image, and then just loop on rows
C  and columns.
C
	NBYTESPERPIXEL=ABS(BITPIX(NMAP,LU))/8
	NBYTESPERROW=NAXIS1(NMAP,LU)*NBYTESPERPIXEL
	NBYTESPERPLANE=NBYTESPERROW*NAXIS2(NMAP,LU)
	BS=BSCALE(NMAP,LU)
	BZ=BZERO(NMAP,LU)
	DO J=1,ILY
	  IBYTEOFFSET1=(ISY+J-2)*NBYTESPERROW+(NPLANE-1)*NBYTESPERPLANE
		!  From beginning of map to row of interest start
	  IBYTEOFFSET1=IBYTEOFFSET1+(ISX-1)*NBYTESPERPIXEL
		!  From beginning of map to row section start
C	  IBYTEOFFSET2=IBYTEOFFSET1+(ILX-1)*NBYTESPERPIXEL
	  IBYTEOFFSET2=IBYTEOFFSET1+ILX*NBYTESPERPIXEL-1
		!  From beginning of map to row section end
	  IRECOFFSET1=IBYTEOFFSET1/IBLKLEN
	  IRECOFFSET2=IBYTEOFFSET2/IBLKLEN
	  IRECSTART=MAPRECORDS(1,NMAP,LU)+IRECOFFSET1
	  IRECEND=MAPRECORDS(1,NMAP,LU)+IRECOFFSET2
	  IBYTEOFFSET3=IBYTEOFFSET1-IRECOFFSET1*IBLKLEN
		!  From start record boundary to row section start
	  IBYTEOFFSET4=IBYTEOFFSET2-IRECOFFSET2*IBLKLEN
		!  From end record boundary to row section end
	  IF (CALL.EQ.'READ') THEN
	    ICOUNT=1
	    DO IR=IRECSTART,IRECEND
	      READ(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	      IF (BITPIX(NMAP,LU).EQ.8) THEN
	        IF (IRECSTART.EQ.IRECEND) THEN
	          ISTART=IBYTEOFFSET3+1
		!  Adds 1 because we want an index in the record
	          IEND=IBYTEOFFSET4+1
	        ELSE
	          IF (IR.EQ.IRECSTART) THEN
	            ISTART=IBYTEOFFSET3+1
	            IEND=IBLKLEN
	          ELSE IF (IR.EQ.IRECEND) THEN
	            ISTART=1
	            IEND=IBYTEOFFSET4+1
	          ELSE
	            ISTART=1
	            IEND=IBLKLEN
	          END IF
	        END IF
	        IF (PRGFMAT.EQ.'R4') THEN
	          DO I=ISTART,IEND
	            IF (PHYSREC(I).LT.0) THEN
	              DATA(ICOUNT,J)=BS*
	1             (PHYSREC(I)+256)+BZ
	            ELSE
	              DATA(ICOUNT,J)=BS*PHYSREC(I)+BZ
	            END IF
	            ICOUNT=ICOUNT+1
	          END DO
	        ELSE IF (PRGFMAT.EQ.'R8') THEN
	          DO I=ISTART,IEND
	            IF (PHYSREC(I).LT.0) THEN
	              DATADP(ICOUNT,J)=BS*
	1             (PHYSREC(I)+256)+BZ
	            ELSE
	              DATADP(ICOUNT,J)=BS*PHYSREC(I)+BZ
	            END IF
	            ICOUNT=ICOUNT+1
	          END DO
	        END IF
	      ELSE IF (BITPIX(NMAP,LU).EQ.16) THEN
	        IF (SYSTEM_ENDIAN.EQ.'L') THEN
	          CALL IBMI2(I2REC,I2REC,IBLKLEN/2)
	        END IF
	        IF (IRECSTART.EQ.IRECEND) THEN
	          ISTART=IBYTEOFFSET3/2+1
	          IEND=IBYTEOFFSET4/2+1
	        ELSE
	          IF (IR.EQ.IRECSTART) THEN
	            ISTART=IBYTEOFFSET3/2+1
	            IEND=IBLKLEN/2
	          ELSE IF (IR.EQ.IRECEND) THEN
	            ISTART=1
	            IEND=IBYTEOFFSET4/2+1
	          ELSE
	            ISTART=1
	            IEND=IBLKLEN/2
	          END IF
	        END IF
	        IF (PRGFMAT.EQ.'R4') THEN
	          DO I=ISTART,IEND
	            DATA(ICOUNT,J)=BS*I2REC(I)+BZ
	            ICOUNT=ICOUNT+1
	          END DO
	        ELSE IF (PRGFMAT.EQ.'R8') THEN
	          DO I=ISTART,IEND
	            DATADP(ICOUNT,J)=BS*I2REC(I)+BZ
	            ICOUNT=ICOUNT+1
	          END DO
	        END IF
	      ELSE IF (BITPIX(NMAP,LU).EQ.32) THEN
	        IF (SYSTEM_ENDIAN.EQ.'L') THEN
	          CALL IBMI4(I4REC,I4REC,IBLKLEN/4)
	        END IF
	        IF (IRECSTART.EQ.IRECEND) THEN
	          ISTART=IBYTEOFFSET3/4+1
	          IEND=IBYTEOFFSET4/4+1
	        ELSE
	          IF (IR.EQ.IRECSTART) THEN
	            ISTART=IBYTEOFFSET3/4+1
	            IEND=IBLKLEN/4
	          ELSE IF (IR.EQ.IRECEND) THEN
	            ISTART=1
	            IEND=IBYTEOFFSET4/4+1
	          ELSE
	            ISTART=1
	            IEND=IBLKLEN/4
	          END IF
	        END IF
	        IF (PRGFMAT.EQ.'R4') THEN
	          DO I=ISTART,IEND
	            DATA(ICOUNT,J)=BS*I4REC(I)+BZ
	            ICOUNT=ICOUNT+1
	          END DO
	        ELSE IF (PRGFMAT.EQ.'R8') THEN
	          DO I=ISTART,IEND
	            DATADP(ICOUNT,J)=BS*I4REC(I)+BZ
	            ICOUNT=ICOUNT+1
	          END DO
	        END IF
	      ELSE IF (BITPIX(NMAP,LU).EQ.-32) THEN
	        IF (SYSTEM_ENDIAN.EQ.'L') THEN
	          CALL IEEE2UNIX(I4REC,IBLKLEN/4,BSWAP,WSWAP)
	        END IF
	        IF (IRECSTART.EQ.IRECEND) THEN
	          ISTART=IBYTEOFFSET3/4+1
	          IEND=IBYTEOFFSET4/4+1
	        ELSE
	          IF (IR.EQ.IRECSTART) THEN
	            ISTART=IBYTEOFFSET3/4+1
	            IEND=IBLKLEN/4
	          ELSE IF (IR.EQ.IRECEND) THEN
	            ISTART=1
	            IEND=IBYTEOFFSET4/4+1
	          ELSE
	            ISTART=1
	            IEND=IBLKLEN/4
	          END IF
	        END IF
	        IF (PRGFMAT.EQ.'R4') THEN
	          DO I=ISTART,IEND
	            DATA(ICOUNT,J)=BS*R4REC(I)+BZ
	            ICOUNT=ICOUNT+1
	          END DO
	        ELSE IF (PRGFMAT.EQ.'R8') THEN
	          DO I=ISTART,IEND
	            DATADP(ICOUNT,J)=BS*R4REC(I)+BZ
	            ICOUNT=ICOUNT+1
	          END DO
	        END IF
	      ELSE IF (BITPIX(NMAP,LU).EQ.-64) THEN
	        IF (SYSTEM_ENDIAN.EQ.'L') THEN
	          CALL IEEE2UNIX64(I4X2REC,IBLKLEN/8,BSWAP,WSWAP)
	        END IF
	        IF (IRECSTART.EQ.IRECEND) THEN
	          ISTART=IBYTEOFFSET3/8+1
	          IEND=IBYTEOFFSET4/8+1
	        ELSE
	          IF (IR.EQ.IRECSTART) THEN
	            ISTART=IBYTEOFFSET3/8+1
	            IEND=IBLKLEN/8
	          ELSE IF (IR.EQ.IRECEND) THEN
	            ISTART=1
	            IEND=IBYTEOFFSET4/8+1
	          ELSE
	            ISTART=1
	            IEND=IBLKLEN/8
	          END IF
	        END IF
	        IF (PRGFMAT.EQ.'R4') THEN
	          DO I=ISTART,IEND
	            DATA(ICOUNT,J)=BS*R8REC(I)+BZ
	            ICOUNT=ICOUNT+1
	          END DO
	        ELSE IF (PRGFMAT.EQ.'R8') THEN
	          DO I=ISTART,IEND
	            DATADP(ICOUNT,J)=BS*R8REC(I)+BZ
	            ICOUNT=ICOUNT+1
	          END DO
	        END IF
	      END IF
	    END DO
C
	  ELSE IF (CALL.EQ.'WRIT') THEN
	    ICOUNT=1
	    DO IR=IRECSTART,IRECEND
	      READ(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	      IF (BITPIX(NMAP,LU).EQ.8) THEN
	        IF (IRECSTART.EQ.IRECEND) THEN
	          ISTART=IBYTEOFFSET3+1
		!  Adds 1 because we want an index in the record
	          IEND=IBYTEOFFSET4+1
	        ELSE
	          IF (IR.EQ.IRECSTART) THEN
	            ISTART=IBYTEOFFSET3+1
	            IEND=IBLKLEN
	          ELSE IF (IR.EQ.IRECEND) THEN
	            ISTART=1
	            IEND=IBYTEOFFSET4+1
	          ELSE
	            ISTART=1
	            IEND=IBLKLEN
	          END IF
	        END IF
	        IF (PRGFMAT.EQ.'R4') THEN
	          DO I=ISTART,IEND
	            PHYSREC(I)=NINT((DATA(ICOUNT,J)-BZ)/BS)
	            ICOUNT=ICOUNT+1
	          END DO
	        ELSE IF (PRGFMAT.EQ.'R8') THEN
	          DO I=ISTART,IEND
	            PHYSREC(I)=NINT((DATADP(ICOUNT,J)-BZ)/BS)
	            ICOUNT=ICOUNT+1
	          END DO
	        END IF
	      ELSE IF (BITPIX(NMAP,LU).EQ.16) THEN
	        IF (SYSTEM_ENDIAN.EQ.'L') THEN
	          CALL IBMI2(I2REC,I2REC,IBLKLEN/2)
	        END IF
	        IF (IRECSTART.EQ.IRECEND) THEN
	          ISTART=IBYTEOFFSET3/2+1
	          IEND=IBYTEOFFSET4/2+1
	        ELSE
	          IF (IR.EQ.IRECSTART) THEN
	            ISTART=IBYTEOFFSET3/2+1
	            IEND=IBLKLEN/2
	          ELSE IF (IR.EQ.IRECEND) THEN
	            ISTART=1
	            IEND=IBYTEOFFSET4/2+1
	          ELSE
	            ISTART=1
	            IEND=IBLKLEN/2
	          END IF
	        END IF
	        IF (PRGFMAT.EQ.'R4') THEN
	          DO I=ISTART,IEND
	            I2REC(I)=NINT((DATA(ICOUNT,J)-BZ)/BS)
	            ICOUNT=ICOUNT+1
	          END DO
	        ELSE IF (PRGFMAT.EQ.'R8') THEN
	          DO I=ISTART,IEND
	            I2REC(I)=NINT((DATADP(ICOUNT,J)-BZ)/BS)
	            ICOUNT=ICOUNT+1
	          END DO
	        END IF
	        IF (SYSTEM_ENDIAN.EQ.'L') THEN
	          CALL IBMI2(I2REC,I2REC,IBLKLEN/2) !  Restores the order
	        END IF
	      ELSE IF (BITPIX(NMAP,LU).EQ.32) THEN
	        IF (SYSTEM_ENDIAN.EQ.'L') THEN
	          CALL IBMI4(I4REC,I4REC,IBLKLEN/4)
	        END IF
	        IF (IRECSTART.EQ.IRECEND) THEN
	          ISTART=IBYTEOFFSET3/4+1
	          IEND=IBYTEOFFSET4/4+1
	        ELSE
	          IF (IR.EQ.IRECSTART) THEN
	            ISTART=IBYTEOFFSET3/4+1
	            IEND=IBLKLEN/4
	          ELSE IF (IR.EQ.IRECEND) THEN
	            ISTART=1
	            IEND=IBYTEOFFSET4/4+1
	          ELSE
	            ISTART=1
	            IEND=IBLKLEN/4
	          END IF
	        END IF
	        IF (PRGFMAT.EQ.'R4') THEN
	          DO I=ISTART,IEND
	            I4REC(I)=NINT((DATA(ICOUNT,J)-BZ)/BS)
	            ICOUNT=ICOUNT+1
	          END DO
	        ELSE IF (PRGFMAT.EQ.'R8') THEN
	          DO I=ISTART,IEND
	            I4REC(I)=NINT((DATADP(ICOUNT,J)-BZ)/BS)
	            ICOUNT=ICOUNT+1
	          END DO
	        END IF
	        IF (SYSTEM_ENDIAN.EQ.'L') THEN
	          CALL IBMI4(I4REC,I4REC,IBLKLEN/4)
	        END IF
	      ELSE IF (BITPIX(NMAP,LU).EQ.-32) THEN
	        IF (SYSTEM_ENDIAN.EQ.'L') THEN
	          CALL IEEE2UNIX(I4REC,IBLKLEN/4,BSWAP,WSWAP)
	        END IF
	        IF (IRECSTART.EQ.IRECEND) THEN
	          ISTART=IBYTEOFFSET3/4+1
	          IEND=IBYTEOFFSET4/4+1
	        ELSE
	          IF (IR.EQ.IRECSTART) THEN
	            ISTART=IBYTEOFFSET3/4+1
	            IEND=IBLKLEN/4
	          ELSE IF (IR.EQ.IRECEND) THEN
	            ISTART=1
	            IEND=IBYTEOFFSET4/4+1
	          ELSE
	            ISTART=1
	            IEND=IBLKLEN/4
	          END IF
	        END IF
	        IF (PRGFMAT.EQ.'R4') THEN
	          DO I=ISTART,IEND
	            R4REC(I)=(DATA(ICOUNT,J)-BZ)/BS
	            ICOUNT=ICOUNT+1
	          END DO
	        ELSE IF (PRGFMAT.EQ.'R8') THEN
	          DO I=ISTART,IEND
	            R4REC(I)=(DATADP(ICOUNT,J)-BZ)/BS
	            ICOUNT=ICOUNT+1
	          END DO
	        END IF
	        IF (SYSTEM_ENDIAN.EQ.'L') THEN
	          CALL UNIX2IEEE(I4REC,IBLKLEN/4,BSWAP,WSWAP)
	        END IF
	      ELSE IF (BITPIX(NMAP,LU).EQ.-64) THEN
	        IF (SYSTEM_ENDIAN.EQ.'L') THEN
	          CALL IEEE2UNIX64(I4X2REC,IBLKLEN/8,BSWAP,WSWAP)
	        END IF
	        IF (IRECSTART.EQ.IRECEND) THEN
	          ISTART=IBYTEOFFSET3/8+1
	          IEND=IBYTEOFFSET4/8+1
	        ELSE
	          IF (IR.EQ.IRECSTART) THEN
	            ISTART=IBYTEOFFSET3/8+1
	            IEND=IBLKLEN/8
	          ELSE IF (IR.EQ.IRECEND) THEN
	            ISTART=1
	            IEND=IBYTEOFFSET4/8+1
	          ELSE
	            ISTART=1
	            IEND=IBLKLEN/8
	          END IF
	        END IF
	        IF (PRGFMAT.EQ.'R4') THEN
	          DO I=ISTART,IEND
	            R8REC(I)=(DATA(ICOUNT,J)-BZ)/BS
	            ICOUNT=ICOUNT+1
	          END DO
	        ELSE IF (PRGFMAT.EQ.'R8') THEN
	          DO I=ISTART,IEND
	            R8REC(I)=(DATADP(ICOUNT,J)-BZ)/BS
	            ICOUNT=ICOUNT+1
	          END DO
	        END IF
	        IF (SYSTEM_ENDIAN.EQ.'L') THEN
	          CALL UNIX2IEEE64(I4X2REC,IBLKLEN/8,BSWAP,WSWAP)
	        END IF
	      END IF
C  Now write the record back
	      WRITE(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	    END DO
	  END IF
	END DO
C
	RETURN
	END
