	SUBROUTINE SAODELS(IDUMUNIT,KEY,MODE)
C
C  Note:  This is the UNIX version

C       This subroutine sends data from a FITS file to SAOimage.
C  It is used by the "I" command of the "TV" option and by
C  other options which require loading of an image.  It functions
C  as an interactive version of SAOLOAD.  A selected part of the
C  image can placed in a selected part of the frame.
C			Donald H. Gudehus, August, 1993
C			Version 1U, D. Gudehus, July 18, 2006
C
C	IDUMUNIT	(C)	The coded unit number.  If it is less than
C				zero, the file is to be opened by this 
C				subroutine;  if it is greater than zero, 
C				the file has already been opened.
C	KEY		(C)	The flag for whether the file name is to be
C				requested from the user (=0), or is to be
C				obtained from the common block DNAME (=1)
C				in the case IDUMUNIT<0.
C	MODE		(C)	The mode for loading an image.  If MODE=0
C				the entire frame buffer is used, and several
C				lines are loaded at a time.  If MODE=1
C				only one line at a time is loaded.  Although
C				this is slower, the line length can be less
C				the buffer width, and allows a picture to be
C				placed next to an existing picture.
C
C  The following information is returned in the COMMON blocks:
C	XLOC_SAO	Starting x location of image measured in the frame
C	YLOC_SAO	Starting y location of image measured in the frame
C	NPTSX_SAO	Number of SAOimage pixels in the x direction in the
C			loaded image
C	NPTSY_SAO	Number of SAOimage pixels in the y direction in the
C			loaded image
C	SCF_SAO		Scale factor for grayscale
C	AMIN_SAO	Lowest intensity
C	AMAX_SAO	Highest intensity
C	IMSX_SAO	Starting column in map
C	IMSY_SAO	Starting row in map
C
	INCLUDE 'miipspar.for'
	INCLUDE 'fitsfacts.for'
	INCLUDE 'saofacts.for' !  COMMON blocks

	CHARACTER*3 PRGFMAT
	CHARACTER*80 FILENAME
	INTEGER*2 I2,J2,ILOC(2)
	INTEGER PLANEKEY,STATUS,ISZ(4)
	REAL*4 AEX(2)
C
        COMMON /MAPN/ IMNUM
        COMMON /PLANEN/ IPNUM
C
	DIMENSION DAT(MAXSIZE)
	COMMON /TV_SCRATCH_BLOCK/ DAT
C
	INTEGER*2 XLOCI,YLOCI
	COMMON /TVTEXT/ XLOCI,YLOCI
C
	CHARACTER*80 DNAME
	COMMON /DFILE/ DNAME
C
	CHARACTER*1 SYSTEM_ENDIAN
	COMMON /SYS_ENDIAN/ SYSTEM_ENDIAN
C
C  Open the file with the map to be plotted, if it was not already opened.
C
	IUNIT=ABS(IDUMUNIT)
	IF (IDUMUNIT.LT.0) THEN
	  CALL MESSAGE(' ')
	  FILENAME=' '
	  MAPKEY=0
	  PLANEKEY=0
	  IF (KEY.EQ.0) THEN
	    CALL OPENFITS(IUNIT,.TRUE.,'FILE=',FILENAME,MAPKEY,PLANEKEY,NMAPS,
	1   NPLNS,NCOLS,NROWS,PRGFMAT,STATUS)
	  ELSE IF (KEY.EQ.1) THEN
	    CALL OPENFITS(IUNIT,.TRUE.,' ',DNAME,MAPKEY,PLANEKEY,NMAPS,
	1   NPLNS,NCOLS,NROWS,PRGFMAT,STATUS)
	  END IF
	END IF
C  Store map reference values.  NPIC_SAO is set in SAOFIRST
	MHED_SAO(1,NPIC_SAO)=1
	MHED_SAO(2,NPIC_SAO)=1
C  Store map name
	TV_FILENAME(NPIC_SAO)=DNAME
C  
   5	NFRAME_SAO=0
C	CALL USRINP(NFRAME_SAO,1,'I',4,1,'FRAME=','Enter the frame number:')
C  For now there is only one frame
C  Set up defaults for the area to be displayed
	ISZ(1)=1
	ISZ(2)=NAXIS1(IMNUM,IUNIT)
	ISZ(3)=1
	ISZ(4)=NAXIS2(IMNUM,IUNIT)
C  Read in the area
	CALL USRINP(ISZ,4,'I',4,1,'BOUNDARIES=',
	1'Enter start-column end-column start-row end-row:'//
	2'  (default = entire map)')
	IMIN=MIN(NAXIS1(IMNUM,IUNIT),MAX(1,ISZ(1)))
	IMAX=MAX(1,MIN(NAXIS1(IMNUM,IUNIT),ISZ(2)))
	JMIN=MIN(NAXIS2(IMNUM,IUNIT),MAX(1,ISZ(3)))
	JMAX=MAX(1,MIN(NAXIS2(IMNUM,IUNIT),ISZ(4)))
	ICOLS=IMAX-IMIN+1  ! Number of map pixels desired in X direction
        IROWS=JMAX-JMIN+1  ! Number of map pixels desired in Y direction
	NPTSX_SAO(NPIC_SAO)=ICOLS
	NPTSY_SAO(NPIC_SAO)=IROWS
	IF (ICOLS.NE.(ISZ(2)-ISZ(1)+1).OR.IROWS.NE.(ISZ(4)-ISZ(3)+1))
	1CALL MESSAGE('Boundary parameter was changed')
	IF (ICOLS.LE.0.OR.IROWS.LE.0) THEN
	  CALL MESSAGE('You can not display an empty set;  try again')
	  GO TO 5
	END IF
	ILOC(1)=0.5*(MAXFB_COLS_SAO-ICOLS)
	ILOC(2)=0.5*(MAXFB_ROWS_SAO+IROWS)-1
C  Read in the lower left starting location
	CALL MESSAGE('The upper left corner of the frame buffer is (0, 0)')
 	CALL MESSAGE('Enter the starting location of the lower left corner')
	CALL MESSAGE('of the map boundary in the frame buffer:)')
	CALL USRINP(ILOC,2,'I',2,1,'LOWER_LEFT=',
	1'(default = map centered in frame buffer)')
C  Make sure XLOC_SAO and YLOC_SAO are within the frame
	XLOC_SAO(NPIC_SAO)=MIN(MAXFB_COLS_SAO-1,MAX(0,ILOC(1)))
	YLOC_SAO(NPIC_SAO)=MIN(MAXFB_ROWS_SAO-1,MAX(0,ILOC(2)))
C  Make sure enough room is available in the frame and if not, try to correct
C  XLOC_SAO by shifting left and lining up the right sides
	XLOC_SAO(NPIC_SAO)=MIN(MAXFB_COLS_SAO-ICOLS,XLOC_SAO(NPIC_SAO))
C  Make sure enough room is available in the frame and if not, try to correct
C  YLOC_SAO by shifting down and lining up the top sides
	YLOC_SAO(NPIC_SAO)=MAX(IROWS-1,YLOC_SAO(NPIC_SAO))
	IF (XLOC_SAO(NPIC_SAO).NE.ILOC(1).OR.YLOC_SAO(NPIC_SAO).NE.ILOC(2)) 
	1CALL MESSAGE('Starting location in frame was changed')
C  Check that the possibly corrected XLOC_SAO is not off the left edge or that
C  the possibly corrected YLOC_SAO is not off the bottom edge of the 
C  frame
	IF (XLOC_SAO(NPIC_SAO).LT.0.OR.YLOC_SAO(NPIC_SAO).GT.
	1 MAXFB_ROWS_SAO-1) THEN
	  CALL MESSAGE('Buffer not wide enough for image;  try again')
	  GO TO 5
	END IF
	IXLOC=XLOC_SAO(NPIC_SAO) !  Converts to I4
        IYLOC=YLOC_SAO(NPIC_SAO) !  Converts to I4
C  Set up the frame dependent starting locations in the map
	IMSX_SAO(NFRAME_SAO,NPIC_SAO)=IMIN
	IMSY_SAO(NFRAME_SAO,NPIC_SAO)=JMIN
C  Scale the intensity
	AMIN_SAO(NFRAME_SAO)=0.0
	AMAX_SAO(NFRAME_SAO)=0.0

	AEX(1)=0.0
	AEX(2)=0.0
	DO WHILE (AEX(1).EQ.AEX(2))
		!  Protects against division by zero
	  AEX(1)=1.0E38
	  AEX(2)=-1.0E38
	  DO J=JMIN,JMAX
	    CALL RDFITSROW(IUNIT,IMNUM,IPNUM,DAT,NAXIS1(IMNUM,IUNIT),J)
	    DO I=IMIN,IMAX
	      IF (DAT(I).LT.AEX(1)) AEX(1)=DAT(I)
	      IF (DAT(I).GT.AEX(2)) AEX(2)=DAT(I)
	    END DO
	  END DO
	  CALL MESSAGE('Enter the minimum and maximum intensities:')
	  CALL USRINP(AEX,2,'F',4,1,'MINIMUM_MAXIMUM=',
	1 '(default = values in chosen region)')
	  AMIN_SAO(NFRAME_SAO)=AEX(1)
	  AMAX_SAO(NFRAME_SAO)=AEX(2)
	END DO
C  Compute the frame dependent intensity scale factor.  200 distinct values
C  can be displayed from the color lookup table;  2 values at the ends
C  of the range will display < and > before the values.
C  Testing of this algorithm can be done by loading tst0001.fits which
C  contains integer values from 0 to 255.  Choose BRIGHTNESS_MIN=0 and
C  BRIGHTNESS_MAX=199 for display of the integer values.
	SCF_SAO(NFRAME_SAO)=(AEX(2)-AEX(1))/199.0
C  Compute the values to set up SAOimage to define the clipping limits.
	AMIN_SAO(NFRAME_SAO)=AEX(1)+SCF_SAO(NFRAME_SAO)
	AMAX_SAO(NFRAME_SAO)=AEX(2)+SCF_SAO(NFRAME_SAO)
c	print *, aex(1),aex(2),amin_sao(nframe_sao),amax_sao(nframe_sao)
c	print *, nframe_sao,scf_sao(nframe_sao)
C	
C  Prepare to load a map into frame NFRAME_SAO
C
C  Set up the new WCS header information
	I=LEN(FILENAME)
	DO WHILE(FILENAME(I:I).NE.' '.AND.I.GE.1)
	  I=I-1
	END DO
	LF=MAX(I,1) !  Length of file name
	IBUFFER_SAO(1)='40000'O !  PACKED and IMC_SAMPLE
	IBUFFER_SAO(2)=-(LF+59)
	IBUFFER_SAO(3)=17 !  imhead.subunit = WCS (World coordinate system)
        IBUFFER_SAO(4)=0
        IBUFFER_SAO(5)=0
        IBUFFER_SAO(6)=0
        IBUFFER_SAO(7)=1 !  Frame number
        IBUFFER_SAO(8)=IFB_SAO-1 !  Frame buffer configuration - 1
	CALL IMTOOLCHECKSUM(IBUFFER_SAO)
	NBYTES=16
C  Send the WCS header
	CALL SI_WRITE(BUFFER_SAO,NBYTES)
C  Set up the WCS data packet
	CBUFFER_SAO(1:LF)=DNAME(1:LF)
	BUFFER_SAO(LF+1)=10
	CBUFFER_SAO(LF+2:LF+14)='1. 0. 0. -1. ' !  xx yx xy yy
	ENCODE (7,1000,CBUFFER_SAO(LF+15:LF+21)) IMIN+0.5-IXLOC !  xo =
	!  X Coordinate at left edge of buffer;  0.5 added to conform with 
	!  corner of pixel being x.0, y.0.
1000	FORMAT(F7.1)
	CBUFFER_SAO(LF+22:LF+22)=' '
	ENCODE (7,1000,CBUFFER_SAO(LF+23:LF+29)) IYLOC+JMIN+0.5 !  yo =
	!  Y Coordinate at top edge of buffer;  0.5 added to conform with 
	!  corner of pixel being x.0, y.0.
	CBUFFER_SAO(LF+30:LF+30)=' '
	ENCODE (12,2000,CBUFFER_SAO(LF+31:LF+42)) AMIN_SAO(NFRAME_SAO) !  low
2000	FORMAT(E12.5)
	CBUFFER_SAO(LF+43:LF+43)=' '
	ENCODE (12,2000,CBUFFER_SAO(LF+44:LF+55)) AMAX_SAO(NFRAME_SAO) !  high
	CBUFFER_SAO(LF+56:LF+57)=' 1' !  w_type (=linear)
	BUFFER_SAO(LF+58)=10
	BUFFER_SAO(LF+59)=0
	NBYTES=LF+59
C  Send the WCS data packet
	CALL SI_WRITE(BUFFER_SAO,NBYTES)
C  In IRAF at this point, the LUT header is sent.  Don't need it in MIIPS.
	IF (MODE.EQ.0) THEN !  A single map is loaded, many rows at a time
			    !  for highest speed
	  DO I=1,MAXNBYTES_SAO
	    BUFFER_SAO(I)=0.0
	  END DO
	  IOFF=IMIN-1
	  JOFF=JMIN+IYLOC
C  Get the initial row in the map to be read
	  JNEXT=JMIN
C  Find out how many rows can be sent at a time.
	  IROWS_AT_A_TIME=MAXNBYTES_SAO/MAXFB_COLS_SAO
C  Start loading several rows at a time into the buffer.  The full width
C  of the display buffer is used.
	  IROWS_REMAINING=IROWS
	  DO WHILE(IROWS_REMAINING.GT.0)
C  Make sure this is not more than the remaining number of rows to be displayed
	    IROWS_AT_A_TIME=MIN(IROWS_REMAINING,IROWS_AT_A_TIME)
	    NBYTES=MAXFB_COLS_SAO*IROWS_AT_A_TIME
	    IHDBUFFER_SAO(1)='71000'O !  = 29184
	    IHDBUFFER_SAO(2)=-NBYTES !  imhead.thingct
	    IHDBUFFER_SAO(3)=1 !  imhead.subunit = MEMORY
	    IHDBUFFER_SAO(4)=0
	    I2=0 !  We always load from the beginning of the buffer
            IHDBUFFER_SAO(5)=IOR(I2,'100000'O) !  x
	    JNEXT_PLUS_DEL=JNEXT+IROWS_AT_A_TIME-1
	    J2=JOFF-JNEXT_PLUS_DEL
	    IHDBUFFER_SAO(6)=IOR(J2,'100000'O) !  y
	    IHDBUFFER_SAO(7)=1
	    IHDBUFFER_SAO(8)=IOR(63,'300'O)
	    CALL IMTOOLCHECKSUM(IHDBUFFER_SAO)
	    NBYTES_HEADER=16
	    CALL SI_WRITE(HDBUFFER_SAO,NBYTES_HEADER)
	    DO J=JNEXT,JNEXT_PLUS_DEL
	      CALL RDFITSROW(IUNIT,IMNUM,IPNUM,DAT,NAXIS1(IMNUM,IUNIT),J)
C  Start the loop to increment the column number in the array
	      DO I=IMIN,IMAX
C  Valid addresses in the color table should have a minimum of 2 because
C  address 1 is reserved for indicating that the value is <= the lower limit.
	        IBUFFER_TEMP_SAO=2+
	1       (DAT(I)-AEX(1))/SCF_SAO(NFRAME_SAO)
C  Don't let any address be too small
	        IF (IBUFFER_TEMP_SAO.LE.1) IBUFFER_TEMP_SAO=1
C  Address 200 is reserved for values >= the upper limit.  Don't let any
C  addresses be larger than this.
	        IF (IBUFFER_TEMP_SAO.GE.200) IBUFFER_TEMP_SAO=200
	        IF (SYSTEM_ENDIAN.EQ.'L') THEN
	          BUFFER_SAO(I-IMIN+IXLOC+1+(JNEXT_PLUS_DEL-J)*
	1         MAXFB_COLS_SAO)=BUFFER_TEMP_SAO(1)
	        ELSE !  Big endian
	          BUFFER_SAO(I-IMIN+IXLOC+1+(JNEXT_PLUS_DEL-J)*
	1         MAXFB_COLS_SAO)=BUFFER_TEMP_SAO(2)
	        END IF
	      END DO
	    END DO
C  Send the rows of data to SAOimage
	    CALL SI_WRITE(BUFFER_SAO,NBYTES)
C  Get the next row in the map to be read
	    JNEXT=JNEXT+IROWS_AT_A_TIME
	    IROWS_REMAINING=IROWS_REMAINING-IROWS_AT_A_TIME
	  END DO
	ELSE IF (MODE.EQ.1) THEN !  One row at a time.  Slower, but allows
C				 !  multiple images to be loaded
C  Start the loop to increment the row number in the map
	  IOFF=IMIN-1
	  JOFF=JMIN+IYLOC
C  We use the auxilliary buffer so that it doesn't have to be reloaded every
C  row.
	  IHDBUFFER_SAO(1)='71000'O !  = 29184
	  IHDBUFFER_SAO(2)=-ICOLS !  imhead.thingct
	  IHDBUFFER_SAO(3)=1 !  imhead.subunit = MEMORY
	  IHDBUFFER_SAO(4)=0
	  I2=IXLOC
          IHDBUFFER_SAO(5)=IOR(I2,'100000'O) !  x
	  IHDBUFFER_SAO(7)=1
	  IHDBUFFER_SAO(8)=IOR(63,'300'O)
	  DO J=JMIN,JMAX
	    CALL RDFITSROW(IUNIT,IMNUM,IPNUM,DAT,NAXIS1(IMNUM,IUNIT),J)
	    J2=JOFF-J
	    IHDBUFFER_SAO(6)=IOR(J2,'100000'O) !  y
	    CALL IMTOOLCHECKSUM(IHDBUFFER_SAO)
	    NBYTES=16
	    CALL SI_WRITE(HDBUFFER_SAO,NBYTES)
	    NBYTES=MIN(MAXFB_COLS_SAO-I2,ICOLS)
C  Start the loop to increment the column number in the map
	    DO I=IMIN,IMAX
	      IBUFFER_TEMP_SAO=2+
	1     (DAT(I)-AEX(1))/SCF_SAO(NFRAME_SAO)
	      IF (IBUFFER_TEMP_SAO.LE.1) IBUFFER_TEMP_SAO=1
	      IF (IBUFFER_TEMP_SAO.GE.200) IBUFFER_TEMP_SAO=200
C  Take note of which endian this system is
	      IF (SYSTEM_ENDIAN.EQ.'L') THEN
	        BUFFER_SAO(I-IOFF)=BUFFER_TEMP_SAO(1)
	      ELSE
	        BUFFER_SAO(I-IOFF)=BUFFER_TEMP_SAO(2)
	      END IF
	    END DO
C  Send the row of data to SAOimage
	    CALL SI_WRITE(BUFFER_SAO,NBYTES)
	  END DO
	END IF
C  Exit
	XLOC_SAO(NPIC_SAO)=IXLOC
	YLOC_SAO(NPIC_SAO)=IYLOC
	RETURN
	END
