	SUBROUTINE WRFITSMHST(LU,MAP_NUMBER,KEYWORD,VALUE,FORMAT,COMMENT,
	1DUPLICATE)
C
C       This subroutine writes header strings to a FITS header.
C			Donald H. Gudehus, June 20, 2005
C			Version 2, D. Gudehus, July 29, 2006
C			Added ability to select which of multiple occurences
C			of a keyword.
C			Version 3, D. Gudehus, January 17, 2008
C			Fixed bug where the value for the keyword XTENSION
C			was sometimes getting corrupted.  Also fixed bug
C			where headers more than one record long resulted in
C			new keywords not being inserted.
C
C	LU		(C)	The logical unit that the file was opened on
C	MAP_NUMBER	(C)	The map number within the file.  If more than
C				one map exists, then there will also be a
C				primary header which can be referenced with
C				MAP_NUMBER=0.
C	KEYWORD		(C)	The FITS keyword to be written
C	VALUE		(C)	The character value to be written.  For the
C				keywords COMMENT and HISTORY, the value will
C				be placed in columns 9 through 80.  For other
C				keywords, e.g. XTENSION, the value (including
C				the required single quotes) will start in
C				column 11 and end not below column 20.  The
C				upper column limit can be as high as 80,
C				but in that case there will be no room for the
C				comment.  For keywords other than COMMENT and
C				HISTORY, the calling routine can optionally
C				include the single quotes that such values
C				require, or more conveniently, omit them in
C				which case this subroutine will insert them.
C	FORMAT		(C)	The desired format.  For writing, use 'C';
C				to replace a keycard with blanks, use 'X'.
C	COMMENT		(C)	The comment to be written.  This is placed
C				in columns 34 through 80.  It should not
C				contain single quotes, and should not contain
C				the forward slash which goes in column 32.
C	DUPLICATE	(C)	Duplicate/create (T) or replace/update (F)
C				existing header value if any.
C
	CHARACTER*(*) KEYWORD,VALUE,FORMAT,COMMENT
	LOGICAL DUPLICATE
C
	INCLUDE 'fitsfacts.for'
C
	BYTE PHYSREC(3000)
	CHARACTER*8 KEYWORD_WORK
	CHARACTER*80 KEYCARD,KEYCARDNEW,KEYCARDSAVE
	CHARACTER*3000 CHARPHYSREC
	EQUIVALENCE (PHYSREC,CHARPHYSREC)
	LOGICAL DONE,FOUND,ENDEXISTS,PUT_IN_COMMENT
C
	IF (LU.LT.LUMIN.OR.LU.GT.LUMAX) THEN
	  CALL ERROR(-4,'LU out of range in WRFITSMHST')
	END IF
	IF (MAP_NUMBER.LT.0.OR.MAP_NUMBER.GT.NUMMAPS(LU))
	1CALL ERROR(-4,'MAP_NUMBER out of range in WRFITSMHST')
	IBLKLEN=2880
C
	IRHSTART=HEADERRECORDS(1,MAP_NUMBER,LU)
	IRHEND=HEADERRECORDS(2,MAP_NUMBER,LU)
C  Build the new keycard
        KEYCARDNEW=' '
	N=LEN(KEYWORD)
	N=MIN(N,8)
        KEYCARDNEW(1:N)=KEYWORD(1:N)
	KEYWORD_WORK=' '
	KEYWORD_WORK(1:N)=KEYWORD(1:N)
	PUT_IN_COMMENT=.TRUE.
C  We don't have to check if this is the END keyword because WRFITSMHV
C  is used for writing that.
	IF (KEYWORD.NE.'COMMENT'.AND.KEYWORD.NE.'HISTORY') THEN
          IF (FORMAT.EQ.'C') THEN
	    KEYCARDNEW(9:9)='='
	    IF (VALUE(1:1).EQ.'''') THEN !  The quotes are already in place
	      KEYCARDNEW(11:30)=VALUE
	      ICSTART=34
	    ELSE !  We need to insert the quotes
	      KEYCARDNEW(11:11)=''''
	      KEYCARDNEW(12:79)=VALUE
C  Find out how many characters are in the value
	      NCHAR=1
	      NL=LEN(VALUE)
	      DO I=NL,1,-1
	        IF (VALUE(I:I).NE.' ') THEN
	          NCHAR=I
	          GO TO 200
	        END IF
	      END DO
 200	      IVEND=11+NCHAR
	      KEYCARDNEW(12:IVEND)=VALUE
	      IF (IVEND.LE.19) THEN
	        KEYCARDNEW(20:20)=''''
	        ICSTART=34
	      ELSE IF (IVEND.LE.29) THEN
	        KEYCARDNEW(30:30)=''''
	        ICSTART=34
	      ELSE IF (IVEND.LE.39) THEN
	        KEYCARDNEW(40:40)=''''
	        ICSTART=44
	      ELSE IF (IVEND.LE.49) THEN
	        KEYCARDNEW(50:50)=''''
	        ICSTART=54
	      ELSE IF (IVEND.LE.59) THEN
	        KEYCARDNEW(60:60)=''''
	        ICSTART=64
	      ELSE
	        KEYCARDNEW(80:80)=''''
	        PUT_IN_COMMENT=.FALSE.
	      END IF
	    END IF
C  Put in the comment
	    IF (PUT_IN_COMMENT) THEN
	      KEYCARDNEW(ICSTART-3:ICSTART-1)=' / '
	      KEYCARDNEW(ICSTART:80)=COMMENT
	    END IF
	  ELSE IF (FORMAT.EQ.'X') THEN
C  Replace with blanks
	    KEYCARDNEW=' '
	  ELSE
	    CALL ERROR(-1,'Wrong format in WRFITSMHST')
	    RETURN
	  END IF
	ELSE !  For COMMENT or HISTORY
	  IF (FORMAT.EQ.'C') THEN
	    KEYCARDNEW(9:80)=VALUE
	  ELSE IF (FORMAT.EQ.'X') THEN
C  Replace with blanks
	    KEYCARDNEW=' '
	  ELSE
	    CALL ERROR(-1,'Wrong format in WRFITSMHST')
	    RETURN
	  END IF
	END IF
C
	IF (.NOT.DUPLICATE) THEN
C  Find the matching existing keyword and write over it.
	  KEYCARD=' '
C  Consider the entire length of the keyword to avoid ambiguities
	  N=8
	  IR=IRHSTART
	  FOUND=.FALSE.
	  NOCCURENCE=0
	  DO WHILE(IR.LE.IRHEND)
	    IBPTR=1
	    READ(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	    IF (ISTAT.NE.0) THEN
	      CALL ERROR(-3,'Error in reading physical record')
	    END IF
	    DO WHILE(KEYCARD(1:3).NE.'END'.AND.IBPTR.LE.2801)
	      KEYCARD=CHARPHYSREC(IBPTR:IBPTR+79)
C  Eliminate control characters
	      DO I=1,80
	        IF (ICHAR(KEYCARD(I:I)).LT.32) KEYCARD(I:I)=' '
	      END DO
	      IF (KEYCARD(1:N).EQ.KEYWORD_WORK(1:N)) THEN
	        NOCCURENCE=NOCCURENCE+1
	        FOUND=.TRUE.
	      END IF
	      IBPTR=IBPTR+80
	    END DO
	    IR=IR+1
	  END DO
C  If there is more than one occurence of the desired keyword, ask the user
C  which occurence is to be written over.
	  NSELECT=1
	  IF (NOCCURENCE.GT.1) THEN
	    WRITE (6,1000) KEYWORD,NOCCURENCE
1000	    FORMAT(' ','The keyword ',A,' occurs ',I4,' times.')
	    WRITE (7,1000) NOCCURENCE
	    CALL USRINP(NSELECT,1,'I',4,0,'OCCURENCE=',
	1   'Which occurence do you wish to overwrite?')
	  END IF
	  IF (FOUND) THEN
	    KEYCARD=' '
	    IR=IRHSTART
	    DONE=.FALSE.
	    NOCCURENCE=0
	    DO WHILE(IR.LE.IRHEND.AND..NOT.DONE)
	      IBPTR=1
	      READ(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	      IF (ISTAT.NE.0) THEN
	        CALL ERROR(-3,'Error in reading physical record')
	      END IF
	      DO WHILE(KEYCARD(1:3).NE.'END'.AND.IBPTR.LE.2801.AND.
	1     .NOT.DONE)
	        KEYCARD=CHARPHYSREC(IBPTR:IBPTR+79)
C  Eliminate control characters
	        DO I=1,80
	          IF (ICHAR(KEYCARD(I:I)).LT.32) KEYCARD(I:I)=' '
	        END DO
	        IF (KEYCARD(1:N).EQ.KEYWORD_WORK(1:N)) THEN
	          NOCCURENCE=NOCCURENCE+1
	          IF (NOCCURENCE.EQ.NSELECT) THEN
C  Write the new keycard to the record and then write the
C  record to the file
	            CHARPHYSREC(IBPTR:IBPTR+79)=KEYCARDNEW(1:80)
	            WRITE(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	            IF (ISTAT.NE.0) THEN
	              CALL ERROR(-3,'Error in writing physical record')
	            END IF
	            DONE=.TRUE.
	          END IF
	        END IF
	        IBPTR=IBPTR+80
	      END DO
	      IR=IR+1
	    END DO
	  END IF
	END IF
C
	IF (DUPLICATE.OR.(.NOT.FOUND.AND.FORMAT.NE.'X')) THEN
C  Check to see if there is room for one more keycard in the record
	  R=NUMKEYCARDS(MAP_NUMBER,LU)/36.0 !  The distance into
			!  the map's header in units of records
	  IF ((R-INT(R).GT.0.0).OR.(R.EQ.0.0)) THEN !  There is room
			!  or the record currently has no keycards
C  Write over the END keyword with the new keyword and then write a
C  new END keyword.
C  IRHEND is measured from the beginning of the file, and is the
C  record containing the END keyword.
	    READ(LU,REC=IRHEND,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	    IF (ISTAT.NE.0) THEN
	      CALL ERROR(-3,'Error in reading physical record')
	    END IF
	    IBPTR=NINT((R-INT(R))*2880)-79 !  This is where END begins
	    KEYCARDSAVE=CHARPHYSREC(IBPTR:IBPTR+79)
	    IF (KEYCARDSAVE(1:3).EQ.'END') THEN
	      ENDEXISTS=.TRUE.
	    ELSE
	      ENDEXISTS=.FALSE.
	    END IF
C  Write the new keycard to the record and then write the
C  record to the file
	    IF (ENDEXISTS) THEN
	      CHARPHYSREC(IBPTR:IBPTR+79)=KEYCARDNEW(1:80)
	      IBPTR=IBPTR+80
	      CHARPHYSREC(IBPTR:IBPTR+79)=KEYCARDSAVE(1:80)
	    ELSE
	      IBPTR=IBPTR+80
	      CHARPHYSREC(IBPTR:IBPTR+79)=KEYCARDNEW(1:80)
	    END IF
	    WRITE(LU,REC=IRHEND,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	    IF (ISTAT.NE.0) THEN
	      CALL ERROR(-3,'Error in writing physical record')
	    END IF
C  Update the common block information in case anything else will be done
C  before closing the file.
	    NUMKEYCARDS(MAP_NUMBER,LU)=
	1   NUMKEYCARDS(MAP_NUMBER,LU)+1
	  ELSE !  No room left, so shift everything up starting at the
	       !  upper end of the file
	    IRLOW=HEADERRECORDS(2,MAP_NUMBER,LU)+1
	    IRHIGH=NUMRECORDS(LU)
	    DO IR=IRHIGH,IRLOW,-1
	      READ(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	      IF (ISTAT.NE.0) THEN
	        CALL ERROR(-1,'Error in reading physical record')
	      ELSE
	        WRITE(LU,REC=IR+1,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	        IF (ISTAT.NE.0) THEN
	          CALL ERROR(-3,'Error in reading physical record')
	        END IF
	      END IF
	    END DO
C  Write the new keycard to the end of header record and then write the
C  record to the file
	    IR=HEADERRECORDS(2,MAP_NUMBER,LU)
	    READ(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	    IBPTR=2880-79
	    KEYCARDSAVE=CHARPHYSREC(IBPTR:IBPTR+79)
	    CHARPHYSREC(IBPTR:IBPTR+79)=KEYCARDNEW(1:80)
	    WRITE(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	    IF (ISTAT.NE.0) THEN
	      CALL ERROR(-3,'Error in writing physical record')
	    END IF
C  Write the END keyword to the beginning of the next record.  This
C  record is otherwise empty.
	    DO I=1,IBLKLEN
	      CHARPHYSREC(I:I)=' ' !  Initializes
	    END DO
	    IBPTR=1
	    CHARPHYSREC(IBPTR:IBPTR+79)=KEYCARDSAVE(1:80)
	    WRITE(LU,REC=IR+1,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	    IF (ISTAT.NE.0) THEN
	      CALL ERROR(-3,'Error in writing physical record')
	    END IF
C  Update the common block information in case anything else will be done
C  before closing the file.
	    NUMRECORDS(LU)=NUMRECORDS(LU)+1
	    NUMKEYCARDS(MAP_NUMBER,LU)=
	1   NUMKEYCARDS(MAP_NUMBER,LU)+1
	    DO M=MAP_NUMBER,NUMMAPS(LU)
	      HEADERRECORDS(2,M,LU)=
	1     HEADERRECORDS(2,M,LU)+1
	      MAPRECORDS(1,M,LU)=MAPRECORDS(1,M,LU)+1
	      MAPRECORDS(2,M,LU)=MAPRECORDS(2,M,LU)+1
	    END DO
	    DO M=MAP_NUMBER+1,NUMMAPS(LU)
	      HEADERRECORDS(1,M,LU)=
	1     HEADERRECORDS(1,M,LU)+1
	    END DO
	  END IF
	END IF
C
	RETURN
	END
