	SUBROUTINE WRFITSMHV(LU,MAP_NUMBER,KEYWORD,VALUE,FORMAT,COMMENT,
	1DUPLICATE)
C
C       This subroutine writes header values to a FITS header.
C
C			Donald H. Gudehus, June 20, 2005
C
C	LU		(C)	The logical unit that the file was opened on
C	MAP_NUMBER	(C)	The map number within the file.  If more than
C				one map exists, then there will also be a 
C				primary header which can be referenced with
C				MAP_NUMBER=0.
C	KEYWORD		(C)	The FITS keyword to be written
C	VALUE		(C)	The value to be written.  This is declared 
C				as REAL*8 so as to work with all numerical 
C				formats.  In the calling program the value
C				should be declared as REAL*8 or if placed in
C				the argument list as a constant should be 
C				written in double precision style, e.g.,
C				1.2345D0.
C	FORMAT		(C)	The desired format, L, I4, R4, or R8
C	COMMENT		(C)	The comment to be written
C	DUPLICATE	(C)	Duplicate/create (T) or replace/update (F) 
C				existing header value.  For a new file, use 
C				DUPLICATE=.TRUE.
C
	
	CHARACTER*(*) KEYWORD,FORMAT,COMMENT
	LOGICAL DUPLICATE
	REAL*8 VALUE
C
	INCLUDE 'fitsfacts.for'
C
	BYTE PHYSREC(3000)
	CHARACTER*8 KEYWORD_WORK
	CHARACTER*80 KEYCARD,KEYCARDNEW,KEYCARDSAVE
	CHARACTER*3000 CHARPHYSREC
	EQUIVALENCE (PHYSREC,CHARPHYSREC)
	INTEGER*2 I2VALUE
	INTEGER IVALUE
	LOGICAL DONE,LVALUE,FOUND,ENDEXISTS
C
	IF (LU.LT.LUMIN.OR.LU.GT.LUMAX) THEN
	  CALL ERROR(-4,'LU out of range in WRFITSMHV')
	END IF
	IF (MAP_NUMBER.LT.0.OR.MAP_NUMBER.GT.NUMMAPS(LU))
	1CALL ERROR(-4,'MAP_NUMBER out of range in WRFITSMHV')
	IBLKLEN=2880
C
	IRHSTART=HEADERRECORDS(1,MAP_NUMBER,LU)
	IRHEND=HEADERRECORDS(2,MAP_NUMBER,LU)
C  Build the new keycard
	KEYCARDNEW=' '
	N=LEN(KEYWORD)
	N=MIN(N,8)
	KEYCARDNEW(1:N)=KEYWORD(1:N)
	KEYWORD_WORK=' '
	KEYWORD_WORK(1:N)=KEYWORD(1:N)
	IF (KEYCARDNEW(1:3).NE.'END') THEN
	  KEYCARDNEW(9:9)='='
	  IF (FORMAT.EQ.'L') THEN
	    LVALUE=NINT(VALUE)
	    IF (LVALUE) THEN
	      ENCODE(20,1010,KEYCARDNEW(11:11))
1010	      FORMAT(19X,1HT)
	    ELSE
	      ENCODE(20,1015,KEYCARDNEW(11:11))
1015	     FORMAT(19X,1HF)
	     END IF
	  ELSE IF (FORMAT.EQ.'I2') THEN
	    I2VALUE=NINT(VALUE)
	    ENCODE(20,1020,KEYCARDNEW(11:30)) I2VALUE
	  ELSE IF (FORMAT.EQ.'I4') THEN
	    IVALUE=NINT(VALUE)
	    ENCODE(20,1020,KEYCARDNEW(11:30)) IVALUE
1020 	    FORMAT(I20)
	  ELSE IF (FORMAT.EQ.'R4') THEN
	    RVALUE=VALUE
	    ENCODE(20,1030,KEYCARDNEW(11:)) RVALUE
1030	    FORMAT(5X,E15.7)
	  ELSE IF (FORMAT.EQ.'R8') THEN
	    ENCODE(20,1040,KEYCARDNEW(11:)) VALUE
1040	    FORMAT(E20.13)
	  ELSE
	    CALL ERROR(-3,'Wrong format in WRFITSMHV')
	  END IF
C  Put in the comment
	  KEYCARDNEW(31:33)=' / '
	  KEYCARDNEW(34:80)=COMMENT
	END IF
C
	IF (.NOT.DUPLICATE) THEN
C  Find the matching existing keyword and write over it.
	  KEYCARD=' '
	  N=8
	  IR=IRHSTART
	  DONE=.FALSE.
	  FOUND=.FALSE.
	  DO WHILE(IR.LE.IRHEND.AND..NOT.DONE)
	    IBPTR=1
	    READ(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	    IF (ISTAT.NE.0) THEN
	      CALL ERROR(-3,'Error in reading physical record')
	    END IF
	    DO WHILE(KEYCARD(1:3).NE.'END'.AND.IBPTR.LE.2801.AND.
	1   .NOT.DONE)
	      KEYCARD=CHARPHYSREC(IBPTR:IBPTR+79)
C  Eliminate control characters
	      DO I=1,80
	        IF (ICHAR(KEYCARD(I:I)).LT.32) KEYCARD(I:I)=' '
	      END DO
	      IF (KEYCARD(1:N).EQ.KEYWORD_WORK(1:N)) THEN
C  Write the new keycard to the record and then write the
C  record to the file
	        CHARPHYSREC(IBPTR:IBPTR+79)=KEYCARDNEW(1:80)
	        WRITE(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	        IF (ISTAT.NE.0) THEN
	          CALL ERROR(-3,'Error in writing physical record')
	        END IF
	        DONE=.TRUE.
	        FOUND=.TRUE.
	      END IF
	      IBPTR=IBPTR+80
	    END DO
	    IR=IR+1
	  END DO
	END IF
C
	IF (DUPLICATE.OR..NOT.FOUND) THEN
C  Check to see if there is room for one more keycard in the record
	  R=NUMKEYCARDS(MAP_NUMBER,LU)/36.0 !  The fraction
			!  of the way into the header
	  IF ((R-INT(R).GT.0.0).OR.(R.EQ.0.0)) THEN !  There was room
			!  or the record currently has no keycards
C  Write over the END keyword (if it exists) with the new keyword and then
C  write a new END keyword.
	    READ(LU,REC=IRHEND,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	    IF (ISTAT.NE.0) THEN
	      CALL ERROR(-3,'Error in reading physical record')
	    END IF
	    IBPTR=NINT(R*2880)-79
c	    IF (IBPTR.LE.0) IBPTR=1
C  Find out what the last keyword is.  If the file is old, it will be
C  END;  if it is new and the keywords are being inserted for the first
C  time, END will not yet exist.
	    KEYCARDSAVE=CHARPHYSREC(IBPTR:IBPTR+79)
	    IF (KEYCARDSAVE(1:3).EQ.'END') THEN
	      ENDEXISTS=.TRUE.
	    ELSE
	      ENDEXISTS=.FALSE.
	    END IF
C  Write the new keycard to the record and then write the
C  record to the file
	    IF (ENDEXISTS) THEN
	      CHARPHYSREC(IBPTR:IBPTR+79)=KEYCARDNEW(1:80)
	      IBPTR=IBPTR+80
	      CHARPHYSREC(IBPTR:IBPTR+79)=KEYCARDSAVE(1:80)
	    ELSE
	      IBPTR=IBPTR+80
	      CHARPHYSREC(IBPTR:IBPTR+79)=KEYCARDNEW(1:80)
	    END IF
	    WRITE(LU,REC=IRHEND,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	    IF (ISTAT.NE.0) THEN
	      CALL ERROR(-3,'Error in writing physical record')
	    END IF
C  Update the common block information in case anything else will be done
C  before closing the file.
            NUMKEYCARDS(MAP_NUMBER,LU)=
	1   NUMKEYCARDS(MAP_NUMBER,LU)+1
	  ELSE !  No room left, so shift everything up starting at the
	       !  upper end of the file
	    IRLOW=HEADERRECORDS(2,MAP_NUMBER,LU)+1
	    IRHIGH=NUMRECORDS(LU)
	    DO IR=IRHIGH,IRLOW,-1
	      READ(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	      IF (ISTAT.NE.0) THEN
	        CALL ERROR(-3,'Error in reading physical record')
	      ELSE
	        WRITE(LU,REC=IR+1,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	        IF (ISTAT.NE.0) THEN
	          CALL ERROR(-3,'Error in reading physical record')
	        END IF
	      END IF
	    END DO
C  Write the new keycard to the end of header record and then write the
C  record to the file
	    IR=HEADERRECORDS(2,MAP_NUMBER,LU)
	    READ(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	    IBPTR=2880-79
	    KEYCARDSAVE=CHARPHYSREC(IBPTR:IBPTR+79) !  Has END keyword
	    CHARPHYSREC(IBPTR:IBPTR+79)=KEYCARDNEW(1:80)
	    WRITE(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	    IF (ISTAT.NE.0) THEN
	      CALL ERROR(-3,'Error in writing physical record')
	    END IF
C  Write the END keyword to the beginning of the next record.  This
C  record is otherwise empty.
	    DO I=1,IBLKLEN
	      CHARPHYSREC(I:I)=' ' !  Initializes
	    END DO
	    IBPTR=1
	    CHARPHYSREC(IBPTR:IBPTR+79)=KEYCARDSAVE(1:80)
	    WRITE(LU,REC=IR+1,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	    IF (ISTAT.NE.0) THEN
	      CALL ERROR(-3,'Error in writing physical record')
	    END IF
C  Update the common block information in case anything else will be done
C  before closing the file.
	    NUMRECORDS(LU)=NUMRECORDS(LU)+1
	    NUMKEYCARDS(MAP_NUMBER,LU)=
	1   NUMKEYCARDS(MAP_NUMBER,LU)+1
	    DO M=MAP_NUMBER,NUMMAPS(LU)
	      HEADERRECORDS(2,M,LU)=
	1     HEADERRECORDS(2,M,LU)+1
	      MAPRECORDS(1,M,LU)=MAPRECORDS(1,M,LU)+1
	      MAPRECORDS(2,M,LU)=MAPRECORDS(2,M,LU)+1
	    END DO
	    DO M=MAP_NUMBER+1,NUMMAPS(LU)
	      HEADERRECORDS(1,M,LU)=
	1     HEADERRECORDS(1,M,LU)+1
	    END DO
	  END IF
	END IF
C
	RETURN
	END	
